/************************************************************************
 *
 * KAction.java -
 *
 * $Id: KAction.java,v 1.1.1.1 2006-06-30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import org.ietf.jgss.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;
import java.rmi.*;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.security.Security;
import javax.security.auth.Subject;
import javax.security.auth.login.*;
import com.sun.security.auth.callback.TextCallbackHandler;

public class KAction implements java.security.PrivilegedExceptionAction {

    public NetDBi netdb;

    public static GSSManager manager;
    public static GSSContext context;

    public KAction(NetDBi netdb) {
        super();

        this.netdb = netdb;
    }

    public Object run() throws Exception {
        try {
        //
        // This Oid is used to represent the Kerberos version 5 GSS-API
        // mechanism. It is defined in RFC 1964. We will use this Oid
        // whenever we need to indicate to the GSS-API that it must
        // use Kerberos for some purpose.
        //
        Oid krb5Oid = new Oid("1.2.840.113554.1.2.2");

        if( manager == null )
            manager = GSSManager.getInstance();

        //
        // Create a GSSName out of the server's name. The null
        // indicates that this application does not wish to make
        // any claims about the syntax of this name and that the
        // underlying mechanism should try to parse it as per whatever
        // default syntax it chooses.
        //
        GSSName serverName =
            manager.createName(System.getProperty(Constants.K5_SERVICE_KEY,
                                                    "service/netdb@stanford.edu"),
                                 null);

        //
        // Create a GSSContext for mutual authentication with the
        // server.
        //    - serverName is the GSSName that represents the server.
        //    - krb5Oid is the Oid that represents the mechanism to
        //      use. The client chooses the mechanism to use.
        //    - null is passed in for client credentials
        //    - DEFAULT_LIFETIME lets the mechanism decide how long the
        //      context can remain valid.
        // Note: Passing in null for the credentials asks GSS-API to
        // use the default credentials. This means that the mechanism
        // will look among the credentials stored in the current Subject
        // to find the right kind of credentials that it needs.
        //
        if( context == null )
            context = manager.createContext(serverName,
                                                krb5Oid,
                                                null,
                                                GSSContext.DEFAULT_LIFETIME);

        // Set the desired optional features on the context. The client
        // chooses these options.

        context.requestMutualAuth(true);  // Mutual authentication
        context.requestConf(true);  // Will use confidentiality later
        context.requestInteg(true); // Will use integrity later

        // Do the context eastablishment loop

        byte[] token = new byte[0];

        while (!context.isEstablished()) {

            // token is ignored on the first call
            Utils.debug("initSecContext "+token.length);
            token = context.initSecContext(token, 0, token.length);

            Utils.debug("token isNull = "+(token==null?"true":"false"));

            // Send a token to the server if one was generated by
            // initSecContext
            if (token != null) {
                Utils.debug("Calling klogin");
                Object retObject = netdb.klogin(token);
                Utils.debug("retObject.getClass() = "+retObject.getClass().getName());

                if( retObject instanceof byte[] ) {
                    token = (byte[])retObject;
                }
                Utils.debug("context.isEstablished() = "+context.isEstablished());
            }

        }

        Utils.debug("Context Established! ");
        Utils.debug("Client is " + context.getSrcName());
        Utils.debug("Server is " + context.getTargName());

        //
        // If mutual authentication did not take place, then only the
        // client was authenticated to the server. Otherwise, both
        // client and server were authenticated to each other.
        //
        if (context.getMutualAuthState()) {
            Utils.debug("Mutual authentication took place!");
            Object retObject = netdb.klogin(null);
            if( !(retObject instanceof NetDBi) )
                throw new Exception("Failed to obtain NetDB handle");
        }

        // successful completion
        return null;

        } finally {
            try {
                context.dispose();
                context = null;
                manager = null;
            } catch (Exception ex) {}
        }
    }
}
