/************************************************************************
 *
 * NetworkThread.java - all network relacted functions
 *
 * $Id: NetworkThread.java,v 1.2 2006-09-26 16:57:50 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.rmi.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import stanford.netdb.utils.*;

public class NetworkThread extends Thread {

    public static final int VERIFY_NAME_AND_IP  = 1;
    public static final int UNLOCK_NETWORK      = 2;
    public static final int REBUILD_AS          = 3;
    public static final int SAVE_NETWORK        = 4;
    public static final int SAVE_AND_CONTINUE   = 5;
    public static final int DELETE_NETWORK      = 6;
    public static final int MOVE_AS             = 7;

    private NetworkModifyFrame parent;
    private NetworkMoveFrame nmParent;
    private NetworkInfoFrame infoParent;
    private int action;
    private Hashtable obj;
    private String successMessage;
    private JComponent focusRequestor;
    private String handle;
    private Timestamp lockedTm = null;
    private boolean addedWarning = false;
    private String msg = "<html><body>";
    private boolean do_enable = true;

    public NetworkThread(NetworkMoveFrame nmParent, int action, String successMessage ) {
        this.nmParent = nmParent;
        this.action = action;
        this.successMessage = successMessage;
    }

    public NetworkThread(NetworkModifyFrame parent, int action,
                         Hashtable obj, String successMessage,
                         JComponent focusRequestor) {
        this.parent = parent;
        this.action = action;
        this.obj = obj;
        this.successMessage = successMessage;
        this.focusRequestor = focusRequestor;
    }

    public NetworkThread(NetworkInfoFrame infoParent, int action,
                         Hashtable obj, String successMessage) {
        this.infoParent = infoParent;
        this.action = action;
        this.obj = obj;
        Hashtable names = (Hashtable)obj.get("names");
        Enumeration enm = names.keys();
        handle = ((String)enm.nextElement()).trim();
        this.successMessage = successMessage;
    }

    public void run() {
        boolean error = true;
        try {
            switch( action ) {
            case DELETE_NETWORK:
                Utils.debug("Deleting network "+handle+" [locking] ...");

                lockedTm = MainWindow.netdb.lock(Constants.NETWORK, handle);
                obj.put("locked", lockedTm);
                Utils.debug("Deleting network "+handle+" [deleting] ...");

                MainWindow.netdb.delete(obj);

                JOptionPane.showMessageDialog(infoParent, "Network has been deleted.",
                          "Info", JOptionPane.INFORMATION_MESSAGE);
                infoParent.dispose();
                break;

            case REBUILD_AS:
                // first remove all panels
                Enumeration enm = parent.asPanels.keys();
                while( enm.hasMoreElements() ) {
                    Hashtable asData = (Hashtable)parent.asPanels.get((String)enm.nextElement());
                    JPanel asPanel = (JPanel)asData.get("big_panel");
                    parent.current_panel.remove(asPanel);
                }
                parent.asPanels.clear();

                // place them all back
                parent.current_gbc.gridx = parent.separator_gridx;
                parent.current_gbc.gridy = parent.separator_gridy;
                java.util.List asIDs = Arrays.asList(parent.addrSpacesH.keySet().toArray());
                Collections.sort(asIDs, parent.as_ip_cmp);
                Object[] asIDsArray = asIDs.toArray();
                for( int i = 0; i < asIDsArray.length; i++ ) {
                    String key = (String)asIDsArray[i];
                    Hashtable asData = parent.makeASPanel(key, (Hashtable)parent.addrSpacesH.get(key));
                    JPanel asPanel = (JPanel)asData.get("big_panel");
                    if( parent.allB.getText().equals("Expand") )
                        parent.collapseASPanel(asData);

                    parent.current_gbc.gridwidth = 2;
                    parent.current_gbc.fill = GridBagConstraints.NONE;
                    parent.current_gbc.anchor = GridBagConstraints.CENTER;
                    asData.put("current_gbc.gridwidth", new Integer(parent.current_gbc.gridwidth));
                    asData.put("current_gbc.fill", new Integer(parent.current_gbc.fill));
                    asData.put("current_gbc.anchor", new Integer(parent.current_gbc.anchor));
                    asData.put("gbc.gridx", new Integer(parent.current_gbc.gridx));
                    asData.put("gbc.gridy", new Integer(parent.current_gbc.gridy));
                    parent.current_gb_layout.setConstraints(asPanel, parent.current_gbc);
                    parent.current_panel.add(asPanel);
                    parent.current_gbc.gridwidth = 1;
                    parent.current_gbc.fill = GridBagConstraints.HORIZONTAL;
                    parent.tr();
                }
                parent.current_panel.revalidate();
                break;

            case UNLOCK_NETWORK:
                if( parent != null ) {
                    parent.closeB.setEnabled(false);
                    // called from NetworkModifyFrame
                    try {
                        MainWindow.netdb.unlock(Constants.NETWORK, parent.handle, parent.lockedTm);
                    } catch (Exception ex) {}
                    // now cancel all reserved ips
                    int totalCount = parent.reservedIPs.size();
                    int count = 0;
                    enm = parent.reservedIPs.keys();
                    while( enm.hasMoreElements() ) {
                        parent.msg.setText("Canceling reserved ip addresses ["+(count++)+"/"+totalCount+"]");
                        try {
                            MainWindow.netdb.cancelIP((String)enm.nextElement());
                        } catch (Exception ex) {}
                    }
                    // now cancel all reserved names
                    totalCount = parent.reservedNames.size();
                    count = 0;
                    enm = parent.reservedNames.keys();
                    while( enm.hasMoreElements() ) {
                        parent.msg.setText("Canceling reserved names ["+
                                           (count++)+"/"+totalCount+"]");
                        try {
                            MainWindow.netdb.cancelName((String)enm.nextElement(),
                                                        Constants.ADDRESS_NAME_TYPE);
                        } catch (Exception ex) {}
                    }
                } else if( nmParent != null ) {
                    // called from NetworkMoveFrame
                    try {
                        MainWindow.netdb.unlock(Constants.NETWORK, nmParent.handle, nmParent.lockedTm);
                    } catch (Exception ex) {}
                }
                break;

            case VERIFY_NAME_AND_IP:
                Utils.debug("VERIFY_NAME_AND_IP");

                // in this case 'obj' is 'asData'
                Hashtable asData = obj;
                if( asData == null )
                    break;
                JPanel dhcp_ip_panel = verifyNameAndIP(asData);
                if( dhcp_ip_panel == null )
                    break;
                dhcp_ip_panel.revalidate();
                dhcp_ip_panel.repaint();
                break;

            case SAVE_NETWORK:
            case SAVE_AND_CONTINUE:

                // pick up network name
                Hashtable names = (Hashtable)parent.newNet.get("names");
                enm = names.keys();
                String old_name = "";
                String old_domain = "";
                if( enm.hasMoreElements() ) {
                    String oldNameKey = (String)enm.nextElement();
                    Hashtable oldName = (Hashtable)names.get(oldNameKey);
                    old_name = ((String)oldName.get("name")).toLowerCase();
                    old_domain = ((String)oldName.get("domain")).toLowerCase();
                }
                Hashtable nameH = new Hashtable();
                String txt = parent.netNameTf.getText().trim();
                if( txt.trim().length() > 0 ) {
                    String name = Utils.name(txt);
                    String domain = Utils.domain(txt, MainWindow.userDefDomain);
                    nameH.put("name", name);
                    nameH.put("domain", domain);
                    names.clear();
                    names.put(name+"."+domain, nameH);

                    if( !name.toLowerCase().equals(old_name) ||
                        !domain.toLowerCase().equals(old_domain) )
                        nameH.put("dirty_name", "");
                } else
                    names.clear();

                // first we have to verify all dhcp ip addresses and their names
                Enumeration as_enm = parent.asPanels.keys();
                while( as_enm.hasMoreElements() ) {
                    String key = (String)as_enm.nextElement();
                    asData = (Hashtable)parent.asPanels.get(key);
                    StringBuffer errMsgB = new StringBuffer("");
                    boolean needs_verification = parent.needVerify(asData, errMsgB);
                    String errMsg = new String(errMsgB);
                    if( errMsg.length() > 0 ) {
                        parent.ePanelLabel.setText(errMsg);
                        parent.header_panel.add(parent.ePanel);
                        parent.header_panel.revalidate();
                        return;
                    }else if( needs_verification ) {
                        dhcp_ip_panel = verifyNameAndIP(asData);
                        if( dhcp_ip_panel == null )
                            break;
                        dhcp_ip_panel.revalidate();
                    }
                }

                parent.msg.setText("Saving network...");

                Hashtable net_warnings = null;
                if( parent.handle == null ) {
                    // this is a network insert
                    net_warnings = MainWindow.netdb.insert(parent.newNet);
                    // setup handle
                    names = (Hashtable)parent.newNet.get("names");
                    Enumeration name_enm = names.keys();
                    if( name_enm.hasMoreElements() ) {
                        nameH = (Hashtable)names.get((String)name_enm.nextElement());
                        parent.handle = nameH.get("name")+"."+nameH.get("domain");
                    }
                } else
                    net_warnings = MainWindow.netdb.update(parent.oldNet, parent.newNet);

                if( net_warnings != null && net_warnings.size() > 0 ) {
                    String w_msg = "<html><body>";
                    int w_size = net_warnings.size();
                    Enumeration w_enm = net_warnings.keys();
                    int i = 0;
                    while( w_enm.hasMoreElements() ) {
                        i++;
                        w_msg += Utils.htmlize(w_enm.nextElement().toString())+(w_size==i?"":"<br>");
                    }
                    JOptionPane.showMessageDialog(parent,
                    w_msg+"</body></html>", "Warning", JOptionPane.WARNING_MESSAGE);
                }
                if( action == SAVE_NETWORK ) {
                    NetworkModifyFrame.netModFrames.remove(parent.handle.toLowerCase());
                    NetworkInfoFrame nif = (NetworkInfoFrame)NetworkInfoFrame.
                        netInfoFrames.get(parent.handle.toLowerCase());
                    if( nif == null ) {
                        // open new info frame, but
                        // first we need to fined new handle
                        names = (Hashtable)parent.newNet.get("names");
                        enm = names.keys();

                        // now open new info frame
                        new NetworkInfoFrame(((String)enm.nextElement()).trim());
                    } else // reload existing info frame
                        nif.reloadB.doClick();
                    parent.dispose();
                }
                break;

            case MOVE_AS:
                MainWindow.netdb.moveAS(nmParent.asList,
                        (String)nmParent.destNet.getSelectedItem());
                try {
                    MainWindow.netdb.unlock(Constants.NETWORK, nmParent.handle, nmParent.lockedTm);
                } catch (Exception ex) {}
                JOptionPane.showMessageDialog(nmParent, successMessage,
                          "Info", JOptionPane.INFORMATION_MESSAGE);

                String handle = nmParent.handle;
                nmParent.dispose();

                NetworkInfoFrame nif = (NetworkInfoFrame)NetworkInfoFrame.
                    netInfoFrames.get(handle.toLowerCase());
                if( nif == null )
                    // now open new info frame
                    new NetworkInfoFrame(handle);
                else // reload existing info frame
                    nif.reloadB.doClick();
                break;

            default:
                break;
            }

            if( action == SAVE_AND_CONTINUE ) {
                parent.central_panel.removeAll();
                parent.central_panel.add(parent.msgPanel);
                parent.central_panel.revalidate();
                // we should pick up new handle becouse name could change
                Hashtable namesH = (Hashtable)parent.newNet.get("names");
                Enumeration enm = namesH.keys();
                Hashtable nameH = (Hashtable)namesH.get((String)enm.nextElement());
                parent.handle = nameH.get("name")+"."+nameH.get("domain");
                parent.setTitle("NetworkModify: "+parent.handle);
                /*
                parent.tool_bar.removeAll();
                parent.tool_bar.add(parent.modifyB);
                parent.tool_bar.add(parent.deleteB);
                parent.tool_bar.add(new JLabel("   "));
                parent.tool_bar.add(parent.reloadB);
                parent.tool_bar.add(new JLabel("   "));
                parent.tool_bar.add(parent.closeB);
                parent.central_panel.remove(parent.mainPanel);
                parent.mainPanel = new JPanel();
                parent.mainPanel.setBackground(Color.white);
                */
                parent.mainPanel.removeAll();
                parent.template = false; // reset template status
                (new Thread(parent)).start();
                do_enable = false;
            }

            return;

        } catch(java.rmi.ServerException exx) {
            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                msg += (addedWarning?"<br>":"");
                int esize = errors.size();
                for( int i = 0; i < esize; i++ )
                    msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                        "</font>"+(i<esize-1?"<br>":"");
                if( parent != null ) {
                    parent.ePanelLabel.setText(msg+"</body></html>");
                    parent.header_panel.add(parent.ePanel);
                    parent.header_panel.revalidate();
                }
                if( infoParent != null ) {
                    JOptionPane.showMessageDialog(infoParent,
                               msg, "Error", JOptionPane.ERROR_MESSAGE);
                    infoParent.central_panel.removeAll();
                    infoParent.central_panel.add(infoParent.mainPanel);
                    infoParent.central_panel.revalidate();
                }
                if( nmParent != null ) {
                    JOptionPane.showMessageDialog(nmParent,
                               msg, "Error", JOptionPane.ERROR_MESSAGE);
                    nmParent.central_panel.removeAll();
                    nmParent.central_panel.add(nmParent.mainPanel);
                    nmParent.central_panel.revalidate();
                    nmParent.modifyB.setEnabled(true);
                }
            } else if( exx.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)exx.getCause();
                msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                    re.getMessage()+"</font>";
                if( parent != null ) {
                    parent.ePanelLabel.setText(msg+"</body></html>");
                    parent.header_panel.add(parent.ePanel);
                    parent.header_panel.revalidate();
                }
                if( infoParent != null ) {
                    JOptionPane.showMessageDialog(infoParent, msg,
                                                  "Error", JOptionPane.ERROR_MESSAGE);
                    infoParent.central_panel.removeAll();
                    infoParent.central_panel.add(infoParent.mainPanel);
                    infoParent.central_panel.revalidate();
                }
                if( nmParent != null ) {
                    JOptionPane.showMessageDialog(nmParent,
                                                  msg, "Error", JOptionPane.ERROR_MESSAGE);
                    nmParent.central_panel.removeAll();
                    nmParent.central_panel.add(nmParent.mainPanel);
                    nmParent.central_panel.revalidate();
                    nmParent.modifyB.setEnabled(true);
                }
            } else {
                msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                    exx.getMessage()+"</font>";
                if( parent != null ) {
                    parent.ePanelLabel.setText(msg+"</body></html>");
                    parent.header_panel.add(parent.ePanel);
                    parent.header_panel.revalidate();
                }
                if( infoParent != null ) {
                    JOptionPane.showMessageDialog(infoParent,
                         msg, "Error", JOptionPane.ERROR_MESSAGE);
                    infoParent.central_panel.removeAll();
                    infoParent.central_panel.add(infoParent.mainPanel);
                    infoParent.central_panel.revalidate();
                }
                if( nmParent != null ) {
                    JOptionPane.showMessageDialog(nmParent,
                         msg, "Error", JOptionPane.ERROR_MESSAGE);
                    nmParent.central_panel.removeAll();
                    nmParent.central_panel.add(nmParent.mainPanel);
                    nmParent.central_panel.revalidate();
                    nmParent.modifyB.setEnabled(true);
                }
            }

        } catch (java.rmi.NoSuchObjectException exx) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException exx) {
            X.handleTimeout();

        } catch(Exception exx) {
            msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                exx.getMessage()+"</font>";
            if( parent != null ) {
                parent.ePanelLabel.setText(msg+"</body></html>");
                parent.header_panel.add(parent.ePanel);
                parent.header_panel.revalidate();
            }
            if( infoParent != null ) {
                JOptionPane.showMessageDialog(infoParent,
                    msg, "Error", JOptionPane.ERROR_MESSAGE);
                infoParent.central_panel.removeAll();
                infoParent.central_panel.add(infoParent.mainPanel);
                infoParent.central_panel.revalidate();
            }
            if( nmParent != null ) {
                JOptionPane.showMessageDialog(nmParent,
                    msg, "Error", JOptionPane.ERROR_MESSAGE);
                nmParent.central_panel.removeAll();
                nmParent.central_panel.add(nmParent.mainPanel);
                nmParent.central_panel.revalidate();
                nmParent.modifyB.setEnabled(true);
            }

        } finally {
            if( parent != null ) {
                if( action == UNLOCK_NETWORK ) {
                    // now close parent window
                    NetworkModifyFrame.netModFrames.remove(parent.handle.toLowerCase());
                    parent.dispose();
                    return;
                }
                parent.mega_pane.removeAll();
                parent.mega_pane.add(parent.scrollpane, BorderLayout.CENTER);
                parent.mega_pane.revalidate();
                parent.mega_pane.repaint();

                if( do_enable ) {
                    parent.reloadB.setEnabled(true);
                    parent.modifyB.setEnabled(true);
                    parent.deleteB.setEnabled(true);
                }

                parent.closeB.setEnabled(true);

                if( focusRequestor != null )
                    focusRequestor.requestFocus();
            }
            if( infoParent != null ) {
                if( lockedTm != null ) {
                    try {
                        MainWindow.netdb.unlock(Constants.NETWORK, handle, lockedTm);
                    } catch (Exception ex) {}
                }
                infoParent.deleteB.setEnabled(true);
                infoParent.reloadB.setEnabled(true);
                infoParent.modifyB.setEnabled(true);
                infoParent.templateB.setEnabled(true);
                infoParent.moveASB.setEnabled(true);
            }
            if( nmParent != null ) {
                if( lockedTm != null ) {
                    try {
                        MainWindow.netdb.unlock(Constants.NETWORK, nmParent.handle, nmParent.lockedTm);
                    } catch (Exception ex) {}
                }
                nmParent.modifyB.setEnabled(true);
                nmParent.closeB.setEnabled(true);
            }
        }

    } // end of run

    private JPanel verifyNameAndIP(Hashtable asData) throws Exception {
        Hashtable as = (Hashtable)parent.addrSpacesH.get((String)asData.get("id"));
        String netPrefix = Utils.ip2str(((Integer)as.get("addr_space_ip")).intValue())+
            "/"+((Integer)as.get("prefix_length"));
        Hashtable dhcpAddrs = (Hashtable)as.get("dhcp_addresses");
        if( dhcpAddrs == null ) {
            dhcpAddrs = new Hashtable();
            as.put("dhcp_addresses", dhcpAddrs);
        }
        // first go over all dhcp addresses
        Enumeration enm = dhcpAddrs.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable dhcpIP = (Hashtable)dhcpAddrs.get(key);
            if( dhcpIP.get("dirty") == null )
                continue;
            // first take care of ip addr.
            if( dhcpIP.get("dirty_ip") != null ) {
                String ipAddr = (String)dhcpIP.get("ip_address");
                Utils.debug("validating "+ipAddr);
                Hashtable result = MainWindow.netdb.reserveIP(netPrefix, ipAddr, 1);

                Hashtable ipH = (Hashtable)((Hashtable)asData.get("dhcp_addressesH")).get(key);
                AJFormattedTextField ftf = (AJFormattedTextField)ipH.get("ip");
                // let see what we've got here and stuff it into text field
                Hashtable ip_addresses = (Hashtable)((Hashtable)result.get(ipAddr)).get("ip_addresses");
                Enumeration enm1 = ip_addresses.keys();
                if( enm1.hasMoreElements() ) {
                    String ipStr = (String)enm1.nextElement();
                    parent.reservedIPs.put(ipStr, "");
                    ftf.setText(ipStr);
                    ((Hashtable)((Hashtable)((Hashtable)parent.addrSpacesH.get((String)asData.get("id"))).
                                 get("dhcp_addresses")).get(ftf.key)).put("ip_address", ipStr);
                }

                // read warnings if any
                Hashtable warnings = (Hashtable)result.get("warnings");
                if( warnings != null ) {
                    enm1 = warnings.keys();
                    boolean first = true;
                    while( enm1.hasMoreElements() ) {
                        if( first && addedWarning ) {
                            first = false;
                            msg += "<br>";
                        }
                        msg += "&nbsp;" + enm1.nextElement() +
                            (enm1.hasMoreElements()?"<br>":"");
                        addedWarning = true;
                    }
                }
                dhcpIP.remove("dirty_ip");
            }
            // now names. we assume only one name per ip
            Hashtable namesH = (Hashtable)dhcpIP.get("names");
            if( namesH != null ) {
                Enumeration enm1 = namesH.keys();
                if( enm1.hasMoreElements() ) {
                    String dhcpNameKey = (String)enm1.nextElement();
                    Hashtable dhcpName = (Hashtable)namesH.get(dhcpNameKey);
                    if( dhcpName == null || dhcpName.get("dirty_name") == null )
                        continue;
                    String nameStr = ((String)dhcpName.get("name")).trim();
                    String domainStr = ((String)dhcpName.get("domain")).trim();
                    String fqdn = nameStr + "." + domainStr;
                    String reservedName = fqdn;
                    if( parent.reserved_and_committed_names.get(fqdn.toLowerCase()) == null )
                        reservedName = MainWindow.netdb.reserveName(fqdn, Constants.ADDRESS_NAME_TYPE);
                    Hashtable namesHash = new Hashtable();
                    dhcpIP.put("names", namesHash);
                    Hashtable nameH = new Hashtable();
                    String name = Utils.name(reservedName);
                    nameH.put("name", name);
                    String domain = Utils.domain(reservedName, MainWindow.userDefDomain);
                    nameH.put("domain", domain);
                    namesHash.put(name+"."+domain, nameH);
                    parent.reserved_and_committed_names.put(reservedName.toLowerCase(), "");
                    parent.reservedNames.put(reservedName.toLowerCase(), "");
                    dhcpName.remove("dirty_name");
                }
            }

            // clean up 'dirty' status
            dhcpIP.remove("dirty");
        }

        // now check fields for adding of new addresses and/or names
        JTextField newIpTf = (JTextField)asData.get("newIpTf");
        String newIpStr = newIpTf.getText().trim();
        // no need to verify cause' it is already done in NetworkModifyFrame.java

        JTextField newCountTf = (JTextField)asData.get("newCountTf");
        String newCountStr = newCountTf.getText().trim();
        if( newCountStr.length() == 0 ) {
            newCountStr = "1";
            if( newIpStr.length() == 0 ) // nothing to do anymore
                return null;
        }
        int newCount = Integer.parseInt(newCountStr);

        // now verify new dhcp ip addresses
        Hashtable reservedIPs = MainWindow.netdb.reserveIP((String)as.get("addr_space_prefix"),
                                                      newIpStr, newCount);
        // read warnings if any
        Hashtable warnings = (Hashtable)((Hashtable)reservedIPs.get(newIpStr)).get("warnings");
        if( warnings != null ) {
            enm = warnings.keys();
            boolean first = true;
            while( enm.hasMoreElements() ) {
                if( first && addedWarning ) {
                    first = false;
                    msg += "<br>";
                }
                msg += "&nbsp;" + enm.nextElement() +
                    (enm.hasMoreElements()?"<br>":"");
                addedWarning = true;
            }
        }
        if( addedWarning ) {
            parent.ePanelLabel.setText(msg+"</body></html>");
            parent.header_panel.add(parent.ePanel);
            parent.header_panel.revalidate();
        }
        Hashtable rips = (Hashtable)((Hashtable)reservedIPs.get(newIpStr)).get("ip_addresses");
        if( rips == null )
            return null;

        newIpTf.setText("");
        newCountTf.setText("");
        // stuff them into network and reference hashes
        int ipCountTotal = rips.size();
        if( ipCountTotal > 0 ) {
            parent.msg.setText("Verifying names [0/"+ipCountTotal+"]");

            enm = rips.keys();
            int ipCount = 0;
            while( enm.hasMoreElements() ) {
                String ipStr = (String)enm.nextElement();
                parent.reservedIPs.put(ipStr, ""); // reference hash
                // make up ip
                String key = "ip "+(parent.available_key++);
                // make up the whole data structure
                Hashtable newDHCPip = new Hashtable();
                dhcpAddrs.put(key, newDHCPip);
                newDHCPip.put("active?", new Boolean(true));
                newDHCPip.put("ip_address", ipStr);
                Utils.debug("reserved ip "+ipStr);
                Hashtable names = new Hashtable();
                newDHCPip.put("names", names);
                // generate and reserve names
                int salt = 0;
                for(;;) { // try 10 times with different salt
                    Hashtable dhcpNameH = MainWindow.netdb.generateHexName(ipStr, salt);
                    try {
                        String dhcpName = dhcpNameH.get("name")+"."+dhcpNameH.get("domain");
                        MainWindow.netdb.reserveName(dhcpName, Constants.ADDRESS_NAME_TYPE);
                        Hashtable namesH = new Hashtable();
                        newDHCPip.put("names", namesH);
                        Hashtable nameH = new Hashtable();
                        namesH.put(dhcpName, nameH);
                        nameH.put("name", (String)dhcpNameH.get("name"));
                        nameH.put("domain", (String)dhcpNameH.get("domain"));
                        parent.reservedNames.put(dhcpName.toLowerCase(), "");
                        break;
                    } catch (Exception exxx) {
                        if( salt > 10 )
                            throw exxx;
                        salt++;
                    }
                }
                parent.msg.setText("Verifying names ["+(++ipCount)+"/"+ipCountTotal+"]");
            }
        }

        // now rebuild whole dhcp panel
        Hashtable dhcp_addressesH = (Hashtable)asData.get("dhcp_addressesH");
        JPanel dhcp_ip_panel = (JPanel)asData.get("dhcp_ip_panel");
        dhcp_ip_panel.removeAll();
        GridLayout dhcp_ip_panel_layout = (GridLayout)asData.get("dhcp_ip_panel_layout");
        Hashtable dhcp_addresses = (Hashtable)as.get("dhcp_addresses");
        if( dhcp_addresses == null )
            dhcp_addresses = new Hashtable();
        dhcp_ip_panel_layout.setRows(dhcp_addresses.size());
        java.util.List list = Arrays.asList(dhcp_addresses.keySet().toArray());
        Collections.sort(list, parent.ip_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String ipStr = (String)it.next();
            Hashtable ipH = new Hashtable();
            dhcp_addressesH.put(ipStr, ipH);

            JPanel ip_panel = new JPanel();
            ip_panel.setBackground(Color.white);
            dhcp_ip_panel.add(ip_panel);
            ipH.put("ip_panel", ip_panel);

            AJCheckBox cb = new AJCheckBox("", true, asData);
            cb.obj2 = ipStr;
            cb.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)e.getSource();
                        if( cb.isSelected() ) return;

                        ((Hashtable)((Hashtable)parent.addrSpacesH.get((String)cb.box.get("id"))).
                         get("dhcp_addresses")).remove((String)cb.obj2);

                        JPanel dhcp_ip_panel = (JPanel)cb.box.get("dhcp_ip_panel");
                        Hashtable ipH = (Hashtable)((Hashtable)cb.box.get("dhcp_addressesH")).
                            remove((String)cb.obj2);
                        dhcp_ip_panel.remove((JPanel)ipH.get("ip_panel"));

                        GridLayout dhcp_ip_panel_layout = (GridLayout)cb.box.get("dhcp_ip_panel_layout");
                        dhcp_ip_panel_layout.setRows(dhcp_ip_panel_layout.getRows() - 1);

                        dhcp_ip_panel.revalidate();
                    }
                });
            cb.setBackground(Color.white);
            ip_panel.add(cb);

            Hashtable addr = (Hashtable)dhcp_addresses.get(ipStr);
            AJFormattedTextField ftf = new AJFormattedTextField((String)addr.get("ip_address"),
                                                                16, null, asData, ipStr);
            ftf.addFocusListener(new FocusAdapter() {
                    public void focusLost(FocusEvent e) {
                        doAction((AJFormattedTextField)e.getSource());
                    }
                    public void doAction(AJFormattedTextField ftf) {
                        Hashtable as = (Hashtable)parent.addrSpacesH.get((String)ftf.box.get("id"));
                        Hashtable dhcpIP = (Hashtable)((Hashtable)as.get("dhcp_addresses")).get(ftf.key);
                        String oldIP = (String)dhcpIP.get("ip_address");
                        String newIP = ftf.getText().trim();
                        if( !oldIP.equals(newIP) ) {
                            dhcpIP.put("ip_address", newIP);
                            dhcpIP.put("dirty_ip", "");
                        }
                    }
                });
            ipH.put("ip", ftf);
            ftf.setEditable(true);
            ftf.setFont(parent.label_font);
            ip_panel.add(ftf);

            JLabel label = new JLabel("  Name");
            label.setFont(parent.label_font);
            ip_panel.add(label);

            Hashtable names = (Hashtable)addr.get("names");
            if( names == null ) {
                names = new Hashtable();
                addr.put("names", names);
            }
            String name = "";
            enm = names.keys();
            if( enm.hasMoreElements() ) {
                Hashtable nameH = (Hashtable)names.get((String)enm.nextElement());
                name = (String)nameH.get("name");
                String domain = (String)nameH.get("domain");
                if( !domain.toLowerCase().trim().equals(MainWindow.userDefDomain) )
                    name += "."+domain;
            }

            AJTextField tf = new AJTextField(name, 17, asData, ipStr);
            tf.addFocusListener(new NetworkDHCPNameFocusListener());
            ipH.put("name", tf);
            tf.setEditable(true);
            tf.setFont(parent.label_font);
            ip_panel.add(tf);

            Boolean active = (Boolean)addr.get("active?");
            if( active == null )
                active = new Boolean(false);
            AJCheckBox activeCB = new AJCheckBox("Active", active.booleanValue(), asData);
            activeCB.obj2 = addr;
            activeCB.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        AJCheckBox cb = (AJCheckBox)e.getSource();
                        if( cb.isSelected() )
                            ((Hashtable)(cb.obj2)).put("active?", new Boolean(true));
                        else
                            ((Hashtable)(cb.obj2)).put("active?", new Boolean(false));
                    }
                });
            activeCB.setBackground(Color.white);
            activeCB.setFont(parent.label_font);
            ip_panel.add(activeCB);
        } // end of while(...)
        return dhcp_ip_panel;
    } // end of verifyNameAndIP(...)
}
