/************************************************************************
 *
 * NodeInfoFrame.java -
 *
 * $Id: NodeInfoFrame.java,v 1.2 2006-09-26 17:07:42 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.text.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class NodeInfoFrame extends XFrame implements Runnable {

    public String handle;
    public Integer idNode = null;
    Hashtable node = null;

    NodeInfoFrame THIS;
    boolean delete = false;
    public JPanel mainPanel = new JPanel();
    public boolean hasAccess = false;
    public JDialog deleteDialogWin;

    public static Hashtable nodeInfoFrames = new Hashtable();

    public void dispose() {
        synchronized(nodeInfoFrames) {
            nodeInfoFrames.remove(handle.toLowerCase());
        }
        super.dispose();
    }

    private void initFrame() {

        init(); // init XTestFrame bowls

        tool_bar.removeAll();
        tool_bar.add(modifyB);
        templateB.setText("Use As Template");
        tool_bar.add(templateB);
        tool_bar.add(deleteB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(reloadB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(closeB);

        templateB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    NodeModifyFrame.MakeNodeModifyFrame(handle, true);
                }
            });

        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    dispose();
                }
            });

        reloadB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    current_panel.removeAll();
                    current_panel.add(label);
                    current_panel.revalidate();
                    current_panel.repaint();

                    node = null;
                    (new Thread(THIS)).start();
                }
            });

        deleteB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    Hashtable node = ((NodeInfoFrame)parent).node;

                    JPanel panel = new JPanel(new BorderLayout());
                    //panel.setBackground(Color.white);
                    panel.add(new JLabel("<html><body>Delete node <i>"+
                                         Utils.htmlize(handle)+"</i> ?<br>&nbsp;</body></html>", JLabel.CENTER),
                              BorderLayout.NORTH);
                    Hashtable mxSlaves = Utils.nHash(node.get("mxSlaves"));
                    if( mxSlaves.size() > 0 ) {
                        JPanel mpanel = new JPanel(new BorderLayout());
                        //mpanel.setBackground(Color.white);
                        mpanel.add(new JLabel("<html><body>To stop directing mail to these mail servers,<br>"+
                                              "check the appropriate boxes.</body></html>"), BorderLayout.NORTH);
                        panel.add(mpanel, BorderLayout.CENTER);
                        JPanel cpanel = new JPanel(new GridLayout(mxSlaves.size(), 1));
                        //cpanel.setBackground(Color.white);
                        mpanel.add(cpanel, BorderLayout.CENTER);
                        Enumeration enm = mxSlaves.keys();
                        while( enm.hasMoreElements() ) {
                            String nodeID = (String)enm.nextElement();
                            Hashtable names = (Hashtable)mxSlaves.get(nodeID);
                            java.util.List list = Arrays.asList(names.keySet().toArray());
                            Collections.sort(list, name_cmp);
                            Object[] nameArray = list.toArray();
                            if( nameArray.length > 0 ) {
                                Hashtable box = new Hashtable();
                                AJCheckBox cb = new AJCheckBox(nameArray[0], false, names);
                                cb.addActionListener(new ActionListener(){
                                    public void actionPerformed(ActionEvent e) {
                                        AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                        if( cb.isSelected() )
                                            cb.box.put(" remove", "");
                                        else
                                            cb.box.remove(" remove");
                                    }
                                });
                                cb.setBackground(cpanel.getBackground());
                                cpanel.add(cb);
                            }
                        }
                    }

                    Object[] delButtons = new JButton[2];
                    delButtons[0] = new JButton("Yes");
                    ((JButton)(delButtons[0])).addActionListener(new AActionListener(THIS) {
                                public void actionPerformed(ActionEvent ev) {
                                    ((NodeInfoFrame)parent).deleteDialogWin.dispose();
                                    deleteB.setEnabled(false);
                                    templateB.setEnabled(false);
                                    modifyB.setEnabled(false);
                                    reloadB.setEnabled(false);
                                    central_panel.removeAll();
                                    central_panel.add(label, BorderLayout.CENTER);
                                    label.setText("Deleting node...");
                                    central_panel.revalidate();
                                    central_panel.repaint();
                                    (new NodeThread(THIS, NodeThread.DELETE_NODE)).start();
                                }
                            });
                    delButtons[1] = new JButton("No");
                    ((JButton)(delButtons[1])).addActionListener(new AActionListener(THIS) {
                                public void actionPerformed(ActionEvent ev) {
                                    ((NodeInfoFrame)parent).deleteDialogWin.dispose();
                                }
                            });

                    JOptionPane delDialog =
                        new JOptionPane(panel, JOptionPane.PLAIN_MESSAGE, JOptionPane.YES_NO_OPTION, null, delButtons);
                    deleteDialogWin = delDialog.createDialog(parent, "Delete Node?");
                    deleteDialogWin.show();
                }
            });

        modifyB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    NodeModifyFrame.MakeNodeModifyFrame(handle, false);
                }
            });

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    THIS.closeB.doClick();
                }
            });
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );

        this.setVisible(true);

        register();
    } // end of initFrame()

    public static NodeInfoFrame MakeNodeInfoFrame(String handle, String action) {
        if( action.equals("info") )
            return NodeInfoFrame.MakeNodeInfoFrame(handle);
        else if( action.equals("delete") ) {
            synchronized(nodeInfoFrames) {
                NodeInfoFrame f = (NodeInfoFrame)nodeInfoFrames.get(handle.toLowerCase());
                if( f != null ) {
                    f.setState(Frame.NORMAL);
                    f.toFront();
                    f.deleteB.doClick();
                    return f;
                }
            }

            NodeInfoFrame nif = NodeInfoFrame.MakeNodeInfoFrame(handle);
            nif.delete = true;
            synchronized(nodeInfoFrames) {
                nodeInfoFrames.put(handle.toLowerCase(), nif);
            }
            return nif;
        } else
            return null;
    }

    public static NodeInfoFrame MakeNodeInfoFrame(String handle) {
        if( handle != null ) {
            synchronized(nodeInfoFrames) {
                NodeInfoFrame f = (NodeInfoFrame)nodeInfoFrames.get(handle.toLowerCase());
                if( f != null ) {
                    f.setState(Frame.NORMAL);
                    f.toFront();
                    return f;
                }
            }
        }

        NodeInfoFrame nif = new NodeInfoFrame(handle);
        synchronized(nodeInfoFrames) {
            nodeInfoFrames.put(handle.toLowerCase(), nif);
        }
        (new Thread(nif)).start();
        return nif;
    }


    private NodeInfoFrame(Hashtable node) {
        super("(Hash)");
        this.node = node;
        Hashtable names = (Hashtable)node.get("names");
        Enumeration enm = names.keys();
        handle = ((String)enm.nextElement()).trim();
        if( this.handle.indexOf(".") < 0 )
            this.handle += ".";
        THIS = this;
        initFrame();
    }

    private NodeInfoFrame(String handle) {
        super("NodeInfo: "+handle);
        this.handle = handle;
        if( this.handle.indexOf(".") < 0 )
            this.handle += ".";
        setTitle("NodeInfo: "+this.handle);
        THIS = this;
        initFrame();
    }

    public void run() {
        try {

            mainPanel.setBackground(Color.white);

            reloadB.setEnabled(false);
            deleteB.setEnabled(false);
            modifyB.setEnabled(false);
            templateB.setEnabled(false);

            if( node == null ) {
                if( idNode == null )
                    node = MainWindow.netdb.getInfo(Constants.NODE, handle);
                else
                    node = MainWindow.netdb.getInfo(Constants.NODE, idNode.intValue());
                if( node == null || node.get("id") == null ) {
                    label.setText("<html><body><font color=\"#ff0000\">Failed to load node. "+
                                  "It probably does not exist.</font></body></html>");
                    reloadB.setEnabled(false);
                    deleteB.setEnabled(false);
                    modifyB.setEnabled(false);
                    templateB.setEnabled(false);
                    return;
                }
                idNode = new Integer((String)node.get("id"));
            }

            label.setText("Verifying access...");
            // check that user has access to the domain
            if( MainWindow.netdb.checkAccess(Constants.NODE) && MainWindow.netdb.checkGroupAccess(node) )
                hasAccess = true;

            Hashtable types = Utils.nHash(node.get("types"));
            Enumeration enm = types.keys();
            while( enm.hasMoreElements() ) {
                String type = ((String)enm.nextElement()).trim();
                if( type.equals("Advanced") && !MainWindow.netdb.checkAccess(Constants.ADVANCED_NODE) )
                    hasAccess = false;
                else if( type.equals("Router") && !MainWindow.netdb.checkAccess(Constants.ROUTER) )
                    hasAccess = false;
                else if( type.equals("IPCprovider") && !MainWindow.netdb.checkAccess(Constants.IPC_PROVIDER) )
                    hasAccess = false;
            }

            // now display what we have got
            current_panel = mainPanel;
            current_panel.removeAll();
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            Hashtable names = (Hashtable)node.get("names");
            java.util.List list = Arrays.asList(names.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] nameArray = list.toArray();
            for( int i = 0; i < nameArray.length; i++ ) {
                // empty row at the top
                XLabel(" ");
                tr();

                Hashtable name = (Hashtable)names.get(nameArray[i]);
                HTMLPanel("Name");
                td();
                if( name.get("domain").toString().trim().toLowerCase().equals(MainWindow.userDefDomain) )
                    XLabel((String)name.get("name"), LEFT);
                else
                    XLabel(name.get("name")+"."+name.get("domain"), LEFT);
                tr();

                Hashtable aliases = (Hashtable)name.get("aliases");
                if( aliases != null ) {
                    java.util.List alist = Arrays.asList(aliases.keySet().toArray());
                    Collections.sort(alist, name_cmp);
                    Object[] aliasArray = alist.toArray();
                    if( aliasArray.length > 0 ) {
                        XLabel(" ");
                        tr();
                        current_gbc.anchor = GridBagConstraints.NORTHWEST;
                        HTMLPanel("Alias(es)");
                        td();
                        JPanel aPanel = new JPanel(new GridLayout(aliasArray.length, 1, 0, 2));
                        aPanel.setBackground(Color.white);
                        for( int j = 0; j < aliasArray.length; j++ ) {
                            Hashtable alias = (Hashtable)aliases.get(aliasArray[j]);
                            String domain = ((String)alias.get("domain")).trim();
                            JLabel label =
                                new JLabel("  "+alias.get("name")+
                                           (domain.toLowerCase().equals(MainWindow.userDefDomain)?"":("."+domain)));
                            label.setFont(label_font);
                            aPanel.add(label);
                        }
                        current_gbc.gridwidth = GridBagConstraints.REMAINDER;
                        current_gb_layout.setConstraints(aPanel, current_gbc);
                        current_panel.add(aPanel);
                        tr();
                    }
                }

                Hashtable mxs = (Hashtable)name.get("mxs");
                if( mxs != null ) {
                    java.util.List mlist = Arrays.asList(mxs.keySet().toArray());
                    Collections.sort(mlist, name_cmp);
                    Object[] mxArray = mlist.toArray();
                    if( mxArray.length > 0 ) {
                            XLabel(" ");
                            tr();
                            current_gbc.anchor = GridBagConstraints.NORTHWEST;
                            XLabel("Receive mail for");
                            td();
                            JPanel mxPanel = new JPanel(new GridLayout(mxArray.length, 1, 0, 2));
                            mxPanel.setBackground(Color.white);
                            for( int j = 0; j < mxArray.length; j++ ) {
                                JPanel labelPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
                                labelPanel.setBackground(Color.white);
                                Hashtable mx = (Hashtable)mxs.get(mxArray[j]);
                                String domain = ((String)mx.get("domain")).trim();
                                JLabel label =
                                    new JLabel("  "+format(mx.get("name")+
                                                           (domain.toLowerCase().equals(MainWindow.userDefDomain)?"":("."+domain)), 25)+
                                               " ("+mx.get("pref")+")");
                                label.setFont(label_font);
                                labelPanel.add(label);
                                //label.setCursor(new Cursor(Cursor.HAND_CURSOR));
                                /*
                                label.addMouseListener(new MouseAdapter() {
                                        public void mouseEntered(MouseEvent event) {
                                            JLabel label = (JLabel)event.getComponent();
                                            label.setForeground(Color.blue);
                                            label.repaint();
                                        }
                                        public void mouseExited(MouseEvent event) {
                                            JLabel label = (JLabel)event.getComponent();
                                            label.setForeground(Color.black);
                                            label.repaint();
                                        }
                                        public void mousePressed(MouseEvent event) {
                                            JLabel label = (JLabel)event.getComponent();
                                            String text = label.getText();
                                            String[] parts = text.split("\\s+");
                                            (new NodeInfoFrame(parts[1])).setVisible(true);
                                        }
                                    });
                                */
                                mxPanel.add(labelPanel);
                            }
                            current_gbc.gridwidth = GridBagConstraints.REMAINDER;
                            current_gb_layout.setConstraints(mxPanel, current_gbc);
                            current_panel.add(mxPanel);
                            tr();
                    }
                }
            }
            tr();
            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            // Node type(s)
            HTMLPanel("Node type(s)");
            td();
            java.util.List tlist = Arrays.asList(types.keySet().toArray());
            Collections.sort(tlist, name_cmp);
            Object[] typesObj = tlist.toArray();
            String typesStr = "";
            for( int j = 0; j < typesObj.length; j++ )
                typesStr += typesObj[j]+(j<typesObj.length-1?", ":"");
            XLabel(typesStr, LEFT);
            tr();

            // Dept.
            HTMLPanel("Department");
            td();
            XLabel(node.get("dept").toString(), LEFT);
            tr();

            // Location
            if( node.get("location") != null ) {
                HTMLPanel("Location");
                td();
                String location = node.get("location").toString();
                if( node.get("room") != null )
                    location += "  Room: "+node.get("room");
                XLabel(location, LEFT);
                tr();
            }

            // Make and Model
            if( node.get("make") != null || node.get("model") != null ) {
                HTMLPanel("Make and Model");
                td();
                String make_and_model = "";
                if( node.get("make") != null )
                    make_and_model += node.get("make") + " ";
                if( node.get("model") != null )
                    make_and_model += node.get("model");
                XLabel(make_and_model, LEFT);
                tr();
            }

            // OS
            HTMLPanel("OS");
            td();
            Hashtable oses = (Hashtable)node.get("oses");
            if( oses != null && oses.size() > 0 ) {
                list = Arrays.asList(oses.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] osArray = list.toArray();
                String oses_string = "";
                for( int i = 0; i < osArray.length; i++ )
                    oses_string += osArray[i]+(i<osArray.length-1?", ":"");
                XLabel(oses_string, LEFT);
            }
            tr();

            XLabel(" ");
            tr();
            // administrators
            HTMLPanel("Administrators");
            td();
            Hashtable admins = (Hashtable)node.get("admins");
            if( admins != null ) {
                Hashtable ateams = (Hashtable)admins.get("ateams");
                if( ateams != null ) {
                    list = Arrays.asList(ateams.keySet().toArray());
                    Collections.sort(list, name_cmp);
                    Object[] ateamsArray = list.toArray();
                    for( int i = 0; i < ateamsArray.length; i++ ) {
                        XLabel(ateamsArray[i].toString(), LEFT);
                        tr();
                        td();
                    }
                }
                admins = (Hashtable)admins.get("admins");
                if( admins != null ) {
                    list = Arrays.asList(admins.keySet().toArray());
                    Collections.sort(list, name_cmp);
                    Object[] keysArray = list.toArray();
                    for( int i = 0; i < keysArray.length; i++ ) {
                        Hashtable person = (Hashtable)admins.get(keysArray[i]);
                        String name = (String)person.get("name");
                        String netID = (String)person.get("NetID");
                        XLabel(name==null?netID:name, LEFT);
                        tr();
                        td();
                    }
                }
            }
            tr();

            XLabel(" ");
            tr();

            // Users
            Hashtable users = (Hashtable)node.get("users");
            if( users != null ) {
                HTMLPanel("Users");
                td();
                list = Arrays.asList(users.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] keysArray = list.toArray();
                for( int i = 0; i < keysArray.length; i++ ) {
                    Hashtable person = (Hashtable)users.get(keysArray[i]);
                    String name = (String)person.get("name");
                    String netID = (String)person.get("NetID");
                    XLabel(name==null?netID:name, LEFT);
                    tr();
                    td();
                }
            }
            tr();

            // custom fields
            Hashtable tags = (Hashtable)node.get("tags");
            if( tags != null ) {
                HTMLPanel("Custom Field Names");
                tr();
                Object[] keys = tags.keySet().toArray();
                for( int i = 0; i < keys.length; i++ ) {
                    HTMLPanel("&nbsp;&nbsp;"+Utils.htmlize((String)keys[i]));
                    td();
                    XLabel("<html><body>"+Utils.htmlize(tags.get(keys[i]).toString())+"</body></html>", LEFT);
                    tr();
                }
            }

            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            String addr_space = (String)node.get("addr_space");
            if( addr_space != null ) {
                HTMLPanel("Addr. Space(Template)");
                td();
                XLabel(addr_space, LEFT);
                tr();

                // empty row
                XLabel(" ");
                tr();

                line();

                XLabel(" ");
                tr();
            }

            // interfaces
            Hashtable intfs = (Hashtable)node.get("interfaces");
            if( intfs != null ) {
                list = Arrays.asList(intfs.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] keysArray = list.toArray();
                for( int i = 0; i < keysArray.length; i++ ) {
                    HTMLPanel("Interface");
                    tr();
                    Hashtable intfc = (Hashtable)intfs.get(keysArray[i]);
                    // interface name(s)
                    Hashtable intfcNames = (Hashtable)intfc.get("names");
                    if( intfcNames != null && intfcNames.size() > 0 ) {
                        // no sorting here as we do not allow
                        // so far multiple names for as interface
                        // to be entered in a FE. It may change in the future
                        Object[] keys = intfcNames.keySet().toArray();
                        for( int j = 0; j < keys.length; j++ ) {
                            Hashtable nameH =
                                (Hashtable)intfcNames.get((String)keys[j]);
                            String name = (String)nameH.get("name");
                            String domain = (String)nameH.get("domain");
                            if( !domain.trim().toLowerCase().equals(MainWindow.userDefDomain) )
                                name += "."+domain;
                            if( j == 0 )
                                HTMLPanel("Name");
                            td();
                            XLabel(name, LEFT);
                            tr();
                        }
                        // also no aliases or mxs are assumed here, just becouse our FE
                        // does not allows entering them for interace name
                    }

                    // HW Addr
                    String hw = (String)intfc.get("hw");
                    if( hw != null ) {
                        hw = hw.toLowerCase();
                        hw = hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+hw.substring(8);
                        HTMLPanel("Hardware Address");
                        td();
                        if( ((Boolean)intfc.get("use-dhcp?")).equals(new Boolean(true)) )
                            hw += " dhcp";
                        if( ((Boolean)intfc.get("roaming?")).equals(new Boolean(true)) )
                            hw += " roam";
                        XLabel(hw, LEFT);
                        tr();
                    }

                    // ips
                    Hashtable ips = (Hashtable)intfc.get("ips");
                    if( ips != null ) {
                        java.util.List list0 = Arrays.asList(ips.keySet().toArray());
                        Collections.sort(list0, ip_cmp);
                        Object[] keys = list0.toArray();
                        for( int j = 0; j < keys.length; j++ ) {
                            if( j == 0 )
                                HTMLPanel("IP Addresses");
                            td();
                            Hashtable ip = (Hashtable)ips.get(keys[j]);
                            // ip may have a name
                            Hashtable ip_names = (Hashtable) ip.get("names");
                            String namesStr = "";
                            if( ip_names != null ) {
                                java.util.List list1 = Arrays.asList(ip_names.keySet().toArray());
                                Collections.sort(list1, name_cmp);
                                Object[] ipNamesArray = list1.toArray();
                                for( int k = 0; k < ipNamesArray.length; k++ ) {
                                    Hashtable ipNameH = (Hashtable)ip_names.get(ipNamesArray[k]);
                                    String ipNameStr = (String)ipNameH.get("name");
                                    String ipDomainStr = (String)ipNameH.get("domain");
                                    namesStr += ipNameStr;
                                    if( !ipDomainStr.trim().toLowerCase().equals(MainWindow.userDefDomain) )
                                        namesStr += "."+ipDomainStr;
                                    if( k < ipNamesArray.length-1 )
                                        namesStr += ", ";
                                }
                                if( namesStr.length() > 0 )
                                    namesStr = " ("+namesStr+")";
                            }
                            boolean active = ((Boolean)ip.get("active?")).booleanValue();
                            XLabel(format(keys[j]+namesStr, 17)+" "+(active?"active":"inactive"), LEFT);
                            tr();
                        }
                    }

                    // DHCP Options
                    Hashtable dhcp_opts = (Hashtable)intfc.get("dhcp-options");
                    if( dhcp_opts != null ) {
                        HTMLPanel("DHCP Options");
                        td();
                        java.util.List list1 = Arrays.asList(dhcp_opts.keySet().toArray());
                        Collections.sort(list1, name_cmp);
                        Object[] optsArray = list1.toArray();
                        for( int j = 0; j < optsArray.length; j++ ) {
                            XLabel(optsArray[j].toString().trim(), LEFT);
                            tr();
                            td();
                        }
                    }

                    // empty row
                    XLabel(" ");
                    tr();

                    line();

                    XLabel(" ");
                    tr();
                }
            }

            // IPC Addresses
            Hashtable ipc_addresses = (Hashtable)node.get("ipc_addresses");
            if( ipc_addresses != null && ipc_addresses.size() > 0 ) {
                HTMLPanel("IPC Addresses");
                td();
                java.util.List list0 = Arrays.asList(ipc_addresses.keySet().toArray());
                Collections.sort(list0, ip_cmp);
                Object[] keys = list0.toArray();
                for( int j = 0; j < keys.length; j++ ) {
                    String ip_addr = keys[j].toString();
                    Hashtable ip_addr_h = (Hashtable)ipc_addresses.get(ip_addr);
                    boolean isActive = ((Boolean)ip_addr_h.get("active?")).booleanValue();
                    String status = isActive?"active":"inactive";
                    ip_addr = format(ip_addr,  15)+" ";
                    Hashtable namesH = (Hashtable)ip_addr_h.get("names");
                    if( namesH != null ) {
                        // peek up only one name, f.e. limitation
                        Object[] nameKeys = namesH.keySet().toArray();
                        if( nameKeys.length > 0 ) {
                            Hashtable name_h    = (Hashtable)namesH.get(nameKeys[0]);
                            String name         = (String)name_h.get("name");
                            String domain       = (String)name_h.get("domain");
                            if( name != null ) {
                                ip_addr += "("+name;
                                if( !domain.trim().toLowerCase().equals(MainWindow.userDefDomain) )
                                    ip_addr += "."+domain;
                                ip_addr += ") ";
                            }
                        }
                    }
                    ip_addr = format(ip_addr, 35)+" "+status;
                    XLabel(ip_addr, LEFT);
                    tr();
                    td();
                }

                // empty row
                XLabel(" ");
                tr();

                line();

                XLabel(" ");
                tr();
            }

            // Exp. date
            HTMLPanel("Expiration Date");
            td();
            // have to write this part
            Object exp_date = node.get("exp.date");
            if( exp_date != null )
                XLabel((new SimpleDateFormat("MM/dd/yyyy")).format(exp_date), LEFT);
            tr();

            // NetDB Groups
            HTMLPanel("NetDB Groups");
            td();
            Hashtable groups = (Hashtable)node.get("groups");
            if( groups != null ) {
                list = Arrays.asList(groups.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] groupsArray = list.toArray();
                for( int i = 0; i < groupsArray.length; i++ ) {
                    XLabel(groupsArray[i].toString(), LEFT);
                    tr();
                    td();
                }
            }
            tr();

            // Comment
            current_gbc.anchor = GridBagConstraints.NORTHWEST;
            HTMLPanel("Comment");
            td();
            Object comment = node.get("comment");
            if( comment != null )
                XLabel("<html><body>"+Utils.htmlize(comment.toString())+"</body></html>", LEFT);
            tr();
            current_gbc.anchor = GridBagConstraints.CENTER;

            // Created By
            HTMLPanel("Created By");
            td();
            Hashtable person = (Hashtable)node.get("created.by");
            String byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+node.get("created").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // Last Modified By
            HTMLPanel("Last Modified By");
            td();
            person = (Hashtable)node.get("mod.by");
            byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+node.get("modified").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            reloadB.setEnabled(true);
            if( hasAccess ) {
                modifyB.setEnabled(true);
                templateB.setEnabled(true);
                deleteB.setEnabled(true);
            } else {
                tool_bar.remove(modifyB);
                tool_bar.remove(templateB);
                tool_bar.remove(deleteB);
                tool_bar.validate();
                tool_bar.repaint();
            }

            central_panel.removeAll();
            central_panel.add(mainPanel, BorderLayout.CENTER);
            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();

            if( delete ) {
                if( hasAccess )
                    deleteB.doClick();
                else
                    JOptionPane.showMessageDialog(THIS,
                              "You do not have rights to delete this node",
                              "Error", JOptionPane.ERROR_MESSAGE);
            }


        } catch (java.rmi.ServerException ex) {
            if( ex.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            JOptionPane.showMessageDialog(THIS,
                    ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            Utils.debug(ex.getMessage());

        } catch (java.rmi.NoSuchObjectException ex) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException ex) {
            X.handleTimeout();

        } catch(Exception ex) {
            Utils.warn(ex.toString());
            ex.printStackTrace();
        }
    } // end of run()

}
