/************************************************************************
 *
 * NodeThread.java - all node related functions
 *
 * $Id: NodeThread.java,v 1.2 2006-09-26 17:08:31 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import stanford.netdb.utils.*;

public class NodeThread extends Thread {

    public static final int UNLOCK_NODE         = 1;
    public static final int MODIFY_NODE         = 2;
    public static final int LOOKUP_ADMINS       = 3;
    public static final int LOOKUP_USERS        = 4;
    public static final int EXCEPTION           = 5;
    public static final int RESERVE_NAME_AND_IP = 6;
    public static final int VERIFY              = 7;
    public static final int DELETE_NODE         = 8;
    public static final int CANCEL_RESERVES     = 9;
    public static final int CREATE_OS           = 10;

    private String msg = "<html><body>";
    private int action;
    private boolean addedWarning = false;
    private Hashtable box;
    private Exception external_error = null;
    private boolean close_final = false;

    public NodeModifyFrame parent;
    public NodeInfoFrame infoParent;

    public NodeThread(NodeInfoFrame infoParent, int action) {
        this.infoParent = infoParent;
        this.action = action;
    }

    public NodeThread(NodeModifyFrame parent, int action) {
        this(parent, action, (Hashtable)null);
    }

    public NodeThread(NodeModifyFrame parent, int action, Exception external_error) {
        this(parent, action, (Hashtable)null);
        this.external_error = external_error;
    }

    public NodeThread(NodeModifyFrame parent, int action, Hashtable box) {
        this.parent = parent;
        this.action = action;
        this.box = box;
    }

    public void run() {
        try {
            switch ( action ) {
            case CREATE_OS:
                MainWindow.netdb.insert(box);
                Hashtable oses = (Hashtable)parent.newNode.get("oses");
                if( oses == null )
                    parent.newNode.put("oses", oses = new Hashtable());
                oses.put(box.get("name"), "");

                // add a check box
                Object[] added_oses = parent.list_of_oses.getSelectedValues();
                Hashtable nodeOSesH = (Hashtable)parent.newNode.get("oses");
                for( int i = 0; i < added_oses.length; i++ )
                    nodeOSesH.put((String)added_oses[i], "");

                parent.oses_panel.removeAll();

                parent.oses_panel_layout.setRows(nodeOSesH.size());
                java.util.List list = Arrays.asList(nodeOSesH.keySet().toArray());
                Collections.sort(list, parent.name_cmp);
                Iterator it = list.iterator();
                while( it.hasNext() ) {
                    String os = (String)it.next();
                    AJCheckBox cb = new AJCheckBox(os, true);
                    cb.setFont(parent.label_font);
                    cb.setBackground(Color.white);
                    cb.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                if( !cb.isSelected() ) {
                                    // remove associated data
                                    ((Hashtable)parent.newNode.get("oses")).remove(cb.getText());
                                    parent.oses_panel.remove(cb);
                                    parent.oses_panel_layout.setRows(parent.oses_panel_layout.getRows() - 1);
                                    parent.oses_panel.doLayout();
                                    parent.current_panel.revalidate();
                                    if( ((Hashtable)parent.newNode.get("oses")).size() == 0 )
                                        parent.add_another_os.doClick();
                                }
                            }
                        });
                    parent.oses_panel.add(cb);
                }

                parent.msg.setText("Reloading list of oses...");
                MainWindow.loadOSes();
                parent.osesH = MainWindow.getOSes();

                reviveParent();
                break;

            case DELETE_NODE:
                Timestamp tm = MainWindow.netdb.lock(Constants.NODE,
                                    Integer.parseInt(infoParent.node.get("id").toString()));
                infoParent.node.put("locked", tm);
                MainWindow.netdb.delete(infoParent.node);

                msg += "<center>Node has been deleted</center><br><br>";
                // now go over mxSlaves
                Hashtable mxSlaves = Utils.nHash(infoParent.node.get("mxSlaves"));
                Enumeration enm = mxSlaves.keys();
                while( enm.hasMoreElements() ) {
                    String nodeID = (String)enm.nextElement();
                    Hashtable node = (Hashtable)mxSlaves.get(nodeID);
                    if( node.get(" remove") == null )
                        continue;
                    int idNode = (new Integer(nodeID)).intValue();
                    Hashtable oldMXingNode = MainWindow.netdb.getInfo(Constants.NODE, idNode);
                    String nodeName = (String)Utils.nHash(oldMXingNode.get("names")).keySet().toArray()[0];
                    if( !MainWindow.netdb.checkGroupAccess(oldMXingNode) ) {
                        msg += "<font color=\"#ff0000\">Warning:</font> Failed to remove mxes from node "+nodeName+". Group access failed.<br>";
                        continue;
                    }
                    Hashtable newMXingNode = Utils.copyHash(oldMXingNode);
                    removeMXes(newMXingNode, node);
                    Timestamp lockedTm = null;
                    try {
                        lockedTm = MainWindow.netdb.lock(Constants.NODE, idNode);
                        oldMXingNode.put("locked", lockedTm);
                        newMXingNode.put("locked", lockedTm);
                        MainWindow.netdb.update(oldMXingNode, newMXingNode);

                    } catch (java.rmi.ServerException ex) {
                        msg += "<font color=\"#ff0000\">Warning:</font> Failed to remove mxes from node "+nodeName+".";
                        if( ex.getCause() instanceof SessionTimeoutException )
                            X.handleTimeout();

                        else if( ex.getCause() instanceof RemoteNetDBException ) {
                            RemoteNetDBException ne = (RemoteNetDBException)ex.getCause();
                            Vector errors = ne.getMessages();
                            msg += (addedWarning?"<br>":"");
                            int esize = errors.size();
                            for( int i = 0; i < esize; i++ )
                                msg += (String)errors.elementAt(i)+" ";
                        } else if( ex.getCause() instanceof RemoteException ) {
                            RemoteException re = (RemoteException)ex.getCause();
                            msg += re.getMessage();
                        } else
                            msg += ex.getMessage();
                        msg += "<br>";

                    } catch (java.rmi.NoSuchObjectException ex) {
                        X.handleTimeout();

                    } catch (java.rmi.MarshalException exx) {
                        X.handleTimeout();

                    } catch (Exception ex) {
                        msg += "<font color=\"#ff0000\">Warning:</font> Failed to remove mxes from node "+nodeName+". "+ex.getMessage()+"<br>";

                    } finally {
                        try {
                            if( lockedTm != null )
                                MainWindow.netdb.unlock(Constants.NODE, idNode, lockedTm);
                        } catch (Exception ex) {}
                    }
                }

                JOptionPane.showMessageDialog(infoParent,
                    msg+"</body></html>", "Info", JOptionPane.INFORMATION_MESSAGE);
                infoParent.closeB.doClick();
                break;

            case LOOKUP_ADMINS:
            case LOOKUP_USERS:
                String[] lines = parent.lookup_ta.getText().split("\n");
                for( int i = 0; i < lines.length; i++ ) {
                    String sString = lines[i].trim();
                    if( sString.length() > 0 ) {
                        if( sString.endsWith(";") && parent.lookupATeams ) {
                            // admin team
                            Hashtable ateam = MainWindow.netdb.getInfo(Constants.ATEAM, sString.substring(0, sString.length()-1));
                            if( ateam != null && ateam.get("object") != null && ((String)ateam.get("object")).equals("ateam") )
                                box.put(ateam.get("name")+" ateam", ateam);
                        } else {
                            try {
                                Hashtable persons = MainWindow.netdb.dirSearch(Constants.LOOKUP, sString);
                                if( !box.equals(parent.looked_up_persons) )
                                    return;
                                String uncked = "";
                                if( persons.size() > 1 )
                                    uncked = " uncheck";
                                enm = persons.keys();
                                while( enm.hasMoreElements() ) {
                                    String key = (String)enm.nextElement();
                                    box.put(key+" person"+uncked, persons.get(key));
                                    //System.out.println("GOT: "+Utils.nHash(persons.get(key)).get("name")+
                                    //                   " RegID: "+Utils.nHash(persons.get(key)).get("RegID")+
                                    //                   " NetID: "+Utils.nHash(persons.get(key)).get("NetID"));
                                }
                            } catch (Exception ex) {
                                // ignore
                            }
                        }
                    }
                }
                if( !box.equals(parent.looked_up_persons) )
                    return;
                parent.personsLookupCenterPanel.removeAll();
                parent.personsLookupGridLayout.setRows(box.size());
                list = Arrays.asList(box.keySet().toArray());
                Collections.sort(list, parent.name_cmp);
                it = list.iterator();
                while( it.hasNext() ) {
                    String key = (String)it.next();
                    Hashtable obj = (Hashtable)box.get(key);
                    obj.put("key", key);
                    String displayName = "";
                    if( obj.get("object") != null )
                        displayName = obj.get("name")+";";
                    else
                        displayName = obj.get("name")+(obj.get("department")!=null?(" ("+obj.get("department")+")"):"");
                    AJCheckBox cb = new AJCheckBox(displayName, key.endsWith("uncheck")?false:true, obj);
                    cb.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                AJCheckBox cb = (AJCheckBox)((JCheckBox)e.getSource()).getParent();
                                if( cb.isSelected() ) {

                                    String key = (String)cb.box.get("key");
                                    String new_key = key.substring(0, key.length() - 8);
                                    box.put(new_key, box.remove(key));
                                    cb.box.put("key", new_key);
                                } else {
                                    // remove associated data
                                    box.remove(cb.box.get("key"));
                                    parent.personsLookupCenterPanel.remove(cb);
                                    parent.personsLookupGridLayout.setRows(parent.personsLookupGridLayout.getRows() - 1);
                                    parent.personsLookupCenterPanel.doLayout();
                                    parent.personsLookupCenterPanel.revalidate();
                                    parent.personsLookupCenterPanel.repaint();
                                }
                            }
                        });
                    cb.setFont(parent.label_font);
                    parent.personsLookupCenterPanel.add(cb);
                }
                parent.personsLookupPanel.revalidate();
                parent.personsLookupCenterPanel.repaint();
                break;

            case CANCEL_RESERVES:
                break;

            case UNLOCK_NODE:
                MainWindow.netdb.unlock(Constants.NODE, parent.handle, parent.lockedTm);
                close_final = true;
                break;

            case MODIFY_NODE:
                // update person table
                // first from users
                Hashtable persons = Utils.nHash(parent.newNode.get("users"));
                updatePersons(persons);

                // and second from admins
                persons = Utils.nHash(Utils.nHash(parent.newNode.get("admins")).get("admins"));
                updatePersons(persons);

                if( parent.template || parent.handle == null )
                    MainWindow.netdb.insert(parent.newNode);
                else
                    MainWindow.netdb.update(parent.oldNode, parent.newNode);

                if( parent.handle != null )
                    NodeModifyFrame.nodeModFrames.remove(parent.handle.toLowerCase());

                NodeInfoFrame nif = null;
                if( parent.handle != null && parent.handle.trim().length() > 0 )
                    nif = (NodeInfoFrame)NodeInfoFrame.
                        nodeInfoFrames.get(parent.handle.toLowerCase());
                if( nif == null ) {
                    // open new info frame, but
                    // first we need to find new handle
                    Hashtable nodeNames = (Hashtable)parent.newNode.get("names");
                    Hashtable nodeName = (Hashtable)nodeNames.get(nodeNames.keys().nextElement());
                    parent.handle = nodeName.get("name")+"."+nodeName.get("domain");

                    // now open new info frame
                    nif = NodeInfoFrame.MakeNodeInfoFrame(parent.handle);
                    nif.reloadB.doClick();
                } else // reload existing info frame
                    nif.reloadB.doClick();

                close_final = true;
                parent.dispose();
                break;

            case EXCEPTION:
                throw external_error;

            case RESERVE_NAME_AND_IP:
                String ipStr = parent.ipAddrReserveIP;
                String ipCountStr = parent.ipAddrCountTf.getText().trim();
                int ipCount = 1;
                if( parent.reserving_ipc ) {
                    if( ipCountStr != null && ipCountStr.length() > 0 )
                        ipCount = Integer.parseInt(ipCountStr);
                    if( ipCount > 1 )
                        parent.msg.setText("Reserving ip addresses...");
                }

                Hashtable result;
                if( ipStr.indexOf("/") > -1 )
                    result = MainWindow.netdb.reserveIP(ipStr, ipStr.substring(0, ipStr.indexOf("/")), ipCount);
                else
                    result = MainWindow.netdb.reserveIP(null, ipStr, ipCount);

                String requestedIP = (String)result.keys().nextElement();
                Hashtable reservedTable = (Hashtable)result.get(requestedIP);
                Hashtable reservedIPs = Utils.nHash(reservedTable.get("ip_addresses"));

                if( reservedTable.get("warnings") != null ) {
                    addedWarning = true;
                    Hashtable warningsH = (Hashtable)reservedTable.get("warnings");
                    enm = warningsH.keys();
                    int index = 0;
                    while( enm.hasMoreElements() ) {
                        index++;
                        msg += "<font color=\"#ff0000\">&nbsp;"+enm.nextElement()+
                            "</font>"+(index<warningsH.size()?"<br>":"");
                    }
                }

                Hashtable ips;
                if( parent.reserving_ipc ) {
                    ips = (Hashtable)parent.newNode.get("ipc_addresses");
                    if( ips == null ) {
                        ips = new Hashtable();
                        parent.newNode.put("ipc_addresses", ips);
                    }
                } else {
                    Hashtable intfc = (Hashtable)((Hashtable)parent.newNode.get("interfaces")).get(parent.currentIntfcStore.get("id"));
                    ips = (Hashtable)intfc.get("ips");
                    if( ips == null ) {
                        ips = new Hashtable();
                        intfc.put("ips", ips);
                    }
                }

                list = Arrays.asList(reservedIPs.keySet().toArray());
                Collections.sort(list, parent.ip_cmp);
                it = list.iterator();
                int counter = 0;
                int total = reservedIPs.size();
                while( it.hasNext() ) {
                    String reservedIP = ((String)it.next());
                    parent.reservedIPs.put(reservedIP.trim(), reservedIP);
                    Hashtable ip = new Hashtable();
                    ip.put("active?", new Boolean(parent.ipAddrActiveRes));
                    ip.put("verified", "");
                    ips.put(reservedIP, ip);

                    String nameToReserve = parent.ipAddrNameReserve;
                    if( counter == 0 ) {
                        if( parent.reserving_ipc &&
                            ( nameToReserve == null || nameToReserve.trim().length() == 0 ) ) {
                            Hashtable genName = MainWindow.netdb.generateHexName(reservedIP, 0);
                            nameToReserve = genName.get("name")+"."+genName.get("domain");
                        }
                    } else {
                        Hashtable genName = MainWindow.netdb.generateHexName(reservedIP, 0);
                        nameToReserve = genName.get("name")+"."+genName.get("domain");
                    }

                    // if advanced or ipc then try to reserve name if given
                    if( nameToReserve.trim().length() > 0 &&
                        ( Utils.nHash(parent.newNode.get("types")).get("Advanced") != null ||
                          parent.reserving_ipc ) )
                        {
                            parent.msg.setText("Reserving name["+counter+"/"+total+"]...");
                            Hashtable names = new Hashtable();
                            ip.put("names", names);
                            try {
                                String reservedName =
                                    MainWindow.netdb.reserveName(nameToReserve,
                                                                 Constants.ADDRESS_NAME_TYPE);
                                Hashtable name = new Hashtable();
                                names.put(reservedName, name);
                                name.put("name", Utils.name(reservedName));
                                name.put("domain", Utils.domain(reservedName, MainWindow.userDefDomain));
                                name.put("verified", "");
                                String nameStr = ((String)name.get("name")).trim()+"."+((String)name.get("domain")).trim();
                                parent.reservedNames.put("IP NAME "+nameStr, nameStr);

                            } catch (java.rmi.ServerException ex) {
                                if( ex.getCause() instanceof SessionTimeoutException )
                                    X.handleTimeout();

                                else if( ex.getCause() instanceof RemoteNetDBException ) {
                                    RemoteNetDBException ne = (RemoteNetDBException)ex.getCause();
                                    Vector errors = ne.getMessages();
                                    msg += (addedWarning?"<br>":"");
                                    int esize = errors.size();
                                    for( int i = 0; i < esize; i++ )
                                        msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                                            "</font>"+(i<esize-1?"<br>":"");

                                } else if( ex.getCause() instanceof RemoteException ) {
                                    RemoteException re = (RemoteException)ex.getCause();
                                    msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                                        re.getMessage()+"</font>";

                                } else
                                    msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                                        ex.getMessage()+"</font>";

                                addedWarning = true;

                            } catch (java.rmi.NoSuchObjectException exx) {
                                X.handleTimeout();

                            } catch (java.rmi.MarshalException exx) {
                                X.handleTimeout();

                            } catch(Exception exx) {
                                msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                                    exx.getMessage()+"</font>";

                                addedWarning = true;
                            }
                        }
                    counter++;
                }

                //((GridLayout)parent.currentIntfcStore.get("ip_layout")).setRows(???);

                //parent.makeIPs(ips, intfcStore, true);
                parent.ipAddrReserveWin.dispose();

                reviveParent();

                parent.header_panel.remove(parent.ePanel);
                if( addedWarning ) {
                    parent.ePanelLabel.setText(msg+"</body></html>");
                    parent.header_panel.add(parent.ePanel);
                } else
                    parent.ePanelLabel.setText("");
                parent.header_panel.revalidate();

                if( parent.reserving_ipc )
                    parent.makeIPCPanel(true);
                else
                    parent.makeIPs(ips, parent.currentIntfcStore, true);

                break;

            case VERIFY:
                //System.out.println("=====================");
                //Utils.printHash(parent.newNode);
                //System.out.println("<<<>>>");
                Vector names_to_reserve = new Vector();
                Vector names_to_cancel = new Vector();
                Vector names_no_change = new Vector();

                Utils.keysIntersect(parent.allOldNames, parent.allNewNames,
                                    names_to_reserve, names_to_cancel, names_no_change);
                // go over names_no_change and pick up dirty names
                /*
                for( int i = 0; i < names_no_change.size(); i++ ) {
                    String key = (String)names_no_change.elementAt(i);
                    Hashtable name = (Hashtable)parent.allNewNames.get(key);
                    if( name.get("dirty") != null )
                        names_to_reserve.addElement(key);
                }
                */
                Vector ips_to_reserve = new Vector();
                Vector ips_to_cancel = new Vector();
                Vector ips_no_change = new Vector();
                Utils.keysIntersect(parent.allOldIPs, parent.allNewIPs,
                                    ips_to_reserve, ips_to_cancel, ips_no_change);

                total = names_to_reserve.size() + names_to_cancel.size() +
                    ips_to_reserve.size() + ips_to_cancel.size();

                parent.msg.setText("Verifying node... [0/"+total+"]");
                int index = 0; // it is used to update msg. label

                for( int i = 0; i < names_to_cancel.size(); i++ ) {
                    String key = (String)names_to_cancel.elementAt(i);

                    // do cancel and remove from list of reserved
                    Hashtable nameH = (Hashtable)parent.allOldNames.get(key);
                    MainWindow.netdb.cancelName(((String)nameH.get("name")).trim()+"."+
                                           ((String)nameH.get("domain")).trim(), nameType(key));
                    parent.reservedNames.remove(key);

                    // update label
                    index++;
                    parent.msg.setText("Verifying node... ["+index+"/"+total+"]");
                }

                for( int i = 0; i < names_to_reserve.size(); i++ ) {
                    String key = (String)names_to_reserve.elementAt(i);
                    Hashtable nameH = (Hashtable)parent.allNewNames.get(key);
                    String name = ((String)nameH.get("name")).trim()+"."+((String)nameH.get("domain")).trim();

                    // do reserve and add to list of reserved
                    MainWindow.netdb.reserveName(name, nameType(key));
                    parent.reservedNames.put(key, name);

                    // update label
                    index++;
                    parent.msg.setText("Verifying node... ["+index+"/"+total+"]");
                }

                for( int i = 0; i < ips_to_cancel.size(); i++ ) {
                    ipStr = (String)ips_to_cancel.elementAt(i);
                    MainWindow.netdb.cancelIP(ipStr);
                    parent.reservedIPs.remove(ipStr.trim());
                    index++;
                    parent.msg.setText("Verifying node... ["+index+"/"+total+"]");
                }

                for( int i = 0; i < ips_to_reserve.size(); i++ ) {
                    ipStr = (String)ips_to_reserve.elementAt(i);
                    result = MainWindow.netdb.reserveIP(null, ipStr, 1);

                    // read warnings if any
                    result = (Hashtable)result.get(ipStr);

                    Hashtable warnings = (Hashtable)result.get("warnings");
                    if( warnings != null ) {
                        enm = warnings.keys();
                        boolean first = true;
                        while( enm.hasMoreElements() ) {
                            if( first && addedWarning ) {
                                first = false;
                                msg += "<br>";
                            }
                            msg += "<font color=\"#ff0000\">&nbsp;"+enm.nextElement()+
                                "</font>"+(index<warnings.size()?"<br>":"");
                            addedWarning = true;
                        }
                    }

                    Hashtable ip_addresses = (Hashtable)result.get("ip_addresses");
                    String reservedIP = (String)ip_addresses.keys().nextElement();
                    parent.reservedIPs.put(reservedIP.trim(), reservedIP);
                    AJFormattedTextField ip_tf = (AJFormattedTextField)parent.ipTextFields.get(ipStr);
                    ip_tf.setText(reservedIP);
                    ip_tf.setFont(parent.bold_em_font);
                    if( ip_tf.key != null ) {
                        ips = Utils.nHash(ip_tf.box.get("ips"));
                        Hashtable ipH = Utils.nHash(ips.remove(ipStr));
                        ipH.put("verified", "");
                        ips.put(reservedIP, ipH);
                        parent.ipTextFields.put(reservedIP, parent.ipTextFields.remove(ipStr));
                    }
                    //System.out.println("------------");
                    //Utils.printHash(ip_tf.box);
                    //System.out.println("------------");

                    index++;
                    parent.msg.setText("Verifying node... ["+index+"/"+total+"]");
                }

                //Utils.printHash(parent.newNode);
                Utils.validate(Constants.NODE, parent.newNode, Constants.NARROW);

                reviveParent();
                break;

            default:
            }

        } catch (java.rmi.ServerException ex) {
            if( ex.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( ex.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)ex.getCause();
                Vector errors = ne.getMessages();
                msg += (addedWarning?"<br>":"");
                int esize = errors.size();
                for( int i = 0; i < esize; i++ )
                    msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                        "</font>"+(i<esize-1?"<br>":"");
            } else if( ex.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)ex.getCause();
                msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                    re.getMessage()+"</font>";
            } else
                msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                    ex.getMessage()+"</font>";

            if( parent != null ) {
                parent.ePanelLabel.setText(msg+"</body></html>");
                parent.header_panel.add(parent.ePanel);
                parent.header_panel.revalidate();
                parent.central_panel.removeAll();
                parent.central_panel.add(parent.mainPanel, BorderLayout.NORTH);
                parent.central_panel.revalidate();
                parent.central_panel.repaint();
                parent.reloadB.setEnabled(true);
                parent.modifyB.setEnabled(true);
                parent.deleteB.setEnabled(true);
            } else {
                JOptionPane.showMessageDialog(MainWindow.win,
                                              msg+"</body></html>",
                                              "Error", JOptionPane.ERROR_MESSAGE);
            }

            if( infoParent != null )
                reviveInfoParent();

        } catch (java.rmi.NoSuchObjectException exx) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException exx) {
            X.handleTimeout();

        } catch (NetDBException exx) {
            Vector errors = exx.getMessages();
            String msg = "<html><body>";
            int esize = errors.size();
            for( int i = 0; i < esize; i++ )
                msg += "<font color=\"#ff0000\">&nbsp;Error: "+(String)errors.elementAt(i)+
                    "</font>"+(i<esize-1?"<br>":"");

            if( action == LOOKUP_ADMINS ) {
                JOptionPane.showMessageDialog(parent, msg, "Error", JOptionPane.ERROR_MESSAGE);

            } else if( parent != null ) {
                parent.ePanelLabel.setText(msg+"</body></html>");
                parent.header_panel.add(parent.ePanel);
                parent.header_panel.revalidate();
                parent.central_panel.removeAll();
                parent.central_panel.add(parent.mainPanel, BorderLayout.NORTH);
                parent.central_panel.revalidate();
                parent.central_panel.repaint();
                parent.reloadB.setEnabled(true);
                parent.modifyB.setEnabled(true);
                parent.deleteB.setEnabled(true);
            }
            if( infoParent != null )
                reviveInfoParent();
            return;

        } catch(Exception exx) {
            msg += (addedWarning?"<br>":"")+"&nbsp;<font color=\"#ff0000\">Error: "+
                exx.getMessage()+"</font>";

            if( action == LOOKUP_ADMINS ) {
                JOptionPane.showMessageDialog(parent, msg, "Error", JOptionPane.ERROR_MESSAGE);

            } else if( parent != null ) {
                parent.ePanelLabel.setText(msg+"</body></html>");
                parent.header_panel.add(parent.ePanel);
                parent.header_panel.revalidate();
                parent.central_panel.removeAll();
                parent.central_panel.add(parent.mainPanel, BorderLayout.NORTH);
                parent.central_panel.revalidate();
                parent.central_panel.repaint();
                parent.reloadB.setEnabled(true);
                parent.modifyB.setEnabled(true);
                parent.deleteB.setEnabled(true);
            }
            if( infoParent != null )
                reviveInfoParent();
            return;

        } finally {
            if( action == LOOKUP_ADMINS || action == LOOKUP_USERS ) {
                parent.lookupButtons[0].setEnabled(true);
                parent.lookupButtons[1].setEnabled(true);

            } else if( parent != null ) {
                if( (action == UNLOCK_NODE || action == MODIFY_NODE) && close_final ) {
                    // now close parent window
                    NodeModifyFrame.nodeModFrames.remove(parent.handle.toLowerCase());
                    parent.dispose();

                    // cancel reserved but not committed names
                    Enumeration enm = parent.reservedNames.keys();
                    while( enm.hasMoreElements() ) {
                        String key = (String)enm.nextElement();
                        try {
                            MainWindow.netdb.cancelName((String)parent.reservedNames.get(key), nameType(key));
                        } catch(Exception ex) {
                            // ignore
                        }
                    }

                    // cancel reserved but not committed ips
                    enm = parent.reservedIPs.keys();
                    while( enm.hasMoreElements() ) {
                        try {
                            MainWindow.netdb.cancelIP((String)enm.nextElement());
                        } catch(Exception ex) {
                            // ignore
                        }
                    }

                    return;
                } else if( action == CANCEL_RESERVES ) {
                    // cancel reserved but not committed names
                    Enumeration enm = parent.reservedNames.keys();
                    while( enm.hasMoreElements() ) {
                        String key = (String)enm.nextElement();
                        try {
                            MainWindow.netdb.cancelName((String)parent.reservedNames.get(key), nameType(key));
                        } catch(Exception ex) {
                            // ignore
                        }
                    }

                    // cancel reserved but not committed ips
                    enm = parent.reservedIPs.keys();
                    while( enm.hasMoreElements() ) {
                        try {
                            MainWindow.netdb.cancelIP((String)enm.nextElement());
                        } catch(Exception ex) {
                            // ignore
                        }
                    }

                    return;
                }
            }
            if( addedWarning && parent != null ) {
                parent.ePanelLabel.setText(msg+"</body></html>");
                parent.header_panel.add(parent.ePanel);
                parent.header_panel.revalidate();
            }
        }
    } // end of run()

    private void reviveParent() {
        if( parent != null ) {
            parent.central_panel.removeAll();
            parent.central_panel.add(parent.mainPanel, BorderLayout.NORTH);
            parent.central_panel.revalidate();
            parent.central_panel.repaint();
            parent.reloadB.setEnabled(true);
            parent.modifyB.setEnabled(true);
            parent.deleteB.setEnabled(true);
        }
        if( infoParent != null ) {
            infoParent.central_panel.removeAll();
            infoParent.add(infoParent.mainPanel, BorderLayout.CENTER);
            infoParent.central_panel.repaint();
            infoParent.scrollpane.validate();
        }
    } // end of reviveParent()

    private void reviveInfoParent() {
        infoParent.central_panel.removeAll();
        infoParent.central_panel.add(infoParent.mainPanel, BorderLayout.CENTER);
        infoParent.central_panel.revalidate();
        infoParent.central_panel.repaint();
        infoParent.scrollpane.validate();
        infoParent.reloadB.setEnabled(false);
        if( infoParent.hasAccess ) {
            infoParent.modifyB.setEnabled(true);
            infoParent.templateB.setEnabled(true);
            infoParent.deleteB.setEnabled(true);
        }
    } // end of reviveInfoParent()

    private String nameType(String key) {
        if (     key.startsWith("NODE NAME ALIAS ") )
            return Constants.NAME_ALIAS_TYPE;
        else if( key.startsWith("NODE NAME MX ") )
            return Constants.MX_TYPE;
        else if( key.startsWith("NODE NAME ") )
            return Constants.NODE_NAME_TYPE;
        else if( key.startsWith("IP NAME ") )
            return Constants.ADDRESS_NAME_TYPE;
        else if( key.startsWith("INTFC NAME ") )
            return Constants.INTERFACE_NAME_TYPE;
        return "";
    } // end of nameType(...)

    private void removeMXes(Hashtable mxingNode, Hashtable mxes) {
        Hashtable names = Utils.nHash(mxingNode.get("names"));
        Enumeration enm = names.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            String nameFQDN = key.toLowerCase().trim();

            Hashtable mxs = null;
            Enumeration enm2 = mxes.keys();
            while( enm2.hasMoreElements() ) {
                String name = (String)enm2.nextElement();
                if( name.equals(nameFQDN) ) {
                    mxs = (Hashtable)mxes.get(name);
                    break;
                }
            }
            if( mxs == null )
                continue;

            Vector keysToRemove = new Vector();
            Hashtable name = (Hashtable)names.get(key);
            Hashtable mxNames = Utils.nHash(name.get("mxs"));
            enm2 = mxNames.keys();
            while( enm2.hasMoreElements() ) {
                String mxKey = (String)enm2.nextElement();

                Enumeration enm3 = mxs.keys();
                while( enm3.hasMoreElements() ) {
                    String mxName = (String)enm3.nextElement();
                    if( mxKey.toLowerCase().trim().equals(mxName.toLowerCase().trim()) ) {
                        keysToRemove.addElement(mxKey);
                        break;
                    }
                }
            }

            // now remove keysToRemove
            for( int i = 0; i < keysToRemove.size(); i++ )
                mxNames.remove(keysToRemove.elementAt(i));
        }
    } // end of removeMXes(...)

    private void updatePersons(Hashtable persons)
        throws Exception, RemoteNetDBException, SessionTimeoutException, java.rmi.NoSuchObjectException, java.rmi.MarshalException
    {
        Enumeration enm = persons.keys();
        while( enm.hasMoreElements() ) {
            Hashtable person = Utils.nHash(persons.get(enm.nextElement()));
            Object oType = person.get("object");
            if( oType != null && (oType instanceof String) &&
                ((String)oType).equals("person") )
                MainWindow.netdb.update(person, person);
        }
    }
}
