/************************************************************************
 *
 * OSReplaceThread.java -
 *
 * $Id: OSThread.java,v 1.2 2006-09-26 17:08:47 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.rmi.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import stanford.netdb.utils.*;

public class OSThread extends Thread {

    private Object[] list1 = null;
    private Hashtable newOS;
    private OSFrame parent;
    private String successMessage;
    private boolean delete = false;

    public OSThread(OSFrame parent, Object[] list1, Hashtable newOS) {
        super();
        this.parent = parent;
        this.list1 = list1;
        this.newOS = newOS;
        successMessage = "OS(es) have been replaced.";
    }

    public OSThread(OSFrame parent, Object[] list1, Hashtable newOS, String successMessage) {
        super();
        this.parent = parent;
        this.list1 = list1;
        this.newOS = newOS;
        this.successMessage = successMessage;
    }

    public OSThread(OSFrame parent, Hashtable newOS, String successMessage) {
        super();
        this.parent = parent;
        this.newOS = newOS;
        this.successMessage = successMessage;
    }

    public OSThread(OSFrame parent, Object[] oses_to_delete) {
        super();
        this.parent = parent;
        this.list1 = oses_to_delete;
        this.successMessage = "OS has been deleted";
        delete = true;
    }

    public void run() {
        boolean gotException = false;
        try {
            if( delete ) {
                successMessage = "<html><body>";
                for( int i = 0; i < list1.length; i++ ) {
                    parent.msgLabel.setText("Deleting OS '"+list1[i]+"' ...");
                    Hashtable os_to_delete = new Hashtable();
                    os_to_delete.put("object", "os");
                    os_to_delete.put("name", list1[i].toString().trim());
                    try {
                        MainWindow.netdb.delete(os_to_delete);
                        successMessage += (i>0?"<br>":"")+"OS <i>"+list1[i].toString().trim()+"</i> has been deleted";

                    } catch(java.rmi.ServerException exx) {
                        if( exx.getCause() instanceof SessionTimeoutException )
                            throw exx;

                    } catch (java.rmi.NoSuchObjectException ex) {
                        throw ex;

                    } catch (java.rmi.MarshalException ex) {
                        throw ex;

                    } catch (Exception ex) {
                        successMessage += (i>0?"<br>":"")+"Failed to delete os <i>"+list1[i].toString().trim()+"</i>";
                    }
                }
                successMessage += "</body></html>";
            } else if( list1 == null )
                MainWindow.netdb.insert(newOS);
            else {
                if( successMessage.equals("OS has been modified") ) {

                    String newOSName = newOS.get("name").toString().trim();
                    String newOSNameLower = newOSName.toLowerCase();
                    if( list1.length > 1 || list1.length < 1 ||
                        !list1[0].toString().trim().toLowerCase().equals(newOSNameLower) )
                        {
                            Enumeration enm = parent.osList.keys();
                            while( enm.hasMoreElements() ) {
                                if( ((String)enm.nextElement()).trim().toLowerCase().equals(newOSNameLower) )
                                    throw new Exception("OS name ["+newOSName+"] already exists");
                            }
                        }
                }
                for( int i = 0; i < list1.length; i++ ) {
                    Utils.debug("Replacing OS ["+list1[i]+"] -> ["+newOS.get("name")+"]");
                    Hashtable oldOS = new Hashtable();
                    oldOS.put("object", "os");
                    oldOS.put("name", list1[i].toString().trim());
                    oldOS.put("comment", ((String)parent.osList.get((String)list1[i])).trim() );
                    MainWindow.netdb.update(oldOS, newOS);
                }
            }

            JOptionPane.showMessageDialog(parent,
                  successMessage, "Info", JOptionPane.INFORMATION_MESSAGE);

            try {
                parent.modifyDialogWin.dispose();
                parent.modifyDialogWin = null;
            } catch (Exception ex) { }

        } catch(java.rmi.ServerException exx) {
            gotException = true;
            if( exx.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            else if( exx.getCause() instanceof RemoteNetDBException ) {
                RemoteNetDBException ne = (RemoteNetDBException)exx.getCause();
                Vector errors = ne.getMessages();
                String[] msg = new String[errors.size()];
                for( int i = 0; i < errors.size(); i++ )
                    msg[i] = (String)errors.elementAt(i);
                JOptionPane.showMessageDialog(parent, msg, "Error", JOptionPane.ERROR_MESSAGE);
            } else if( exx.getCause() instanceof RemoteException ) {
                RemoteException re = (RemoteException)exx.getCause();
                JOptionPane.showMessageDialog(parent, re.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            } else
                JOptionPane.showMessageDialog(parent, exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            return;

        } catch (java.rmi.NoSuchObjectException exx) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException exx) {
            X.handleTimeout();

        } catch(Exception exx) {
            gotException = true;
            JOptionPane.showMessageDialog(parent,
                    exx.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
            return;

        } finally {
            try {
                parent.msgLabel.setText("Loading list of OSes...");
                // reload list
                MainWindow.loadOSes();
                parent.osList = MainWindow.getOSes();
                java.util.List list = Arrays.asList(parent.osList.keySet().toArray());
                Collections.sort(list, parent.name_cmp);

                String[] oses = new String[parent.osList.size()];
                Iterator it = list.iterator();
                int i = 0;
                while( it.hasNext() ) {
                    oses[i++] = (String)it.next();
                }
                parent.osList1.setListData(oses);
                parent.osList2.setListData(oses);

            } catch (Exception ex) {
                Utils.warn(ex.toString());
            }
            parent.central_panel.removeAll();
            parent.central_panel.add(parent.osPanel);
            parent.central_panel.revalidate();
            parent.central_panel.repaint();
            parent.osList1.requestFocus();

            if( gotException && parent.modifyDialogWin != null )
                parent.modifyDialogWin.show();
        }
    }
}
