/************************************************************************
 *
 * QSPanel.java - quick search panel
 *
 * $Id: QSPanel.java,v 1.1.1.1 2006-06-30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class QSPanel extends QSPanelI {

    public JButton helpB                        = new JButton("Help");
    public JPopupMenu popupMenu                 = new JPopupMenu("actions");
    public KeyListener tableKeyListener;

    public JTableHeader tbh                     = null;

    // used for sorting
    public Comparator cmp1                      = null;
    public Comparator cmp2                      = null;
    public Comparator cmp3                      = null;
    public Comparator cmp4                      = null;
    public Comparator cmp5                      = null;
    public Comparator cmp6                      = null;

    // if cmp_key is null then sorting will happen on a keys themself
    // otherwise on a value associated with the key
    public String cmp_key                       = null;
    public boolean sorted_down                  = true;

    public String[] colSuffix                   = {"", "", "", "", "", "", "", "", "", ""};

    public Set keys                             = null;

    public QSPanel THIS;

    public QSPanel() {
        THIS = this;

        progressBar.setIndeterminate(true);

        cmp1 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    if( cmp_key == null ) {
                        if( sorted_down )
                            return ((String)o1).compareToIgnoreCase((String)o2);
                        else
                            return ((String)o2).compareToIgnoreCase((String)o1);
                    } else {
                        String s1 = (String)((Hashtable)records.get((String)o1)).get(cmp_key);
                        String s2 = (String)((Hashtable)records.get((String)o2)).get(cmp_key);
                        if( s1 == null ) s1 = "";
                        if( s2 == null ) s2 = "";
                        if( sorted_down )
                            return s1.compareToIgnoreCase(s2);
                        else
                            return s2.compareToIgnoreCase(s1);
                    }
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        cmp2 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable h1 = (Hashtable)((Hashtable)records.get((String)o1)).get("node_type");
                    Hashtable h2 = (Hashtable)((Hashtable)records.get((String)o2)).get("node_type");
                    String s1 = h1.get(cmp_key)!=null?"a":"b";
                    String s2 = h2.get(cmp_key)!=null?"a":"b";
                    if( sorted_down)
                        return s1.compareToIgnoreCase(s2);
                    else
                        return s2.compareToIgnoreCase(s1);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        cmp3 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable h1 = (Hashtable)records.get((String)o1);
                    Hashtable h2 = (Hashtable)records.get((String)o2);
                    long v1 = ((Long)h1.get(cmp_key)).longValue();
                    long v2 = ((Long)h2.get(cmp_key)).longValue();
                    if( sorted_down )
                        return (int)(v1 - v2);
                    else
                        return (int)(v2 - v1);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        cmp4 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable h1 = (Hashtable)records.get((String)o1);
                    Hashtable h2 = (Hashtable)records.get((String)o2);
                    String ip1 = (String)h1.get(cmp_key);
                    String ip2 = (String)h2.get(cmp_key);
                    if( ip1 == null || ip1.trim().length() == 0 ) ip1 = "0.0.0.0";
                    if( ip2 == null || ip2.trim().length() == 0 ) ip2 = "0.0.0.0";
                    long cmp_val = Utils.ipstr2long(ip1) - Utils.ipstr2long(ip2);

                    if( sorted_down )
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    else
                        return cmp_val<0?1:(cmp_val>0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        cmp5 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable h1 = (Hashtable)records.get((String)o1);
                    Hashtable h2 = (Hashtable)records.get((String)o2);
                    int v1 = ((Integer)h1.get(cmp_key)).intValue();
                    int v2 = ((Integer)h2.get(cmp_key)).intValue();
                    if( sorted_down )
                        return v1 - v2;
                    else
                        return v2 - v1;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        cmp6 = new Comparator() {
                public int compare(Object o1, Object o2) {
                    String s1 = (String)((Hashtable)records.get((String)o1)).get("net_name");
                    String s2 = (String)((Hashtable)records.get((String)o2)).get("net_name");
                    if( s1 == null ) s1 = (String)((Hashtable)records.get((String)o1)).get("handle");
                    if( s2 == null ) s2 = (String)((Hashtable)records.get((String)o2)).get("handle");
                    if( s1 == null ) s1 = "";
                    if( s2 == null ) s2 = "";
                    if( sorted_down )
                        return s1.compareToIgnoreCase(s2);
                    else
                        return s2.compareToIgnoreCase(s1);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        name_cmp = cmp1;

        tableModel = new AbstractTableModel() {
                public int getColumnCount() {
                    switch( lastSearchIndex ) {
                    case 1 : return 8; // node
                    case 2 : return 3; // mx
                    case 3 : return 3; // network
                    case 4 : return 2; // user ( name, netid )
                    case 5 :
                    case 6:
                    case 7:
                        return 1; // group, ateam & domain
                    default:
                        return 0;
                    }
                }
                public int getRowCount() {
                    if( records == null )
                        return 0;
                    return records.size();
                }
                public Object getValueAt(int row, int col) {
                    switch( lastSearchIndex ) {
                    case 1: // NODE
                        switch( col ) {
                        case 0: // name
                            return ((Hashtable)records.get(keysArray[row])).get("name");
                        case 1: // name type
                            return ((Hashtable)records.get(keysArray[row])).get("name_type");
                        case 2: // ip
                            return (String)((Hashtable)records.get(keysArray[row])).get("ip");
                        case 3:
                            return ((Hashtable)((Hashtable)records.get(keysArray[row])).get("node_type")).get("Advanced")!=null?" advanced":"";
                        case 4:
                            return ((Hashtable)((Hashtable)records.get(keysArray[row])).get("node_type")).get("Router")!=null?" router":"";
                        case 5:
                            return ((Hashtable)((Hashtable)records.get(keysArray[row])).get("node_type")).get("IPCprovider")!=null?" ipc provider":"";
                        case 6:
                            return ((Hashtable)((Hashtable)records.get(keysArray[row])).get("node_type")).get("Template")!=null?" template":"";
                        case 7:
                            return ((Hashtable)records.get(keysArray[row])).get("handle");
                        case 8:
                            return ((Hashtable)records.get(keysArray[row])).get("NodeID");
                        default:
                            return "";
                        }

                    case 2: // MX
                        switch( col ) {
                        case 0:
                            return ((Hashtable)records.get(keysArray[row])).get("mx");
                        case 1:
                            return ((Hashtable)records.get(keysArray[row])).get("handle");
                        case 2:
                            return ((Hashtable)records.get(keysArray[row])).get("pref");
                        };

                    case 3: // Network
                        switch( col ) {
                        case 0:
                            String net_name = (String)((Hashtable)records.get(keysArray[row])).get("net_name");
                            if( net_name != null )
                                return net_name;
                            else
                                return ((Hashtable)records.get(keysArray[row])).get("handle");

                        case 1: return ((Hashtable)records.get(keysArray[row])).get("dhcp_name");
                        case 2:
                            String ip = (String)((Hashtable)records.get(keysArray[row])).get("ip");
                            if( ip != null )
                                return ip;
                            else
                                return "";
                        }


                    case 4: // User
                        switch( col ) {
                        case 0: // name
                            return ((Hashtable)records.get(keysArray[row])).get("name");
                        case 1: // netid
                            return ((Hashtable)records.get(keysArray[row])).get("NetID");
                        }

                    case 5: // Group
                    case 6: // ATeam
                    case 7: // Domain
                        return keysArray[row]; // only one column

                    default:
                        return "";
                    }
                }
                public String getColumnName(int col) {
                    for( int i = 0; i < colSuffix.length; i++ )
                        colSuffix[i] = "";
                    if( last_column_sort > -1 ) {
                        if( sorted_down )
                            colSuffix[last_column_sort] = "  [down]";
                        else
                            colSuffix[last_column_sort] = "  [up]";
                    }
                    if( lastSearchIndex == 1 ) // Node
                        switch(col) {
                        case 0: return "Name"+colSuffix[col];
                        case 1: return "Name Type"+colSuffix[col];
                        case 2: return "IP"+colSuffix[col];
                        case 3: return "Adv?"+colSuffix[col];
                        case 4: return "Router?"+colSuffix[col];
                        case 5: return "IPC?"+colSuffix[col];
                        case 6: return "Template?"+colSuffix[col];
                        case 7: return "Handle"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 2 ) // MX
                        switch(col) {
                        case 0: return "MX"+colSuffix[col];
                        case 1: return "Handle"+colSuffix[col];
                        case 2: return "Pref"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 3 ) // Network
                        switch(col) {
                        case 0: return "Network Name"+colSuffix[col];
                        case 1: return "Dynamic DHCP Address Name"+colSuffix[col];
                        case 2: return "Dynamic DHCP/Network Address"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 4 )
                        switch(col) {
                        case 0: return "Name"+colSuffix[col];
                        case 1: return "NetID"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 5 )
                        switch(col) {
                        case 0: return "Group"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 6 )
                        switch(col) {
                        case 0: return "Admin Team"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else if( lastSearchIndex == 7 )
                        switch(col) {
                        case 0: return "Domain"+colSuffix[col];
                        default:
                            return "(null)";
                        }

                    else
                        return "(null)";
                }
            };

        table = new JTable(tableModel);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setShowVerticalLines(false);

        tableKeyListener = new KeyAdapter() {
                public void keyPressed(KeyEvent event) {
                    try {
                        char key = event.getKeyChar();
                        if( key == 'i' || key == 'I' || event.getKeyCode() == KeyEvent.VK_ENTER ) {
                            // info frame
                            XFrame infoFrame = null;
                            switch( lastSearchIndex ) {
                            case 1:
                            case 2: // NodeInfo
                                infoFrame =
                                    NodeInfoFrame.MakeNodeInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"));
                                break;

                            case 3: // NetworkInfo
                                infoFrame =
                                    new NetworkInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"));
                                break;

                            case 4: // User
                                infoFrame =
                                    new UserInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("NetID"));
                                break;

                            case 5: // GroupInfo
                                infoFrame =
                                    new GroupInfoFrame(keysArray[table.getSelectedRow()].toString());
                                break;

                            case 6: // ATeam
                                infoFrame =
                                    new ATeamInfoFrame(keysArray[table.getSelectedRow()].toString());
                                break;

                            case 7: // DomainInfo
                                infoFrame =
                                    new DomainInfoFrame(keysArray[table.getSelectedRow()].toString());
                                break;
                            }
                            if( infoFrame != null )
                                infoFrame.setVisible(true);

                        } else if ( key == 'm' || key == 'M' ) {
                            switch( lastSearchIndex ) {
                            case 1:
                            case 2: // Node
                                NodeModifyFrame.MakeNodeModifyFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), false);
                                break;

                            case 3: // Network
                                new NetworkModifyFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), false);
                                break;

                            case 4: // User
                                //new UserModifyFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("NetID"));
                                break;
                            }

                        } else if ( key == 't' || key == 'T' ) {
                            switch( lastSearchIndex ) {
                            case 1:
                            case 2: // Node
                                NodeModifyFrame.MakeNodeModifyFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), true);
                                break;

                            case 3: // Network
                                new NetworkModifyFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), true);
                                break;
                            }

                        } else if ( key == 'd' || key == 'D' ) {
                            switch( lastSearchIndex ) {
                            case 1:
                            case 2: // Node
                                NodeInfoFrame.MakeNodeInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), "delete");
                                return;

                            case 3: // Network
                                new NetworkInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"), true);
                                break;
                            }
                        }
                    } catch(Exception ex) {}
                }
            };

        JMenuItem mi = new JMenuItem("info");
        mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    table.requestFocus();
                    tableKeyListener.keyPressed(new KeyEvent(THIS, KeyEvent.KEY_TYPED,
                                                             (new java.util.Date()).getTime(), 0,
                                                             KeyEvent.VK_UNDEFINED, 'i'));
                }
            });
        popupMenu.add(mi);

        mi = new JMenuItem("modify");
        mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    table.requestFocus();
                    tableKeyListener.keyPressed(new KeyEvent(THIS, KeyEvent.KEY_TYPED,
                                                             (new java.util.Date()).getTime(), 0,
                                                             KeyEvent.VK_UNDEFINED, 'm'));
                }
            });
        popupMenu.add(mi);

        mi = new JMenuItem("tmplate");
        mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    table.requestFocus();
                    tableKeyListener.keyPressed(new KeyEvent(THIS, KeyEvent.KEY_TYPED,
                                                             (new java.util.Date()).getTime(), 0,
                                                             KeyEvent.VK_UNDEFINED, 't'));
                }
            });
        popupMenu.add(mi);

        mi = new JMenuItem("delete");
        mi.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    table.requestFocus();
                    tableKeyListener.keyPressed(new KeyEvent(THIS, KeyEvent.KEY_TYPED,
                                                             (new java.util.Date()).getTime(), 0,
                                                             KeyEvent.VK_UNDEFINED, 'd'));
                }
            });
        popupMenu.add(mi);
        popupMenu.pack();

        tbh = table.getTableHeader();
        tbh.setReorderingAllowed(false);
        tbh.addMouseListener( new MouseAdapter() {
                public void mouseClicked(MouseEvent event) {
                    int column = tbh.columnAtPoint(event.getPoint());
                    java.util.List list = Arrays.asList(records.keySet().toArray());
                    if( last_column_sort == column )
                        sorted_down = sorted_down?false:true;
                    else
                        sorted_down = true;
                    last_column_sort = column;
                    switch( lastSearchIndex ) {
                    case 1: // Node
                        switch(column) {
                        case 0: // name w/o def. domain
                            cmp_key = null;
                            name_cmp = cmp1;
                            break;

                        case 1: // name type
                            cmp_key = "name_type";
                            name_cmp = cmp1;
                            break;

                        case 2: // ip address
                            cmp_key = "ip";
                            name_cmp = cmp4;
                            break;

                        case 3: // Adv?
                            cmp_key = "Advanced";
                            name_cmp = cmp2;
                            break;

                        case 4: // Router?
                            cmp_key = "Router";
                            name_cmp = cmp2;
                            break;

                        case 5: // IPC?
                            cmp_key = "IPCprovider";
                            name_cmp = cmp2;
                            break;

                        case 6: // Template?
                            cmp_key = "Template";
                            name_cmp = cmp2;
                            break;

                        case 7: // Handle
                            cmp_key = "handle";
                            name_cmp = cmp1;
                            break;

                        case 8: // NodeID
                            cmp_key = "NodeID";
                            name_cmp = cmp3;
                            break;

                        default:
                            name_cmp = cmp1;
                            cmp_key = null;
                        }
                        break;

                    case 2: // MX
                        switch(column) {
                        case 0: // MX
                            cmp_key = "mx";
                            name_cmp = cmp1;
                            break;

                        case 1: // Handle
                            cmp_key = "handle";
                            name_cmp = cmp1;
                            break;

                        case 2: // Pref
                            cmp_key = "pref";
                            name_cmp = cmp5;
                            break;

                        default:
                            name_cmp = cmp1;
                            cmp_key = null;
                        }
                        break;

                    case 3: // Network
                        switch(column) {
                        case 0: // MX
                            name_cmp = cmp6;
                            break;

                        case 1: // Handle
                            cmp_key = "dhcp_name";
                            name_cmp = cmp1;
                            break;

                        case 2: // Pref
                            cmp_key = "ip";
                            name_cmp = cmp4;
                            break;

                        default:
                            name_cmp = cmp1;
                            cmp_key = null;
                        }
                        break;

                    case 4: // Users
                        switch(column) {
                        case 0: // Name
                            cmp_key = "name";
                            name_cmp = cmp1;
                            break;

                        case 1: // NetID
                            cmp_key = "NetID";
                            name_cmp = cmp1;
                            break;

                        default:
                            name_cmp = cmp1;
                            cmp_key = null;
                        }
                        break;

                    default:
                        name_cmp = cmp1;
                        cmp_key = null;
                    }

                    Collections.sort(list, name_cmp);
                    keysArray = list.toArray();
                    tableModel.fireTableStructureChanged();
                    table.repaint();
                }
            });
        table.addKeyListener(tableKeyListener);
        table.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent event) {

                    if( event.getClickCount() == 1 ) {
                        if( event.getButton() == 3 || event.getButton() == 2 ) {
                            Point point = event.getPoint();
                            int row = table.rowAtPoint(point);
                            if( row != -1 ) {
                                table.setRowSelectionInterval(row, row);
                                popupMenu.show(table, point.x, point.y);
                                popupMenu.setVisible(true);
                            }
                        }

                    } else {
                        XFrame infoFrame = null;
                        switch( lastSearchIndex ) {
                        case 1:
                        case 2: // NodeInfo
                            infoFrame =
                                NodeInfoFrame.MakeNodeInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"));
                            break;

                        case 3: // NetworkInfo
                            infoFrame =
                                new NetworkInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("handle"));
                            break;

                        case 4: // User
                            infoFrame =
                                new UserInfoFrame(""+((Hashtable)records.get(keysArray[table.getSelectedRow()])).get("NetID"));
                            break;

                        case 5: // GroupInfo
                            infoFrame =
                                new GroupInfoFrame(keysArray[table.getSelectedRow()].toString());
                            break;

                        case 6: // ATeam
                            infoFrame =
                                new ATeamInfoFrame(keysArray[table.getSelectedRow()].toString());
                            break;

                        case 7: // Domain
                            infoFrame =
                                new DomainInfoFrame(keysArray[table.getSelectedRow()].toString());
                            break;
                        }
                        if( infoFrame != null )
                            infoFrame.setVisible(true);

                    }
                }
            });
        JScrollPane scrollpane = new JScrollPane(table);

        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        setLayout(layout);

        //what.addItem("All Types");
        if( MainWindow.enable_node ) {
            what.addItem("Nodes");
            what.addItem("MXes");
        }
        what.addItem("Networks");
        //what.addItem("Users");
        //what.addItem("Groups");
        //what.addItem("Admin Teams");
        //what.addItem("Domains");

        what.setSelectedIndex(0);
        JPanel top_panel = new JPanel();
        top_panel.setBackground(MainWindow.tool_bar_color);
        top_panel.add(new JLabel("Search by name, hw or ip address"));
        top_panel.add(qString);
        top_panel.add(what);
        top_panel.add(submitB);
        top_panel.add(cancelB);
        cancelB.setEnabled(false);
        cancelB.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    (new SearchThread(THIS, 0, SearchThread.CANCEL)).start();
                }
            });
        top_panel.add(new JLabel("   "));

        //top_panel.add(helpB);
     /*
        helpB.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    JOptionPane.showMessageDialog(MainWindow.win,
                                                  "<html><body><center><font size=\"+2\">Quick search help:</font></center><br>"+
                        "<blockquote><font size=\"+1\">Key binings:</font></blockquote><br><ul><li>i - info, or mouse double click</li>"+
                        "<li>m - modify</li><li>t - use as a template</li><li>d - delete</li></ul><ul>"+
                        "<li>You can click right button mouse to evoke context menu.</li></ul><br>"+
                        "<blockquote>Also, once you are in the info screen you may<br>press ESC to close it.</blockquote></body></html>",
                        "Help", JOptionPane.INFORMATION_MESSAGE);
                }
            });
     */
        layout.setConstraints(top_panel, c);
        add(top_panel);

        c.weighty = 1.0;
        c.fill = GridBagConstraints.BOTH;
        c.gridy = 1;
        layout.setConstraints(scrollpane, c);
        add(scrollpane);

        status_panel.setBackground(MainWindow.tool_bar_color);
        status_panel.add(status_label);
        c.weighty = 0;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridy = 2;
        layout.setConstraints(status_panel, c);
        add(status_panel);

        submitB.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    qString.postActionEvent();
                }
            });

        qString.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    try {
                        if( qString.getText().trim().equals("") ) // no nothing on empty string
                            return;
                        cmp_key = null;
                        name_cmp = cmp1;
                        last_column_sort = 0;
                        sorted_down = true;

                        searchException  = null;
                        Thread sth = new SearchThread(THIS, THIS.whatToID(what.getSelectedItem().toString()), SearchThread.SEARCH);
                        if( searchException != null )
                            throw searchException;
                        qString.setEnabled(false);
                        what.setEnabled(false);
                        submitB.setEnabled(false);
                        status_panel.add(progressBar);
                        status_panel.validate();
                        status_panel.repaint();

                        sth.start();
                    } catch(Exception ex) {
                        JOptionPane.showMessageDialog(MainWindow.win,
                              ex, "Error", JOptionPane.ERROR_MESSAGE);

                    } finally {
                        tableModel.fireTableStructureChanged();
                        table.revalidate();
                        table.repaint();
                    }
                };
            });
    }

    private int whatToID(String selection) {
        if( selection.equals("Nodes") )
            return 0;
        else if( selection.equals("MXes") )
            return 1;
        else if( selection.equals("Networks") )
            return 2;
        else
            return -1;
    }

}
