/************************************************************************
 *
 * SearchThread.java -
 *
 * $Id: SearchThread.java,v 1.1.1.1 2006-06-30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class SearchThread extends Thread {

    private QSPanelI qsp;
    private int what;
    private int action;
    private String qString;
    private String oldLabel;

    public static final int SEARCH = 1;
    public static final int CANCEL = 2;

    public SearchThread(QSPanelI panel, int what, int action) {
        super("");
        qsp = panel;
        this.what = what;
        this.action = action;
    }

    public void run() {
        try {
            if( action == CANCEL ) {
                MainWindow.netdb.cancelSearch(qsp.searchID);
                return;
            }
            qsp.cancelB.setEnabled(true);
            oldLabel = qsp.status_label.getText();
            qsp.status_label.setText("");
            qString = new String(qsp.qString.getText());
            switch( what ) {
            case 0: // node
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.NODE, qString, qsp.searchID);

                java.util.List list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                //keys = records.keySet();
                // sort here may be
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 1;
                break;

            case 1: // mx
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.MX, qString, qsp.searchID);

                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 2;
                break;

            case 2: // networks
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.NETWORK, qString, qsp.searchID);
                // shuffle records for quick search display
                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 3;

                break;

            case 3: // users
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.USER, qString, qsp.searchID);
                // shuffle records for quick search display
                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 4;
                qsp.last_column_sort = -1;

                break;

            case 4: // groups
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.GROUP, qString, qsp.searchID);
                // shuffle records for quick search display
                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 5;

                break;

            case 5: // admin teams
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.ATEAM, qString, qsp.searchID);
                // shuffle records for quick search display
                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 6;

                break;

            case 6: // domains
                qsp.searchID = MainWindow.netdb.createSearchID();
                qsp.records = MainWindow.netdb.simpleSearch(Constants.DOMAIN, qString, qsp.searchID);
                // shuffle records for quick search display
                list = Arrays.asList(qsp.records.keySet().toArray());
                Collections.sort(list, qsp.name_cmp);
                qsp.keysArray = list.toArray();
                qsp.lastSearchIndex = 7;

                break;

            default:
                throw new Exception("Unsupported search type");
            }

            qsp.status_label.setText("<html><body>Search for [<tt>"+Utils.htmlize(qString)+
                                     "</tt>] found "+qsp.records.size()+" records</body></html>");

        } catch (java.rmi.ServerException ex) {
            if( ex.getCause() instanceof SessionTimeoutException )
                X.handleTimeout();

            String msg = ex.getCause().getMessage();
            if( msg.indexOf(":") == -1 )
                JOptionPane.showMessageDialog(MainWindow.win,
                  "<html><body>"+Utils.htmlize(msg)+
                  "</body></html>", "Error", JOptionPane.ERROR_MESSAGE);
            Utils.debug(ex.getMessage());
            qsp.status_label.setText(oldLabel);
            qsp.searchException = ex;

        } catch (java.rmi.NoSuchObjectException ex) {
            X.handleTimeout();

        } catch (java.rmi.MarshalException ex) {
            X.handleTimeout();

        } catch (Exception ex) {
            Utils.debug(ex.getMessage());
            qsp.status_label.setText(oldLabel);
            qsp.searchException = ex;

        } finally {
            qsp.qString.setEnabled(true);
            qsp.what.setEnabled(true);
            qsp.submitB.setEnabled(true);
            qsp.cancelB.setEnabled(false);
            qsp.searchID = null;
            qsp.status_panel.remove(qsp.progressBar);
            qsp.status_panel.validate();
            qsp.status_panel.repaint();

            qsp.tableModel.fireTableStructureChanged();
            qsp.table.revalidate();
            qsp.table.repaint();
        }
    }
}
