/************************************************************************
 *
 * UserInfoFrame.java -
 *
 * $Id: UserInfoFrame.java,v 1.1.1.1 2006-06-30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.io.*;
import java.sql.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.table.*;
import stanford.netdb.utils.*;

public class UserInfoFrame extends XFrame implements Runnable {

    public String handle;
    Hashtable user              = null;
    //public JButton becomeB      = new JButton("Become");

    // used for sorting
    private Comparator name_cmp = null;

    public UserInfoFrame(Hashtable user) {
        super("(Hash)");
        this.user = user;
        initFrame();
    }

    public UserInfoFrame(String handle) {
        super("UserInfo: "+handle);
        this.handle = handle;
        initFrame();
    }

    private void initFrame() {

        init(); // init XFrame bowls
        //tool_bar.add(becomeB);
        modifyB.setEnabled(false);

        modifyB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    //new UserModifyFrame(handle);
                }
            });

        reloadB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    ((UserInfoFrame)parent).user = null;
                    (new Thread((UserInfoFrame)parent)).start();
                }
            });

        deleteB.addActionListener( new AActionListener(this) {
                public void actionPerformed(ActionEvent event) {
                    Hashtable user = ((UserInfoFrame)parent).user;
                    if( JOptionPane.showConfirmDialog(parent, "Delete User "+user.get("name")+
                                                      " ("+user.get("NetID")+") ?",
                                                      "Delete User ?",
                                                      JOptionPane.YES_NO_OPTION) == 0 ) {
                        boolean unlocked = false;
                        int id = -1;
                        Hashtable result = new Hashtable();
                        Timestamp tm = null;
                        try {
                            id = Integer.parseInt(user.get("id").toString());
                            tm = MainWindow.netdb.lock(Constants.USER, id);
                            result = MainWindow.netdb.delete(Constants.USER, user.get("id").toString());
                            MainWindow.netdb.unlock(Constants.USER, id, tm);
                            unlocked = true;
                            JOptionPane.showMessageDialog(parent,
                                 "User has been deleted.", "Info", JOptionPane.INFORMATION_MESSAGE);
                            ((JFrame)parent).dispose();

                        } catch(RemoteNetDBException exx) {
                            Vector errors = exx.getMessages();
                            String[] msg = new String[errors.size()];
                            for( int i = 0; i < errors.size(); i++ )
                                msg[i] = (String)errors.elementAt(i);
                            exx.printStackTrace();
                            JOptionPane.showMessageDialog(parent,
                              msg, "Error", JOptionPane.ERROR_MESSAGE);

                        } catch (Exception ex) {
                            ex.printStackTrace();
                            JOptionPane.showMessageDialog(parent,
                              ex.toString(), "Error", JOptionPane.ERROR_MESSAGE);

                        } finally {
                            if( !unlocked && tm != null ) {
                                // try to unlock
                                try {
                                    MainWindow.netdb.unlock(Constants.USER, id, tm);
                                } catch(Exception exx) {
                                }
                            }
                        }
                    }
                }
            });

        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.pack();
        this.setSize(900, 700);
        Dimension scrnsize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation( (int)((scrnsize.getWidth()-getWidth())/2),
                     (int)((scrnsize.getHeight()-getHeight())/2) );
        this.show();

        (new Thread(this)).start();
    } // end of initFrame()

    public void run() {
        try {
            boolean hasAccess = false;
            reloadB.setEnabled(false);
            //becomeB.setEnabled(false);

            if( user == null ) {
                user = MainWindow.netdb.getInfo(Constants.USER, handle);
                if( user == null || user.get("id") == null ) {
                    label.setText("Failed to load user. It probably does not exist.");
                    label.setForeground(Color.red);
                    reloadB.setEnabled(false);
                    deleteB.setEnabled(false);
                    //becomeB.setEnabled(false);
                    return;
                }
            }

            // check that user has access to the domain
            if( MainWindow.netdb.checkAccess(Constants.USER) && MainWindow.netdb.checkGroupAccess(user) )
                hasAccess = true;

            current_panel = central_panel;
            current_panel.removeAll();
            current_panel.setLayout(current_gb_layout);

            current_gbc.fill = GridBagConstraints.HORIZONTAL;
            current_gbc.weightx = 1.0;
            current_gbc.weighty = 0;
            current_gbc.gridx = 0;
            current_gbc.gridy = 0;

            // empty row at the top
            XLabel(" ");
            tr();

            XLabel("Name");
            td();
            XLabel(user.get("name").toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            XLabel("NetID");
            td();
            XLabel(user.get("NetID").toString(), LEFT);
            tr();

            XLabel("Email");
            td();
            if( user.get("email") != null )
                XLabel(user.get("email").toString(), LEFT);
            tr();

            XLabel("Phone");
            td();
            if( user.get("phone") != null )
                XLabel(user.get("phone").toString(), LEFT);
            tr();

            XLabel("Account");
            td();
            XLabel(((Boolean)user.get("active?")).booleanValue()?"Active":"Inactive", LEFT);
            tr();

            XLabel("Default Domain");
            td();
            if( user.get("default_domain") != null )
                XLabel(user.get("default_domain").toString(), LEFT);
            tr();

            XLabel("Default Addr. Space");
            td();
            if( user.get("default_addr_space") != null )
                XLabel(user.get("default_addr_space").toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            XLabel("LNA departments");
            Hashtable deps = (Hashtable)user.get("departments");
            if( deps != null ) {
                java.util.List dlist = Arrays.asList(deps.keySet().toArray());
                Collections.sort(dlist, name_cmp);
                Object[] dArray = dlist.toArray();
                for( int i = 0; i < dArray.length; i++ ) {
                    String depname = (String)dArray[i];
                    Hashtable department = (Hashtable)deps.get(depname);
                    if( !((Boolean)department.get("official?")).booleanValue() )
                        continue;
                    td();

                    // collect consultants
                    XLabel(depname, LEFT);
                    tr();
                }
            }
            tr();

            XLabel(" ");
            tr();

            XLabel("Other departments");
            if( deps != null ) {
                java.util.List dlist = Arrays.asList(deps.keySet().toArray());
                Collections.sort(dlist, name_cmp);
                Object[] dArray = dlist.toArray();
                for( int i = 0; i < dArray.length; i++ ) {
                    String depname = (String)dArray[i];
                    Hashtable department = (Hashtable)deps.get(depname);
                    if( ((Boolean)department.get("official?")).booleanValue() )
                        continue;
                    td();

                    // collect consultants
                    XLabel(depname, LEFT);
                    tr();
                }
            }
            tr();

            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            XLabel("Default Group");
            td();
            XLabel(user.get("default_group").toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            XLabel("Member Groups");
            Object obj = user.get("all_groups?");
            if( obj != null && obj instanceof Boolean && ((Boolean)obj).booleanValue() ) {
                td();
                XLabel("All Groups", LEFT);
                tr();
            }

            Hashtable groups = (Hashtable)user.get("groups");
            if( groups != null ) {
                java.util.List glist = Arrays.asList(groups.keySet().toArray());
                Collections.sort(glist, name_cmp);
                Object[] gArray = glist.toArray();
                for( int i = 0; i < gArray.length; i++ ) {
                    td();
                    XLabel(gArray[i].toString(), LEFT);
                    tr();
                }
            }
            tr();

            XLabel(" ");
            tr();

            XLabel("Records");
            obj = user.get("all_objects?");
            if( obj != null && obj instanceof Boolean && ((Boolean)obj).booleanValue() ) {
                td();
                XLabel("All Records", LEFT);
                tr();
            }
            Hashtable recs = (Hashtable)user.get("records");
            if( recs != null ) {
                java.util.List rlist = Arrays.asList(recs.keySet().toArray());
                Collections.sort(rlist, name_cmp);
                Object[] rArray = rlist.toArray();
                for( int i = 0; i < rArray.length; i++ ) {
                    td();

                    // collect consultants
                    XLabel(recs.get(rArray[i].toString()).toString(), LEFT);
                    tr();
                }
            }
            tr();

            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            XLabel("Comment");
            td();
            obj = user.get("comment");
            if( obj != null )
                XLabel(obj.toString(), LEFT);
            tr();

            XLabel(" ");
            tr();

            line();

            XLabel(" ");
            tr();

            XLabel("Last Login");
            td();
            obj = user.get("last_login");
            if( obj != null )
                XLabel(obj.toString().substring(0, 16), LEFT);
            tr();

            XLabel(" ");
            tr();

            // Created By
            XLabel("Created By");
            td();
            Hashtable person = (Hashtable)user.get("created.by");
            String byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+user.get("created").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // Last Modified By
            XLabel("Last Modified By");
            td();
            person = (Hashtable)user.get("mod.by");
            byStr = "";
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += " ("+user.get("modified").toString().substring(0, 16)+")";
            XLabel(byStr, LEFT);
            tr();

            // empty row
            XLabel("");
            tr();

            reloadB.setEnabled(true);
            //becomeB.setEnabled(true);

            if( hasAccess ) {
                modifyB.setEnabled(true);
                deleteB.setEnabled(true);
            } else {
                tool_bar.remove(modifyB);
                tool_bar.remove(deleteB);
                tool_bar.validate();
                tool_bar.repaint();
            }

            central_panel.validate();
            central_panel.repaint();
            scrollpane.validate();

        } catch(Exception ex) {
            Utils.warn(ex.toString());
            ex.printStackTrace();
        }
    } // end of run()
}
