/************************************************************************
 *
 * XFrame.java - it is recommended that all frames be derived from
 *                   this one, as it holds some common tools.
 *
 * $Id: XFrame.java,v 1.1.1.1 2006-06-30 23:38:03 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import stanford.netdb.utils.*;

public class XFrame extends JFrame {

    public JPanel header_panel = new JPanel(new BorderLayout());
    public JPanel tool_bar = new JPanel(new FlowLayout(FlowLayout.CENTER));
    public JPanel mega_pane;

    public static Properties props      = new Properties();

    public static int LEFT              = SwingConstants.LEFT;
    public static int CENTER            = SwingConstants.CENTER;
    public static int RIGHT             = SwingConstants.RIGHT;

    public JButton closeB               = new BButton("Close");
    public JButton modifyB              = new BButton("Modify");
    public JButton deleteB              = new BButton("Delete");
    public JButton reloadB              = new BButton("Reload");
    public JButton templateB            = new BButton("Template");

    public KeyListener klistener;

    protected GridBagLayout layout      = new GridBagLayout();

    protected int min_label_length              = 20;
    public Font bold_font                       = null;
    public Font bold_em_font                    = null;
    protected Font label_font                   = null;
    protected Color label_color                 = null;
    protected GridBagLayout current_gb_layout   = new GridBagLayout();
    protected GridBagConstraints current_gbc    = new GridBagConstraints();
    protected JPanel current_panel              = new JPanel(new BorderLayout());

    public JPanel central_panel                 = new JPanel(new BorderLayout());
    public JScrollPane scrollpane;

    public JLabel label         = new JLabel("Loading info...", SwingConstants.CENTER);

    // used for sorting
    public static Comparator name_cmp = null;
    public static Comparator number_cmp = null;
    public static Comparator ip_cmp = null;
    public static Comparator name_number_cmp = null;

    public static Hashtable allFrames   = new Hashtable();
    private static long nextFrameID = 1;
    public String key = null;
    public JMenuItem winMenuItem = null;

    public void setTitle(String title) {
        if( key != null ) {
            ((Hashtable)allFrames.get(key)).put("title", title);
        }
        super.setTitle(title);
        if( winMenuItem != null )
            winMenuItem.setText(title);
    }

    public void register() {
        register(false);
    }

    public void register(boolean main) {
        synchronized(allFrames) {
            Hashtable frame = new Hashtable();
            winMenuItem = new JMenuItem(getTitle());
            frame.put("title", getTitle());
            frame.put("frame", this);
            frame.put("winMenuItem", winMenuItem);
            if( main )
                frame.put("main", "");
            key = getTitle()+" "+(nextFrameID++);
            allFrames.put(key, frame);
        }
        winMenuItem.addActionListener(new AActionListener(this) {
                public void actionPerformed(ActionEvent e) {
                    ((JFrame)parent).setState(Frame.NORMAL);
                    ((JFrame)parent).toFront();
                }
            });
        X.windowsMenu.add(winMenuItem);
    }

    public void dispose() {
        synchronized(allFrames) {
            if( winMenuItem != null )
                X.windowsMenu.remove(winMenuItem);
            if( key != null )
                allFrames.remove(key);
        }
        super.dispose();
    }

    public XFrame(String name) {
        super(name);
        klistener = new KeyAdapter() {
                public void keyPressed(KeyEvent event) {
                    if( event.getKeyCode() == KeyEvent.VK_ESCAPE )
                        dispose();
                }
            };

        addKeyListener(klistener);

        // close frame on ESC
        closeB.addActionListener(new ActionListener(){
                public void actionPerformed(ActionEvent event) {
                    dispose();
                }
            });

        if( name_cmp == null )
            name_cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        return ((String)o1).compareToIgnoreCase((String)o2);
                    }
                    public boolean equals(Object obj) {
                        return true;
                    }
                };

        if( number_cmp == null )
            number_cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        return (int)((new Long((String)o1)).longValue() -
                                     (new Long((String)o2)).longValue());
                    }
                    public boolean equals(Object obj) {
                        return true;
                    }
                };

        if( ip_cmp == null )
            ip_cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        return Utils.ipstr2num((String)o1) - Utils.ipstr2num((String)o2);
                    }
                    public boolean equals(Object obj) {
                        return true;
                    }
                };

        if( name_number_cmp == null )
            name_number_cmp = new Comparator() {
                    public int compare(Object o1, Object o2) {
                        try {
                            long lo1 = (new Long(((String)o1).trim())).longValue();
                            long lo2 = (new Long(((String)o2).trim())).longValue();
                            return (int)(lo2 - lo1);
                        } catch (Exception ex) { }
                        return ((String)o1).compareToIgnoreCase((String)o2);
                    }
                    public boolean equals(Object obj) {
                        return true;
                    }
                };

        try {
            bold_font = new Font("Monospaced", Font.BOLD, 14);
        } catch (Exception ex) {
            try {
                bold_font = new Font("fixed", Font.BOLD, 14);
            } catch (Exception exx) { }
            Utils.warn("Got exception while loading bold font. "+ex);
        }

        try {
            bold_em_font = new Font("Monospaced", Font.BOLD|Font.ITALIC, 14);
        } catch (Exception ex) {
            try {
                bold_font = new Font("fixed", Font.BOLD, 14);
            } catch (Exception exx) { }
            Utils.warn("Got exception while loading bold font. "+ex);
        }
    }

    /** loads default values for protected variables and sets default properties */
    public void init() {
        current_panel.setLayout(current_gb_layout);

        try {
            label_font = new Font(props.getProperty("font_name", "Monospaced"),
                                  Font.PLAIN,
                                  Integer.parseInt(props.getProperty("font_size", "14")));
        } catch (Exception ex) {
            label_font = new Font("fixed", Font.BOLD, 14);
            Utils.warn("Got exception while loading label font. "+ex);
        }

        label_color = Color.getColor(props.getProperty("label_color", "black"), Color.black);

        getContentPane().setLayout(layout);

        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.weighty = 0;
        c.gridx = 0;
        c.gridy = 0;

        modifyB.setEnabled(false);
        deleteB.setEnabled(false);
        reloadB.setEnabled(false);

        tool_bar.add(modifyB);
        tool_bar.add(deleteB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(reloadB);
        tool_bar.add(new JLabel("   "));
        tool_bar.add(closeB);

        tool_bar.setBackground(MainWindow.tool_bar_color);
        header_panel.add(tool_bar, BorderLayout.NORTH);
        header_panel.setBackground(Color.white);
        layout.setConstraints(header_panel, c);
        getContentPane().add(header_panel);

        c.gridy = 1;
        c.weighty = 1;
        c.fill = GridBagConstraints.BOTH;
        central_panel.setBackground(Color.white);
        central_panel.add(label, BorderLayout.CENTER);

        scrollpane = new JScrollPane(central_panel);
        mega_pane = new JPanel(new BorderLayout());
        mega_pane.setBackground(Color.white);
        mega_pane.add(scrollpane, BorderLayout.CENTER);
        layout.setConstraints(mega_pane, c);
        getContentPane().add(mega_pane);
    }

    /** start next line in GridBagLayout, a la html */
    public void tr() {
        current_gbc.gridy++;
        current_gbc.gridx = 0;
        current_gbc.gridwidth = GridBagConstraints.RELATIVE;
    }

    /** go to next 'table' cell, a la html*/
    public void td() {
        current_gbc.gridx++;
    }

    /** go to next 'table' cell with custom gridwidth */
    public void td(int gridwidth) {
        current_gbc.gridx++;
        current_gbc.gridwidth = gridwidth;
    }

    /** create centered formatted label and display it */
    public JLabel XLabel(String txt) {
        return XLabel(txt, CENTER);
    }

    /**
     * create formatted label with custom alignment.
     * @param alignment - {LEFT, CENTER, RIGHT}
     */
    public JLabel XLabel(String txt, int alignment) {
        return XLabel(txt, alignment, label_color);
    }

    /**
     * create formatted label with custom alignment, and foreground color.
     * @param alignment - {LEFT, CENTER, RIGHT}
     * @param color     - Color
     */
    public JLabel XLabel(String txt, int alignment, Color color) {
        return XLabel(format(txt, min_label_length), alignment, color, label_font);
    }

    /**
     * create unformatted label with custom alignment, foreground color and font
     * @param alignment - {LEFT, CENTER, RIGHT}
     * @param color     - Color
     */
    public JLabel XLabel(String txt, int alignment, Color color, Font font) {
        JLabel label = new JLabel(txt, alignment);
        if( font != null )
            label.setFont(font);
        if( color != null )
            label.setForeground(color);
        current_gb_layout.setConstraints(label, current_gbc);
        current_panel.add(label);
        return label;
    }

    public JPanel HTMLPanel(String txt) {
        return HTMLPanel(txt, label_font, 1);
    }

    public JPanel HTMLPanel(String txt, int grids) {
        return HTMLPanel(txt, label_font, grids);
    }

    public JPanel HTMLPanel(String txt, Font font, int grids) {
        JPanel panel = new JPanel(new GridLayout(1, grids*2+1));
        panel.setBackground(Color.white);
        for( int i = 0; i < grids; i++ )
            panel.add(new JLabel(""));
        JLabel label = new JLabel("<html><body>"+txt+"</body></html>");
        if( font != null )
            label.setFont(font);
        panel.add(label);
        for( int i = 0; i < grids; i++ )
            panel.add(new JLabel(""));
        current_gb_layout.setConstraints(panel, current_gbc);
        current_panel.add(panel);
        return panel;
    }

    /** adds extra free space to the right of text if its length less then min_length */
    public static String format(String str, int min_length) {
        if( str.length() >= min_length )
            return str;

        int extra_length = min_length - str.length();
        char[] extra = new char[extra_length];
        for( int i = 0; i < extra_length; i++ )
            extra[i] = ' ';

        return str.concat(new String(extra));
    }

    /** adds separating line */
    public JSeparator line() {
        int oldGridwidth = current_gbc.gridwidth;
        current_gbc.gridwidth = GridBagConstraints.REMAINDER;
        JSeparator line = new JSeparator();
        current_gb_layout.setConstraints(line, current_gbc);
        current_panel.add(line);
        tr();
        return line;
    }

        public AJLabel makeLabel(Object labelObj, Hashtable box, Font label_font, Color label_color) {
        AJLabel label;
        if( labelObj instanceof Icon )
            label = new AJLabel((Icon)labelObj, box);
        else
            label = new AJLabel(labelObj.toString(), box);
        if( label_font != null )
            label.setFont(label_font);
        if( label_color != null )
            label.setForeground(label_color);
        return label;
    }

    public AJLabel makeLabel(Object labelObj, Hashtable box, Font label_font) {
        return makeLabel(labelObj, box, label_font, label_color);
    }

    public AJLabel makeLabel(Object labelObj, Hashtable box) {
        return makeLabel(labelObj, box, label_font, label_color);
    }

    public AJLabel makeLabel(Object labelObj, Font label_font) {
        return makeLabel(labelObj, null, label_font, label_color);
    }

    public AJLabel makeLabel(Object labelObj) {
        return makeLabel(labelObj, null, label_font, label_color);
    }

    public Component add(Component comp) {
        current_gb_layout.setConstraints(comp, current_gbc);
        current_panel.add(comp);
        return comp;
    }

    public Component add(Component comp, int fill, int anchor) {
        int saved_fill = current_gbc.fill;
        int saved_anchor = current_gbc.anchor;
        current_gbc.fill = fill;
        current_gbc.anchor = anchor;
        add(comp);
        current_gbc.fill = saved_fill;
        current_gbc.anchor = saved_anchor;
        return comp;
    }
}
