-- NetDB_Default.sql --- contains functions to manipulate GlobalDefault table

create or replace package body NetDB_Default as

    -- lists content of the GlobalDefault table
    function list return NetDB.ref_cursor
    as
        out_record result_set       := result_set ();
        ret_cursor NetDB.ref_cursor := null;

        v_label      GlobalDefault.label%TYPE     := null;
        v_str_val    GlobalDefault.str_value%TYPE := null;
        v_int_val    GlobalDefault.int_value%TYPE := 0;
        v_data_type  varchar(7) := null;

        cursor data_cursor is
            select label, str_value, int_value from GlobalDefault;
    begin
        open data_cursor;
        loop
            fetch data_cursor into v_label, v_str_val, v_int_val;
            exit when data_cursor%notfound;

            if (v_int_val is not null) then v_data_type := 'integer';
            else                            v_data_type := 'string';
            end if;

            out_record.extend(1);
            out_record(out_record.COUNT) := result_row ('key', v_label, null, v_data_type, v_str_val, null, v_int_val);
        end loop;
        close data_cursor;

        open ret_cursor for
            select * from table (cast (out_record as result_set));

        return ret_cursor;
    end list;


    function get_str (p_label in varchar)  return varchar
    as
        v_value GlobalDefault.str_value%TYPE;
    begin
        select str_value into v_value from GlobalDefault where lower (label) = lower (p_label);

        return v_value;
    exception
        when NO_DATA_FOUND or TOO_MANY_ROWS then
            return null;
    end get_str;


    function get_int (p_label in varchar)  return int
    as
        v_value GlobalDefault.int_value%TYPE;
    begin
        select int_value into v_value from GlobalDefault where lower(label) = lower(p_label);

        return v_value;
    exception
        when NO_DATA_FOUND or TOO_MANY_ROWS then
            return null;
    end get_int;


    function get_id (p_label in varchar) return int
    as
        v_value GlobalDefault.id_value%TYPE;
    begin
        select id_value into v_value from GlobalDefault where lower(label) = lower(p_label);

        return v_value;
    exception
        when NO_DATA_FOUND or TOO_MANY_ROWS then
            return null;
    end get_id;

end NetDB_Default;
/
