-- NetDB_Make.sql --- contains functions related to Make table

create or replace package body NetDB_Make as

    function list return NetDB.ref_cursor
    as
        ret_cursor  NetDB.ref_cursor := null;
        out_record  result_set       := result_set ();
    begin
        for r in (select * from Make)
        loop
            out_record.extend(1);
            out_record(out_record.COUNT) :=
                result_row ('key', r.make, null, 'string', ' ', null, null);
        end loop;

        open ret_cursor for
            select * from table (cast (out_record as result_set));

        return ret_cursor;
    end list;


    -- deletes make
    procedure del ( p_op_uid in int,
                    p_make in varchar )
    as
        v_make Make.make%TYPE := verify_or_die (p_make);
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'Make');

        delete Make where make = p_make;
    exception
        when NetDB.children_exist then
            NetDB_Util.throw_still_in_use ('make', p_make);
    end del;


    -- inserts make
    procedure ins ( p_op_uid in int,
                    p_make   in varchar ) as
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'Make');

        NetDB_Util.validate (p_make, 'make name', 'A make must have a name');

        insert into Make (make) values (translate (p_make, '"', ''''));
    exception
        when NetDB.unique_constraint_failed then
            NetDB_Util.throw_record_already_exists ('make', p_make);
        when NetDB.value_too_large then
            NetDB_Util.throw_field_too_long (sqlerrm);
    end ins;


    -- updates makes
    procedure upd ( p_op_uid   in int,
                    p_make     in varchar,
                    p_make_new in varchar )
    as
        v_make Make.make%TYPE := verify_or_die (p_make);
    begin
        NetDB_Access.has_access_or_die (p_op_uid, 'Make');

        NetDB_Util.validate (p_make_new, 'make name', 'A make must have a name');

        update Make set make = translate (p_make_new, '"', '''')
         where make = p_make;
    exception
        when NetDB.unique_constraint_failed then
            NetDB_Util.throw_record_already_exists ('make', p_make_new);
        when NetDB.value_too_large then
            NetDB_Util.throw_field_too_long (sqlerrm);
    end upd;


    -- given make verifies that it exists and returns itself
    function verify_or_die (p_make in varchar) return varchar
    as
        v_make Make.make%TYPE;
    begin
        select make into v_make from Make where lower (make) = lower (translate (p_make, '"', ''''));

        return v_make;
    exception
        when NO_DATA_FOUND then
            NetDB_Util.throw_record_does_not_exist ('make', p_make);
    end verify_or_die;

end NetDB_Make;
/
