-- NetDB_ModelType.sql --- contains functions related to ModelType table

create or replace package body NetDB_ModelType as

    -- List all model types.
    function list return NetDB.ref_cursor
    as
        out_record  result_set := result_set ();
        ret_cursor  NetDB.ref_cursor;
    begin
        for r in (select * from ModelType)
        loop
            out_record.extend(1);
            out_record(out_record.COUNT) := result_row ('key', r.modeltype, null, 'string', r.description, null, null);
        end loop;

        open ret_cursor for
            select * from table (cast (out_record as result_set));

        return ret_cursor;
    end list;


    -- Verify a modeltype esists, or complain.
    function verify_or_die (p_modeltype in varchar) return varchar
    as
        v_modeltype ModelType.modeltype%TYPE;
    begin
        if (p_modeltype is null) then
            NetDB_Util.throw_no_record_id_handle ();
        end if;

        select modeltype into v_modeltype from ModelType
         where lower (modeltype) = lower (p_modeltype);

        return v_modeltype;
    exception
        when NO_DATA_FOUND then
            NetDB_Util.throw_record_does_not_exist ('modeltype', p_modeltype);
        when TOO_MANY_ROWS then
            NetDB_Util.throw_single_record_expected ('modeltype');
    end verify_or_die;

end NetDB_ModelType;
/
