-- Simple.sql --- Simple, simplified, and SQL-ized (not using PL/SQL types) functions
--                For use by reporting tools, etc; not for use by the middleware!

create or replace package body Simple as

    -- see if a user is a member of a group with access to a record.
    function i_check_ownership_access ( p_user_id              in int,
                                        p_netid                in varchar,
                                        p_record_id            in int,
                                        p_record_type          in varchar,
                                        p_check_all_group_flag in int := 1,
                                        p_check_active_state   in int := 1 ) return int as
    begin
        return Util.to_int ( NetDB_Owner.check_access ( NetDB_User.verify_or_die(p_user_id,p_netid),
                                                        p_record_id,
                                                        p_record_type,
                                                        Util.to_boolean(p_check_all_group_flag),
                                                        Util.to_boolean(p_check_active_state) ));
    exception when NetDB.record_not_found then
        return -1;
    end i_check_ownership_access;

    function check_ownership_access ( p_user_id              in int,
                                      p_record_id            in int,
                                      p_record_type          in varchar,
                                      p_check_all_group_flag in int := 1,
                                      p_check_active_state   in int := 1 ) return int as
    begin
        return i_check_ownership_access ( p_user_id, null, p_record_id, p_record_type,
                                          p_check_all_group_flag, p_check_active_state );
    end check_ownership_access;

    function check_ownership_access ( p_netid                in varchar,
                                      p_record_id            in int,
                                      p_record_type          in varchar,
                                      p_check_all_group_flag in int := 1,
                                      p_check_active_state   in int := 1 ) return int as
    begin
        return i_check_ownership_access ( null, p_netid, p_record_id, p_record_type,
                                          p_check_all_group_flag, p_check_active_state );
    end check_ownership_access;

    -- see if a user is active and has a particular type of access.
    function i_check_recordtype_access ( p_user_id               in int,
                                         p_netid                 in varchar,
                                         p_access_type           in varchar,
                                         p_record_id             in int := null,
                                         p_check_all_record_flag in int := 1 ) return int as
    begin
        return Util.to_int ( NetDB_Access.check_access ( NetDB_User.verify_or_die(p_user_id,p_netid),
                                                         p_access_type,
                                                         p_record_id,
                                                         Util.to_boolean(p_check_all_record_flag) ));
    exception when NetDB.record_not_found then
        return -1;
    end i_check_recordtype_access;

    function check_recordtype_access ( p_user_id               in int,
                                       p_access_type           in varchar,
                                       p_record_id             in int := null,
                                       p_check_all_record_flag in int := 1 ) return int as
    begin
        return i_check_recordtype_access ( p_user_id, null, p_access_type,
                                           p_record_id, p_check_all_record_flag );
    end check_recordtype_access;

    function check_recordtype_access ( p_netid                 in varchar,
                                       p_access_type           in varchar,
                                       p_record_id             in int := null,
                                       p_check_all_record_flag in int := 1 ) return int as
    begin
        return i_check_recordtype_access ( null, p_netid, p_access_type,
                                           p_record_id, p_check_all_record_flag );
    end check_recordtype_access;

    -- returns list of address spaces to which user has access
    function i_IPaddressSpace_list ( p_user_id in int, p_netid varchar ) return simple_table_of_varchar
    as
        spaces simple_table_of_varchar;
    begin
        select value_string bulk collect into spaces
          from table(NetDB_IPaddressSpace.list_pipelined(NetDB_User.verify_or_die(p_user_id,p_netid)))
          where key='prefix';
        return spaces;
    exception when NetDB.record_not_found then
        select '' bulk collect into spaces from dual;
        return spaces;
    end i_IPaddressSpace_list;

    function IPaddressSpace_list ( p_user_id in int ) return simple_table_of_varchar as begin
        return i_IPaddressSpace_list ( p_user_id, null );
    end IPaddressSpace_list;

    function IPaddressSpace_list ( p_netid in varchar ) return simple_table_of_varchar as begin
        return i_IPaddressSpace_list ( null, p_netid );
    end IPaddressSpace_list;

end Simple;
/

