-- NetDB_Domain.pkg --- Domain package spec.

create or replace package NetDB_Domain
as
    function can_use ( p_user_id   in int,
                       p_domain_id in int ) return boolean;

    function can_use_as_name ( p_user_id   in int,
                               p_domain_id in int ) return boolean;

    function is_limited (p_domain_id in int) return boolean;

    procedure locked_or_die ( p_op_uid    in int,
                              p_lock      in timestamp,
                              p_domain_id in int );

    function lock_record ( p_op_uid    in int,
                           p_domain_id in int ) return NetDB.ref_cursor;

    procedure unlock_record ( p_op_uid    in int,
                              p_lock      in timestamp,
                              p_domain_id in int );

    function list (p_user_id in int := null) return NetDB.ref_cursor;

    function info (p_domain_id in int) return NetDB.ref_cursor;
    function info_pipelined (p_domain_id in int) return result_set pipelined;

    procedure del ( p_op_uid    in int,
                    p_lock      in timestamp,
                    p_domain_id in int );

    function ins ( p_op_uid  in int,
                   p_domain  in varchar,
                   p_limited in numeric := 1,
                   p_comment in varchar := null) return NetDB.ref_cursor;

    procedure upd ( p_op_uid     in int,
                    p_lock       in timestamp,
                    p_domain_id  in int,
                    p_domain_new in varchar,
                    p_limited    in numeric := 1,
                    p_comment    in varchar := null );


    procedure verify_and_log ( p_op_uid    in int,
                               p_domain_id in int );

    function verify_or_die ( p_domain_id in int     := null,
                             p_domain    in varchar := null ) return int;

    function handle (p_domain_id in int) return varchar;

end NetDB_Domain;
/

grant execute on NetDB_Domain to netdb_execute;
