-- NetDB_IPaddressSpace.sql --- functions related to IPaddressSpace table

create or replace package NetDB_IPaddressSpace as

    procedure move ( p_op_uid       in int,
                     p_lock         in timestamp,
                     p_ias_id       in int     := null,
                     p_prefix       in varchar := null,
                     p_network_id   in int     := null,
                     p_network_name in varchar := null,
                     p_net_lock     in timestamp );

    function is_MidRes (p_address in varchar) return boolean;
    function after_MidRes (p_address in varchar) return varchar;

    function containing (p_ip in varchar) return int;

    procedure validate_join ( p_prefix     in varchar := null,
                              p_network_id in int     := null,
                              p_network    in varchar := null );

    procedure validate_split (p_prefix in varchar);

    function get_parent_id (p_ias_id in int) return int;

    procedure locked_or_die ( p_op_uid in int,
                              p_lock   in timestamp,
                              p_ias_id in int );

    function info (p_ias_id in int) return NetDB.ref_cursor;
    function info_pipelined (p_ias_id in int) return result_set pipelined;

    function list (p_user_id in int := null) return NetDB.ref_cursor;
    function list_pipelined (p_user_id in int := null) return result_set pipelined;

    procedure del ( p_op_uid in int,
                    p_lock   in timestamp,
                    p_ias_id in int );

    function ins ( p_op_uid     in int,
                   p_lock       in timestamp,
                   p_prefix     in varchar,
                   p_network_id in int,
                   p_comment    in varchar,
                   p_lowres     in int,
                   p_highres    in int ) return NetDB.ref_cursor;

    function upd ( p_op_uid     in int,
                   p_lock       in timestamp,
                   p_ias_id     in int,
                   p_new_prefix in varchar,
                   p_network_id in int,
                   p_net_lock   in timestamp,
                   p_comment    in varchar,
                   p_lowres     in int,
                   p_highres    in int ) return NetDB.ref_cursor;

    function verify_or_die ( p_ias_id in int     := null,
                             p_prefix in varchar := null ) return int;

    function handle ( p_ias_id      in int,
                      user_readable in boolean := false) return varchar;

end NetDB_IPaddressSpace;
/

grant execute on NetDB_IPaddressSpace to netdb_execute;
