-- NetDB_Name.sql --- functions dealing with names (related to the Name table)

create or replace package NetDB_Name as

    function reserve ( p_op_uid in int,
                       p_name   in varchar,
                       p_domain in varchar,
                       p_type   in varchar ) return NetDB.ref_cursor;

    procedure accept ( p_op_uid        in int,
                       p_name_id       in int       := null,
                       p_fullname      in varchar   := null,
                       p_name_type     in varchar,
                       p_referent_id   in int       := null,
                       p_referent_name in varchar   := null,
                       p_lock          in timestamp,
                       p_mx_pref       in int       := null );

    procedure cancel ( p_op_uid   in int,
                       p_name_id  in int     := null,
                       p_fullname in varchar := null,
                       p_type     in varchar := null );

    procedure get_alias_terminal_parent ( p_name_id      in int,
                                          p_parent_id   out int,
                                          p_parent_type out varchar );

    function generate_fullname (p_name_id in int) return varchar;
    function fullname (p_name_id in int) return varchar;

    function related_node    (p_name_id in int) return int;
    function related_network (p_name_id in int) return int;

    function aliases ( p_name_id    in int,
                       p_hash_level in int ) return result_set pipelined;

    function names ( p_ip         in int := null,
                     p_interface  in int := null,
                     p_node       in int := null,
                     p_network    in int := null,
                     p_hash_level in int := 0 ) return result_set pipelined;

    procedure locked_or_die ( p_op_uid  in int,
                              p_lock    in timestamp,
                              p_name_id in int );

    procedure del ( p_op_uid   in int,
                    p_lock     in timestamp,
                    p_name_id  in int     := null,
                    p_fullname in varchar := null,
                    p_type     in varchar );

    procedure update_domain_name ( p_op_uid   in int,
                                   p_lock     in timestamp,
                                   p_name_id  in int,
                                   p_name_new in varchar );

    function verify_or_die ( p_name_id      in int     := null,
                             p_fullname     in varchar := null,
                             p_label        in varchar := '%',
                             check_reserved in boolean := false ) return int;

end NetDB_Name;
/

grant execute on NetDB_Name to netdb_execute;
