-- NetDB_Owner.sql --- Package specification for Owners

create or replace package NetDB_Owner as

    procedure add ( p_op_uid      in int,
                    p_lock        in timestamp,
                    p_owner_id    in int     := null,
                    p_name        in varchar := null,
                    p_record_id   in int,
                    p_record_type in varchar );

    procedure remove ( p_op_uid      in int,
                       p_lock        in timestamp,
                       p_owner_id    in int     := null,
                       p_name        in varchar := null,
                       p_record_id   in int,
                       p_record_type in varchar );

    function check_access ( p_user_id              in int,
                            p_record_id            in int,
                            p_record_type          in varchar,
                            p_check_all_group_flag in boolean := true,
                            p_check_active_state   in boolean := true ) return boolean;

    procedure has_access_or_die ( p_user_id              in int,
                                  p_record_id            in int,
                                  p_record_type          in varchar ,
                                  p_check_all_group_flag in boolean := true,
                                  p_check_active_state   in boolean := true );

    function is_member_of_group ( p_user_id  in int,
                                  p_owner_id in int ) return boolean;

    procedure locked_or_die ( p_op_uid   in int,
                              p_lock     in timestamp,
                              p_owner_id in int );

    function lock_record ( p_op_uid   in int,
                           p_owner_id in int ) return NetDB.ref_cursor;

    procedure unlock_record ( p_op_uid   in int,
                              p_lock     in timestamp,
                              p_owner_id in int );

    function list ( p_filter  in varchar := null,
                    p_user_id in int ) return NetDB.ref_cursor;

    function info (p_owner_id in int) return NetDB.ref_cursor;
    function info_pipelined (p_owner_id in int) return result_set pipelined;

    procedure del ( p_op_uid   in int,
                    p_lock     in timestamp ,
                    p_owner_id in int );

    function ins ( p_op_uid      in int,
                   p_name        in varchar,
                   p_description in varchar := null ) return NetDB.ref_cursor;

    procedure upd ( p_op_uid      in int,
                    p_lock        in timestamp,
                    p_owner_id    in int,
                    p_name_new    in varchar,
                    p_description in varchar );

    function verify_or_die ( p_owner_id in int     := null,
                             p_name     in varchar := null ) return int;

    function handle (p_owner_id in int) return varchar;

end NetDB_Owner;
/

grant execute on NetDB_Owner to netdb_execute;
