-- NetDB_User.sql --- contains functions related to users

create or replace package NetDB_User as

    procedure update_prefix;
    procedure update_prefix ( old_prefix in varchar,
                              new_prefix in varchar := null );

    function is_active (p_user_id in int) return boolean;
    function has_all_groups (p_user_id in int) return boolean;
    function has_all_records (p_user_id in int) return boolean;
    function def_domain (p_user_id in int) return int;

    procedure upd_profile ( p_op_uid      in int,
                            p_def_group   in varchar,
                            p_def_domain  in varchar,
                            p_def_prefix  in varchar := null );

    procedure org ( p_action   in varchar,
                    p_op_uid   in int,
                    p_lock     in timestamp,
                    p_user_id  in int,
                    p_org_id   in int,
                    p_official in int := 0 );

    procedure users_access ( p_action      in varchar,
                             p_op_uid      in int,
                             p_lock        in timestamp,
                             p_user_id     in int,
                             p_access_id   in int     := null,
                             p_access_name in varchar := null);

    function login ( p_user_id in int     := null,
                     p_netid   in varchar := null ) return NetDB.ref_cursor;

    procedure locked_or_die ( p_op_uid  in int,
                              p_lock    in timestamp,
                              p_user_id in int );

    function lock_record ( p_op_uid  in int,
                           p_user_id in int ) return NetDB.ref_cursor;

    procedure unlock_record ( p_op_uid  in int,
                              p_lock    in timestamp,
                              p_user_id in int );

    function info (p_user_id in int) return NetDB.ref_cursor;
    function info_pipelined (p_user_id in int) return result_set pipelined;

    procedure del ( p_op_uid  in int,
                    p_lock    in timestamp,
                    p_user_id in int );

    function ins ( p_op_uid      in int,
                   p_person_id   in varchar,
                   p_def_group   in varchar,
                   p_def_domain  in varchar,
                   p_def_prefix  in varchar,
                   p_active      in int,
                   p_all_records in int     := 0,
                   p_all_groups  in int     := 0,
                   p_comment     in varchar := null ) return NetDB.ref_cursor;

    procedure upd ( p_op_uid      in int,
                    p_lock        in timestamp,
                    p_user_id     in int,
                    p_def_group   in varchar,
                    p_def_domain  in varchar,
                    p_def_prefix  in varchar,
                    p_active      in int,
                    p_all_records in int     := 0,
                    p_all_groups  in int     := 0,
                    p_comment     in varchar := null );

    procedure verify_and_log ( p_op_uid  in int,
                               p_user_id in int );

    function verify_or_die ( p_user_id in int     := null,
                             p_netid   in varchar := null ) return int;

    function handle (p_user_id in int) return varchar;

end NetDB_User;
/

grant execute on NetDB_User to netdb_execute;
