set serveroutput on size 100000;
savepoint start_DHCPservice_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- values for insert and update.
    test_service   DHCPservice.label%TYPE := 'unlikely DHCPservice name - test';
    test_service_2 DHCPservice.label%TYPE := 'test - unlikely DHCPservice name';
    test_desc      DHCPservice.description%TYPE := 'a description';

    test_id     int := null;
    verified_id int := null;

    existing_service DHCPservice%ROWTYPE;

    unrecoverable_error EXCEPTION;
begin
    select count (*) into v_count from DHCPservice where label in (test_service, test_service_2);

    if (v_count > 0) then
        err_msg := 'The test DHCP service "' || test_service || '" exists. If it is not a valid ' ||
                   'DHCP service, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    select * into existing_service from DHCPservice where rownum = 1;
    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'DHCPservice');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a DHCP service without permission';

        test_cursor := NetDB_DHCPservice.ins (NetDB_Test.user_1.id, test_service, test_desc);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a DHCP service without permission.';

        NetDB_DHCPservice.upd (NetDB_Test.user_1.id, existing_service.id, test_service_2, test_desc);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a DHCP service without permission.';

        NetDB_DHCPservice.del (NetDB_Test.user_1.id, existing_service.id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'DHCPservice');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a DHCP service';

        test_cursor := NetDB_DHCPservice.ins (NetDB_Test.user_1.id, test_service, test_desc);
        result  := NetDB_Test.get_generic_row (test_cursor);
        test_id := result.value_string;
        NetDB_Test.empty_cursor (test_cursor);

        verified_id := NetDB_DHCPservice.verify_or_die (test_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a DHCP service.';

        NetDB_DHCPservice.upd (NetDB_Test.user_1.id, test_id, test_service_2, test_desc);

        verified_id := NetDB_DHCPservice.verify_or_die (test_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a DHCP service.';

        NetDB_DHCPservice.del (NetDB_Test.user_1.id, test_id);

        verified_id := NetDB_DHCPservice.verify_or_die (test_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_DHCPservice_tests;
