set serveroutput on size 100000;
savepoint start_mx_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;
    v_lock      timestamp    := null;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- MX info for insert and update.
    test_MX       varchar(256)       := 'test-mx-name';
    test_domain   Name.name%TYPE     := NetDB_Test.domain_1_name;
    test_fullname Name.fullname%TYPE := NetDB_Util.fullname (test_MX, test_domain);
    test_id       int;
    test_name_id  int;
    test_reserve  timestamp;

    ip_name       Name%ROWTYPE;
    ipaddress_row IPaddress%ROWTYPE;
    referent_id   int;

    v_os          OS.os%TYPE;
    loc           Location.name%TYPE;
    v_state       State.state%TYPE;
    v_make        Model.make%TYPE;
    v_model       Model.modelname%TYPE;

    unrecoverable_error EXCEPTION;
begin
    select os              into v_os            from OS       where rownum = 1;
    select name            into loc             from Location where rownum = 1;
    select state           into v_state         from State    where rownum = 1;
    select make, modelname into v_make, v_model from Model    where rownum = 1;

    NetDB_Test.reset_address (NetDB_Test.name_intfc_1.interface, 'interface');
    ipaddress_row := NetDB_Test.get_next_address (NetDB_Test.name_intfc_1.interface, 'interface');

    begin
        select * into ip_name from Name where ipaddress = ipaddress_row.id;
    exception
        when NO_DATA_FOUND then
            ipaddress_row := NetDB_Test.get_next_address (NetDB_Test.name_intfc_1.interface, 'interface');

            select * into ip_name from Name where ipaddress = ipaddress_row.id;
    end;

    select count (*) into v_count from Name where lower (name) = lower (test_fullname);

    if (v_count > 0) then
        err_msg := 'The test name "' || test_fullname || '" exists. If it is not a valid ' ||
                   'name, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Network');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Node');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Router');
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdvancedNode');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at an alias (tests MX insertion).';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', NetDB_Test.name_alias_1.id, null, systimestamp, 10);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.bad_MX_target then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at a domain (tests MX insertion).';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', NetDB_Test.domain_1.id, null, systimestamp, 10);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.bad_MX_target then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at a interface (tests MX insertion).';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', NetDB_Test.name_intfc_1.id, null, v_lock, 10);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at an address (tests MX insertion).';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        if (ipaddress_row.ipcnode is not null) then
            referent_id := ipaddress_row.ipcnode;
            test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, referent_id);
        elsif (ipaddress_row.interface is not null) then
            referent_id := NetDB_Interface.get_parent_id (ipaddress_row.interface);
            test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, referent_id);
        elsif (ipaddress_row.ipaddressspace is not null) then
            referent_id := NetDB_IPaddressSpace.get_parent_id (ipaddress_row.ipaddressspace);
            test_cursor := NetDB_Network.lock_record (NetDB_Test.user_1.id, referent_id);
        end if;

        result := NetDB_Test.get_generic_row (test_cursor);
        v_lock := result.value_timestamp;

        test_cursor := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', ip_name.id, null, v_lock, 10);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at a network (tests MX insertion).';

        test_cursor := NetDB_Network.lock_record (NetDB_Test.user_1.id, NetDB_Test.network_1.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', NetDB_Test.name_net_1.id, null, v_lock, 10);
        NetDB_Network.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.network_1.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Reserve a name as an MX pointed at a node (tests MX insertion).';

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_MX, NetDB_Test.domain_1.domain, 'MX');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_id, null, 'MX', NetDB_Test.name_node_3.id, null, v_lock, 10);
        NetDB_Node.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an MX.';

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_MXrecord.upd (NetDB_Test.user_1.id, v_lock, null, test_fullname, NetDB_Test.name_node_3.id, null, 20);
        NetDB_Node.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an MX.';

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_MXrecord.del (NetDB_Test.user_1.id, v_lock, null, test_fullname, NetDB_Test.name_node_3.id);
        NetDB_Node.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an MX using an MX that shares the name with an "A" name record.';

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_MXrecord.ins (NetDB_Test.user_1.id, v_lock, null, test_fullname, NetDB_Test.name_node_3.id, null, 10);
        NetDB_Node.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_mx_tests;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an MX using an MX that shares the name with a node and a domain.';

        -- Create a new domain.
        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_0.id, test_MX, test_domain, 'Domain');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_id      := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Domain.ins (NetDB_Test.user_0.id, test_fullname, 1);
        result      := NetDB_Test.get_generic_row (test_cursor); -- we already have the ID.
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_0.id, test_id, null, 'Domain', test_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_0.id, v_lock, NetDB_Test.group_2.id, null, test_id, 'Domain');
        NetDB_Owner.add (NetDB_Test.user_0.id, v_lock, NetDB_Test.group_2.id, null, test_id, 'domain_use');
        NetDB_Domain.verify_and_log (NetDB_Test.user_0.id, test_id);

        -- Create a new node.
        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_0.id, test_MX, test_domain, 'Node');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_name_id := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_Node.ins (NetDB_Test.user_0.id, NetDB_Test.dept_1_name, loc, 'room', v_state, v_make, v_model);
        result  := NetDB_Test.get_generic_row (test_cursor);
        test_id := result.value_string;
        result  := NetDB_Test.get_generic_row (test_cursor);
        v_lock  := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_0.id, test_name_id, null, 'Node', test_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_0.id, v_lock, NetDB_Test.group_2.id, null, test_id, 'Node');
        NetDB_Node.admin ('add', NetDB_Test.user_0.id, v_lock, test_id, null, null, null, NetDB_Test.user_0.id);
        NetDB_Node.OS ('add', NetDB_Test.user_0.id, v_lock, test_id, null, v_os, null);
        NetDB_Node.verify_and_log (NetDB_Test.user_0.id, test_id);

        -- Create the MX.
        test_cursor := NetDB_Name.reserve (NetDB_Test.user_0.id, test_MX, test_domain, 'MX');
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_0.id, null, test_fullname, 'MX', null, test_fullname, v_lock, 10);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an MX that shares the name with a node and a domain.';

         NetDB_MXrecord.del (NetDB_Test.user_0.id, v_lock, null, test_fullname, null, test_fullname);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_mx_tests;
