set serveroutput on size 100000;
savepoint start_node_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_count     int          := 0;
    v_reserved  timestamp    := null;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- Values for insert and update.
    test_node      Name.name%TYPE     := 'test-node-name';
    test_node_full Name.fullname%TYPE;
    test_node_id   int;
    test_name_id   int;

    room          Node.room%TYPE     := 'the room';
    a_comment     Node.comments%TYPE := 'a comment';
    a_new_comment Node.comments%TYPE := 'a new comment';
    loc           Location.name%TYPE;
    v_state       State.state%TYPE;
    v_make        Model.make%TYPE;
    v_model       Model.modelname%TYPE;
    v_os          OS.os%TYPE;
    v_os_2        OS.os%TYPE;

    ipc_name      Name.name%TYPE           := 'IPC-address-name';
    valid_addr    IPaddress.ipaddress%TYPE := '191.64.20.18';
    ip            IPaddress.ipaddress%TYPE;
    reserve       timestamp;

    v_service     DHCPservice.label%TYPE         := 'campus';
    v_addr_space  IPaddressSpace.prefix_txt%TYPE := '191.64.20.0/24';

    tag_key       NodeTag.label%TYPE := 'tag key';
    tag_value     NodeTag.value%TYPE := 'tag value';

    unrecoverable_error EXCEPTION;
begin
    select count (*) into v_count from Name
     where lower (name) like lower (test_node) or lower (name) like lower (ipc_name);

    if (v_count > 0) then
        err_msg := 'The test name "' || test_node || '" or "' || ipc_name || '" exists. If they ' ||
                   'are not valid names, remove the entries and run the tests again. ' ||
                   'If they are valid, the tests must be modified.';
        raise unrecoverable_error;
    end if;

    select name            into loc             from Location where rownum = 1;
    select state           into v_state         from State    where rownum = 1;
    select make, modelname into v_make, v_model from Model    where rownum = 1;
    select os              into v_os            from OS       where rownum = 1;
    select os              into v_os_2          from OS       where rownum = 1 and os != v_os;

    NetDB_Test.test_info ('Node', test_number);
    NetDB_Test.test_locking ('Node', test_number);

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [2].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_2.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [3].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_3.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [4].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_4.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [5].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_5.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [6].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_6.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get info on a record with valid arguments [7].';

        test_cursor := NetDB_Node.info (NetDB_Test.node_7.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'Node');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a node without permissions.';

        test_cursor := NetDB_Node.ins (NetDB_Test.user_1.id, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_comment, null, v_service, v_addr_space);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a node without record type permissions.';

        NetDB_Node.upd (NetDB_Test.user_1.id, systimestamp, NetDB_Test.node_1.id, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_comment, null, v_service, v_addr_space);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a node without record type permissions.';

        NetDB_Node.del (NetDB_Test.user_1.id, systimestamp, NetDB_Test.node_1.id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Node');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a node without record permissions.';

        NetDB_Node.upd (NetDB_Test.user_1.id, systimestamp, NetDB_Test.node_2.id, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_comment, null, v_service, v_addr_space);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a node without record permissions.';

        NetDB_Node.del (NetDB_Test.user_1.id, systimestamp, NetDB_Test.node_2.id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to lock an advanced node without access to all sub-types.';

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_1.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Attempt to lock an advanced node with access to all sub-types.';

        NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdvancedNode');
        NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Router');

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_1.id);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a node.';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, test_node, NetDB_Test.domain_1.domain, 'Node');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_name_id := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        v_reserved   := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        select fullname into test_node_full from Name where name = test_node;

        test_cursor  := NetDB_Node.ins (NetDB_Test.user_1.id, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_comment, null, v_service, v_addr_space);
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_node_id := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        v_lock       := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_name_id, null, 'Node', test_node_id, null, v_lock);
        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, test_node_id, 'Node');
        NetDB_Node.admin ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, null, null, NetDB_Test.user_1.id);
        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, v_os, null);

        NetDB_Node.verify_and_log (NetDB_Test.user_1.id, test_node_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a Node a template.';

        NetDB_Node.type ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, 'Template');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

     begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a template node, a normal node.';

        NetDB_Node.type ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, 'Template');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a Node an Advanced Node without the correct access groups.';

        NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'AdvancedNode');
        NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'Router');
        NetDB_Node.type ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, 'AdvancedNode');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a Node an Advanced Node.';

        NetDB_Test.grant_access (NetDB_Test.user_1.id, 'AdvancedNode');
        NetDB_Node.type ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, 'Advanced');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a node.';

        NetDB_Node.upd (NetDB_Test.user_1.id, v_lock, test_node_id, NetDB_Test.dept_1_name, loc, room, v_state, v_make, v_model, a_new_comment, null, v_service, v_addr_space);
        NetDB_Node.verify_and_log (NetDB_Test.user_1.id, test_node_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an OS to a node.';

        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, v_os_2, null);

        select count (*) into v_count from Node_OS where node = test_node_id and os = v_os_2;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an OS to a node.';

        NetDB_Node.OS ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, v_os_2, null);

        select count (*) into v_count from Node_OS where node = test_node_id and os = v_os_2;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an Admin (person) to a node.';

        NetDB_Node.admin ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, null, null, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_Admin where node = test_node_id and person = NetDB_Test.person_2.id;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an Admin (person) to a node.';

        NetDB_Node.admin ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, null, null, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_Admin where node = test_node_id and person = NetDB_Test.person_2.id;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an Admin (team) to a node.';

        NetDB_Node.admin ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.ateam_1.id, NetDB_Test.ateam_1.name);

        select count (*) into v_count from Node_Admin where node = test_node_id and adminteam = NetDB_Test.ateam_1.id;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an Admin (team) to a node.';

        NetDB_Node.admin ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.ateam_1.id, NetDB_Test.ateam_1.name);

        select count (*) into v_count from Node_Admin where node = test_node_id and adminteam = NetDB_Test.ateam_1.id;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add an Admin (person and team) to a node.';

        NetDB_Node.admin ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.ateam_1.id, NetDB_Test.ateam_1.name, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_Admin
         where node = test_node_id and (person = NetDB_Test.person_2.id or adminteam = NetDB_Test.ateam_1.id);

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove an Admin (person and team) to a node.';

        NetDB_Node.admin ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.ateam_1.id, NetDB_Test.ateam_1.name, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_Admin
         where node = test_node_id and (person = NetDB_Test.person_2.id or adminteam = NetDB_Test.ateam_1.id);

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add a user to a node.';

        NetDB_Node.user ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_User where node = test_node_id and person = NetDB_Test.person_2.id;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove a user from a node.';

        NetDB_Node.user ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, NetDB_Test.person_2.id, NetDB_Test.person_2.person_regid);

        select count (*) into v_count from Node_User where node = test_node_id and person = NetDB_Test.person_2.id;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add a tag to a node.';

        NetDB_Node.tag ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, tag_key, tag_value);

        select count (*) into v_count from NodeTag where node = test_node_id and label = tag_key;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Add a tag, which already exists, to a node.';

        NetDB_Node.tag ('add', NetDB_Test.user_1.id, v_lock, test_node_id, null, tag_key, tag_value);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_already_exists then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Remove a tag from a node.';

        NetDB_Node.tag ('remove', NetDB_Test.user_1.id, v_lock, test_node_id, null, tag_key, tag_value);

        select count (*) into v_count from NodeTag where node = test_node_id and label = tag_key;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert some IPC addresses without a name.';

        test_cursor := NetDB_IPaddress.reserve (NetDB_Test.user_1.id, valid_addr, 1);
        result  := NetDB_Test.get_generic_row (test_cursor); -- useless row

        result  := NetDB_Test.get_generic_row (test_cursor);
        ip      := result.key;
        reserve := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_IPaddress.accept (NetDB_Test.user_1.id, ip, 1, 0, test_node_id, v_lock, 'node');
        NetDB_Node.verify_and_log (NetDB_Test.user_1.id, test_node_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_requires_name then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Name the IPC address.';

        test_cursor  := NetDB_Name.reserve (NetDB_Test.user_1.id, ipc_name, NetDB_Test.domain_1.domain, 'Address');
        result       := NetDB_Test.get_generic_row (test_cursor);
        test_name_id := result.value_string;
        result       := NetDB_Test.get_generic_row (test_cursor);
        v_reserved   := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Name.accept (NetDB_Test.user_1.id, test_name_id, null, 'address', null, ip, v_lock);
        NetDB_Node.verify_and_log (NetDB_Test.user_1.id, test_node_id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a node.';

        NetDB_Node.del (NetDB_Test.user_1.id, v_lock, test_node_id);
        test_node_id := NetDB_Node.verify_or_die (test_node_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_node_tests;
