set serveroutput on size 100000;
savepoint start_os_test;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_count     int          := 0;
    v_count_2   int          := 0;
    v_lock      timestamp    := null;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- Values for insert/update.
    test_os     OS.os%TYPE          := 'test OS that is unique';
    test_os_new OS.os%TYPE          := 'new test OS that is unique';
    test_desc   OS.description%TYPE := 'the test os';
begin
    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Get the list of all OSes.';

        test_cursor := NetDB_OS.list ();
        NetDB_Test.empty_cursor (test_cursor);

        if (NetDB_Test.rows_skipped > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'List one OS.';

        test_cursor := NetDB_OS.list ('IOS');
        NetDB_Test.empty_cursor (test_cursor);

        if (NetDB_Test.rows_skipped = 1) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'OS');
    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'Node');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an OS without sufficient privileges.';

        NetDB_OS.ins (NetDB_Test.user_1.id, test_os);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an OS without sufficient privileges.';

        NetDB_OS.upd (NetDB_Test.user_1.id, 'IOS', 'IOS new');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an OS without sufficient privileges.';

        NetDB_OS.del (NetDB_Test.user_1.id, 'IOS');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'Node');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an OS with node access.';

        NetDB_OS.ins (NetDB_Test.user_1.id, test_os);

        select count (*) into v_count from OS where os = test_os;

        if (v_count = 1) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an OS with node access.';

        NetDB_OS.upd (NetDB_Test.user_1.id, 'IOS', 'IOS new');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an OS with node access.';

        NetDB_OS.del (NetDB_Test.user_1.id, 'IOS');

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_os_test;
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'OS');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an OS.';

        NetDB_OS.ins (NetDB_Test.user_1.id, test_os);

        select count (*) into v_count from OS where os = test_os;

        if (v_count = 1) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an OS.';

        NetDB_OS.upd (NetDB_Test.user_1.id, test_os, test_os_new, test_desc);

        select count (*) into v_count   from OS where os = test_os;
        select count (*) into v_count_2 from OS where os = test_os_new;

        if (v_count = 0 and v_count_2 = 1) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an OS.';

        NetDB_OS.del (NetDB_Test.user_1.id, test_os_new);

        select count (*) into v_count from OS where os = test_os_new;

        if (v_count = 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an OS to an existing OS.';

        NetDB_OS.ins (NetDB_Test.user_1.id, test_os);
        NetDB_OS.ins (NetDB_Test.user_1.id, test_os_new);

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id, null, test_os, null);
        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id, null, test_os_new, null);

        NetDB_OS.upd (NetDB_Test.user_1.id, test_os, test_os_new);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_already_exists then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    rollback to start_os_test;
    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'OS');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Replace an OS with a new OS, when both OSes exist on the same node.';

        NetDB_OS.ins (NetDB_Test.user_1.id, test_os);
        NetDB_OS.ins (NetDB_Test.user_1.id, test_os_new);

        test_cursor := NetDB_Node.lock_record (NetDB_Test.user_1.id, NetDB_Test.node_3.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id, null, test_os, null);
        NetDB_Node.OS ('add', NetDB_Test.user_1.id, v_lock, NetDB_Test.node_3.id, null, test_os_new, null);

        NetDB_OS.supplant (NetDB_Test.user_1.id, test_os, test_os_new);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

exception
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_os_test;
