set serveroutput on size 100000;
savepoint start_owner_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_count     int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- values for insert/update tests.
    test_owner     Owner.label%TYPE       := 'unused group name';
    test_owner_new Owner.label%TYPE       := 'unused different group name';
    test_desc      Owner.description%TYPE := 'the description';

    test_id     int := 0;

    shared_mx_id int := 0;
    unique_mx_id int := 0;
    reverse_name Name.name%TYPE;

    access boolean;

    unrecoverable_error EXCEPTION;

    function empty_owner_cursor (cursor_to_empty in NetDB.ref_cursor) return int
    as
        skipped_rows int := 0;
        rec_id       int;
        owner_id     int;
    begin
        loop
            fetch cursor_to_empty into rec_id, owner_id;
            exit when cursor_to_empty%NOTFOUND;

            skipped_rows := skipped_rows + 1;
        end loop;
        close cursor_to_empty;

        return skipped_rows;
    end empty_owner_cursor;
begin
    select count (*) into v_count from Owner where label = test_owner;
    if (v_count > 0) then
        err_msg := 'The test group "' || test_owner || '" exists. If it is not a valid ' ||
                   'group, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    select count (*) into v_count from Owner where label = test_owner_new;
    if (v_count > 0) then
        err_msg := 'The test group "' || test_owner_new || '" exists. If it is not a valid ' ||
                   'group, remove the entry and run the tests again. If it is valid, the ' ||
                   'tests must be modified.';
        raise unrecoverable_error;
    end if;

    v_lock := NetDB_Test.force_lock (NetDB_Test.user_0.id, NetDB_Test.node_3.id, 'node');
    select reverse (NetDB_Test.name_node_3.name) into reverse_name from dual;

    -- Create an MX that shares a name with a node.
    test_cursor  := NetDB_Name.reserve (NetDB_Test.user_0.id, NetDB_Test.name_node_3.name, NetDB_Domain.handle (NetDB_Test.name_node_3.domain), 'MX');
    result       := NetDB_Test.get_generic_row (test_cursor);
    shared_mx_id := result.value_string;
    NetDB_Test.empty_cursor (test_cursor);

    NetDB_Name.accept (NetDB_Test.user_0.id, shared_mx_id, null, 'MX', NetDB_Test.name_node_3.id, null, v_lock, 10);

    -- Create a unique MX.
    test_cursor  := NetDB_Name.reserve (NetDB_Test.user_0.id, reverse_name, NetDB_Domain.handle (NetDB_Test.name_node_3.domain), 'MX');
    result       := NetDB_Test.get_generic_row (test_cursor);
    unique_mx_id := result.value_string;
    NetDB_Test.empty_cursor (test_cursor);

    NetDB_Name.accept (NetDB_Test.user_0.id, unique_mx_id, null, 'MX', NetDB_Test.name_node_3.id, null, v_lock, 10);

    for r in ( select NetDB_Test.user_1.id user_id, NetDB_Test.ateam_1.id       record_id, 'adminteam'    record_type, 'adminteam[1.1]'        test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.ateam_1.id       record_id, 'adminteam'    record_type, 'adminteam[2.1]'        test_name, 'fail' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_1.id      record_id, 'domain'       record_type, 'Domain[1.1]'           test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_2.id      record_id, 'domain'       record_type, 'Domain[1.2]'           test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.domain_1.id      record_id, 'domain'       record_type, 'Domain[2.1]'           test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.domain_2.id      record_id, 'domain'       record_type, 'Domain[2.2]'           test_name, 'pass' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.net_1_space_1.id record_id, 'addressspace' record_type, 'IP Address Space[1.1]' test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.net_1_space_2.id record_id, 'addressspace' record_type, 'IP Address Space[1.2]' test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.net_2_space_1.id record_id, 'addressspace' record_type, 'IP Address Space[1.3]' test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.net_2_space_2.id record_id, 'addressspace' record_type, 'IP Address Space[1.4]' test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.net_1_space_1.id record_id, 'addressspace' record_type, 'IP Address Space[2.1]' test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.net_1_space_2.id record_id, 'addressspace' record_type, 'IP Address Space[2.2]' test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.net_2_space_1.id record_id, 'addressspace' record_type, 'IP Address Space[2.3]' test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.net_2_space_2.id record_id, 'addressspace' record_type, 'IP Address Space[2.4]' test_name, 'pass' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.name_mx_1.id     record_id, 'mx'           record_type, 'MX[1.1]'               test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, shared_mx_id                record_id, 'mx'           record_type, 'MX[1.2]'               test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, unique_mx_id                record_id, 'mx'           record_type, 'MX[1.3]'               test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.name_mx_1.id     record_id, 'mx'           record_type, 'MX[2.1]'               test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, shared_mx_id                record_id, 'mx'           record_type, 'MX[2.2]'               test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, unique_mx_id                record_id, 'mx'           record_type, 'MX[2.3]'               test_name, 'fail' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.network_1.id     record_id, 'network'      record_type, 'Network[1.1]'          test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.network_2.id     record_id, 'network'      record_type, 'Network[1.2]'          test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.network_1.id     record_id, 'network'      record_type, 'Network[2.1]'          test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.network_2.id     record_id, 'network'      record_type, 'Network[2.2]'          test_name, 'pass' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.node_1.id        record_id, 'node'         record_type, 'Node[1.1]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_2.id        record_id, 'node'         record_type, 'Node[1.2]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_3.id        record_id, 'node'         record_type, 'Node[1.3]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_4.id        record_id, 'node'         record_type, 'Node[1.4]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_5.id        record_id, 'node'         record_type, 'Node[1.5]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_6.id        record_id, 'node'         record_type, 'Node[1.6]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.node_7.id        record_id, 'node'         record_type, 'Node[1.7]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_1.id        record_id, 'node'         record_type, 'Node[2.1]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_2.id        record_id, 'node'         record_type, 'Node[2.2]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_3.id        record_id, 'node'         record_type, 'Node[2.3]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_4.id        record_id, 'node'         record_type, 'Node[2.4]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_5.id        record_id, 'node'         record_type, 'Node[2.5]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_6.id        record_id, 'node'         record_type, 'Node[2.6]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.node_7.id        record_id, 'node'         record_type, 'Node[2.7]'             test_name, 'pass' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_2.id      record_id, 'resname'      record_type, 'ResName[1.1]'          test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_2.id      record_id, 'resname'      record_type, 'ResName[1.2]'          test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_2.id      record_id, 'resname'      record_type, 'ResName[1.3]'          test_name, 'fail' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.domain_2.id      record_id, 'resname'      record_type, 'ResName[1.4]'          test_name, 'fail' expected from dual union

               select NetDB_Test.user_1.id user_id, NetDB_Test.user_1.id        record_id, 'user'         record_type, 'User[1.1]'             test_name, 'pass' expected from dual union
               select NetDB_Test.user_1.id user_id, NetDB_Test.user_2.id        record_id, 'user'         record_type, 'User[1.2]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.user_1.id        record_id, 'user'         record_type, 'User[2.1]'             test_name, 'fail' expected from dual union
               select NetDB_Test.user_2.id user_id, NetDB_Test.user_2.id        record_id, 'user'         record_type, 'User[2.2]'             test_name, 'pass' expected from dual
                order by test_name, user_id, record_id )
    loop
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Access check: ' || r.test_name;

        if (lower (r.expected) = 'pass') then
            begin
                NetDB_Owner.has_access_or_die (r.user_id, r.record_id, r.record_type, false);
                NetDB_Test.pass ();
            exception
                when others then
                    NetDB_Test.failed (err_msg, sqlerrm);
            end;
        else
            begin
                NetDB_Owner.has_access_or_die (r.user_id, r.record_id, r.record_type, false);
                NetDB_Test.failed (err_msg);
            exception
                when NetDB.no_access_to_record then
                    NetDB_Test.pass ();
                when others then
                    NetDB_Test.failed (err_msg, sqlerrm);
            end;
        end if;
    end loop;

    NetDB_Test.test_info ('owner', test_number);
    NetDB_Test.test_locking ('owner', test_number);

    NetDB_Test.revoke_access (NetDB_Test.user_1.id, 'owner');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an owner without privileges.';

        test_cursor := NetDB_Owner.ins (NetDB_Test.user_1.id, test_owner, test_desc);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an owner without privileges.';

        NetDB_Owner.upd (NetDB_Test.user_1.id, null, NetDB_Test.group_1.id, null, test_desc);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an owner without privileges.';

        NetDB_Owner.del (NetDB_Test.user_1.id, null, NetDB_Test.group_1.id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'owner');

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an owner.';

        test_cursor := NetDB_Owner.ins (NetDB_Test.user_1.id, test_owner, test_desc);
        result      := NetDB_Test.get_generic_row (test_cursor);
        test_id     := result.value_string;
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update an owner.';

        NetDB_Owner.upd (NetDB_Test.user_1.id, v_lock, test_id, test_owner_new, test_desc);
        test_id := NetDB_Owner.verify_or_die (null, test_owner_new);

        NetDB_Owner.upd (NetDB_Test.user_1.id, v_lock, test_id, test_owner, test_desc);
        test_id := NetDB_Owner.verify_or_die (null, test_owner);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an owner.';

        NetDB_Owner.del (NetDB_Test.user_1.id, v_lock, test_id);
        test_id := NetDB_Owner.verify_or_die (null, test_owner);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    for r in ( select NetDB_Test.group_1.id group_id, NetDB_Test.ateam_1.id       record_id, 'adminteam'    record_type from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.domain_1.id      record_id, 'domain'       record_type from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.net_1_space_1.id record_id, 'addressspace' record_type from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.network_1.id     record_id, 'network'      record_type from dual union
               select NetDB_Test.group_1.id group_id, NetDB_Test.node_2.id        record_id, 'node'         record_type from dual union
               select NetDB_Test.group_1.id group_id, NetDB_Test.res_name_1.id    record_id, 'resname'      record_type from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.user_1.id        record_id, 'user'         record_type from dual )
    loop
        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Add a group to a[n] ' || r.record_type || '.';

            v_lock := NetDB_Test.force_lock (NetDB_Test.user_0.id, r.record_id, r.record_type);

            NetDB_Owner.add (NetDB_Test.user_0.id, v_lock, r.group_id, null, r.record_id, r.record_type);

            if (r.record_type != 'user') then
                access := NetDB_Owner.check_access (NetDB_Test.user_1.id, r.record_id, r.record_type, false);
            else
                access := NetDB_Owner.is_member_of_group (r.record_id, r.group_id);
            end if;

            if (access) then
                NetDB_Test.pass ();
            else
                NetDB_Test.failed (err_msg);
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;
    end loop;

    for r in ( select NetDB_Test.group_1.id group_id, NetDB_Test.ateam_1.id       record_id, 'adminteam'    record_type, 'adminteam_owner'      table_name, 'adminteam'      column_name from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.domain_1.id      record_id, 'domain'       record_type, 'domain_owner'         table_name, 'domain'         column_name from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.net_1_space_1.id record_id, 'addressspace' record_type, 'ipaddressspace_owner' table_name, 'ipaddressspace' column_name from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.network_1.id     record_id, 'network'      record_type, 'network_owner'        table_name, 'network'        column_name from dual union
               select NetDB_Test.group_1.id group_id, NetDB_Test.node_2.id        record_id, 'node'         record_type, 'node_owner'           table_name, 'node'           column_name from dual union
               select NetDB_Test.group_1.id group_id, NetDB_Test.res_name_1.id    record_id, 'resname'      record_type, 'resname_owner'        table_name, 'resname'        column_name from dual union
               select NetDB_Test.group_2.id group_id, NetDB_Test.user_1.id        record_id, 'user'         record_type, 'users_owner'          table_name, 'userid'         column_name from dual )
    loop
        begin
            NetDB_Test.output_and_increment_test (test_number);
            err_msg := 'Remove a group from a[n] ' || r.record_type || '.';

            v_lock := NetDB_Test.force_lock (NetDB_Test.user_0.id, r.record_id, r.record_type);

            NetDB_Owner.remove (NetDB_Test.user_0.id, v_lock, r.group_id, null, r.record_id, r.record_type);

            open test_cursor for 'select * from ' || r.table_name ||
                                 ' where ' || r.column_name || ' = ' || r.record_id || ' and owner = ' || r.group_id;

            if (empty_owner_cursor (test_cursor) > 0) then
                NetDB_Test.failed (err_msg);
            else
                NetDB_Test.pass ();
            end if;
        exception
            when others then
                NetDB_Test.failed (err_msg, sqlerrm);
        end;
    end loop;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_owner_tests;
