create or replace package NetDB_Test as

-- Begin test data key definitions.
    -- User 0 must have "all_groups", "all_records" and "system management" access
    user_0_netid   Users.netid%TYPE := 'willbarber';
    user_1_netid   Users.netid%TYPE := 'hgcogswell';
    user_2_netid   Users.netid%TYPE := 'astrodoggy';

    ateam_1_name   AdminTeam.name%TYPE := 'Spacely''s Nuts';

    domain_1_name  Domain.domain%TYPE := 'Cogswell-Cogs.COM';
    domain_2_name  Domain.domain%TYPE := 'Spacely-Sprockets.COM';

    dept_1_name    Organization.organization%TYPE := 'Gears, Cogs, and Sprockets';
    dept_2_name    Organization.organization%TYPE := 'Gears, Sprockets, and Cogs';

    group_1_name   Owner.label%TYPE := 'Cogswell Cogs';
    group_2_name   Owner.label%TYPE := 'Spacely Sprokets';

    network_1_name Name.name%TYPE := 'cogswell-cogs-net.Cogswell-Cogs.COM';
    network_2_name Name.name%TYPE := 'spacely-sprockets-net.Spacely-Sprockets.COM';
    net_1_prefix_1 IPaddressSpace.prefix_txt%TYPE := '191.64.20.0/24';
    net_1_prefix_2 IPaddressSpace.prefix_txt%TYPE := '10.254.20.0/24';
    net_2_prefix_1 IPaddressSpace.prefix_txt%TYPE := '191.64.132.0/23';
    net_2_prefix_2 IPaddressSpace.prefix_txt%TYPE := '10.254.132.0/24';

    node_1_name    Name.name%TYPE := 'astro-gw.Cogswell-Cogs.COM';
    node_2_name    Name.name%TYPE := 'spacely-template.Spacely-Sprockets.COM';
    node_3_name    Name.name%TYPE := 'pc-cogswell.Cogswell-Cogs.COM';
    node_4_name    Name.name%TYPE := 'mac-spacely.Spacely-Sprockets.COM';
    node_5_name    Name.name%TYPE := 'spacely-printer.Spacely-Sprockets.COM';
    node_6_name    Name.name%TYPE := 'spacewell-gears.Spacely-Sprockets.COM';
    node_7_name    Name.name%TYPE := 'jetson-mbdsl.Spacely-Sprockets.COM';

    -- Interface name for astro-gw.Cogswell-Cogs.COM
    interface_1_name Name.name%TYPE := 'astro-gw-1.Cogswell-Cogs.COM';

    -- Alias for 'mac-spacely.Spacely-Sprockets.COM'
    alias_1_name   Name.name%TYPE := 'WWW.Spacely-Sprockets.COM';

    -- Mail sent to CogswellCogs.COM is received by pc-cogswell.Cogswell-Cogs.COM
    mx_1_name      Name.name%TYPE := 'Cogswell-Cogs.COM';

    -- Restricted name
    restricted_name ResName.name%TYPE := '^phb$';
-- End keys

-- Begin raw test data variables from the database
    user_0        Users%ROWTYPE;
    user_1        Users%ROWTYPE;
    user_2        Users%ROWTYPE;

    ateam_1       AdminTeam%ROWTYPE;

    person_1      Person%ROWTYPE;
    person_2      Person%ROWTYPE;

    name_domain_1 Name%ROWTYPE;
    name_domain_2 Name%ROWTYPE;
    domain_1      Domain%ROWTYPE;
    domain_2      Domain%ROWTYPE;

    dept_1        Organization%ROWTYPE;
    dept_2        Organization%ROWTYPE;

    group_1       Owner%ROWTYPE;
    group_2       Owner%ROWTYPE;

    name_net_1    Name%ROWTYPE;
    name_net_2    Name%ROWTYPE;
    network_1     Network%ROWTYPE;
    network_2     Network%ROWTYPE;
    net_1_space_1 IPaddressSpace%ROWTYPE;
    net_1_space_2 IPaddressSpace%ROWTYPE;
    net_2_space_1 IPaddressSpace%ROWTYPE;
    net_2_space_2 IPaddressSpace%ROWTYPE;

    name_node_1   Name%ROWTYPE;
    name_node_2   Name%ROWTYPE;
    name_node_3   Name%ROWTYPE;
    name_node_4   Name%ROWTYPE;
    name_node_5   Name%ROWTYPE;
    name_node_6   Name%ROWTYPE;
    name_node_7   Name%ROWTYPE;

    name_intfc_1  Name%ROWTYPE;

    node_1        Node%ROWTYPE;
    node_2        Node%ROWTYPE;
    node_3        Node%ROWTYPE;
    node_4        Node%ROWTYPE;
    node_5        Node%ROWTYPE;
    node_6        Node%ROWTYPE;
    node_7        Node%ROWTYPE;

    name_alias_1  Name%ROWTYPE;
    name_mx_1     Name%ROWTYPE;

    res_name_1    ResName%ROWTYPE;
-- End raw data

    -- The number of characters that was printed out by the
    -- last call to output_and_increment_test (...).
    test_text_length int;

    -- The number of rows that were fetched in an empty_cursor (...)
    -- call that have been sent to the great digital void.
    rows_skipped int;

    -- The test number of the last test skipped;
    skipped_test int;

    -- These hold the info for the last calls to
    -- get_next_interface (...) and get_next_address (...).
    type list_type is table of int index by varchar(16);

    node_list    list_type;
    IPspace_list list_type;
    intfc_list   list_type;

    procedure set_package_variables;
    function  get_generic_row (result_cursor in NetDB.ref_cursor) return result_row;
    procedure empty_cursor (cursor_to_empty in NetDB.ref_cursor);

    procedure output_and_increment_test (test_number in out int);
    procedure pass;

    procedure failed ( err_msg in varchar,
                       sql_err in varchar := null );

    procedure skipped ( test_number in int,
                        err_msg     in varchar );

    function force_lock ( user_id     in int,
                          record_id   in int,
                          record_type in varchar ) return timestamp;

    procedure force_unlock ( record_id   in int,
                             record_type in varchar );

    procedure grant_access ( user_id     in int,
                             record_type in varchar );

    procedure revoke_access ( user_id     in int,
                              record_type in varchar );

    procedure add_to_group ( user_id  in int,
                             group_id in int );

    procedure remove_from_group ( user_id  in int,
                                  group_id in int );

    procedure reset_interface (node_id in int);
    procedure reset_address ( record_id   in int,
                              record_type in varchar );

    function get_next_interface (node_id in int) return Interface%ROWTYPE;
    function get_next_address ( record_id   in int,
                                record_type in varchar ) return IPaddress%ROWTYPE;

    procedure test_locking ( record_type in     varchar,
                             test_number in out int );

    procedure test_info ( record_type in     varchar,
                          test_number in out int );

end NetDB_Test;
/

create or replace package body NetDB_Test as

    procedure set_package_variables is begin
        select * into user_0        from Users          where netid        = user_0_netid;
        select * into user_1        from Users          where netid        = user_1_netid;
        select * into user_2        from Users          where netid        = user_2_netid;
        select * into ateam_1       from AdminTeam      where name         = ateam_1_name;
        select * into person_1      from Person         where netid        = user_1_netid;
        select * into person_2      from Person         where netid        = user_2_netid;
        select * into domain_1      from Domain         where domain       = domain_1_name;
        select * into domain_2      from Domain         where domain       = domain_2_name;
        select * into name_domain_1 from Name           where id           = domain_1.id;
        select * into name_domain_2 from Name           where id           = domain_2.id;
        select * into dept_1        from Organization   where organization = dept_1_name;
        select * into dept_2        from Organization   where organization = dept_2_name;
        select * into group_1       from Owner          where label        = group_1_name;
        select * into group_2       from Owner          where label        = group_2_name;
        select * into name_net_1    from Name           where fullname     = network_1_name and label = 'Network';
        select * into name_net_2    from Name           where fullname     = network_2_name and label = 'Network';
        select * into network_1     from Network        where id           = name_net_1.network;
        select * into network_2     from Network        where id           = name_net_2.network;
        select * into net_1_space_1 from IPaddressSpace where prefix_txt   = net_1_prefix_1;
        select * into net_1_space_2 from IPaddressSpace where prefix_txt   = net_1_prefix_2;
        select * into net_2_space_1 from IPaddressSpace where prefix_txt   = net_2_prefix_1;
        select * into net_2_space_2 from IPaddressSpace where prefix_txt   = net_2_prefix_2;
        select * into name_node_1   from Name           where fullname     = node_1_name and label = 'Node';
        select * into name_node_2   from Name           where fullname     = node_2_name and label = 'Node';
        select * into name_node_3   from Name           where fullname     = node_3_name and label = 'Node';
        select * into name_node_4   from Name           where fullname     = node_4_name and label = 'Node';
        select * into name_node_5   from Name           where fullname     = node_5_name and label = 'Node';
        select * into name_node_6   from Name           where fullname     = node_6_name and label = 'Node';
        select * into name_node_7   from Name           where fullname     = node_7_name and label = 'Node';
        select * into name_intfc_1  from Name           where fullname     = interface_1_name;
        select * into node_1        from Node           where id           = name_node_1.node;
        select * into node_2        from Node           where id           = name_node_2.node;
        select * into node_3        from Node           where id           = name_node_3.node;
        select * into node_4        from Node           where id           = name_node_4.node;
        select * into node_5        from Node           where id           = name_node_5.node;
        select * into node_6        from Node           where id           = name_node_6.node;
        select * into node_7        from Node           where id           = name_node_7.node;
        select * into name_alias_1  from Name           where fullname     = alias_1_name and label    = 'Alias';
        select * into name_mx_1     from Name           where fullname     = mx_1_name    and dnslabel = 'MX';
        select * into res_name_1    from ResName        where name         = restricted_name;
    end set_package_variables;

    function get_generic_row (result_cursor in NetDB.ref_cursor) return result_row
    is
        r result_row := result_row (null, null, null, null, null, null, null);
    begin
        fetch result_cursor into r.type, r.key, r.hash_level,
                                 r.value_type, r.value_string, r.value_timestamp, r.value_int;
        return r;
    end get_generic_row;


    -- Empty a cursor and then close it.
    procedure empty_cursor (cursor_to_empty in NetDB.ref_cursor)
    is
        the_row result_row := result_row (null, null, null, null, null, null, null);
    begin
        rows_skipped := 0;

        if (cursor_to_empty is not null) then
            loop
                the_row := get_generic_row (cursor_to_empty);
                exit when cursor_to_empty%NOTFOUND;

                rows_skipped := rows_skipped + 1;
            end loop;
            close cursor_to_empty;
        end if;
    end empty_cursor;


    procedure output_and_increment_test (test_number in out int)
    is
        the_text varchar (64) := 'Test ';
    begin
        if (test_number < 100) then the_text := the_text || ' '; end if;
        if (test_number < 10)  then the_text := the_text || ' '; end if;

        the_text         := the_text || test_number || ' ... ';

        dbms_output.put (the_text);

        test_text_length := length (the_text);
        test_number      := test_number + 1;
    end output_and_increment_test;


    procedure pass is begin
        dbms_output.put_line ('OK');
    end pass;


    procedure failed ( err_msg in varchar,
                       sql_err in varchar := null ) is
    begin
        dbms_output.put_line ('FAIL: ' || err_msg);

        if (sql_err is not null) then
            dbms_output.put_line ('Unknown exception: ' || sql_err);
        end if;
    end failed;


    procedure skipped ( test_number in int,
                        err_msg     in varchar ) is
    begin
        dbms_output.put_line ('Skipped: ' || err_msg);

        skipped_test := test_number;
    end skipped;


    -- Lock a record by directly accessing the database, instead of using the API's.
    function force_lock ( user_id     in int,
                          record_id   in int,
                          record_type in varchar ) return timestamp
    is
        v_now   timestamp := systimestamp;
        db_lock timestamp;
        rec_id  int;
    begin
        case lower (record_type)
            -- Necessary for the Ownership tests.
            when 'resname'      then null;
            when 'addressspace' then
                select network into rec_id from IPaddressSpace where id = record_id;
                db_lock := force_lock (user_id, rec_id, 'network');

            when 'adminteam' then update AdminTeam set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            when 'domain'    then update Domain    set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            when 'node'      then update Node      set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            when 'network'   then update Network   set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            when 'owner'     then update Owner     set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            when 'user'      then update Users     set locked = v_now, locked_by = user_id where id = record_id returning locked into db_lock;
            else
                NetDB_Util.throw_unknown_record_type (record_type);
        end case;

        return db_lock;
    end force_lock;


    -- Unlock a record by directly accessing the database, instead of using the API's.
    procedure force_unlock ( record_id   in int,
                             record_type in varchar )
    is
        rec_id int;
    begin
        case lower (record_type)
            -- added for symetry with force_lock.
            when 'resname'      then null;
            when 'addressspace' then
                select network into rec_id from IPaddressSpace where id = record_id;
                force_unlock (rec_id, 'network');

            when 'adminteam' then update AdminTeam set locked = null, locked_by = null where id = record_id;
            when 'domain'    then update Domain    set locked = null, locked_by = null where id = record_id;
            when 'node'      then update Node      set locked = null, locked_by = null where id = record_id;
            when 'network'   then update Network   set locked = null, locked_by = null where id = record_id;
            when 'owner'     then update Owner     set locked = null, locked_by = null where id = record_id;
            when 'user'      then update Users     set locked = null, locked_by = null where id = record_id;
            else
                NetDB_Util.throw_unknown_record_type (record_type);
        end case;
    end force_unlock;


    -- Using user_0, grant access to a record type for a user.
    procedure grant_access ( user_id     in int,
                             record_type in varchar )
    is
        lock_time Users.locked%TYPE := force_lock (user_0.id, user_id, 'user');
    begin
        NetDB_User.users_access ('add', user_0.id, lock_time, user_id, null, record_type);
        force_unlock (user_id, 'user');
    exception
        when NetDB.record_already_exists then null;
    end grant_access;


    -- Using user_0, revoke access from a record type for a user.
    procedure revoke_access ( user_id     in int,
                              record_type in varchar )
    is
        lock_time Users.locked%TYPE := force_lock (user_0.id, user_id, 'user');
    begin
        NetDB_User.users_access ('remove', user_0.id, lock_time, user_id, null, record_type);
        force_unlock (user_id, 'user');
    end revoke_access;

    -- Using user_0, add a user to a group.
    procedure add_to_group ( user_id  in int,
                             group_id in int )
    is
        lock_time Users.locked%TYPE := force_lock (user_0.id, user_id, 'user');
    begin
        NetDB_Owner.add (user_0.id, lock_time, group_id, null, user_id, 'user');
        force_unlock (user_id, 'user');
    exception
        when NetDB.record_already_exists then
            null;
    end add_to_group;


    -- Using user_0, remove a user from a group.
    procedure remove_from_group ( user_id  in int,
                                  group_id in int )
    is
        lock_time Users.locked%TYPE := force_lock (user_0.id, user_id, 'user');
    begin
        NetDB_Owner.remove (user_0.id, lock_time, group_id, null, user_id, 'user');
        force_unlock (user_id, 'user');
    end remove_from_group;


    procedure reset_interface (node_id in int) is begin
        node_list(node_id) := 0;
    end reset_interface;


    procedure reset_address ( record_id   in int,
                              record_type in varchar ) is
    begin
        case lower (record_type)
            when 'addressspace' then
                IPspace_list(record_id) := 0;
            when 'interface' then
                intfc_list(record_id) := 0;
            else
                NetDB_Util.throw_unknown_record_type (record_type);
        end case;
    end reset_address;


    -- Gets the next interface listed for a node.
    function get_next_interface (node_id in int) return Interface%ROWTYPE
    is
        next_interface Interface%ROWTYPE := null;
        last_row       int := 0;
    begin
        begin
            last_row := node_list(node_id);
        exception
            when NO_DATA_FOUND then null;
        end;

        select * into next_interface from Interface
         where node = node_id and rownum = last_row + 1
         order by id;

        node_list(node_id) := last_row + 1;

        return next_interface;
    exception
        when NO_DATA_FOUND then
            return null;
    end get_next_interface;


    -- Get the next IP address for a record.
    function get_next_address ( record_id   in int,
                                record_type in varchar ) return IPaddress%ROWTYPE
    is
        rec_type varchar(32)       := lower (record_type);
        next_ip  IPaddress%ROWTYPE := null;
        last_row int := 0;
    begin
        begin
            case rec_type
                when 'addressspace' then
                    last_row := IPspace_list(record_id);
                when 'interface' then
                    last_row := intfc_list(record_id);
                else
                    NetDB_Util.throw_unknown_record_type (record_type);
            end case;
        exception
            when NO_DATA_FOUND then null;
        end;

        case rec_type
            when 'addressspace' then
            /* What the logic SHOULD be:
             *
             *  select * into next_ip from IPaddress
             *   where ipaddressspace = record_id and rownum = last_row + 1
             *   order by ipaddress;
             */

                -- What we have to use:
                select id,          IPaddress, IPaddress_txt, active,
                       PTRpref,     IPCnode,   interface,     IPaddressSpace,
                       reserved_by, reserved
                  into next_ip
                  from ( select rownum row_num, ip.* from IPaddress ip
                          where ipaddressspace = record_id order by ipaddress )
                 where row_num = last_row + 1;

                IPspace_list(record_id) := last_row + 1;

            when 'interface' then
            /* What the logic SHOULD be:
             *
             *  select * into next_ip from IPaddress
             *   where interface = record_id and rownum = last_row + 1
             *   order by ipaddress;
             */

                -- What we have to use:
                select id,          IPaddress, IPaddress_txt, active,
                       PTRpref,     IPCnode,   interface,     IPaddressSpace,
                       reserved_by, reserved
                  into next_ip
                  from ( select rownum row_num, ip.* from IPaddress ip
                          where interface = record_id order by ipaddress )
                 where row_num = last_row + 1;

                intfc_list(record_id) := last_row + 1;
        end case;

        return next_ip;
    exception
        when NO_DATA_FOUND then
            return null;
    end get_next_address;


    /* Test suite for locking.
     * Tests:
     *   - lock an unlocked record with a user that doesn't have the require access
     *   - unlock an unlocked record
     *   - lock an unlocked record
     *   - Unlock a locked record
     *   - Lock a locked record
     */
    procedure test_locking ( record_type in     varchar,
                             test_number in out int )
    is
        test_cursor NetDB.ref_cursor;

        err_msg varchar(256) := null;
        rec     varchar(64)  := lower (record_type);

        db_lock      timestamp := null;
        db_locked_by int       := null;

        result result_row := null;
    begin
        savepoint lock_start;

        skipped_test := null;

        revoke_access (user_1.id, record_type);
        revoke_access (user_2.id, record_type);

        begin
            output_and_increment_test (test_number);
            err_msg := 'Lock an unlocked record with a user that does not have record access.';

            case rec
                when 'adminteam' then test_cursor := NetDB_AdminTeam.lock_record (user_2.id, ateam_1.id);
                when 'domain'    then test_cursor := NetDB_Domain.lock_record    (user_1.id, domain_1.id);
                when 'node'      then test_cursor := NetDB_Node.lock_record      (user_1.id, node_1.id);
                when 'network'   then test_cursor := NetDB_Network.lock_record   (user_1.id, network_1.id);
                when 'owner'     then test_cursor := NetDB_Owner.lock_record     (user_1.id, group_1.id);
                when 'user'      then test_cursor := NetDB_User.lock_record      (user_1.id, user_1.id);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                empty_cursor (test_cursor);
                failed (err_msg);
            end if;
        exception
            when NetDB.no_access_to_record_type then
                pass ();
            when others then
                failed (err_msg, sqlerrm);
        end;

        grant_access (user_1.id, record_type);
        grant_access (user_2.id, record_type);

        begin
            output_and_increment_test (test_number);
            err_msg := 'Unlock an unlocked record.';

            case rec
                when 'adminteam' then NetDB_AdminTeam.unlock_record (user_2.id, systimestamp, ateam_1.id);
                when 'domain'    then NetDB_Domain.unlock_record    (user_1.id, systimestamp, domain_1.id);
                when 'node'      then NetDB_Node.unlock_record      (user_1.id, systimestamp, node_1.id);
                when 'network'   then NetDB_Network.unlock_record   (user_1.id, systimestamp, network_1.id);
                when 'owner'     then NetDB_Owner.unlock_record     (user_1.id, systimestamp, group_1.id);
                when 'user'      then NetDB_User.unlock_record      (user_1.id, systimestamp, user_1.id);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                failed (err_msg);
            end if;
        exception
            when NetDB.record_not_locked then
                pass ();
            when others then
                failed (err_msg, sqlerrm);
        end;

        begin
            output_and_increment_test (test_number);
            err_msg := 'Lock an unlocked record.';

            case rec
                when 'adminteam' then test_cursor := NetDB_AdminTeam.lock_record (user_2.id, ateam_1.id);
                when 'domain'    then test_cursor := NetDB_Domain.lock_record    (user_1.id, domain_1.id);
                when 'node'      then test_cursor := NetDB_Node.lock_record      (user_1.id, node_3.id);
                when 'network'   then test_cursor := NetDB_Network.lock_record   (user_1.id, network_1.id);
                when 'owner'     then test_cursor := NetDB_Owner.lock_record     (user_1.id, group_1.id);
                when 'user'      then test_cursor := NetDB_User.lock_record      (user_1.id, user_1.id);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                result  := get_generic_row (test_cursor);
                db_lock := result.value_timestamp;
                empty_cursor (test_cursor);

                pass ();
            end if;
        exception
            when others then
                failed (err_msg, sqlerrm);
        end;

        begin
            output_and_increment_test (test_number);
            err_msg := 'Unlock a locked record.';

            case rec
                when 'adminteam' then NetDB_AdminTeam.unlock_record (user_2.id, db_lock, ateam_1.id);
                when 'domain'    then NetDB_Domain.unlock_record    (user_1.id, db_lock, domain_1.id);
                when 'node'      then NetDB_Node.unlock_record      (user_1.id, db_lock, node_3.id);
                when 'network'   then NetDB_Network.unlock_record   (user_1.id, db_lock, network_1.id);
                when 'owner'     then NetDB_Owner.unlock_record     (user_1.id, db_lock, group_1.id);
                when 'user'      then NetDB_User.unlock_record      (user_1.id, db_lock, user_1.id);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                pass ();
            end if;
        exception
            when others then
                failed (err_msg, sqlerrm);
        end;

        begin
            output_and_increment_test (test_number);
            err_msg := 'Lock a locked record.';

            case rec
                when 'adminteam' then
                    db_lock     := force_lock (user_1.id, ateam_1.id, rec);
                    test_cursor := NetDB_AdminTeam.lock_record (user_2.id, ateam_1.id);

                when 'domain' then
                    db_lock     := force_lock (user_2.id, domain_1.id, rec);
                    test_cursor := NetDB_Domain.lock_record (user_1.id, domain_1.id);

                when 'node' then
                    db_lock     := force_lock (user_2.id, node_3.id, rec);
                    test_cursor := NetDB_Node.lock_record (user_1.id, node_3.id);

                when 'network' then
                    db_lock     := force_lock (user_2.id, network_1.id, rec);
                    test_cursor := NetDB_Network.lock_record (user_1.id, network_1.id);

                when 'owner' then
                    db_lock     := force_lock (user_2.id, group_1.id, rec);
                    test_cursor := NetDB_Owner.lock_record (user_1.id, group_1.id);

                when 'user' then
                    db_lock     := force_lock (user_2.id, user_1.id, rec);
                    test_cursor := NetDB_User.lock_record (user_1.id, user_1.id);

                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                empty_cursor (test_cursor);
                failed (err_msg);
            end if;
        exception
            when NetDB.record_locked_by_other then
                pass ();
            when others then
                failed (err_msg, sqlerrm);
        end;

        -- Revert the DB to the state in which it was given to the procedure.
        rollback to lock_start;
    end test_locking;


    -- Test suite for info. Verifies that the info function will complete and tests verify_or_die (...).
    -- There is no data validation (ie the results of info must be manually checked).
    procedure test_info ( record_type in     varchar,
                          test_number in out int )
    is
        test_cursor NetDB.ref_cursor;

        err_msg varchar(256) := null;
        rec     varchar(64)  := lower (record_type);

        intfc Interface%ROWTYPE;
        ip    IPaddress%ROWTYPE;
    begin
        select Interface.* into intfc
          from Interface
          join Name on Name.node = Interface.node
         where rownum = 1             and
               hw_address is not null and
               fullname in ( name_node_1.fullname, name_node_2.fullname, name_node_3.fullname,
                             name_node_4.fullname, name_node_5.fullname, name_node_6.fullname,
                             name_node_7.fullname );

        select IPaddress.* into ip
          from IPaddress
          join Name on Name.node = IPaddress.ipcnode
         where rownum = 1 and
               fullname in ( name_node_1.fullname, name_node_2.fullname, name_node_3.fullname,
                             name_node_4.fullname, name_node_5.fullname, name_node_6.fullname,
                             name_node_7.fullname );

        savepoint info_start;

        skipped_test := null;

        begin
            output_and_increment_test (test_number);
            err_msg := 'Get info on a record with invalid argument.';

            case rec
                when 'address'      then test_cursor := NetDB_IPaddress.info      (null);
                when 'adminteam'    then test_cursor := NetDB_AdminTeam.info      (null);
                when 'domain'       then test_cursor := NetDB_Domain.info         (null);
                when 'interface'    then test_cursor := NetDB_Interface.info      (null);
                when 'addressspace' then test_cursor := NetDB_IPaddressSpace.info (null);
                when 'network'      then test_cursor := NetDB_Network.info        (null);
                when 'node'         then test_cursor := NetDB_Node.info           (null);
                when 'owner'        then test_cursor := NetDB_Owner.info          (null);
                when 'user'         then test_cursor := NetDB_User.info           (null);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                empty_cursor (test_cursor);
                failed (err_msg);
            end if;
        exception
            when NetDB.bad_parameter then
                pass ();
            when others then
                failed (err_msg, sqlerrm);
        end;

        begin
            output_and_increment_test (test_number);
            err_msg := 'Get info on a record with valid argument.';

            case rec
                when 'address'      then test_cursor := NetDB_IPaddress.info      (ip.id);
                when 'adminteam'    then test_cursor := NetDB_AdminTeam.info      (ateam_1.id);
                when 'domain'       then test_cursor := NetDB_Domain.info         (domain_1.id);
                when 'interface'    then test_cursor := NetDB_Interface.info      (intfc.id);
                when 'addressspace' then test_cursor := NetDB_IPaddressSpace.info (net_1_space_1.id);
                when 'node'         then test_cursor := NetDB_Node.info           (node_1.id);
                when 'network'      then test_cursor := NetDB_Network.info        (network_1.id);
                when 'owner'        then test_cursor := NetDB_Owner.info          (group_1.id);
                when 'user'         then test_cursor := NetDB_User.info           (user_1.id);
                else
                    skipped (test_number, err_msg);
            end case;

            if (skipped_test is null or skipped_test != test_number) then
                empty_cursor (test_cursor);
                pass ();
            end if;
        exception
            when others then
                failed (err_msg, sqlerrm);
        end;

        -- Revert the DB to the state in which it was given to the procedure.
        rollback to info_start;
    end test_info;

begin
    set_package_variables ();
    test_text_length := 0;
end NetDB_Test;
/

show errors package NetDB_Test;
show errors package body NetDB_Test;
