set serveroutput on size 100000;
savepoint start_user_tests;

declare
    test_number int          := 1;
    err_msg     varchar(256) := null;
    v_lock      timestamp    := null;
    v_count     int          := 0;
    v_other_org int          := 0;
    v_user_id   int          := 0;

    test_cursor NetDB.ref_cursor;
    result      result_row := result_row (null, null, null, null, null, null, null);

    -- user info for insert and update.
    person_id    int;
    new_regid    Person.person_regid%TYPE := 'unique person regid';
    new_name     Person.name%TYPE         := 'no name mcgee';
    new_netid    Person.netid%TYPE        := 'alongnamenobodywouldwant';
    new_email    Person.email%TYPE        := 'noname@email.com';
    new_phone    Person.phone%TYPE        := '(555) 555-5555';
    new_lastname Person.lastname%TYPE     := 'mcgee';
    new_active   Person.active%TYPE       := 0;
    new_dirid    Person.directoryid%TYPE  := 'directory id';

    user_row Users%ROWTYPE;

    unrecoverable_error EXCEPTION;
begin
    NetDB_Test.test_info ('User', test_number);
    NetDB_Test.test_locking ('User', test_number);

    NetDB_Test.grant_access (NetDB_Test.user_1.id, 'User');
    NetDB_Test.add_to_group (NetDB_Test.user_1.id, NetDB_Test.group_2.id);

    begin -- non-test
        err_msg := 'Locking the user "' || NetDB_Test.user_2.netid || '".';

        test_cursor := NetDB_User.lock_record (NetDB_Test.user_1.id, NetDB_Test.user_2.id);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);
    exception
        when others then
            raise unrecoverable_error;
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a user an LNA for a department they are already an LNA for.';

        NetDB_User.org ('insert', NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id, NetDB_Test.dept_1.id);

        select count (*) into v_count from LNA
         where lna = NetDB_Test.user_2.id and organization = NetDB_Test.dept_1.id;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an LNA from an organiaztion.';

        NetDB_User.org ('delete', NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id, NetDB_Test.dept_1.id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.user_must_be_LNA then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Make a user an LNA for a department.';

        NetDB_User.org ('insert', NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id, NetDB_Test.dept_2.id);

        select count (*) into v_count from LNA
         where lna = NetDB_Test.user_2.id and organization = NetDB_Test.dept_2.id;

        if (v_count > 0) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update the official flag for an LNA.';

        NetDB_User.org ('update', NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id, NetDB_Test.dept_2.id, 1);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete an LNA from an organization.';

        NetDB_User.org ('delete', NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id, NetDB_Test.dept_2.id);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    -- non-test
    err_msg     := 'Unlocking the user "' || NetDB_Test.user_2.netid || '".';
    NetDB_User.unlock_record (NetDB_Test.user_1.id, v_lock, NetDB_Test.user_2.id);
    -- end

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert a new user.';

        test_cursor := NetDB_Person.ins (NetDB_Test.user_1.id, new_regid, new_name, new_netid, new_email, new_phone, new_active, new_lastname, new_dirid);
        result      := NetDB_Test.get_generic_row (test_cursor);
        person_id   := result.value_string;
        NetDB_Test.empty_cursor (test_cursor);

        test_cursor := NetDB_User.ins (NetDB_Test.user_1.id, person_id, NetDB_Test.group_1_name, NetDB_Test.domain_1_name, null, new_active);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_user_id   := to_number (result.value_string);
        result      := NetDB_Test.get_generic_row (test_cursor);
        v_lock      := result.value_timestamp;
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Owner.add (NetDB_Test.user_1.id, v_lock, NetDB_Test.group_1.id, null, v_user_id, 'user');

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Insert an existing user.';

        test_cursor := NetDB_User.ins (NetDB_Test.user_1.id, person_id, NetDB_Test.group_1_name, NetDB_Test.domain_1_name, null, new_active);
        NetDB_Test.empty_cursor (test_cursor);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_already_exists then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a user.';

        NetDB_User.upd (NetDB_Test.user_1.id, v_lock, v_user_id, NetDB_Test.group_1_name, NetDB_Test.domain_1_name, null, 1);

        NetDB_Test.pass ();
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a user profile with a bad group.';

        NetDB_User.upd_profile (v_user_id, NetDB_Test.group_2.label, NetDB_Test.domain_2.domain);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.not_a_member_of_group then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.add_to_group (v_user_id, NetDB_Test.group_2.id);
        v_lock := NetDB_Test.force_lock (NetDB_Test.user_1.id, v_user_id ,'user');

        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Update a user profile.';

        NetDB_User.upd_profile (v_user_id, NetDB_Test.group_2.label, NetDB_Test.domain_2.domain);

        select * into user_row  from Users where id = v_user_id;

        if (user_row.domain = NetDB_Test.domain_2.id and user_row.owner = NetDB_Test.group_2.id) then
            NetDB_Test.pass ();
        else
            NetDB_Test.failed (err_msg);
        end if;
    exception
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a user with a user that doesn''t have "user" access.';

        NetDB_User.del (NetDB_Test.user_2.id, v_lock, v_user_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.no_access_to_record_type then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;

    begin
        NetDB_Test.output_and_increment_test (test_number);
        err_msg := 'Delete a user.';

        NetDB_User.del (NetDB_Test.user_1.id, v_lock, v_user_id);
        v_user_id := NetDB_User.verify_or_die (v_user_id);

        NetDB_Test.failed (err_msg);
    exception
        when NetDB.record_not_found then
            NetDB_Test.pass ();
        when others then
            NetDB_Test.failed (err_msg, sqlerrm);
    end;
exception
    when unrecoverable_error then
        dbms_output.put_line ('ERROR: All remaining tests aborted. An unrecoverable error occured: ' || err_msg);
    when others then
        dbms_output.put_line ('ERROR: Unhandled exception "' || sqlcode || '" with message "' || sqlerrm || '"');
        dbms_output.put_line (err_msg);
end;
/

rollback to start_user_tests;
