-- Name.sql --- DNS names

create table Name
(
    id           int           PRIMARY KEY,
    name         varchar(64)   NOT NULL,    -- as entered by the user
    domain       int           NOT NULL,    -- references Domain on delete restrict,
                                            -- add constraint *after* adding root domain
    fullname     varchar(256)  NOT NULL,    -- fully-qualified, mixed case version of the name
    fullname_lc  varchar(256)  NOT NULL,
    label        varchar(16)   NOT NULL,
    DNSlabel     varchar(16)   NOT NULL,
    IPaddress    int           references IPaddress on delete cascade,
    aliasee      int           references Name      on delete cascade,
    interface    int           references Interface on delete cascade,
    node         int           references Node      on delete cascade,
    network      int           references Network   on delete cascade,
    reserved_by  int           references Person    on delete SET NULL,
    reserved     timestamp(3),
    related_node int           references Node,
    FOREIGN KEY (label, DNSlabel) references NameType
);

create sequence name_id_seq;

-- create index name_name_idx    on Name (name);    -- we probably don't need this
create index name_lower_name_idx on Name (lower(name));
-- create index name_domain_idx  on Name (domain);  -- slows down DNS queries bigtime on PostgreSQL
create index name_fqname_lc_idx  on Name (fullname_lc);
create index name_ipaddress_idx  on Name (IPaddress);
create index name_aliasee_idx    on Name (aliasee);
create index name_interface_idx  on Name (interface);
create index name_node_idx       on Name (node);
create index name_network_idx    on Name (network);
create index name_reserved_idx   on Name (reserved);

create unique index name_fqname_DNSlabel_idx on Name (fullname_lc, DNSlabel);

grant select on Name to netdb_select;

