-- IPaddressSpace_b_iu_r.sql --- IPaddressSpace table before insert or update trigger

create or replace trigger IPaddressSpace_b_iu_r
  before insert or update on IPaddressSpace
  for each row
declare
    minlen int;
    maxlen int;
begin

    -- make sure the user is not setting derivatives
    if INSERTING then
        NetDB_Util.sysmaint_check( 'prefix_txt',    :NEW.prefix_txt    );
        NetDB_Util.sysmaint_check( 'address',       :NEW.address       );
        NetDB_Util.sysmaint_check( 'address_txt',   :NEW.address_txt   );
        NetDB_Util.sysmaint_check( 'broadcast',     :NEW.broadcast     );
        NetDB_Util.sysmaint_check( 'broadcast_txt', :NEW.broadcast_txt );
    end if;

    -- make sure the prefix length is within bounds
    if (IP.family(:NEW.prefix) = 4) then
        minlen := NetDB_Default.get_int('MinPrefixLength');
        maxlen := NetDB_Default.get_int('MaxPrefixLength');
    else
        -- subtle hint to update this trigger for IPv6
        raise_application_error(NetDB.ex_IPv6_not_implemented,
          'Address space prefix length checking for IPv6 has not been considered/implemented.');
    end if;

    if (IP.masklen(:NEW.prefix) < minlen) then
        raise_application_error(NetDB.ex_prefix_len_out_of_bounds,
          'Address space prefix length must be at least ' || minlen || '.');
    elsif (IP.masklen(:NEW.prefix) > maxlen) then
        raise_application_error(NetDB.ex_prefix_len_out_of_bounds,
          'Address space prefix length must be no more than ' || maxlen || '.');
    end if;

    -- normalize prefix and set derivatives
    :NEW.prefix        := IP.normalize(:NEW.prefix);
    :NEW.prefix_txt    := IP.text(:NEW.prefix);
    :NEW.address       := IP.normalize(IP.host(:NEW.prefix));
    :NEW.address_txt   := IP.host(:NEW.prefix);
    :NEW.broadcast     := IP.normalize(IP.broadcast(:NEW.prefix));
    :NEW.broadcast_txt := IP.broadcast(:NEW.prefix);

    -- normalize high and low reserve values
    :NEW.lowres  := IP.normalize(:NEW.lowres);
    :NEW.highres := IP.normalize(:NEW.highres);

    -- make sure the reserved areas are actually in the address space
    if (:NEW.lowres not between :NEW.address and :NEW.broadcast) then
        raise_application_error(NetDB.ex_reserve_out_of_bounds,
          'The low reserve address, ' || IP.host(:NEW.lowres) ||
          ', is not within the address space ' || :NEW.prefix_txt || '.');
    elsif (:NEW.highres not between :NEW.address and :NEW.broadcast) then
        raise_application_error(NetDB.ex_reserve_out_of_bounds,
          'The high reserve address, ' || IP.host(:NEW.highres) ||
          ', is not within the address space ' || :NEW.prefix_txt || '.');
    end if;

end;
/

