-- IPaddress_b_u_r.sql --- IPaddress table before update trigger

create or replace trigger IPaddress_b_u_r
  before update
    of id, IPaddress_txt, IPCnode, interface
    on IPaddress
  for each row
begin
    -- user must not change the value of the id column
    if (:NEW.id != :OLD.id) then
        raise_application_error(NetDB.ex_system_supplies_id, 'ID values are supplied by the system and cannot be changed.');
    end if;

    -- user must not update derived column IPaddress_txt
    if (UPDATING('address_txt')) then
        NetDB_Util.sysmaint_check('address_txt','address_txt');
    end if;

    -- IP addresses cannot move between IPC nodes (to preserve Name.related_node)
    if (UPDATING('IPCnode') and
        :OLD.IPCnode is not null and :NEW.IPCnode != :OLD.IPCnode) then
        raise_application_error(NetDB.ex_no_moving_IPaddresses, 'IP addresses cannot be moved between nodes.');
    end if;

    -- IP addresses cannot move between interfaces (to preserve Name.related_node)
    --   We might want to ease up on this someday and allow addresses to be moved
    --   between interfaces on the same node.
    if (UPDATING('interface') and
        :OLD.interface is not null and :NEW.interface != :OLD.interface) then
        raise_application_error(NetDB.ex_no_moving_IPaddresses, 'IP addresses cannot be moved between interfaces.');
    end if;
end;
/

