-- Save the triggers.
-- Create an, incomplete (everything in this table is of type "trigger"), mirror of the user_source table.
create table patch_triggers ( name   varchar(30),
                              line   int,
                              source varchar(4000) );

insert into patch_triggers (name, line, source)
select name, line, text from user_source
 where type = 'TRIGGER' and name in ( select trigger_name from user_triggers
                                       where table_name = upper ('NodeType') );

-- Rename the table.
alter table NodeType rename to old_NodeType;

-- Rename all indexes on the table.
begin
    for r in (select * from user_indexes where table_name = upper ('old_NodeType'))
    loop
       execute immediate 'alter index ' || r.index_name || ' rename to old_' || r.index_name;
    end loop;
end;
/

-- Load the new table.
@ &1/tables/NodeType.sql;

-- Copy the data from the old table to the new table.
insert into NodeType (type, extkey,      plural,              abbrev, rank, description, accessgroup)
select                type, description, description || 's', 'abbr',  0,    '',          accessgroup from old_NodeType;

update NodeType set plural = 'Advanced', abbrev = 'Adv',  rank = '100' where type = 'Advanced';
update NodeType set                      abbrev = 'IPCP', rank = '200' where type = 'IPCprovider';
update NodeType set                      abbrev = 'Rtr',  rank = '300' where type = 'Router';
update NodeType set                      abbrev = 'Tmpl', rank = '400' where type = 'Template';

-- Move all foreign key constraints to the new table.
begin
    for r in ( select uc.table_name,
                      utc.column_name,
                      uc.constraint_type,
                      uc.constraint_name,
                      uc.r_constraint_name,
                      utc.data_type || '(' || utc.data_length || ')' data_type
                 from user_constraints  uc
                 join user_cons_columns ucc on ucc.constraint_name = uc.constraint_name
                 join user_tab_columns  utc on utc.table_name      = uc.table_name and
                                               utc.column_name     = ucc.column_name
                where uc.r_constraint_name = ( select constraint_name from user_constraints
                                                where table_name = upper ('old_NodeType') and constraint_type = 'P' ) )
    loop
        -- drop the existing contraint
        execute immediate 'alter table ' || r.table_name || ' drop constraint ' || r.constraint_name;

        -- recreate the constraint, referencing the new table.
        execute immediate 'alter table ' || r.table_name ||
                          ' modify ' || r.column_name || ' ' || r.data_type || ' references NodeType';
    end loop;
end;
/

-- Drop the old table (and triggers).
drop table old_NodeType purge;

-- Reload the saved triggers.
declare
    source long;
begin
    for t in (select distinct name from patch_triggers)
    loop
        source := null;

        for r in (select * from patch_triggers where name = t.name order by line)
        loop
            source := source || r.source;
        end loop;

        execute immediate 'create or replace ' || source;
    end loop;
end;
/

drop table patch_triggers purge;

-- Swap the data in the type and extkey columns. The trigger will correct the Node_NodeType table.
update NodeType set type = extkey, extkey = type;
