-- Save the triggers.
-- Create an, incomplete (everything in this table is of type "trigger"), mirror of the user_source table.
create table patch_triggers ( name   varchar(30),
                              line   int,
                              source varchar(4000) );

insert into patch_triggers (name, line, source)
select name, line, text from user_source
 where type = 'TRIGGER' and name in ( select trigger_name from user_triggers
                                       where table_name = upper ('Domain') );

-- Rename the table
alter table Domain rename to old_Domain;

-- Rename all indexes on the table.
begin
    for r in (select * from user_indexes where table_name = upper ('old_Domain'))
    loop
       execute immediate 'alter index ' || r.index_name || ' rename to old_' || r.index_name;
    end loop;
end;
/

-- Load the new table.
@ &1/tables/Domain.sql;

-- Copy the data from the old table to the new table.
insert into Domain (id, domain, created_by, modified_by, created, modified, limited, delegated, locked_by, locked, comments)
select              id, domain, created_by, modified_by, created, modified, limited, 1,         locked_by, locked, comments from old_Domain;

-- Move all foreign key constraints to the new table.
declare
    cascade_delete varchar(64);
begin
    for r in ( select uc.table_name,
                      utc.column_name,
                      uc.constraint_type,
                      uc.constraint_name,
                      uc.r_constraint_name,
                      uc.delete_rule,
                      utc.data_type || case when utc.data_type = 'VARCHAR2' then '(' || utc.data_length    || ')'
                                            when utc.data_type = 'NUMBER'   then '(' || nvl(to_char(utc.data_precision),'*') ||
                                                                                 ',' || utc.data_scale || ')'
                                       end data_type
                 from user_constraints  uc
                 join user_cons_columns ucc on ucc.constraint_name = uc.constraint_name
                 join user_tab_columns  utc on utc.table_name      = uc.table_name and
                                               utc.column_name     = ucc.column_name
                where uc.r_constraint_name = ( select constraint_name from user_constraints
                                                where table_name = upper ('old_Domain') and constraint_type = 'P' ) )
    loop
        select decode (r.delete_rule, 'CASCADE', ' on delete cascade', null) into cascade_delete from dual;

        -- drop the existing contraint
        execute immediate 'alter table ' || r.table_name || ' drop constraint ' || r.constraint_name;

        -- recreate the constraint, referencing the new table.
        execute immediate 'alter table ' || r.table_name ||
                          ' modify ' || r.column_name || ' ' || r.data_type || ' references Domain' || cascade_delete;
    end loop;
end;
/

-- Drop the old table (and triggers).
drop table old_Domain purge;

-- Reload the saved triggers.
declare
    source long;
begin
    for t in (select distinct name from patch_triggers)
    loop
        source := null;

        for r in (select * from patch_triggers where name = t.name order by line)
        loop
            source := source || r.source;
        end loop;

        execute immediate 'create or replace ' || source;
    end loop;
end;
/

drop table patch_triggers purge;
