/************************************************************************
 *
 * ATeamDepartmentsCommand.java - db call NetDB_AdminTeam.org
 *
 * $Id: ATeamDepartmentsCommand.java,v 1.2 2008-03-15 02:55:53 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** This class is used to add and remove departments to/from admin team. */
class ATeamDepartmentsCommand extends DBCall implements ComplexDBCall {

    private static final int ACTION_INDEX        = 0; // p_action
    private static final int ID_OP_USER_INDEX    = 1; // p_op_uid
    private static final int LOCK_INDEX          = 2; // p_lock
    private static final int ATEAM_ID_INDEX      = 3; // p_ateam_id
    private static final int DEPARTMENT_ID_INDEX = 4; // p_org_id

    //NetDB.NetDB_AdminTeam.org('add', 15734, 2006-01-05 14:06:14.645, 20)

    // commands
    public static final String INSERT             = "add";
    public static final String DELETE             = "remove";

    private Vector departments;

    /**
     * Constructor for ATeamMemeberCommand class.
     *
     * @param command     - ATeamDepartmentsCommand.{INSERT, DELETE}
     * @param departments - Vector of department names
     * @param teamID      - id of admin team we are acting upon.
     */
    public ATeamDepartmentsCommand(stanford.netdb.utils.Logger logger,
                                   String action,  Vector departments,
                                   Integer teamID, Timestamp lock,
                                   Integer op_uid)
    throws Exception, IllegalArgumentException {
        super(logger);
        this.departments = departments;
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.ORG);

        values = new Object[5];
        values[ACTION_INDEX]        = action;
        values[ID_OP_USER_INDEX]    = op_uid;
        values[LOCK_INDEX]          = lock;
        values[ATEAM_ID_INDEX]      = teamID;
        values[DEPARTMENT_ID_INDEX] = new Null(Types.NUMERIC);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.ORG));
    }

    /** performs actuall db call for each department in a vector of departments */
    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        for( int i = 0; i < departments.size(); i++ ) {
            values[DEPARTMENT_ID_INDEX] = ((NetDB)logger).verify_or_die (NetDB.DEPARTMENT, (String)departments.elementAt(i));
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;
        }
        return new Hashtable();
    }
}
