/************************************************************************
 *
 * ATeamInsertCommand.java - db call to NetDB_AdminTeam.master 'insert', ...
 *
 * $Id: ATeamInsertCommand.java,v 1.3 2008-03-15 02:55:54 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call to NetDB_AdminTeam.ins */
class ATeamInsertCommand extends DBCall implements ComplexDBCall {

    private static final int ID_OP_USER_INDEX = 1; // p_op_uid
    private static final int NAME_INDEX       = 2; // p_name
    private static final int EMAIL_INDEX      = 3; // p_email
    private static final int PHONE_INDEX      = 4; // p_phone
    private static final int COMMENT_INDEX    = 5; // p_comments

    public Hashtable newATeam;

    public ATeamInsertCommand(stanford.netdb.utils.Logger logger,
                              Hashtable newATeam, Integer op_uid)
    {
        super(logger);
        this.newATeam = newATeam;
        db_call_type  = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.INSERT);

        values = new Object[6];
        values[0]                = ref_cursor;
        values[ID_OP_USER_INDEX] = op_uid;
        values[NAME_INDEX]       = ((String)Utils.n2s(newATeam.get("name"))).trim();
        values[EMAIL_INDEX]      = ((String)Utils.n2s(newATeam.get("email"))).trim();
        values[PHONE_INDEX]      = ((String)Utils.n2s(newATeam.get("phone"))).trim();
        values[COMMENT_INDEX]    = o2v(newATeam.get("comment"));
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.INSERT));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;
        int id = 0;
        Timestamp lock = null;

        try {
            logger.info("inserting ateam "+values[NAME_INDEX]);
            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            id = Integer.parseInt((String)result.get("id"));
            lock = (Timestamp)result.get("locked");

            // now groups
            Hashtable newGroups = (Hashtable)newATeam.get("groups");
            Vector groups_to_insert = new Vector(newGroups.keySet());

            // insert groups to insert
            logger.debug("inserting groups");
            ComplexDBCall cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS,
                                       id, ObjectGroupCommand.ATEAM,
                                       lock, groups_to_insert,
                                       (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update members
            Hashtable newMembers = Utils.nHash(newATeam.get("members"));
            Vector m2i = new Vector();
            Enumeration enm = newMembers.keys();
            while( enm.hasMoreElements() ) {
                // keys are Person IDs
                m2i.addElement( ((Hashtable)newMembers.get(enm.nextElement())).get("id") );
            }

            // insert m2i (members to insert)
            logger.debug("inserting members");
            cmd = new ATeamMembersCommand(logger, ATeamMembersCommand.INSERT, m2i,
                                          new Integer(id), lock,
                                          (Integer)values[ID_OP_USER_INDEX]);

            cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update organizations/departments
            Hashtable newDepts = (Hashtable)newATeam.get("departments");
            if( newDepts == null )
                newDepts = new Hashtable();
            Vector depts_to_insert = new Vector(newDepts.keySet());

            // insert depts to insert
            logger.debug("inserting depts");
            cmd = new ATeamDepartmentsCommand(logger, ATeamDepartmentsCommand.INSERT,
                                              depts_to_insert, new Integer(id),
                                              lock, (Integer)values[ID_OP_USER_INDEX]);

            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // now do check
            logger.debug("do check");
            DBCall verify = new VerifyCommand(logger, VerifyCommand.ATEAM,
                                              new Integer(id), (Integer)values[ID_OP_USER_INDEX]);

            result = con.callDB(verify);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            commit_finally = true;
            logger.debug("done");

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("inserting ateam "+values[NAME_INDEX]+" done");
                    // Unlock the admin team now that we're done.
                    try {
                        DBCall call =
                            new ATeamUnlockCommand(logger, id, lock,
                                                   (Integer)values[ID_OP_USER_INDEX]);
                        call.processResultSet(con.callDB(call));
                    } catch (Exception exxx) { }
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("inserting ateam "+values[NAME_INDEX]+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }
}
