/************************************************************************
 *
 * ATeamMembersCommand.java - db call NetDB_AdminTeam.member ...
 *
 * $Id: ATeamMembersCommand.java,v 1.2 2008-03-15 02:55:55 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** This class is used to add and remove members to/from admin team. */
class ATeamMembersCommand extends DBCall implements ComplexDBCall {

    private static final int ACTION_INDEX     = 0;  // p_action
    private static final int ID_OP_USER_INDEX = 1;  // p_op_uid
    private static final int LOCK_INDEX       = 2;  // p_lock
    private static final int ATEAM_ID_INDEX   = 3;  // p_ateam_id
    private static final int ID_MEMBER_INDEX  = 4;  // p_person_id

    // commands
    public static final String INSERT        = "add";
    public static final String DELETE        = "remove";

    private Vector members;

    public ATeamMembersCommand(stanford.netdb.utils.Logger logger,
                               String action,  Vector members, Integer teamID,
                               Timestamp lock, Integer op_uid)
    throws Exception, IllegalArgumentException {
        super(logger);
        this.members = members;
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.MEMBER);

        values = new Object[5];
        values[ACTION_INDEX]     = action;
        values[ID_OP_USER_INDEX] = op_uid;
        values[LOCK_INDEX]       = lock;
        values[ATEAM_ID_INDEX]   = teamID;
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.MEMBER));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        for( int i = 0; i < members.size(); i++ ) {
            values[ID_MEMBER_INDEX] = members.elementAt(i);
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;
        }
        return new Hashtable();
    }
}
