/************************************************************************
 *
 * ATeamModifyCommand.java - call to NetDB_AdminTeam.master 'update', ...
 *
 * $Id: ATeamModifyCommand.java,v 1.2 2008-03-15 02:55:55 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call NetDB_AdminTeam.upd */
class ATeamModifyCommand extends DBCall implements ComplexDBCall {

    public Hashtable oldATeam;
    public Hashtable newATeam;

    private static final int ID_OP_USER_INDEX = 0; // p_op_uid
    private static final int LOCKED_INDEX     = 1; // p_lock
    private static final int ATEAM_ID_INDEX   = 2; // p_ateam_id
    private static final int NEW_NAME_INDEX   = 3; // p_name_new
    private static final int EMAIL_INDEX      = 4; // p_email
    private static final int PHONE_INDEX      = 5; // p_phone
    private static final int COMMENT_INDEX    = 6; // p_comments

    public ATeamModifyCommand(stanford.netdb.utils.Logger logger,
                              Hashtable oldATeam, Hashtable newATeam,
                              Integer op_uid)
    {
        super(logger);
        this.oldATeam = oldATeam;
        this.newATeam = newATeam;
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.UPDATE);

        values = new Object[7];
        values[ID_OP_USER_INDEX] = op_uid;
        values[LOCKED_INDEX]     = newATeam.get("locked");
        values[ATEAM_ID_INDEX]   = new Integer((String)newATeam.get("id"));
        values[NEW_NAME_INDEX]   = newATeam.get("name");
        values[EMAIL_INDEX]      = ((String)Utils.n2s(newATeam.get("email"))).trim();
        values[PHONE_INDEX]      = ((String)Utils.n2s(newATeam.get("phone"))).trim();
        values[COMMENT_INDEX]    = o2v(newATeam.get("comment"));
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_ADMIN_TEAM, Function_Map.UPDATE));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;

        try {
            logger.info("modifying admin team "+oldATeam.get("name"));
            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            Timestamp lock = (Timestamp)newATeam.get("locked");

            // now groups
            Hashtable newGroups = (Hashtable)newATeam.get("groups");
            Hashtable oldGroups = (Hashtable)oldATeam.get("groups");
            Vector groups_to_insert = new Vector();
            Vector groups_to_delete = new Vector();
            Vector groups_to_keep = new Vector();
            Utils.keysIntersect(oldGroups, newGroups, groups_to_insert, groups_to_delete, groups_to_keep);

            // insert groups_to_insert
            logger.debug("inserting groups_to_insert");
            ComplexDBCall cmd =
                new ObjectGroupCommand(logger, ObjectGroupCommand.ADD_GROUPS,
                                       Integer.parseInt((String)newATeam.get("id")),
                                       ObjectGroupCommand.ATEAM,
                                       lock, groups_to_insert,
                                       (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // delete groups_to_delete
            logger.debug("deleting groups_to_delete");
            cmd = new ObjectGroupCommand(logger, ObjectGroupCommand.REMOVE_GROUPS,
                                         Integer.parseInt((String)newATeam.get("id")),
                                         ObjectGroupCommand.ATEAM,
                                         lock, groups_to_delete,
                                         (Integer)values[ID_OP_USER_INDEX]);
            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update members
            Hashtable oldMembers = (Hashtable)oldATeam.get("members");
            if( oldMembers == null )
                oldMembers = new Hashtable();
            Hashtable newMembers = (Hashtable)newATeam.get("members");
            if( newMembers == null )
                newMembers = new Hashtable();
            Vector members_to_insert = new Vector();
            Vector members_to_delete = new Vector();
            Vector members_to_keep = new Vector();
            Utils.keysIntersect(oldMembers, newMembers, members_to_insert, members_to_delete, members_to_keep);

            Vector m2d = new Vector();
            for( int i = 0; i < members_to_delete.size(); i++ )
                m2d.addElement( new Integer((String)((Hashtable)oldMembers.get(members_to_delete.elementAt(i))).get("id")) );

            Vector m2i = new Vector();
            for( int i = 0; i < members_to_insert.size(); i++ )
                m2i.addElement( new Integer((String)((Hashtable)newMembers.get(members_to_insert.elementAt(i))).get("id")) );

            // delete m2d (members_to_delete)
            logger.debug("deleting members_to_delete");
            cmd = new ATeamMembersCommand(logger, ATeamMembersCommand.DELETE, m2d,
                                          (Integer)values[ATEAM_ID_INDEX], lock,
                                          (Integer)values[ID_OP_USER_INDEX]);

            cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // insert m2i (members_to_insert)
            logger.debug("inserting members_to_insert");
            cmd = new ATeamMembersCommand(logger, ATeamMembersCommand.INSERT, m2i,
                                          (Integer)values[ATEAM_ID_INDEX], lock,
                                          (Integer)values[ID_OP_USER_INDEX]);

            cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // update organizations/departments
            Hashtable oldDepts = (Hashtable)oldATeam.get("departments");
            if( oldDepts == null )
                oldDepts = new Hashtable();
            Hashtable newDepts = (Hashtable)newATeam.get("departments");
            if( newDepts == null )
                newDepts = new Hashtable();
            Vector depts_to_insert = new Vector();
            Vector depts_to_delete = new Vector();
            Vector depts_to_keep = new Vector();
            Utils.keysIntersect(oldDepts, newDepts, depts_to_insert, depts_to_delete, depts_to_keep);

            // delete depts_to_delete
            logger.debug("deleting depts_to_delete");
            cmd = new ATeamDepartmentsCommand(logger, ATeamDepartmentsCommand.DELETE,
                                              depts_to_delete, (Integer)values[ATEAM_ID_INDEX],
                                              lock, (Integer)values[ID_OP_USER_INDEX]);

            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // insert depts_to_insert
            logger.debug("inserting depts_to_insert");
            cmd = new ATeamDepartmentsCommand(logger, ATeamDepartmentsCommand.INSERT,
                                              depts_to_insert, (Integer)values[ATEAM_ID_INDEX],
                                              lock, (Integer)values[ID_OP_USER_INDEX]);

            result = cmd.doIt(con);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            // now do check
            logger.debug("do check");
            DBCall verify = new VerifyCommand(logger, VerifyCommand.ATEAM,
                                              (Integer)values[ATEAM_ID_INDEX], (Integer)values[ID_OP_USER_INDEX]);

            result = con.callDB(verify);

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            DBCall call = new ATeamUnlockCommand(logger, ((Integer)values[ATEAM_ID_INDEX]).intValue(),
                                                 lock, (Integer)values[ID_OP_USER_INDEX]);
            result = call.processResultSet(con.callDB(call));

            // check for errors
            error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            ((NetDBii)logger).unregister_lock(lock, new Integer(Constants.ATEAM));

            commit_finally = true;
            logger.debug("done");

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("modifying admin team "+oldATeam.get("name")+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("modifying admin team "+oldATeam.get("name")+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }

        return new Hashtable();
    }
}
