/************************************************************************
 *
 * AddrSpaceMoveCommand.java - db call NetDB_IPaddressSpace.move
 *
 * $Id: AddrSpaceMoveCommand.java,v 1.2 2008-03-15 02:55:56 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** db call NetDB_IPaddressSpace.move */
class AddrSpaceMoveCommand extends DBCall implements ComplexDBCall {

    private static final int OP_USER_INDEX         = 0;  // p_op_uid
    private static final int LOCK_INDEX            = 1;  // p_lock
    private static final int AS_ID_INDEX           = 2;  // p_ias_id
    private static final int PREFIX_INDEX          = 3;  // p_prefix
    private static final int TARGET_NET_ID_INDEX   = 4;  // p_network_id
    private static final int TARGET_NET_NAME_INDEX = 5;  // p_network_name
    private static final int TARGET_NET_LOCK_INDEX = 6;  // p_net_lock

    private Vector asPrefixList;

    public AddrSpaceMoveCommand(stanford.netdb.utils.Logger logger,
                                Vector asPrefixList,   String target_net_name,
                                Timestamp target_net_lock, Integer op_uid) {
        super(logger);

        db_call_type      = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.MOVE);
        this.asPrefixList = asPrefixList;

        values = new Object[7];
        values[OP_USER_INDEX]         = op_uid;
        values[AS_ID_INDEX]           = new Null(Types.NUMERIC);
        values[TARGET_NET_ID_INDEX]   = new Null(Types.NUMERIC);
        values[TARGET_NET_NAME_INDEX] = target_net_name;
        values[TARGET_NET_LOCK_INDEX] = target_net_lock;
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.MOVE));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        boolean commit_finally = false;
        try {
            logger.info("moving address spaces");
            logger.debug("BeginTransacton");
            BeginTransaction(con.con);

            for( int i = 0; i < asPrefixList.size(); i++ ) {
                Hashtable asToMove = (Hashtable)asPrefixList.elementAt(i);
                values[PREFIX_INDEX] = asToMove.get("prefix");
                values[LOCK_INDEX]   = asToMove.get("lock");
                logger.debug("Moving AS "+values[PREFIX_INDEX]);
                printSQL();
                checkErrors(processResultSet(con.callDB(this)));
            }

            commit_finally = true;

            return new Hashtable();

        } catch(MiniException ex) {
            return ex.result;

        } finally {
            if( commit_finally ) {
                try {
                    logger.info("moving address spaces done");
                    logger.debug("CommitTransaction");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            } else
                try {
                    logger.info("moving address spaces failed");
                    logger.debug("RollbackTransaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
    }

    public void checkErrors(Hashtable result) throws MiniException {
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 )
            throw new MiniException(result);
    }

    class MiniException extends Exception {
        public Hashtable result;

        public MiniException(Hashtable result) {
            this.result = result;
        }
    } // end of class MiniException
}
