/************************************************************************
 *
 * CheckAccessCommand.java - db call NetDB_Access.check_access
 *
 * $Id: CheckAccessCommand.java,v 1.2 2008-03-15 02:55:57 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.sql.*;

/** db call NetDB_Access.check_access */
class CheckAccessCommand extends DBCall {

    private static final int ID_USER_INDEX = 1;  // p_user_id
    private static final int TYPE_INDEX    = 2;  // p_access_type

    /**
     * @param objectType - NetDB.{NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN,
     *                     ADVANCED_NODE, IPC_PROVIDER, ROUTER, SYSTEM_MANAGEMENT}
     * @param op_uid     - ID of the operating user.
     */
    public CheckAccessCommand(stanford.netdb.utils.Logger logger, int objectType,
                              Integer op_uid)
        throws InternalException
    {
        super(logger);
        db_call_type = ORA_FUN_IGNORE_RET_CALL;
        values = new Object[3];

        values[0]                       = new OutParam(Types.NUMERIC);
        switch(objectType) {
        case Constants.NODE:
            values[TYPE_INDEX]          = "node";
            break;
        case Constants.USER:
            values[TYPE_INDEX]          = "users";
            break;
        case Constants.NETWORK:
            values[TYPE_INDEX]          = "network";
            break;
        case Constants.GROUP:
            values[TYPE_INDEX]          = "owner";
            break;
        case Constants.ATEAM:
            values[TYPE_INDEX]          = "adminteam";
            break;
        case Constants.DOMAIN:
            values[TYPE_INDEX]          = "domain";
            break;
        case Constants.ADVANCED_NODE:
            values[TYPE_INDEX]          = "advancednode";
            break;
        case Constants.IPC_PROVIDER:
            values[TYPE_INDEX]          = "ipcprovider";
            break;
        case Constants.ROUTER:
            values[TYPE_INDEX]          = "router";
            break;
        case Constants.SYSTEM_MANAGEMENT:
            values[TYPE_INDEX]          = "systemmanagement";
            break;
        default:
            throw new InternalException("Illegal object type passed to the CheckAccessCommand constructor");
        }

        values[ID_USER_INDEX]           = op_uid;
    }

    public String getSQL() {
        return proc_call_sql("$(SCHEMA)Util.to_int($(SCHEMA)NetDB_Access.check_access");
    }
}
