/************************************************************************
 *
 * DepartmentModifyCommand.java - db call NetDB_Organization.master 'update'
 *
 * $Id: DepartmentModifyCommand.java,v 1.4 2008-03-15 02:55:59 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;

/** db call NetDB_Organization.master 'update' */
class DepartmentModifyCommand extends DBCall implements ComplexDBCall {

    private static final int MODIFY_ACTION  = 1;
    private static final int REPLACE_ACTION = 2;

    public Hashtable oldDep;
    public Hashtable newDep;
    public Integer op_uid;
    public int action;
    public int procedure;

    public DepartmentModifyCommand(stanford.netdb.utils.Logger logger,
                                   Hashtable oldDep, Hashtable newDep,
                                   Integer op_uid)
    {
        super(logger);

        this.oldDep  = oldDep;
        this.newDep  = newDep;
        this.op_uid  = op_uid;

        String oldID = (String)oldDep.get("id");
        String newID = (String)newDep.get("id");
        if( !oldID.equals(newID) )
        {
            action = REPLACE_ACTION;
            procedure = Function_Map.SUPPLANT;

            values = new Object[4];
            values[0] = ref_cursor;
            values[1] = op_uid;
            values[2] = new Integer(oldID);
            values[3] = new Integer(newID);
        }
        else
        {
            action = MODIFY_ACTION;
            procedure = Function_Map.UPDATE;

            values = new Object[5];
            values[0] = ref_cursor;
            values[1] = op_uid;
            values[2] = new Integer(oldID);
            values[3] = newDep.get("name");
            values[4] = o2v(newDep.get("RegID"));
        }

        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_ORGANIZATION, procedure);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_ORGANIZATION, procedure));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        BeginTransaction(con.con);
        boolean commit_finally = false;

        try {
            // delete old consultants
            logger.info("updating department "+oldDep.get("name"));
            logger.debug("deleting old consultants");
            Hashtable cons = Utils.nHash(oldDep.get("consultants"));
            Enumeration enm = cons.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                Hashtable person = (Hashtable)cons.get(key);
                ConsultantCommand cmd =
                    new ConsultantCommand(logger, ConsultantCommand.DELETE_ACTION, op_uid,
                                          new Integer((String)oldDep.get("id")),
                                          new Integer((String)person.get("id")),
                                          (Boolean)person.get("primary?"));
                logger.debug((String)person.get("RegID")+" "+(String)person.get("NetID"));
                Hashtable result = cmd.processResultSet(con.callDB(cmd));

                // check for errors
                Vector error_set = (Vector)result.get("errors");
                if( error_set != null && error_set.size() > 0 )
                    return result;
            }

            logger.debug("do sigle value attributes");
            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;

            if( ((String)Utils.n2s(result.get("action"))).equals("replace") )
                action = REPLACE_ACTION;

            if( action == MODIFY_ACTION ) {
                // insert new consultants
                logger.debug("inserting new consultants");
                cons = (Hashtable)newDep.get("consultants");
                if( cons == null ) cons =  new Hashtable();
                enm = cons.keys();
                while( enm.hasMoreElements() ) {
                    String key = (String)enm.nextElement();
                    Hashtable person = (Hashtable)cons.get(key);
                    ConsultantCommand cmd =
                        new ConsultantCommand(logger, ConsultantCommand.INSERT_ACTION, op_uid,
                                              new Integer((String)newDep.get("id")),
                                              new Integer((String)person.get("id")),
                                              (Boolean)person.get("primary?"));
                    logger.debug((String)person.get("RegID")+" "+(String)person.get("NetID"));
                    result = cmd.processResultSet(con.callDB(cmd));

                    // check for errors
                    error_set = (Vector)result.get("errors");
                    if( error_set != null && error_set.size() > 0 )
                        return result;
                }
            }
            commit_finally = true;

        } catch(Exception ex) {
            ex.printStackTrace();
            try {
                logger.debug("Rollback Transaction");
                RollbackTransaction(con.con);
            } catch(Exception exx) { }
            throw ex;

        } finally {
            if( commit_finally )
                try {
                    logger.info("updating department "+oldDep.get("name")+" done");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }
            else
                try {
                    logger.info("updating department "+oldDep.get("name")+" failed");
                    logger.debug("Rollback Transaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }
        return new Hashtable();
    }

    public int errorResultSetIndex() {
        return 1;
    }
}
