package stanford.netdb.lib;

import java.util.*;

public class Oracle_Function_Map extends Function_Map
{
    private static String generate_key (int record_type, int action) {
        return record_type + "." + action; }

    private SQL_Function get_sql_function  (int record_type, int action) {
        return mapping.get (generate_key (record_type, action)); }

    public String get_function_name (int record_type, int action)
    {
        SQL_Function func = get_sql_function (record_type, action);

        if (func == null) {
            return null; }

        return func.name;
    }

    public int get_function_type (int record_type, int action)
    {
        SQL_Function func = get_sql_function (record_type, action);

        if (func == null) {
            return -1; }

        return func.type;
    }

    private static Hashtable<String, SQL_Function> mapping = new Hashtable<String, SQL_Function> ();

    static
    {
        mapping.put (generate_key (Function_Map.PACKAGE_ACCESS,         Function_Map.CHECK_ACCESS),              new SQL_Function ("NetDB_Access.check_access",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ACCESS,         Function_Map.HANDLE),                    new SQL_Function ("NetDB_Access.handle",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ACCESS,         Function_Map.HAS_ACCESS_OR_DIE),         new SQL_Function ("NetDB_Access.has_access_or_die",       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ACCESS,         Function_Map.LIST),                      new SQL_Function ("NetDB_Access.list",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ACCESS,         Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Access.verify_or_die",           DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.DELETE),                    new SQL_Function ("NetDB_AdminTeam.del",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.HANDLE),                    new SQL_Function ("NetDB_AdminTeam.handle",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.INFO),                      new SQL_Function ("NetDB_AdminTeam.info",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.INSERT),                    new SQL_Function ("NetDB_AdminTeam.ins",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_AdminTeam.locked_or_die",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_AdminTeam.lock_record",          DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.MEMBER),                    new SQL_Function ("NetDB_AdminTeam.member",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.ORG),                       new SQL_Function ("NetDB_AdminTeam.org",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_AdminTeam.unlock_record",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.UPDATE),                    new SQL_Function ("NetDB_AdminTeam.upd",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.VERIFY_AND_LOG),            new SQL_Function ("NetDB_AdminTeam.verify_and_log",       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ADMIN_TEAM,     Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_AdminTeam.verify_or_die",        DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.ADD),                       new SQL_Function ("NetDB_DHCPoption.add",                 DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.APPLICATION_ADD),           new SQL_Function ("NetDB_DHCPoption.application_add",     DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.APPLICATION_REMOVE),        new SQL_Function ("NetDB_DHCPoption.application_remove",  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.DELETE),                    new SQL_Function ("NetDB_DHCPoption.del",                 DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.HANDLE),                    new SQL_Function ("NetDB_DHCPoption.handle",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.INSERT),                    new SQL_Function ("NetDB_DHCPoption.ins",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.LIST),                      new SQL_Function ("NetDB_DHCPoption.list",                DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.REMOVE),                    new SQL_Function ("NetDB_DHCPoption.remove",              DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.UPDATE),                    new SQL_Function ("NetDB_DHCPoption.upd",                 DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPOPTION,     Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_DHCPoption.verify_or_die",       DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_DHCPSERVICE,    Function_Map.HANDLE),                    new SQL_Function ("NetDB_DHCPservice.handle",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPSERVICE,    Function_Map.LIST),                      new SQL_Function ("NetDB_DHCPservice.list",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DHCPSERVICE,    Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_DHCPservice.verify_or_die",      DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.CAN_USE),                   new SQL_Function ("NetDB_Domain.can_use",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.CAN_USE_AS_NAME),           new SQL_Function ("NetDB_Domain.can_use_as_name",         DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.DELETE),                    new SQL_Function ("NetDB_Domain.del",                     DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.HANDLE),                    new SQL_Function ("NetDB_Domain.handle",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.INFO),                      new SQL_Function ("NetDB_Domain.info",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.INSERT),                    new SQL_Function ("NetDB_Domain.ins",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.IS_LIMITED),                new SQL_Function ("NetDB_Domain.is_limited",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.LIST),                      new SQL_Function ("NetDB_Domain.list",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Domain.locked_or_die",           DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_Domain.lock_record",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_Domain.unlock_record",           DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.UPDATE),                    new SQL_Function ("NetDB_Domain.upd",                     DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.VERIFY_AND_LOG),            new SQL_Function ("NetDB_Domain.verify_and_log",          DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_DOMAIN,         Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Domain.verify_or_die",           DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.DELETE),                    new SQL_Function ("NetDB_Interface.del",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.GET_PARENT_ID),             new SQL_Function ("NetDB_Interface.get_parent_id",        DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.HANDLE),                    new SQL_Function ("NetDB_Interface.handle",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.INFO),                      new SQL_Function ("NetDB_Interface.info",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.INSERT),                    new SQL_Function ("NetDB_Interface.ins",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Interface.locked_or_die",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.PRETTY_HANDLE),             new SQL_Function ("NetDB_Interface.pretty_handle",        DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.UPDATE),                    new SQL_Function ("NetDB_Interface.upd",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACE,      Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Interface.verify_or_die",        DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACETYPE,  Function_Map.DELETE),                    new SQL_Function ("NetDB_InterfaceType.del",              DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACETYPE,  Function_Map.INSERT),                    new SQL_Function ("NetDB_InterfaceType.ins",              DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACETYPE,  Function_Map.UPDATE),                    new SQL_Function ("NetDB_InterfaceType.upd",              DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_INTERFACETYPE,  Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_InterfaceType.verify_or_die",    DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.ACCEPT),                    new SQL_Function ("NetDB_IPaddress.accept",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.CANCEL),                    new SQL_Function ("NetDB_IPaddress.cancel",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.CLOSEST_NAME),              new SQL_Function ("NetDB_IPaddress.closest_name",         DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.DELETE),                    new SQL_Function ("NetDB_IPaddress.del",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.HANDLE),                    new SQL_Function ("NetDB_IPaddress.handle",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.INFO),                      new SQL_Function ("NetDB_IPaddress.info",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_IPaddress.locked_or_die",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.RESERVE),                   new SQL_Function ("NetDB_IPaddress.reserve",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.UPDATE),                    new SQL_Function ("NetDB_IPaddress.upd",                  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESS,      Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_IPaddress.verify_or_die",        DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.AFTER_MIDRES),              new SQL_Function ("NetDB_IPaddressSpace.after_midres",    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.CONTAINING),                new SQL_Function ("NetDB_IPaddressSpace.containing",      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.DELETE),                    new SQL_Function ("NetDB_IPaddressSpace.del",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.GET_PARENT_ID),             new SQL_Function ("NetDB_IPaddressSpace.get_parent_id",   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.HANDLE),                    new SQL_Function ("NetDB_IPaddressSpace.handle",          DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.INFO),                      new SQL_Function ("NetDB_IPaddressSpace.info",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.INSERT),                    new SQL_Function ("NetDB_IPaddressSpace.ins",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.IS_MIDRES),                 new SQL_Function ("NetDB_IPaddressSpace.is_midres",       DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.LIST),                      new SQL_Function ("NetDB_IPaddressSpace.list",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_IPaddressSpace.locked_or_die",   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.MOVE),                      new SQL_Function ("NetDB_IPaddressSpace.move",            DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.UPDATE),                    new SQL_Function ("NetDB_IPaddressSpace.upd",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.VALIDATE_JOIN),             new SQL_Function ("NetDB_IPaddressSpace.validate_join",   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.VALIDATE_SPLIT),            new SQL_Function ("NetDB_IPaddressSpace.validate_split",  DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_IPADDRESSSPACE, Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_IPaddressSpace.verify_or_die",   DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.DELETE),                    new SQL_Function ("NetDB_Location.del",                   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.HANDLE),                    new SQL_Function ("NetDB_Location.handle",                DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.INSERT),                    new SQL_Function ("NetDB_Location.ins",                   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.LIST),                      new SQL_Function ("NetDB_Location.list",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.UPDATE),                    new SQL_Function ("NetDB_Location.upd",                   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOCATION,       Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Location.verify_or_die",         DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_LOG,            Function_Map.GET_ACTION),                new SQL_Function ("NetDB_Log.get_action",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOG,            Function_Map.INSERT),                    new SQL_Function ("NetDB_Log.ins",                        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_LOG,            Function_Map.SET_ACTION),                new SQL_Function ("NetDB_Log.set_action",                 DBCall.ORA_PROC_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_MAKE,           Function_Map.DELETE),                    new SQL_Function ("NetDB_Make.del",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MAKE,           Function_Map.INSERT),                    new SQL_Function ("NetDB_Make.ins",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MAKE,           Function_Map.LIST),                      new SQL_Function ("NetDB_Make.list",                      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MAKE,           Function_Map.UPDATE),                    new SQL_Function ("NetDB_Make.upd",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MAKE,           Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Make.verify_or_die",             DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.DELETE),                    new SQL_Function ("NetDB_Model.del",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.INSERT),                    new SQL_Function ("NetDB_Model.ins",                      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.LIST),                      new SQL_Function ("NetDB_Model.list",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.SUPPLANT),                  new SQL_Function ("NetDB_Model.supplant",                 DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.UPDATE),                    new SQL_Function ("NetDB_Model.upd",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODEL,          Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Model.verify_or_die",            DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_MODELTYPE,      Function_Map.LIST),                      new SQL_Function ("NetDB_ModelType.list",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MODELTYPE,      Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_ModelType.verify_or_die",        DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_MXRECORD,       Function_Map.DELETE),                    new SQL_Function ("NetDB_MXrecord.del",                   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MXRECORD,       Function_Map.INSERT),                    new SQL_Function ("NetDB_MXrecord.ins",                   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_MXRECORD,       Function_Map.UPDATE),                    new SQL_Function ("NetDB_MXrecord.upd",                   DBCall.ORA_PROC_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.ACCEPT),                    new SQL_Function ("NetDB_Name.accept",                    DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.CANCEL),                    new SQL_Function ("NetDB_Name.cancel",                    DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.DELETE),                    new SQL_Function ("NetDB_Name.del",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.FULLNAME),                  new SQL_Function ("NetDB_Name.fullname",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.GENERATE_FULLNAME),         new SQL_Function ("NetDB_Name.generate_fullname",         DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.GET_ALIAS_TERMINAL_PARENT), new SQL_Function ("NetDB_Name.get_alias_terminal_parent", DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Name.locked_or_die",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.RELATED_NETWORK),           new SQL_Function ("NetDB_Name.related_network",           DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.RELATED_NODE),              new SQL_Function ("NetDB_Name.related_node",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.RESERVE),                   new SQL_Function ("NetDB_Name.reserve",                   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.UPDATE_DOMAIN_NAME),        new SQL_Function ("NetDB_Name.update_domain_name",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NAME,           Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Name.verify_or_die",             DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.DELETE),                    new SQL_Function ("NetDB_Network.del",                    DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.HANDLE),                    new SQL_Function ("NetDB_Network.handle",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.INFO),                      new SQL_Function ("NetDB_Network.info",                   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.INSERT),                    new SQL_Function ("NetDB_Network.ins",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.LIST),                      new SQL_Function ("NetDB_Network.list",                   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.LOCATION),                  new SQL_Function ("NetDB_Network.location",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Network.locked_or_die",          DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_Network.lock_record",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_Network.unlock_record",          DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.UPDATE),                    new SQL_Function ("NetDB_Network.upd",                    DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.VERIFY_AND_LOG),            new SQL_Function ("NetDB_Network.verify_and_log",         DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NETWORK,        Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Network.verify_or_die",          DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.ADMIN),                     new SQL_Function ("NetDB_Node.admin",                     DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.DELETE),                    new SQL_Function ("NetDB_Node.del",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.HANDLE),                    new SQL_Function ("NetDB_Node.handle",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.INFO),                      new SQL_Function ("NetDB_Node.info",                      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.INSERT),                    new SQL_Function ("NetDB_Node.ins",                       DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Node.locked_or_die",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_Node.lock_record",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.OS),                        new SQL_Function ("NetDB_Node.os",                        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.TAG),                       new SQL_Function ("NetDB_Node.tag",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.TYPE),                      new SQL_Function ("NetDB_Node.type",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_Node.unlock_record",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.UPDATE),                    new SQL_Function ("NetDB_Node.upd",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.USER),                      new SQL_Function ("NetDB_Node.user",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.VERIFY_AND_LOG),            new SQL_Function ("NetDB_Node.verify_and_log",            DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODE,           Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Node.verify_or_die",             DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_NODETYPE,       Function_Map.EXISTS),                    new SQL_Function ("NetDB_NodeType.exists",                DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODETYPE,       Function_Map.LIST),                      new SQL_Function ("NetDB_NodeType.list",                  DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_NODETYPE,       Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_NodeType.verify_or_die",         DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.CONSULTANT),                new SQL_Function ("NetDB_Organization.consultant",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.DELETE),                    new SQL_Function ("NetDB_Organization.del",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.HANDLE),                    new SQL_Function ("NetDB_Organization.handle",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.INFO),                      new SQL_Function ("NetDB_Organization.info",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.INSERT),                    new SQL_Function ("NetDB_Organization.ins",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.LIST),                      new SQL_Function ("NetDB_Organization.list",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.SUPPLANT),                  new SQL_Function ("NetDB_Organization.supplant",          DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.UPDATE),                    new SQL_Function ("NetDB_Organization.upd",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_ORGANIZATION,   Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Organization.verify_or_die",     DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.DELETE),                    new SQL_Function ("NetDB_OS.del",                         DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.INSERT),                    new SQL_Function ("NetDB_OS.ins",                         DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.LIST),                      new SQL_Function ("NetDB_OS.list",                        DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.SUPPLANT),                  new SQL_Function ("NetDB_OS.supplant",                    DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.UPDATE),                    new SQL_Function ("NetDB_OS.upd",                         DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OS,             Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_OS.verify_or_die",               DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.ADD),                       new SQL_Function ("NetDB_Owner.add",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.CHECK_ACCESS),              new SQL_Function ("NetDB_Owner.check_access",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.DELETE),                    new SQL_Function ("NetDB_Owner.del",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.HANDLE),                    new SQL_Function ("NetDB_Owner.handle",                   DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.HAS_ACCESS_OR_DIE),         new SQL_Function ("NetDB_Owner.has_access_or_die",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.INFO),                      new SQL_Function ("NetDB_Owner.info",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.INSERT),                    new SQL_Function ("NetDB_Owner.ins",                      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.IS_MEMBER_OF_GROUP),        new SQL_Function ("NetDB_Owner.is_member_of_group",       DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.LIST),                      new SQL_Function ("NetDB_Owner.list",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_Owner.locked_or_die",            DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_Owner.lock_record",              DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.REMOVE),                    new SQL_Function ("NetDB_Owner.remove",                   DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_Owner.unlock_record",            DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.UPDATE),                    new SQL_Function ("NetDB_Owner.upd",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_OWNER,          Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Owner.verify_or_die",            DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.DELETE),                    new SQL_Function ("NetDB_Person.del",                     DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.INFO),                      new SQL_Function ("NetDB_Person.info",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.INSERT),                    new SQL_Function ("NetDB_Person.ins",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.NAME),                      new SQL_Function ("NetDB_Person.name",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.UPDATE),                    new SQL_Function ("NetDB_Person.upd",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_PERSON,         Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Person.verify_or_die",           DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.CHECK_ACCESS),              new SQL_Function ("NetDB_State.check_access",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.DELETE),                    new SQL_Function ("NetDB_State.del",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.HAS_ACCESS_OR_DIE),         new SQL_Function ("NetDB_State.has_access_or_die",        DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.INSERT),                    new SQL_Function ("NetDB_State.ins",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.LIST),                      new SQL_Function ("NetDB_State.list",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.UPDATE),                    new SQL_Function ("NetDB_State.upd",                      DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_STATE,          Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_State.verify_or_die",            DBCall.ORA_FUN_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.DEF_DOMAIN),                new SQL_Function ("NetDB_User.def_domain",                DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.DELETE),                    new SQL_Function ("NetDB_User.del",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.HANDLE),                    new SQL_Function ("NetDB_User.handle",                    DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.HAS_ALL_GROUPS),            new SQL_Function ("NetDB_User.has_all_groups",            DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.HAS_ALL_RECORDS),           new SQL_Function ("NetDB_User.has_all_records",           DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.INFO),                      new SQL_Function ("NetDB_User.info",                      DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.INSERT),                    new SQL_Function ("NetDB_User.ins",                       DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.IS_ACTIVE),                 new SQL_Function ("NetDB_User.is_active",                 DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.LOCKED_OR_DIE),             new SQL_Function ("NetDB_User.locked_or_die",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.LOGIN),                     new SQL_Function ("NetDB_User.login",                     DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.ORG),                       new SQL_Function ("NetDB_User.org",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.UPDATE),                    new SQL_Function ("NetDB_User.upd",                       DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.UPDATE_PREFIX),             new SQL_Function ("NetDB_User.update_prefix",             DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.UPDATE_PROFILE),            new SQL_Function ("NetDB_User.upd_profile",               DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.USERS_ACCESS),              new SQL_Function ("NetDB_User.users_access",              DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.VERIFY_AND_LOG),            new SQL_Function ("NetDB_User.verify_and_log",            DBCall.ORA_PROC_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_User.verify_or_die",             DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.LOCK_RECORD),               new SQL_Function ("NetDB_User.lock_record",               DBCall.ORA_FUN_CALL));
        mapping.put (generate_key (Function_Map.PACKAGE_USER,           Function_Map.UNLOCK_RECORD),             new SQL_Function ("NetDB_User.unlock_record",             DBCall.ORA_PROC_CALL));

        mapping.put (generate_key (Function_Map.PACKAGE_UTIL,           Function_Map.VERIFY_OR_DIE),             new SQL_Function ("NetDB_Util.verify_or_die",             DBCall.ORA_FUN_CALL));
    }
}
