/************************************************************************
 *
 * GroupModifyCommand.java - db call call NetDB_Owner.master 'update'
 *
 * $Id: GroupModifyCommand.java,v 1.2 2008-03-15 02:56:02 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;

/** db call call NetDB_Owner.master 'update' */
class GroupModifyCommand extends DBCall implements ComplexDBCall {

    private Hashtable oldGroup;
    private Hashtable newGroup;

    private static final int OP_USER_INDEX       = 0; // p_op_uid
    private static final int LOCKED_INDEX        = 1; // p_lock
    private static final int OWNER_ID_INDEX      = 2; // p_owner_id
    private static final int NEW_NAME_INDEX      = 3; // p_name_new
    private static final int DESCRIPTION_INDEX   = 4; // p_description

    public GroupModifyCommand(stanford.netdb.utils.Logger logger,
                              Hashtable oldGroup, Hashtable newGroup,
                              Integer op_uid) {
        super(logger);
        this.oldGroup = oldGroup;
        this.newGroup = newGroup;

        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_OWNER, Function_Map.UPDATE);

        values = new Object[5];
        values[OP_USER_INDEX] = op_uid;

        String new_name = (String)newGroup.get("name");
        if( new_name.length() > 255 )
            new_name = new_name.substring(0, 254);
        values[NEW_NAME_INDEX] = new_name;

        String comment = (String)newGroup.get("comment");
        if( comment != null && comment.length() > 255 )
            comment = comment.substring(0, 254);
        values[DESCRIPTION_INDEX] = o2v(comment);

        values[OWNER_ID_INDEX] = new Integer((String)newGroup.get("id"));
        values[LOCKED_INDEX]   = oldGroup.get("locked");
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_OWNER, Function_Map.UPDATE));
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        Hashtable result = con.callDB(this);

        // check for errors
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 )
            return result;

        DBCall call = new GroupUnlockCommand(logger,
                                             new Integer ((String)oldGroup.get ("id")),
                                             (java.sql.Timestamp)values[LOCKED_INDEX], (Integer)values[OP_USER_INDEX]);
        result = call.processResultSet(con.callDB(call));
        error_set = (Vector)result.get("errors");
        if( error_set == null || error_set.size() == 0 )
            ((NetDBii)logger).unregister_lock((java.sql.Timestamp)values[LOCKED_INDEX],
                                              new Integer(Constants.GROUP));

        return new Hashtable();
    }
}
