/************************************************************************
 *
 * IPAddressAcceptCommand.java - db call NetDB_IPaddress.accept
 *
 * $Id: IPAddressAcceptCommand.java,v 1.4 2008-03-15 02:56:03 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.lang.*;
import java.sql.*;

/** db call NetDB_IPaddress.accept */
class IPAddressAcceptCommand extends DBCall {

    public static final String IPC_TYPE         = "node";
    public static final String INTERFACE_TYPE   = "interface";
    public static final String ADDR_SPACE_TYPE  = "addressspace";

    private static final int ID_OP_USER_INDEX    = 0; // p_op_uid
    private static final int ADDRESS_INDEX       = 1; // p_address
    private static final int ACTIVE_INDEX        = 2; // p_active
    private static final int PTR_PREF_INDEX      = 3; // p_ptr_pref
    private static final int REFERENT_ID_INDEX   = 4; // p_referent_id
    private static final int LOCK_INDEX          = 5; // p_lock
    private static final int REFERENT_TYPE_INDEX = 6; // p_referent_type

    public IPAddressAcceptCommand(stanford.netdb.utils.Logger logger,
                                  String ip,        Boolean active,
                                  Integer referent, Timestamp lock,
                                  String ref_type,  String ptr_pref,
                                  Integer op_uid) {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_IPADDRESS, Function_Map.ACCEPT);
        values = new Object[7];

        values[ID_OP_USER_INDEX]    = op_uid;
        values[ADDRESS_INDEX]       = ip;
        values[ACTIVE_INDEX]        = new Short(active.booleanValue()?"1":"0");
        if( ptr_pref != null && ptr_pref.trim().length() > 0 )
            values[PTR_PREF_INDEX]  = new Integer(ptr_pref);
        else
            values[PTR_PREF_INDEX]  = new Integer(System.getProperty("DefaultPTRpref", "0"));
        values[REFERENT_ID_INDEX]   = referent;
        values[LOCK_INDEX]          = lock;
        values[REFERENT_TYPE_INDEX] = ref_type;
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_IPADDRESS, Function_Map.ACCEPT));
    }
}
