/************************************************************************
 *
 * InterfaceInsertCommand.java - db call NetDB_Interface.master 'insert'
 *
 * $Id: InterfaceInsertCommand.java,v 1.2 2008-03-15 02:56:05 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.lang.*;
import java.sql.*;

/** db call NetDB_Interface.master 'insert' */
class InterfaceInsertCommand extends DBCall {

    private static final int ID_OP_USER_INDEX     = 1;  // p_op_uid
    private static final int LOCK_INDEX           = 2;  // p_lock
    private static final int INTFC_TYPE_INDEX     = 3;  // p_type
    private static final int NODE_ID_INDEX        = 4;  // p_node_id
    private static final int ROAMING_FLAG_INDEX   = 5;  // p_use_roam
    private static final int DHCP_FLAG_INDEX      = 6;  // p_use_dhcp
    private static final int HW_ADDRESS_INDEX     = 7;  // p_hw_addr
    private static final int COMMENT_INDEX        = 8;  // p_comment

    public InterfaceInsertCommand(stanford.netdb.utils.Logger logger, Timestamp lock,
                                  Integer nodeID, Boolean dhcp, Boolean roaming,
                                  String hw, String type, String comment, Integer op_uid)
    {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_INTERFACE, Function_Map.INSERT);

        dhcp    = dhcp==null?Boolean.FALSE:dhcp;
        roaming = roaming==null?Boolean.FALSE:roaming;

        values = new Object[9];
        values[0]                  = ref_cursor;
        values[ID_OP_USER_INDEX]   = op_uid;
        values[LOCK_INDEX]         = lock;
        values[INTFC_TYPE_INDEX]   = type;
        values[NODE_ID_INDEX]      = nodeID;
        values[ROAMING_FLAG_INDEX] = new Integer(roaming.booleanValue()?1:0);
        values[DHCP_FLAG_INDEX]    = new Integer(dhcp.booleanValue()?1:0);
        values[HW_ADDRESS_INDEX]   = o2v(hw);
        values[COMMENT_INDEX]      = o2v(comment);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_INTERFACE, Function_Map.INSERT));
    }
}
