/************************************************************************
 *
 * LocationInsertCommand.java - db call NetDB_Location.master 'insert'
 *
 * $Id: LocationInsertCommand.java,v 1.2 2008-03-15 02:56:05 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;

/** NetDB_Location.master 'insert' */
class LocationInsertCommand extends DBCall {

    private static final int OP_USER_INDEX = 1; // p_op_uid
    private static final int NAME_INDEX    = 2; // p_name
    private static final int SITE_INDEX    = 3; // p_site
    private static final int CODE_INDEX    = 4; // p_code

    public LocationInsertCommand(stanford.netdb.utils.Logger logger,
                                 Hashtable location, Integer op_uid)
        throws NetDBException
   {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_LOCATION, Function_Map.INSERT);

        values = new Object[5];
        values[0]             = ref_cursor;
        values[OP_USER_INDEX] = op_uid;
        values[NAME_INDEX]    = location.get("location");

        Vector errors = new Vector();
        if( ((String)values[NAME_INDEX]).length() > 65 )
            errors.addElement("location field may not be longer than 65 characters");

        String site = (String)location.get("site");
        if( site == null ) site = "";
        values[SITE_INDEX] = site.trim();
        if( site.trim().length() > 16 )
            errors.addElement("site field may not be longer than 16 characters");

        String code = (String)location.get("code");
        if( code == null ) code = "";
        values[CODE_INDEX] = code.trim();
        if( code.trim().length() > 16 )
            errors.addElement("code field may not be longer than 16 characters");

        if( errors.size() > 0 )
            throw new NetDBException("", errors);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_LOCATION, Function_Map.INSERT));
    }
}
