/************************************************************************
 *
 * LocationModifyCommand.java - db call pLocation 'update'
 *
 * $Id: LocationModifyCommand.java,v 1.2 2008-03-15 02:56:06 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;

/** db call pLocation 'update' */
class LocationModifyCommand extends DBCall {

    private static final int OP_USER_INDEX  = 0; // p_op_uid
    private static final int ID_INDEX       = 1; // p_location_id
    private static final int NEW_NAME_INDEX = 2; // p_name_name
    private static final int SITE_INDEX     = 3; // p_site
    private static final int CODE_INDEX     = 4; // p_code

    /**
     * Constructor for LocationModifyCommand class.
     *
     * @param oldLoc     old location object
     * @param newLoc     old location object
     * @param regID      regID of operating user
     */
    public LocationModifyCommand(stanford.netdb.utils.Logger logger,
                                 Hashtable oldLoc, Hashtable newLoc,
                                 Integer op_uid)
        throws Exception, NetDBException
    {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_LOCATION, Function_Map.UPDATE);

        if (oldLoc.get("id") == null)
            oldLoc.put ("id", ((NetDB)logger).verify_or_die (NetDB.LOCATION, (String)oldLoc.get("location")));

        values = new Object[5];
        values[OP_USER_INDEX] = op_uid;
        values[ID_INDEX]      = oldLoc.get("id");

        Vector errors = new Vector();
        values[NEW_NAME_INDEX] = newLoc.get("location");
        if( ((String)values[NEW_NAME_INDEX]).length() > 65 )
            errors.addElement("location field may not be longer than 65 characters");

        String site = (String)newLoc.get("site");
        if( site == null ) site = "";
        values[SITE_INDEX] = site.trim();
        if( site.trim().length() > 16 )
            errors.addElement("site field may not be longer than 16 characters");

        String code = (String)newLoc.get("code");
        if( code == null ) code = "";
        values[CODE_INDEX] = code.trim();
        if( code.trim().length() > 16 )
            errors.addElement("code field may not be longer than 16 characters");

        if( errors.size() > 0 )
            throw new NetDBException("", errors);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_LOCATION, Function_Map.UPDATE));
    }
}
