/************************************************************************
 *
 * ModelModifyCommand.java - db call NetDB_Model.master 'update'
 *
 * $Id: ModelModifyCommand.java,v 1.2 2008-03-15 02:56:09 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;

/** db call NetDB_Model.master 'update' */
class ModelModifyCommand extends DBCall {

    private static final int ID_OP_USER_INDEX     = 0; // p_op_uid
    private static final int MODEL_ID_INDEX       = 1; // p_model_id
    private static final int MAKE_NEW_INDEX       = 2; // p_make_new
    private static final int MODEL_NAME_NEW_INDEX = 3; // p_modelname_new
    private static final int MODEL_TYPE_INDEX     = 4; // p_type

    private String old_id;
    private String new_id;

    public ModelModifyCommand(stanford.netdb.utils.Logger logger,
                              Hashtable oldModel, Hashtable newModel,
                              Integer op_uid)
    {
        super(logger);

        old_id = (String)oldModel.get("id");
        new_id = (String)newModel.get("id");

        if (old_id.equals (new_id)) // update
        {
            db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_MODEL, Function_Map.UPDATE);

            values = new Object[5];
            values[ID_OP_USER_INDEX]     = op_uid;
            values[MODEL_ID_INDEX]       = new Integer((String)oldModel.get("id"));
            values[MAKE_NEW_INDEX]       = newModel.get("make");
            values[MODEL_NAME_NEW_INDEX] = newModel.get("model");

            String type = ((String)Utils.n2s(newModel.get("type"))).trim();
            values[MODEL_TYPE_INDEX] = o2v(type);
        }
        else // supplant
        {
            db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_MODEL, Function_Map.SUPPLANT);

            values = new Object[4];
            values[ID_OP_USER_INDEX]     = op_uid;
            values[MODEL_ID_INDEX]       = new Integer((String)oldModel.get("id"));
            values[MAKE_NEW_INDEX]       = newModel.get("make");
            values[MODEL_NAME_NEW_INDEX] = newModel.get("model");
        }
    }

    public String getSQL()
    {
        int function = Function_Map.UPDATE;

        if (!old_id.equals (new_id))
            function = Function_Map.SUPPLANT;

        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_MODEL, function));
    }
}
