/************************************************************************
 *
 * NetDB.java -
 *
 * $Id: NetDB.java,v 1.11 2008-03-15 02:56:10 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.math.*;
import java.util.*;
import java.lang.*;
import java.sql.*;
import java.io.*;
import java.lang.reflect.*;
import su.netdb.parser.Parser;

/**
 * Public methods in this object present coherent view of the database to
 * the programmer.
 */
class NetDB implements NetDBii, NetDBpi, stanford.netdb.utils.Logger {

    // default logger class
    static stanford.netdb.utils.Logger logger = null;

    // db connection properties for netdb_client
    static Properties db_props = new Properties();

    // all netdb connection properties loaded from netdb.properties
    static Properties netdb_props = new Properties();

    // db connection properties for netdb_search
    static Properties dbs_props = new Properties();

    private static Boolean init_config = new Boolean(false);

    private Integer searchID;

    private Hashtable searches;

    private static Hashtable ptr_pref = new Hashtable();

    // keeps time of the last call; is used to calculate time out
    private long last_call = Calendar.getInstance().getTime().getTime();
    private long last_method_call_at = last_call;
    private String last_method_call = "init";

    static {
        try {
            //DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
            Class.forName("oracle.jdbc.OracleDriver").newInstance();
        } catch (Exception ex) {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    // Indicates if pool of connections is to be used or not.
    static boolean use_connection_pool = false;

    // used to parse string for quick search
    static Parser parser = new Parser();

    /** Id (SUNetID in Stanford) of logged in person */
    String NetID = null;

    /** Hashtable of logged in person */
    Hashtable user = null;

    /**
     * default paremeters stored in a way key -> value,
     * where key is always a String and value is either String of Long
     */
    Hashtable defs = null;

    Integer custom_fields_total = null;

    /** holds all reserved name */
    Hashtable reservedNames = new Hashtable();

    /** holds all reserved ip addresses */
    Hashtable reservedIPs = new Hashtable();

    /** holds all active locks for a given session,
     * when object is destroyed we ulock all objects listed here
     */
    Hashtable locks = new Hashtable();

    // dir. provider class
    private static Class dirProviderClass;

    // db connection url
    static String db_url = "";

    // lots of constants
    static final int ID                  = Constants.ID;
    static final int NAME                = Constants.NAME;

    static final int NODE                = Constants.NODE;
    static final int USER                = Constants.USER;
    static final int NETWORK             = Constants.NETWORK;
    static final int GROUP               = Constants.GROUP;
    static final int ATEAM               = Constants.ATEAM;
    static final int DOMAIN              = Constants.DOMAIN;
    static final int ALL                 = Constants.ALL;
    static final int OS                  = Constants.OS;
    static final int MAKE                = Constants.MAKE;
    static final int MX                  = Constants.MX;
    static final int IPADDRSPACE         = Constants.IPADDRSPACE;
    static final int ADVANCED_NODE       = Constants.ADVANCED_NODE;
    static final int IPC_PROVIDER        = Constants.IPC_PROVIDER;
    static final int ROUTER              = Constants.ROUTER;
    static final int SYSTEM_MANAGEMENT   = Constants.SYSTEM_MANAGEMENT;
    static final int DEPARTMENT          = Constants.DEPARTMENT;
    static final int LOCATION            = Constants.LOCATION;
    static final int MODEL               = Constants.MODEL;
    static final int INTERFACE           = Constants.INTERFACE;
    static final int DHCP_SERVICE        = Constants.DHCP_SERVICE;
    static final int NET_ONLY            = Constants.NET_ONLY;
    static final int PERSON              = Constants.PERSON;
    static final int DHCP_OPTIONS        = Constants.DHCP_OPTIONS;
    static final int LOG                 = Constants.LOG;
    static final int MODEL_TYPE          = Constants.MODEL_TYPE;
    static final int FULL_SEARCH_TYPE    = Constants.FULL_SEARCH_TYPE;
    static final int STATE               = Constants.STATE;
    static final int NODE_TYPE           = Constants.NODE_TYPE;

    static final int IP                  = Constants.IP;
    static final int STRING              = Constants.STRING;

    // targets of quick search
    static final String SEARCH_FOR_NAME                  = Constants.SEARCH_FOR_NAME;
    static final String SEARCH_FOR_IP_ADDR               = Constants.SEARCH_FOR_IP_ADDR;
    static final String SEARCH_FOR_HW_ADDR               = Constants.SEARCH_FOR_HW_ADDR;
    static final String SEARCH_FOR_NAME_NO_MX            = Constants.SEARCH_FOR_NAME_NO_MX;
    static final String SEARCH_FOR_IP_ADDR_NEW           = Constants.SEARCH_FOR_IP_ADDR_NEW;
    static final String SEARCH_FOR_NAME_NO_MX_NEW        = Constants.SEARCH_FOR_NAME_NO_MX_NEW;
    static final String SEARCH_FOR_NET_NAME_NO_MX_ONLY   = Constants.SEARCH_FOR_NET_NAME_NO_MX_ONLY;

    // search constraints for quick search
    static final int WI_NETWORK                          = Constants.WI_NETWORK;
    static final int WI_NODE                             = Constants.WI_NODE;
    static final int WI_USERS                            = Constants.WI_USERS;
    static final int WI_DOMAIN                           = Constants.WI_DOMAIN;
    static final int WI_ADMINTEAM                        = Constants.WI_ADMINTEAM;
    static final int WI_OWNER                            = Constants.WI_OWNER; // group

    // validation constants
    static final int WIDE                                = Constants.WIDE;
    static final int NARROW                              = Constants.NARROW;
    static final int FULL                                = Constants.FULL;

    static final String NETDB_PROPERTIES_FILENAME        = Constants.NETDB_PROPERTIES_FILENAME;
    static final String NETDB_CSS_FILE                   = Constants.NETDB_CSS_FILE;

    // keys in netdb.properties
    static final String DO_SYSLOG_KEY                    = Constants.DO_SYSLOG_KEY;
    static final String DBCONNECTION_USER_KEY            = Constants.DBCONNECTION_USER_KEY;
    static final String DBCONNECTION_PASSWORD_KEY        = Constants.DBCONNECTION_PASSWORD_KEY;
    static final String DBCONNECTION_SEARCH_USER_KEY     = Constants.DBCONNECTION_SEARCH_USER_KEY;
    static final String DBCONNECTION_SEARCH_PASSWORD_KEY = Constants.DBCONNECTION_SEARCH_PASSWORD_KEY;
    static final String DBCONNECTION_URL_KEY             = Constants.DBCONNECTION_URL_KEY;
    static final String DBSCHEMA_KEY                     = Constants.DBSCHEMA_KEY;
    static final String TIMEOUT_KEY                      = Constants.TIMEOUT_KEY;
    static final String SYSLOG_HOST_KEY                  = Constants.SYSLOG_HOST_KEY;
    static final String SYSLOG_PORT_KEY                  = Constants.SYSLOG_PORT_KEY;
    static final String SYSLOG_FACILITY_KEY              = Constants.SYSLOG_FACILITY_KEY;
    static final String SYSLOG_PRIORITY_KEY              = Constants.SYSLOG_PRIORITY_KEY;
    static final String EUID_KEY                         = Constants.UID_KEY;
    static final String EGID_KEY                         = Constants.GID_KEY;
    static final String DIR_PROVIDER_CLASS_KEY           = Constants.DIR_PROVIDER_CLASS_KEY;

    // constants for loggin
    static final int DEBUG                               = Constants.DEBUG;
    static final int WARN                                = Constants.WARN;
    static final int ERROR                               = Constants.ERROR;

    public static final int NETID                        = Constants.NETID;
    public static final int REGID                        = Constants.REGID;
    public static final int LOOKUP                       = Constants.LOOKUP;

    public static final String ETHERNET_TYPE = "Ethernet";
    public static final String RETURN_VALUE  = "return_value";

    private long timestamp; // timestamp when instance is created

    /**
     * This sould throw Exception when called directlly,
     * not throw createInstance(...)
     */
    NetDB() throws Exception {
        if( logger == null )
            logger = new DefaultLogger();
        timestamp  = Calendar.getInstance().getTimeInMillis();
        searches = new Hashtable();
        searchID = new Integer(0);
        debug("NetDB BEGIN");
    }

    /** loads data from GlobalDefault */
    Hashtable loadDefaults() throws Exception {
        NetDBConnection con = getConnection();
        DBCall call = new ListDefaultsCommand(this);
        Hashtable result;
        try {
            result = call.processResultSet(con.callDB(call));
            return result;
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns content of GlobalDefault table. Note that it is cached on login */
    public Hashtable getGlobalDefaults() throws Exception {
        return Utils.copyHash(Utils.nHash(this.defs));
    }

    public Hashtable getCurrentUser() throws Exception {
        recordTime();
        return Utils.copyHash(this.user);
    }

    public Hashtable getUser() {
        return this.user;
    }

    public String getDefaultDomain() {
        recordTime();
        return new String((String)defs.get("DefaultDomain"));
    }

    public void storeSearch(String searchID, Statement stmt, Connection con) {
        synchronized (searches) {
            Hashtable search = new Hashtable();
            search.put("stmt", stmt);
            search.put("con", con);
            debug("Storing search "+searchID);
            searches.put(new String(searchID), search);
        }
    }

    public void cancelSearch(String searchID) {
        recordTime();
        debug("Canceling search "+searchID);
        synchronized (searches) {
            Hashtable search = (Hashtable)searches.get(searchID);
            if( search == null )
                return;

            Statement stmt = (Statement)search.get("stmt");
            Connection con = (Connection)search.get("con");

            try {
                stmt.cancel();
                stmt.close();
                con.close();
            } catch (Exception ex) {
            } finally {
                try {
                    con.close();
                } catch (Exception ex) {}
                try {
                    searches.remove(searchID);
                } catch (Exception ex) {}
            }
        }
    } // end of cancelSearch(...)

    public String createSearchID() {
        recordTime();
        synchronized (searchID) {
            String retVal = new String((searchID).toString());
            searchID = new Integer(searchID.intValue() + 1);
            return retVal;
        }
    }

    public Hashtable simpleSearch(int what, String searchStr)
        throws Exception, NetDBException
    {
        // recordTime() called bellow
        return simpleSearch(what, searchStr, createSearchID());
    }

    public Hashtable simpleSearch(int what, String searchStr, StringBuffer sstype)
        throws Exception, NetDBException
    {
        // recordTime() called bellow
        return simpleSearch(what, searchStr, createSearchID(), sstype);
    }

    public Hashtable simpleSearch(int what, String searchStr, String searchID)
        throws Exception, NetDBException
    {
        return simpleSearch(what, searchStr, searchID, null);
    }

    /**
     * Performs simple search based on passed search_params. Where 'what' can
     * be one of {ALL, NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN, DEPARTMENT}.
     */
    public Hashtable simpleSearch(int what, String searchStr, String searchID, StringBuffer sstype)
        throws Exception, NetDBException
    {
        recordTime();
        debug(what+", ["+searchStr+"]");

        if( searchStr.trim().length() == 0 )
            throw new Exception("Invalid search string");

        if( what == DEPARTMENT ) {
            String filter = null;
            if( searchStr == null || searchStr.trim().length() == 0 )
                filter = "%";
            else
                filter = searchStr.replace('*', '%').replace('?', '_');

            NetDBConnection con = checkWhat(what);
            DBCall call = new DepartmentsListCommand(this, filter);
            Hashtable result = call.processResultSet(con.callDB(call));
            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Search for departments failed", errors);
            }
            return result;
        }
        SimpleSearchCommand call = (SimpleSearchCommand)getSimpleSearchCommand(what);
        if( what == USER || what == GROUP || what == ATEAM || what == DOMAIN ) {

            // parsing is simple in this case: * -> %, ? -> _
            String sstring = searchStr.replace('*', '%');
            sstring = sstring.replace('?', '_');

            call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX, sstring);
            call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, new Null(Types.VARCHAR));
            if( what !=  MX ) // db call for MX search has only three arguments
                call.setParam(SimpleSearchCommand.SEARCH_FOR_INDEX, new Null(Types.NUMERIC));

        } else if ( what == MX ) { // MX
            debug("Searching for mx...");
            String sstring = searchStr.replace('*', '%');
            sstring = sstring.replace('?', '_');
            call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX, sstring);
            call.setParam(1, user.get("RegID").toString());
            debug(call.getSQL()+":::"+SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX+"="+sstring+", 1="+
                  user.get("RegID").toString());

        } else if( what == NET_ONLY ) {
            String sstring = searchStr.replace('*', '%');
            sstring = sstring.replace('?', '_');
            call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX, sstring);
            call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, new Null(Types.VARCHAR));
            call.setParam(SimpleSearchCommand.SEARCH_FOR_INDEX, new Integer(SEARCH_FOR_NET_NAME_NO_MX_ONLY));

        } else { // NODE or NETWORK

            Hashtable parsing_result = null;

            try {
                parsing_result = parser.parse(searchStr);
            } catch (Exception ex) {
                if (ex.getMessage().equals("String passed to ipstr2num(...) is not an ip address."))
                    throw new Exception ("\""+searchStr+"\" is not a recognized IP address format.");
            }

            if( parsing_result.get("hw") != null ) {
                if( what != NODE ) // hw search valid only for NODE
                    return new Hashtable();
                call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX, parsing_result.get("hw"));
                call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, "");
                call.setParam(SimpleSearchCommand.SEARCH_FOR_INDEX, SEARCH_FOR_HW_ADDR);
                if( sstype != null )
                    sstype.append("hw address");

            } else if( parsing_result.get("name") != null ) {
                call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX, parsing_result.get("original_name"));
                String domain = (String)parsing_result.get("domain");
                if( domain == null || what == NODE ) {
                    call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, new Null(Types.VARCHAR));
                } else {
                    domain = domain.toLowerCase();
                    call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, domain);
                }

                call.setParam(SimpleSearchCommand.SEARCH_FOR_INDEX, SEARCH_FOR_NAME);

                if( sstype != null )
                    sstype.append("name");

            } else if( parsing_result.get("ip_low") != null ) {
                if( what != NODE && what != NETWORK ) // valid only for NODE and NETWORK searches
                    return new Hashtable();
                call.setParam(SimpleSearchCommand.SEARCH_VALUE_LOW_INDEX,
                              parsing_result.get("ip_low"));
                String ip_high = (String)parsing_result.get("ip_high");
                if( ip_high == null )
                    call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, new Null(Types.VARCHAR));
                else
                    call.setParam(SimpleSearchCommand.SEARCH_VALUE_HIGH_INDEX, ip_high);
                call.setParam(SimpleSearchCommand.SEARCH_FOR_INDEX, SEARCH_FOR_IP_ADDR);

                if( sstype != null )
                    sstype.append("ip address");

            } else {
                throw new Exception("Invalid search string");
            }
            call.setParam(SimpleSearchCommand.OP_USER_ID_INDEX, new Integer((String)user.get("id")));
        }

        NetDBConnection con = checkQuickSearchWhat(what);
        Hashtable resultSet = null;
        Hashtable search_result = null;
        try {
            if( searchID != null )
                resultSet = con.callDB(call, searchID);
            else
                resultSet = con.callDB(call);
            search_result = call.processResultSet(resultSet);

            Vector error_set = (Vector)search_result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Search failed", errors);
            }

        } finally {
            try {
                con.releaseConnection();
                if( searchID != null ) {
                    synchronized(searches) {
                        try {
                            searches.remove(searchID);
                        } catch (Exception ex) {}
                    }
                }
            } catch (Exception ex) {}
        }

        if( search_result == null )
            search_result = new Hashtable();
        return search_result;
    } // end of simpleSearch(...)

    /**
     * returns instance of SimpleSearchCommand,
     * what = {NODE, MX, NETWORK, USER, GROUP, ATEAM, DOMAIN, ALL}.
     */
    DBCall getSimpleSearchCommand(int what) throws Exception {
        switch( what ) {
        case NODE:
            return new NodeSimpleSearchCommand(this);

        case MX:
            return new MXSimpleSearchCommand(this);

        case NETWORK:
        case NET_ONLY:
            return new NetSimpleSearchCommand(this);

        case USER:
            return new UserSimpleSearchCommand(this);

        case GROUP:
            return new GroupSimpleSearchCommand(this);

        case ATEAM:
            return new ATeamSimpleSearchCommand(this);

        case DOMAIN:
            return new DomainSimpleSearchCommand(this);

        case ALL:
            throw new Exception("NetDB::getSimpleSearchCommand(ALL) not implemented yet.");

        default:
            throw new Exception("NetDB::getSimpleSearchCommand invalid 'what'.");
        }
    }

    /**
     * Performs lookup of person in a directory.
     */
    public Hashtable dirSearch(int byWhat, String sString) throws Exception {
        DirectoryProvider dir = null;
        try {
            recordTime();
            if( dirProviderClass == null )
                throw new Exception("Could not find directory provider");

            synchronized( dirProviderClass ) {
                dir = (DirectoryProvider)dirProviderClass.newInstance();
            }
            dir.setLogger(this);

            if( dir == null )
                throw new Exception("Could not instantiate directory provider class");

            dir.connect(this);

            debug("dirSearch for "+sString);
            Hashtable list = null;
            switch( byWhat ) {
            case NETID:
                list = Utils.nHash(dir.getByUID(sString));
                break;

            case REGID:
                list = Utils.nHash(dir.getByKey(sString));
                break;

            case LOOKUP:
                list = Utils.nHash(dir.lookup(sString));
                break;

            default:
                throw new Exception("Illegal byWhat");
            }
            return list;

        } finally {
            if( dir != null )
                dir.disconnect();
        }
    } // end of dirSearch(...)

    /**
     * Performs full search based on passed search_params. Where 'what' can
     * be one of {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN}.
     */
    public Hashtable
        fullSearch(int what, Hashtable search_params, String searchID)
        throws Exception
    {
        recordTime();
        NetDBConnection con = getSearchConnection();
        try {
            FullSearchCommand call = null;
            switch(what) {
            case DOMAIN:
                call = new DomainFullSearchCommand(this, search_params);
                break;

            case ALL:
                call = new RecordsByOwnerFullSearchCommand(this, search_params);
                break;

            case ATEAM:
                call = new ATeamFullSearchCommand(this, search_params);
                break;

            case GROUP:
                call = new GroupFullSearchCommand(this, search_params);
                break;

            case NETWORK:
                call = new NetworkFullSearchCommand(this, search_params);
                break;

            case USER:
                call = new UserFullSearchCommand(this, search_params);
                break;

            case NODE:
                call = new NodeFullSearchCommand(this, search_params);
                break;

            case LOG:
                call = new LogFullSearchCommand(this, search_params);
                break;

            default:
                throw new Exception("This full search not implemented yet");
            }

            return call.processResultSet(con.callDB(call));

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}

            if( searchID != null ) {
                synchronized(searches) {
                    try {
                        searches.remove(searchID);
                    } catch (Exception ex) {}
                }
            }
        }
    } // end of fullSearch(...)

    public Hashtable fullSearch(Hashtable search_params, String searchID)
        throws Exception
    {
        throw new Exception("Not implemented.");
    } // end of fullSearch(Hashtable search_params, String searchID)

    public String findIPAddrSpace(String ip) throws Exception, NetDBException {
        recordTime();
        NetDBConnection con = getConnection();
        try {
            DBCall call = new AddrSpacePrefix(this, ip);
            Hashtable result = call.processResultSet(con.callDB(call));
            checkForDBErrors("Failed to find address space.", result);
            return (String)call.get(0);
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of findIPAddrSpace(...)

    /**
     * Performes move of address space(s) listed in asList vector as their prefixes to
     * destination network lableled by its FQDN targetNetFQDN
     */
    public void moveAS(Vector asList, String targetNetFQDN)
        throws Exception, NetDBException
    {
        recordTime();
        NetDBConnection con = getConnection();
        Hashtable result = new Hashtable();
        Timestamp lockTm = null;
        try {
            lockTm = lock(Constants.NETWORK, targetNetFQDN);
            if( lockTm == null )
                throw new Exception("Failed to lock network "+targetNetFQDN);

            ComplexDBCall call = new AddrSpaceMoveCommand(this, asList,
                      targetNetFQDN, lockTm, new Integer((String)user.get("id")));
            result = call.doIt(con);
        } finally {
            try {
                if( lockTm != null )
                    unlock(Constants.NETWORK, targetNetFQDN, lockTm);
            } catch (Exception ex) {}
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }

        // check for errors
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            Vector errors = new Vector();
            for( int i = 0; i < error_set.size(); i++ )
                errors.addElement(error_set.elementAt(i).toString().trim());
            throw new NetDBException("Failed to move address spaces", errors);
        }
    }

    /**
     * Locks NetDB object of type 'what' = {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN}
     * 'handle' - for user is NetID
     */
    public Timestamp lock(int what, String handle) throws Exception, NetDBException {
        return lock (what, verify_or_die(what, handle));
    } // end of lock(...)

    /**
     * Locks NetDB object of type 'what' = {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN}
     * and object id - oid. Throws Exception if lock fails for some reason.
     */
    public Timestamp lock(int what, int oid) throws Exception, NetDBException {
        recordTime();
        DBCall call = null;
        Integer user_id = new Integer((String)user.get("id"));
        switch(what) {
        case ATEAM:
            call = new ATeamLockCommand(this, oid, new Integer((String)user.get("id")));
            break;

        case GROUP:
            call = new GroupLockCommand(this, new Integer(oid), new Integer((String)user.get("id")));
            break;

        case DOMAIN:
            call = new DomainLockCommand(this, new Integer(oid), new Integer((String)user.get("id")));
            break;

        case USER:
            call = new UserLockCommand(this, new Integer(oid), new Integer((String)user.get("id")));
            break;

        case NETWORK:
            call = new NetworkLockCommand(this, oid, new Integer((String)user.get("id")));
            break;

        case NODE:
            call = new NodeLockCommand(this, oid, new Integer((String)user.get("id")));
            break;

        default:
            throw new Exception("Unsupported what passed to lock");
        }

        NetDBConnection con = getConnection();
        Hashtable result = new Hashtable();
        try {
            result = call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            String ex_msg = "";
            for( int i = 0; i < error_set.size(); i++ )
                ex_msg += error_set.elementAt(i).toString().trim()+" ";
            throw new NetDBException(ex_msg, error_set);
        }

        java.sql.Timestamp lock_tm = (java.sql.Timestamp)result.get("locked");

        register_lock(lock_tm, new Integer(what), null, new Integer(oid));

        return lock_tm;
    } // end of lock(...)

    /**
     * Unlocks NetDB object of type 'what' = {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN}
     * 'handle' - for user is NetID
     */
    public Timestamp unlock(int what, String handle, Timestamp tm) throws Exception, NetDBException {
        unlock (what, verify_or_die(what, handle), tm);
        return new java.sql.Timestamp(0);
    } // end of unlock(...)

    /**
     * Unlocks NetDB object of type 'what' = {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN}
     * and object id - oid. Throws Exception if fails to unlock object for some reason,
     * for example the object is already unlocked.
     */
    public void unlock(int what, int oid, Timestamp tm) throws Exception, NetDBException {
        recordTime();
        DBCall call = null;
        Integer user_id = new Integer((String)user.get("id"));
        switch(what) {
        case NETWORK:
            call = new NetworkUnlockCommand(this, oid, tm, user_id);
            break;

        case NODE:
            call = new NodeUnlockCommand(this, oid, tm, user_id);
            break;

        case DOMAIN:
            call = new DomainUnlockCommand(this, oid, tm, user_id);
            break;

        case ATEAM:
            call = new ATeamUnlockCommand(this, oid, tm, user_id);
            break;

        case USER:
            call = new UserUnlockCommand(this, oid, tm, user_id);
            break;

        case GROUP:
            call = new GroupUnlockCommand(this, oid, tm, user_id);
            break;

        default:
            throw new Exception("Unsupported what passed to unlock");
        }

        NetDBConnection con = getConnection();
        Hashtable result = new Hashtable();
        try {
            result = call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            String ex_msg = "";
            for( int i = 0; i < error_set.size(); i++ )
                ex_msg += error_set.elementAt(i).toString().trim()+" ";
            throw new NetDBException(ex_msg, error_set);
        }

        unregister_lock(tm, new Integer(what));
    } // end of unlock(...)

    /**
     * Given Address Space (AS) datastructure it splits AS into two equal parts and
     * returns Hashtable := {ip_addr_space_1(String) -> <AS1>, ip_addr_space_2(String) -> <AS2>}
     * This method is made as a part of NetDB class becouse it has to perform db call in order
     * to make sure that splitting is allowed.
     * Note, that <AS> Hashtable is an obvious part of Network hashtable.
     * Also, we are not validating 'as', thus please make sure that you are passing
     * correct datastructure.
     */
    public Hashtable splitAS(Hashtable as) throws Exception, NetDBException {
        recordTime();
        Hashtable as1 = new Hashtable();
        Hashtable as2 = new Hashtable();

        Hashtable as_groups = (Hashtable)as.get("groups");
        if( as_groups == null )
            as_groups = new Hashtable();

        Hashtable as_dhcp_options = (Hashtable)as.get("dhcp_options");
        if( as_dhcp_options == null )
            as_dhcp_options = new Hashtable();

        String as_comment = (String)as.get("comment");
        if( as_comment == null )
            as_comment = "";

        Hashtable as_dhcp_addresses = (Hashtable)as.get("dhcp_addresses");
        if( as_dhcp_addresses == null )
            as_dhcp_addresses = new Hashtable();

        int as_prefix_length = ((Integer)as.get("prefix_length")).intValue();
        int as_new_prefix_length = as_prefix_length+1;
        // if     as_ip          = xxxxxxxx...xxxxxxx00000
        // then   as_prefix_mask = 00000000...000000011111
        // and    as_net_mask    = 11111111...111111100000
        //                         \________________/
        //                          as_prefix_length
        //
        int as_net_mask = 0xffffffff << (32 - as_prefix_length);
        int as_prefix_mask =  ~as_net_mask;

        int as_new_net_mask = 0xffffffff << (32 - as_new_prefix_length);
        int as_new_prefix_mask =  ~as_new_net_mask;

        // find out low res. indent of original address space
        int as_ip = Utils.ipstr2num((String)as.get("addr_space_ip"));

        // Note, that network ip addr. is always reserved even if low_res_indent = 0
        // the same is valid for broadcast as well
        Integer as_low_res_indent  = (Integer)as.get("low_res");
        Integer as_high_res_indent = (Integer)as.get("high_res");

        // fill up as1
        int as1_bcast_ip = as_ip | as_new_prefix_mask;
        String as1_prefix = Utils.ip2str(as_ip)+"/"+(as_prefix_length+1);
        debug("as1 "+as1_prefix+" := ["+Utils.ip2str(as_ip)+"] ["+Utils.ip2str(as1_bcast_ip)+"]");
        as1.put("addr_space_ip", Utils.ip2str(as_ip));
        as1.put("addr_space_prefix", Utils.ip2str(as_ip)+"/"+(as_prefix_length+1));
        as1.put("prefix_length", new Integer(as_prefix_length+1));
        as1.put("low_res", as_low_res_indent); // inherited from parent
        as1.put("high_res", new Integer(0)); // default value - 0
        as1.put("comment", as_comment); // inherited from parent
        as1.put("groups", Utils.copyHash(as_groups)); // inherited from parent
        // copy appropriate dhcp addresses
        Hashtable as1_dhcp_addresses = new Hashtable();
        Enumeration enm = as_dhcp_addresses.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable dhcp_addr = (Hashtable)as_dhcp_addresses.get(key);
            String dhcp_ip_Str = (String)dhcp_addr.get("ip_address");
            int dhcp_ip = Utils.ipstr2num(dhcp_ip_Str);
            if( dhcp_ip < as_ip || dhcp_ip > as1_bcast_ip )
                continue;
            if( dhcp_ip == as_ip )
                throw new Exception("Address space '"+as1_prefix+
                                    "' maps to an existing dhcp address.");
            if( dhcp_ip == as1_bcast_ip )
                throw new Exception("Broadcast address of '"+as1_prefix+
                                    "' addr. space maps to an existing dhcp address.");
            as1_dhcp_addresses.put(key, dhcp_addr);
        }
        as1.put("dhcp_addresses", as1_dhcp_addresses);
        as1.put("dhcp_options", Utils.copyHash(as_dhcp_options));

        // fill up as2
        int as2_ip = as_ip | (0x00000001 << (32 - as_new_prefix_length));
        int as2_bcast_ip = as2_ip | as_new_prefix_mask;
        String as2_prefix = Utils.ip2str(as2_ip)+"/"+(as_prefix_length+1);
        debug("as1 "+as2_prefix+" := ["+Utils.ip2str(as2_ip)+"] ["+Utils.ip2str(as2_bcast_ip)+"]");
        as2.put("addr_space_ip", Utils.ip2str(as2_ip));
        as2.put("addr_space_prefix", Utils.ip2str(as2_ip)+"/"+(as_prefix_length+1));
        as2.put("prefix_length", new Integer(as_prefix_length+1));
        as2.put("low_res", new Integer(1)); // default value for low res is 1
        as2.put("high_res", as_high_res_indent); // inherited from parent
        as2.put("comment", as_comment);
        as2.put("groups", Utils.copyHash(as_groups)); // inherited from parent
        // copy appropriate dhcp addresses
        Hashtable as2_dhcp_addresses = new Hashtable();
        enm = as_dhcp_addresses.keys();
        while( enm.hasMoreElements() ) {
            String key = (String)enm.nextElement();
            Hashtable dhcp_addr = (Hashtable)as_dhcp_addresses.get(key);
            String dhcp_ip_Str = (String)dhcp_addr.get("ip_address");
            int dhcp_ip = Utils.ipstr2num(dhcp_ip_Str);
            if( dhcp_ip < as2_ip || dhcp_ip > as2_bcast_ip )
                continue;
            if( dhcp_ip == as2_ip )
                throw new Exception("The address space '"+as2_prefix+
                                    "' maps to an existing dhcp address.");
            if( dhcp_ip == as2_bcast_ip )
                throw new Exception("The broadcast address of '"+as2_prefix+
                                    "' addr. space maps to an existing dhcp address.");
            as2_dhcp_addresses.put(key, dhcp_addr);
        }
        as2.put("dhcp_addresses", as2_dhcp_addresses);
        as2.put("dhcp_options", Utils.copyHash(as_dhcp_options));

        // now call db to validate resulting addr. spaces against other existing ip addresses
        NetDBConnection con = getConnection();
        try {
            // call for first addr. space
            DBCall call = new AddressSpaceValidateCommand(this, as1_prefix);
            Hashtable result = call.processResultSet(con.callDB(call));

            // check for errors
            checkForDBErrors("Failed to split address space", result);

            // call for second addr. space
            call = new AddressSpaceValidateCommand(this, as2_prefix);
            result = call.processResultSet(con.callDB(call));

            // check for errors
            checkForDBErrors("Failed to split address space", result);

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }

        Hashtable newSet = new Hashtable();
        newSet.put(as1_prefix, as1);
        newSet.put(as2_prefix, as2);
        return newSet;
    } // end of splitAS(...)

    /**
     * Updates NetDB object. Based on known oldObj. This schema implies following method
     * of updating. Fisrt you search for object using either quick or full search then
     * you lock it then get info for object, then you clone it and modify entries in
     * cloned copy (newObj) then you call update, which will unlock object on successful
     * update.
     */
    public Hashtable update(Hashtable oldObj, Hashtable newObj) throws Exception, NetDBException {
        recordTime();
        String otype = (String)oldObj.get("object");
        String ntype = (String)newObj.get("object");
        if( otype == null || ntype == null )
            throw new NetDBException("object field is null", new Vector());

        if( !otype.equals(ntype) )
            throw new Exception("types of oldObj and newObj do not match");

        int what = obj2what(otype);

        DBCall call = null;
        NetDBConnection con = checkWhat(what);
        DirectoryProvider dir = null;
        boolean updProfile = false;
        try {
            Integer op_uid = new Integer((String)user.get("id"));
            switch(what) {
            case NODE:
                if( otype.equals("nodes") ) {
                    // we are modifying two nodes at the same time
                    Hashtable oldNode1 = Utils.nHash(Utils.nHash(oldObj.get("nodes")).get("1"));
                    Utils.validate(NODE, oldNode1, WIDE);
                    Hashtable oldNode2 = Utils.nHash(Utils.nHash(oldObj.get("nodes")).get("2"));
                    Utils.validate(NODE, oldNode2, WIDE);

                    Hashtable newNode1 = Utils.nHash(Utils.nHash(newObj.get("nodes")).get("1"));
                    newNode1.put("check empty interfaces", "");
                    newNode1.remove("no ip format check");
                    Utils.validate(NODE, newNode1, FULL);
                    check_node_tags(newNode1, custom_fields_total);

                    Hashtable newNode2 = Utils.nHash(Utils.nHash(newObj.get("nodes")).get("2"));
                    newNode2.put("check empty interfaces", "");
                    newNode2.remove("no ip format check");
                    Utils.validate(NODE, newNode2, FULL);
                    check_node_tags(newNode2, custom_fields_total);

                    call = new Node2ModifyCommand(this, oldNode1, newNode1,
                                                  oldNode2, newNode2, op_uid);

                } else {
                    Utils.validate(NODE, oldObj, WIDE);
                    Hashtable newNewObj = Utils.copyHash(newObj);
                    newNewObj.put("check empty interfaces", "");
                    newNewObj.remove("no ip format check");
                    Utils.validate(NODE, newNewObj, FULL);
                    check_node_tags(newNewObj, custom_fields_total);
                    set_ptr_prefs(newNewObj);
                    call = new NodeModifyCommand(this, oldObj, newNewObj, op_uid);
                }
                break;

            case NETWORK:
                Hashtable oldOldObj = Utils.copyHash(oldObj);
                oldOldObj.put("ips do not have to have names", "true");
                Utils.validate(NETWORK, oldOldObj, WIDE);
                newObj.remove("ips do not have to have names");
                Utils.validate(NETWORK, newObj, WIDE);
                call = new NetworkModifyCommand(this, oldOldObj, newObj, op_uid);
                break;

            case USER:
                if( newObj.get("updProfile") != null ) {
                    updProfile = true;
                    Utils.validate(USER, newObj, NARROW);
                    newObj.put("RegID", user.get("RegID")); // XXX do we need it?
                    call = new UserProfileModifyCommand(this, newObj);
                } else {
                    Utils.validate(USER, oldObj, WIDE);
                    Utils.validate(USER, newObj, WIDE);
                    call = new UserModifyCommand(this, oldObj, newObj, op_uid);
                }
                break;

            case GROUP:
                Utils.validate(GROUP, oldObj, WIDE);
                Utils.validate(GROUP, newObj, WIDE);
                call = new GroupModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case DOMAIN:
                Utils.validate(DOMAIN, oldObj, WIDE);
                Utils.validate(DOMAIN, newObj, WIDE);
                call = new DomainModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case ATEAM:
                Utils.validate(ATEAM, oldObj, WIDE);
                Utils.validate(ATEAM, newObj, WIDE);
                call = new ATeamModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case OS:
                Utils.validate(OS, oldObj, NARROW); // NARROW becouse obj. does not have 'id' field
                Utils.validate(OS, newObj, NARROW);
                call = new OSModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case MAKE:
                Utils.validate(MAKE, oldObj, NARROW); // NARROW becouse obj. does not have 'id' field
                Utils.validate(MAKE, newObj, NARROW);
                call = new MakeModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case MODEL:
                Utils.validate(MODEL, oldObj, WIDE);
                Utils.validate(MODEL, newObj, NARROW); // NARROW becouse obj. does not have 'id' field
                call = new ModelModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case LOCATION:
                Utils.validate(LOCATION, oldObj, NARROW); // NARROW becouse obj. does not have 'id' field
                Utils.validate(LOCATION, newObj, NARROW);
                call = new LocationModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case STATE:
                Utils.validate(STATE, oldObj, NARROW); // NARROW becouse obj. does not have 'id' field
                Utils.validate(STATE, newObj, NARROW);
                call = new StateModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case DEPARTMENT:
                Utils.validate(DEPARTMENT, oldObj, WIDE);
                Utils.validate(DEPARTMENT, newObj, WIDE);
                call = new DepartmentModifyCommand(this, oldObj, newObj, op_uid);
                break;

            case PERSON:
                if( newObj.get("RegID") == null )
                    throw new Exception("newObj must have 'RegID' field");

                if( dirProviderClass == null )
                    throw new Exception("Could not find directory provider");

                synchronized( dirProviderClass ) {
                    dir = (DirectoryProvider)dirProviderClass.newInstance();
                }
                dir.setLogger(this);

                if( dir == null )
                    throw new Exception("Could not instantiate directory provider class");

                dir.connect(this);

                debug("Verifying person "+newObj.get("RegID")+" ("+newObj.get("NetID")+")");
                Hashtable person = dir.verify(newObj);

                // update persons records
                if( person == null || person.get("RegID") == null ) {
                    String msg = (String)person.get("warning");
                    Vector errors = new Vector();
                    if( msg == null )
                        errors.addElement("Failed to verify person "+(String)newObj.get("RegID"));
                    else
                        errors.addElement(msg);
                    throw new NetDBException("Failed to update person", errors);
                }

                call = new PersonInsertUpdateCommand(this, person, op_uid);
                break;

            default:
                throw new Exception("update for this type of objects is not implemented yet");
            }

            // actually do the command
            Hashtable result;
            if( call instanceof ComplexDBCall )
                result = ((ComplexDBCall)call).doIt(con);
            else
                result = call.processResultSet(con.callDB(call));

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Failed to update "+otype, errors);
            }

            if( updProfile ) {
                synchronized (searchID) { // use searchID as a lock variable
                    user = getInfo(USER, (String)user.get("NetID"));
                    if( user == null || user.get("NetID") == null )
                        throw new Exception(user.get("NetID")+" is not a valid user.");
                }
            }

            // at this point result it may contain only warnings or be empty
            // except for case of updating persons, in which case it will have
            // internal persons id
            return result;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}

            try {
                if( dir != null )
                    dir.disconnect();
            } catch (Exception ex) {}
        }
    } // end of update(...)

    /**
     * This method will delete object from database, based on object id - oid and
     * object type = {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN, ...OS?}
     * It returns non empty Hashtable only in case if there were some errors and
     * delete failed.
     */
    public Hashtable delete(int what, String oid) throws Exception, NetDBException {
        recordTime();
        NetDBConnection con = checkWhat(what);
        try {
            DBCall call = null;
            String otype = "";
            if( con == null )
                throw new InternalException("NetDB::getInfo(...) - 'con' is null");

            switch( what ) {
            case NODE:
                throw new Exception("To delete node use delete(Hashtable obj)");

            case NETWORK:
                throw new Exception("To delete network use delete(Hashtable obj)");

            case USER:
                throw new Exception("To delete user use delete(Hashtable obj)");

            case GROUP:
                throw new Exception("To delete group use delete(Hashtable obj)");

            case ATEAM:
                throw new Exception("To delete admin team use delete(Hashtable obj)");

            case DOMAIN:
                throw new Exception("To delete domain use delete(Hashtable obj)");

            case OS:
                throw new Exception("Use delete(Hashtable obj) for deleting of OS.");

            case STATE:
                throw new Exception("Use delete(Hashtable obj) for deleting of STATE.");

            case MAKE:
                throw new Exception("Use delete(Hashtable obj) for deleting of Make.");

            case DEPARTMENT:
                otype = "department";
                call = new DepartmentDeleteCommand(this, new Integer(oid), new Integer((String)user.get("id")));
                break;

            case LOCATION:
                throw new Exception("Use delete(Hashtable obj) for deleting of Location.");

            case MODEL:
                otype = "model";
                call = new ModelDeleteCommand(this, new Integer(oid), new Integer((String)user.get("id")));
                break;

            default:
                // there is no way it can end up here
            }

            Hashtable result = call.processResultSet(con.callDB(call));
            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Failed to delete "+otype, errors);
            }

            return result;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of delete(...)

    /**
     * Deletes netdb object based on node object (Hashtable). Deleting will happen
     * using oid present in a hashtable.
     */
    public Hashtable delete(Hashtable obj) throws Exception, NetDBException {
        recordTime();
        DBCall call = null;
        NetDBConnection con = null;
        String otype = "";

        if ( obj == null )
            throw new NullPointerException("Object passed to NetDB::delete(obj) is null");

        String whatObj = (String)obj.get("object");
        if( whatObj == null )
            throw new Exception("Object passed to NetDB::delete(obj) does not have \"object\" key");
        int what = obj2what(whatObj);

        Integer op_uid = new Integer((String)user.get("id"));

        switch( what ) {
        case NETWORK:
            otype = "network";
            Utils.validate(NETWORK, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new NetworkDeleteCommand(this, new Integer((String)obj.get("id")),
                                            (Timestamp)obj.get("locked"), op_uid);
            break;

        case NODE:
            otype = "node";
            Utils.validate(NODE, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new NodeDeleteCommand(this, new Integer((String)obj.get("id")),
                                         (Timestamp)obj.get("locked"), op_uid);
            break;

        case GROUP:
            otype = "group";
            Utils.validate(GROUP, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new GroupDeleteCommand(this, new Integer((String)obj.get("id")),
                                          (Timestamp)obj.get("locked"), op_uid);
            break;

        case USER:
            otype = "user";
            Utils.validate(USER, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new UserDeleteCommand(this, new Integer((String)obj.get("id")),
                                         (Timestamp)obj.get("locked"), op_uid);
            break;

        case ATEAM:
            otype = "ateam";
            Utils.validate(ATEAM, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new ATeamDeleteCommand(this, new Integer((String)obj.get("id")),
                                          (Timestamp)obj.get("locked"), op_uid);
            break;

        case DOMAIN:
            otype = "domain";
            Utils.validate(DOMAIN, obj, FULL); // must have fields 'id' and 'locked'
            con = checkWhat(what);
            call = new DomainDeleteCommand(this, new Integer((String)obj.get("id")),
                                           (Timestamp)obj.get("locked"), op_uid);
            break;

        case OS:
            otype = "os";
            Utils.validate(OS, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            con = checkWhat(what);
            call = new OSDeleteCommand(this, obj, op_uid);
            break;

        case MAKE:
            otype = "make";
            Utils.validate(MAKE, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            con = checkWhat(what);
            call = new MakeDeleteCommand(this, obj, op_uid);
            break;

        case LOCATION:
            otype = "location";
            Utils.validate(LOCATION, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            con = checkWhat(what);
            call = new LocationDeleteCommand(this, obj, op_uid);
            break;

        case STATE:
            otype = "state";
            Utils.validate(STATE, obj, NARROW); // NARROW because obj. does not have 'id' field
            con = checkWhat(what);
            call = new StateDeleteCommand(this, obj, op_uid);
            break;

        default:
            String id = (String)obj.get("id");
            if( id == null )
                throw new Exception("Object passed to NetDB::delete(obj) does not have \"id\" key");
            return delete(what, id);
        }

        if( con == null )
            throw new InternalException("NetDB::delete(...) - 'con' is null");

        try {
            Hashtable result = call.processResultSet(con.callDB(call));
            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Failed to delete "+otype, errors);
            }

            try {
                unregister_lock((java.sql.Timestamp)obj.get("locked"), new Integer(what));
            } catch (Exception ex) { }

            return result;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of delete(...)

    /**
     * Inserts object, where 'obj' is appropriate object's Hashtable.
     */
    public Hashtable insert(Hashtable obj) throws Exception, NetDBException {
        recordTime();
        DBCall call = null;
        NetDBConnection con = null;
        String otype = "";

        if ( obj == null )
            throw new NullPointerException("Object passed to NetDB::delete(obj) is null");

        String whatObj = (String)obj.get("object");
        if( whatObj == null )
            throw new Exception("Object passed to NetDB::delete(obj) does not have \"object\" key");
        int what = obj2what(whatObj);

        con = checkWhat(what);
        if( con == null )
            throw new InternalException("NetDB::insert(...) - 'con' is null");

        Integer op_uid = new Integer((String)user.get("id"));

        switch( what ) {
        case ATEAM:
            otype = "admin team";
            Utils.validate(ATEAM, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new ATeamInsertCommand(this, obj, op_uid);
            break;

        case GROUP:
            otype = "group";
            Utils.validate(GROUP, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new GroupInsertCommand(this, obj, op_uid);
            break;

        case DOMAIN:
            otype = "domain";
            Utils.validate(DOMAIN, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new DomainModifyCommand(this, null, obj, op_uid);
            break;

        case OS:
            otype = "os";
            Utils.validate(OS, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new OSInsertCommand(this, obj, op_uid);
            break;

        case MAKE:
            otype = "make";
            Utils.validate(MAKE, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new MakeInsertCommand(this, obj, op_uid);
            break;

        case MODEL:
            otype = "model";
            Utils.validate(MODEL, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new ModelInsertCommand(this, obj, op_uid);
            break;

        case LOCATION:
            otype = "location";
            Utils.validate(LOCATION, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new LocationInsertCommand(this, obj, op_uid);
            break;

        case DEPARTMENT:
            otype = "department";
            Utils.validate(DEPARTMENT, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new DepartmentInsertCommand(this, obj, op_uid);
            break;

        case NETWORK:
            otype = "network";
            Utils.validate(NETWORK, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new NetworkInsertCommand(this, obj, op_uid);
            break;

        case USER:
            otype = "user";
            Utils.validate(USER, obj, NARROW); // NARROW becouse obj. does not have 'id' field
            call = new UserInsertCommand(this, obj, op_uid);
            break;

        case NODE:
            otype = "node";
            Hashtable newObj = Utils.copyHash(obj);
            newObj.put("check empty interfaces", "");
            newObj.remove("no ip format check");
            Utils.validate(NODE, newObj, NARROW); // NARROW becouse obj. does not have 'id' field
            set_ptr_prefs(newObj);
            check_node_tags(newObj, custom_fields_total);

            call = new NodeInsertCommand(this, newObj, op_uid);
            break;

        case STATE:
            otype = "state";
            Utils.validate(STATE, obj, NARROW); // NARROW because obj does not have 'id' field
            call = new StateInsertCommand(this, obj, op_uid);
            break;

        default:
            throw new IllegalArgumentException("Object passed to the insert(...) method is not supported");
        }

        try {
            Hashtable result;
            if( call instanceof ComplexDBCall )
                result = ((ComplexDBCall)call).doIt(con);
            else {
                result = call.processResultSet(con.callDB(call));

                if (result != null && result.containsKey("id")) {
                    try {
                        Integer id = new Integer((String)result.get("id"));
                        Timestamp lock = (Timestamp)result.get("locked");

                        unlock (what, id.intValue (), lock);
                    } catch (Exception ex) {
                        // If the exception is the "Unsupported..." exception, there was no
                        // record locking for this type of object. Ignore it and continue.
                        // If it's not, re-raise the exception and let someone else deal with it.
                        if (! ex.getMessage().toLowerCase().startsWith("unsupported")) {
                            throw ex;
                        }
                    }
                }
            }

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                Vector errors = new Vector();
                for( int i = 0; i < error_set.size(); i++ )
                    errors.addElement(error_set.elementAt(i).toString().trim());
                throw new NetDBException("Failed to insert "+otype, errors);
            }

            return result;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}

            //try {
            //  unlock(what, new Integer((String)obj.get("id")), (java.sql.Timestamp)obj.get("locked"))
            //} catch (Exception ex) {}
        }
    } // end of insert(...)

    /**
     * This methos does not release(close) database connection
     */
    public Hashtable reserveIP(NetDBConnection con, String asPrefix, String ip, int count) throws Exception, NetDBException {
        // If the user didn't enter an IP address, we don't want to warn them that the address wasn't available.
        boolean passed_ip = true;
        if (ip == (String)null)
            passed_ip = false;

        if( asPrefix != null && ip == null ) {
            // now we need to find base address for given prefix as it may be in a middle of some
            // other address space
            if( asPrefix.indexOf("/") > -1 )
                ip = asPrefix.substring(0, asPrefix.indexOf("/"));
            else
                ip = asPrefix;
            DBCall call = new AddrSpacePrefix(this, ip);
            Hashtable result = call.processResultSet(con.callDB(call));
            checkForDBErrors("Failed to find address space.", result);
            ip = (String)call.get(0);
            ip = ip.substring(0, ip.indexOf("/"));
        }
        DBCall call = new IPAddressReserveCommand(this, count, ip, false, false,
                                                  new Integer((String)user.get("id")));
        Hashtable retval = new Hashtable();
        Hashtable result = new Hashtable();
        debug("Reserving ip "+ip);
        result = call.processResultSet(con.callDB(call));
        checkForDBErrors("Failed to reserve an ip address.", result);
        Hashtable ip_addresses = (Hashtable)result.get("ip_addresses");
        boolean found_requested = false;
        if( ip_addresses != null ) {
            Enumeration enm = ip_addresses.keys();
            while( enm.hasMoreElements() ) {
                String reserved_ip_addr = (String)enm.nextElement();
                register_reserved_ip(reserved_ip_addr);

                if( reserved_ip_addr.equals(ip) )
                    found_requested = true;
            }
        }
        retval.put(ip, result);

        if (passed_ip && ! found_requested) {
            Hashtable warning = new Hashtable();
            warning.put("Warning: The IP address "+ip+" was not available. The next available address was used.", "");
            result.put("warnings", warning);
        }

        return retval;
    } // end of reserveIP(...)

    /**
     * Reserves 'count' ip addresses in the given address space 'asPrefix' begining with 'ip'.
     * Returns:
     * (Hashtable)
     * %requested ip% -> (Hashtable)
     *                   'ip_addresses' -> (Hashtable)
     *                                     %ip% - (String)
     *                   'warnings'     -> (Hashtable)
     *                                     %warning%
     **/
    public Hashtable reserveIP(String asPrefix, String ip, int count) throws Exception, NetDBException {
        recordTime();
        NetDBConnection con = getConnection();
        try {
            return reserveIP(con, asPrefix, ip, count);
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /**
     * Cancels previously reseved ip.
     */
    public void cancelIP(String ip) throws Exception, NetDBException {
        recordTime();
        DBCall call = new IPAddressCancelCommand(this, ip, new Integer((String)user.get("id")));
        NetDBConnection con = getConnection();
        try {
            debug("Canceling ip "+ip);
            call.processResultSet(con.callDB(call));
            unregister_reserved_ip(ip);
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of cancelIP(...)

    public void register_reserved_ip (String ip) {
        synchronized (reservedIPs) {
            reservedIPs.put(Utils.ip2norm(Utils.nStr(ip)), "");
        }
    }

    public void unregister_reserved_ip (String ip) {
        synchronized (reservedIPs) {
            try {
                String rip = (String)reservedIPs.remove(Utils.ip2norm(Utils.nStr(ip)));
                if( rip != null )
                    debug("removed ip "+ip+" from list of reserved");
                else
                    debug("failed to remove ip "+ip+" from list of reserved (failed)");
            } catch (Exception ex) {
                debug("failed to remove ip "+ip+" from list of reserved (exception)");
            }
        }
    }


    /**
     * Reserves 'fqdnName' (FQDN format, if domain part is missing then uses default domain) of
     * type 'nameType'. There are several constants to be used in place of name 'nameType':
     * Constants.{DOMAIN_TYPE, NODE_NAME_TYPE, ADDRESS_NAME_TYPE, INTERFACE_NAME_TYPE,
     *            NETWORK_NAME_TYPE, NAME_ALIAS_TYPE, MX_TYPE}
     * Returns actually reserved name, which is usefull in case when reserving name without
     * given domain, it will attach def. domain and return it.
     */
    public Hashtable reserveName(NetDBConnection con, String fqdnName, String nameType)
        throws Exception, NetDBException {
        recordTime();
        // split name and domain
        String name;
        String domain;
        int dotIndex = fqdnName.indexOf(".");
        if( dotIndex > -1 ) {
            name = fqdnName.substring(0, dotIndex).trim();
            domain = fqdnName.substring(dotIndex + 1).trim();
        } else {
            name = fqdnName.trim();
            domain = (String)user.get("default_domain");
        }

        // check if domain is at least TLD
        if( !nameType.equals(Constants.DOMAIN_TYPE) && (domain.matches("^\\.+$") || domain.length() == 0 ) )
            throw new Exception("Name must have domain.");

        // do reserve
        debug("Reserving name "+fqdnName);
        DBCall call = new NameReserveCommand(this, name, domain, nameType, new Integer((String)user.get("id")));
        Hashtable result = new Hashtable();
        result = call.processResultSet(con.callDB(call));
        checkForDBErrors("Failed to reserve name.", result);
        Hashtable nameH = new Hashtable();
        nameH.put("fqdn", name.trim().toLowerCase()+"."+domain.trim().toLowerCase());
        nameH.put("type", nameType);
        synchronized (reservedNames) {
            reservedNames.put(nameH, ""); // at the end of the session all reserverd names will be canceled.
        }

        Hashtable retVal = new Hashtable();
        retVal.put("name", name+"."+domain);
        retVal.put("id",   result.get("id"));
        return retVal;
    } // end of reserveName(...)

    public String reserveName(String fqdnName, String nameType) throws Exception, NetDBException {
        recordTime();
        NetDBConnection con = getConnection();
        try {
            return (String)reserveName(con, fqdnName, nameType).get("name");
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of reserveName(...)

    /** Cancels previously reserved name */
    public void cancelName(String fqdnName, String nameType) throws Exception, NetDBException {
        recordTime();
        NetDBConnection con = getConnection();
        try {
            cancelName(con, fqdnName, nameType);
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Cancels previously reserved name. Does not close db connection */
    public void cancelName(NetDBConnection con, String fqdnName, String nameType) throws Exception, NetDBException {
        // split name and domain
        String name;
        String domain;
        int dotIndex = fqdnName.indexOf(".");
        if( dotIndex > -1 ) {
            name = fqdnName.substring(0, dotIndex).trim();
            domain = fqdnName.substring(dotIndex + 1).trim();
        } else {
            name = fqdnName.trim();
            domain = (String)defs.get("DefaultDomain");
        }
        // do cancel
        debug("Canceling name "+fqdnName);
        DBCall call = new NameCancelCommand(this, name, domain, nameType, new Integer((String)user.get("id")));
        Hashtable result = new Hashtable();
        result = call.processResultSet(con.callDB(call));
        checkForDBErrors("Failed to cancel name.", result);
        Hashtable nameH = new Hashtable();
        nameH.put("fqdn", name.trim().toLowerCase()+"."+domain.trim().toLowerCase());
        nameH.put("type", nameType);
        synchronized (reservedNames) {
            reservedNames.remove(nameH);
        }
    } // end of cancelName(...)

    public void finalize() throws Throwable {
        // cancel reserved names
        Enumeration enm = reservedNames.keys();
        while( enm.hasMoreElements() ) {
            Hashtable name = (Hashtable)enm.nextElement();
            try {
                cancelName((String)name.get("fqdn"), (String)name.get("type"));
            } catch (Exception  ex) {
                error(ex.getMessage());
            }
        }

        // cancel reserved ip addresses
        enm = reservedIPs.keys();
        while( enm.hasMoreElements() ) {
            try {
                cancelIP((String)enm.nextElement());
            } catch (Exception ex) {
                error(ex.getMessage());
            }
        }

        // unlock all locked objects
        enm = locks.keys();
        while( enm.hasMoreElements() ) {
            Hashtable lock = (Hashtable)locks.get(enm.nextElement());
            try {
                if( lock.get("handle") != null )
                    unlock(((Integer)lock.get("what")).intValue(),
                           (String)lock.get("handle"), (Timestamp)lock.get("lock"));
                else if( lock.get("id") != null )
                    unlock(((Integer)lock.get("what")).intValue(),
                           ((Integer)lock.get("id")).intValue(), (Timestamp)lock.get("lock"));
            } catch (Exception ex) {
                ex.printStackTrace();
                error(ex.getMessage());
            }
        }

        info("NetDB END "+timestamp);
        super.finalize();
    } // end of finalize()

    /**
     * Checks if user has access to the specific type of the objects.
     * @param objectType - NetDB.{SYSTEM_MANAGEMENT, ...}
     * @return true or false indicating that user does/doesn't have access to the objectType
     */
    public boolean checkAccess(int objectType) throws Exception, NetDBException, InternalException {
        recordTime();
        NetDBConnection con = checkWhat(NODE);
        DBCall call = null;
        try {
            call = new CheckAccessCommand(this, objectType, new Integer((String)user.get("id")));

            Hashtable result = call.processResultSet(con.callDB(call));
            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return false;

            Integer retval = (Integer)call.get(0);
            if( retval != null )
                return retval.intValue()==1?true:false;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
        return false;
    }

    /**
     * Checks if user has access rights to the specific object. Normally in order to check that user
     * has rights to modify or delete object both conditions checkAccess and checkGroupAccess should
     * be checked.
     *
     * @param object   Object to check access to.
     * @return         true or false indicating that user does/doesn't have access to the objectType
     */
    public boolean checkGroupAccess(Hashtable object) throws Exception, NetDBException, InternalException {
        recordTime();
        NetDBConnection con = checkWhat(NODE);
        DBCall call = null;
        try {
            call = new CheckGroupAccessCommand(this, object,
                                               new Integer((String)user.get("id")));
            Hashtable result = call.processResultSet(con.callDB(call));
            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 ) {
                return false;
            }

            Integer retval = (Integer)call.get(0);
            if( retval != null )
                return retval.intValue()==1?true:false;

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
        return false;
    } // end of checkGroupAccess(...)

    /**
     * Same as getInfo(USER,...) but it also updates last login column in a Users table
     */
    public Hashtable login(String netID) throws Exception, InternalException {
        recordTime();
        NetDBConnection con = checkWhat(USER);
        DBCall call = null;

        if( con == null )
            throw new InternalException("NetDB::getInfo(...) - 'con' is null");

        try {
            call = new UserLoginCommand(this, null, netID);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /**
     * Returns info for a particular object based on object's type and id.
     *
     * @param what - one of {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN, ...}
     * @param oid  - object id
     */
    public Hashtable getInfo(int what, int oid) throws Exception, InternalException {
        recordTime();
        NetDBConnection con = checkWhat(what);
        DBCall call = null;

        if( con == null )
            throw new InternalException("NetDB::getInfo(...) - 'con' is null");

        try {
            switch( what ) {
            case NODE:
                call = new NodeInfoCommand(this, new Integer(oid),
                                           new Integer((String)user.get("id")));
                break;

            case NETWORK:
                call = new NetworkInfoCommand(this, new Integer(oid));
                break;

            case USER:
                call = new UserInfoCommand(this, new Integer(oid));
                break;

            case GROUP:
                call = new GroupInfoCommand(this, new Integer(oid));
                break;

            case ATEAM:
                call = new ATeamInfoCommand(this, new Integer(oid));
                break;

            case DOMAIN:
                call = new DomainInfoCommand(this, new Integer(oid));
                break;

            case OS:
                call = new OSListCommand(this, null);
                break;

            case MAKE:
                call = new MakeListCommand(this);
                break;

            case MODEL:
                call = new ModelListCommand(this);
                break;

            case MODEL_TYPE:
                call = new ModelTypeListCommand(this);
                break;

            case DEPARTMENT:
                call = new DepartmentInfoCommand(this, new Integer(oid));
                break;

            case LOCATION:
                call = new LocationListCommand(this, null);
                break;

            case DHCP_OPTIONS:
                call = new DHCPOptionListCommand(this);
                break;

            default:
                // there is no way it can end up here
            }

            if( what == NODE )
                return fill_in_ptr_pref_strings(call.processResultSet(con.callDB(call)));

            return call.processResultSet(con.callDB(call));

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of getInfo(...)

    /**
     * Returns info for a particular object based on object's type and string handle.
     *
     * @param what - one of {NODE, USER, NETWORK, GROUP, ATEAM, DOMAIN, ...}
     * @param handle - string handle to the object, for axample if case of node
     *                 it is 'name.domain'
     */
    public Hashtable getInfo(int what, String handle) throws Exception {
        recordTime();
        if( handle == null )
            throw new Exception("NetDB::getInfo(...) - 'handle' is null");
        info(const2str(what)+", ["+handle+"]");
        NetDBConnection con = checkWhat(what);
        DBCall call = null;

        if( con == null )
            throw new InternalException("NetDB::getInfo(...) - 'con' is null");

        String name = "";
        String domain = "";
        switch( what ) {
        case NODE:
        case NETWORK:
            int domainIndex = handle.indexOf(".");
            if( domainIndex < 1 )
                throw new Exception("NetDB::getInfo(...) - 'handle' format is 'node.domain'"+domainIndex);
            name = handle.substring(0, domainIndex);
            domain = handle.substring(domainIndex+1);
        default:
        }

        String filter = null;
        if( what == OS || what == MODEL || what == LOCATION ) {
            if( handle.trim().length() == 0 )
                filter = "%";
            else {
                filter = handle.replace('*', '%');
                filter = filter.replace('?', '_');
            }
        }

        try {
            switch( what ) {
            case NODE:

                call = new NodeInfoCommand(this, verify_or_die(what, handle),
                                           new Integer((String)user.get("id")));
                break;

            case NETWORK:
                call = new NetworkInfoCommand(this, verify_or_die(what, handle));
                break;

            case USER:
                call = new UserInfoCommand(this, verify_or_die(what, handle));
                break;

            case GROUP:
                call = new GroupInfoCommand(this, verify_or_die(what, handle));
                break;

            case ATEAM:
                call = new ATeamInfoCommand(this, verify_or_die(what, handle));
                break;

            case DOMAIN:
                call = new DomainInfoCommand(this, verify_or_die(what, handle));
                break;

            case OS:
                call = new OSListCommand(this, filter);
                break;

            case MAKE:
                call = new MakeListCommand(this);
                break;

            case MODEL:
                call = new ModelListCommand(this);
                break;

            case MODEL_TYPE:
                call = new ModelTypeListCommand(this);
                break;

            case DEPARTMENT:
                if( handle != null && handle.trim().length() > 0 )
                    call = new DepartmentInfoCommand(this, verify_or_die(what, handle));
                else
                    call = new DepartmentsListCommand(this, null);
                break;

            case LOCATION:
                call = new LocationListCommand(this, filter);
                break;

            case STATE:
                call = new StateInfoCommand(this, handle);
                break;

            case DHCP_OPTIONS:
                call = new DHCPOptionListCommand(this);
                break;

            default:
                // there is no way it can end up here
            }

            if( what == NODE )
                return fill_in_ptr_pref_strings(call.processResultSet(con.callDB(call)));

            return call.processResultSet(con.callDB(call));

        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of getInfo(...)


    /** Returns list of little O objects */
    public Hashtable list(int what, String filter, boolean mine) throws Exception {
        filter = Utils.nStr(filter).replace('*', '%').replace('?', '_');
        recordTime();
        Integer uid = !mine?null:(new Integer((String)user.get("id")));
        NetDBConnection con = checkWhat(STATE);
        DBCall call = null;
        try {
            switch( what ) {
            case NODE_TYPE:
                call = new NodeTypeListCommand(this, uid);
            default:
            }

            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of list(...)

    /** Returns list of states to which user have group access. */
    public Hashtable listMyStates(String filter) throws Exception {
        if(filter != null)
            filter = filter.replace('*', '%').replace('?', '_');
        recordTime();
        NetDBConnection con = checkWhat(STATE);

        try {
            DBCall call = new StateListCommand(this, new Integer((String)user.get("id")), filter);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns list of all states . */
    public Hashtable listStates() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(STATE);

        try {
            DBCall call = new StateListCommand(this, null, null);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns list of networks to which user have group access. */
    public Hashtable listMyNetworks() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(IPADDRSPACE);

        try {
            DBCall call = new ListUserNetworks(this, new Integer((String)user.get("id")));
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    public Hashtable   listMyDomains() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(DOMAIN);

        try {
            DBCall call = new DomainListCommand(this, new Integer((String)user.get("id")));
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    public Hashtable listMyGroups(String filter) throws Exception, NetDBException {
        recordTime();
        if( filter == null )
            filter = "%";

        NetDBConnection con = checkWhat(USER);

        try {
            DBCall call =
                new ListUserGroups(this, filter.replace('*', '%').replace('?', '_'),
                                   new Integer((Utils.nStr(user.get("id")))));

            Hashtable result = call.processResultSet(con.callDB(call));
            checkForDBErrors("Failed to verify address space", result);
            return result;
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    public Hashtable verifyAddressSpace(String prefix, Integer idNet) throws Exception, NetDBException {
        NetDBConnection con = getConnection();

        try {
            DBCall call = new AddrSpaceVerifyCommand(this, prefix, idNet);
            checkForDBErrors("Failed to verify address space", call.processResultSet(con.callDB(call)));
            return new Hashtable();
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns address spaces for user of this session */
    public Hashtable listMyAddrSpaces() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(IPADDRSPACE);

        try {
            DBCall call = new AddrSpaceListCommand(this, new Integer((String)user.get("id")));
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns all address spaces for user of this session */
    public Hashtable listAddrSpaces() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(IPADDRSPACE);

        try {
            DBCall call = new AddrSpaceListCommand(this, null);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns all available dhcp services */
    public Hashtable listDHCPServices() throws Exception {
        recordTime();
        // check is here itself is bogus but we need to get db connection
        NetDBConnection con = checkWhat(NETWORK);

        try {
            DBCall call = new DHCPServiceListCommand(this);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /** Returns list of all records, a.k.a. access groups */
    public Hashtable listRecords() throws Exception {
        recordTime();
        NetDBConnection con = checkWhat(IPADDRSPACE);

        try {
            DBCall call = new RecordsListCommand(this);
            return call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    }

    /**
     * given ip address (string representaion) generates hexademal name used in dhcp ip addresses,
     * 'salt' can be used to generation slitly different name.
     */
    public Hashtable generateHexName(String ip, int salt) {
        recordTime();
        Hashtable result = new Hashtable();
        String[] parts = ip.split("\\.");
        String name = "DN";
        for( int i = 0; i < parts.length; i++ ) {
            parts[i] = "0" + Integer.toHexString(Integer.parseInt(parts[i])+salt);
            name += parts[i].substring(parts[i].length() - 2);
        }
        result.put("name", name);
        long ipL = Utils.ipstr2long(ip);

        try {
            String special_addr_space = System.getProperty("special.addr.space");
            if( special_addr_space == null ) {
                result.put("domain", user.get("default_domain"));
                return result;
            }
            String[] aParams = special_addr_space.trim().split("\\ +");
            for( int i = 0; i < aParams.length; i += 2 ) {
                if( i+1 < aParams.length ) {
                    String sas = aParams[i];
                    String sdomain = aParams[i+1];

                    long as_ip = Utils.ipstr2long(sas);
                    int prefixLength = Integer.parseInt(sas.substring(sas.indexOf("/")+1));
                    long as_bcast = as_ip + (((long)1 << ((32 - prefixLength)))) - 1;
                    if( ipL >= as_ip && ipL <= as_bcast ) {
                        result.put("domain", sdomain);
                        break;
                    }
                }
            }
            if( result.get("domain") == null )
                result.put("domain", user.get("default_domain"));

        } catch (Exception ex) {
            error("Failed to give proper domain to a hex name. "+ex.getMessage());
            result.put("domain", user.get("default_domain"));
        }
        return result;
    }

    /**
     * Returns internal state of a particular instance of NetDB object.
     * Format of reuslting Hashtable is to be defined.
     */
    public Hashtable getState() {
        recordTime();
        Hashtable state = new Hashtable();
        state.put("timestamp",        timestamp+"");
        state.put("last_method_call", last_method_call);
        state.put("last_call_at",     last_method_call_at+"");
        state.put("release version",  Constants.RELEASE_VERSION);
        state.put("release name",     Constants.RELEASE_NAME);
        state.put("release id",       Constants.RELEASE_ID);
        state.put("build date",       Constants.BUILD_DATE);
        return state;
    } // end of getState()

    public void reloadConfig() throws Exception {
        recordTime();
        reloadConfigInt(false);
    }

    /**
     * Realoads config file, if 'init' is true then it is done only once per
     * JVM instance.
     */
    static void reloadConfigInt(boolean init) throws Exception {
        synchronized(init_config) {
            if( init )
                if( init_config.booleanValue() )
                    return;
                else
                    init_config = new Boolean(true);
            else
                init_config = new Boolean(true);
            //  also try to load jdbc driver
            FileInputStream in = null;
            String exceptionString = null;
            Properties system_props = System.getProperties();
            try {
                // Load the properties from a file.
                netdb_props = new Properties();
                in = new FileInputStream(System.getProperty("config.file",
                                                            Constants.NETDB_PROPERTIES_FILENAME));
                netdb_props.load(in);

            } catch (IOException ioex) {
                ioex.printStackTrace();
                exceptionString = ioex.getMessage();

            } finally {
                try {
                    if (in != null)
                        in.close();
                } catch (Exception ex) { }
            }

            if( exceptionString != null )
                throw new Exception(exceptionString);

            // override by system properties
            Enumeration enm = system_props.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                netdb_props.put(key, system_props.get(key));
            }

            // now override system properties
            enm = netdb_props.keys();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                system_props.put(key, netdb_props.get(key));
            }

            // syslog related data
            if( logger == null )
                logger = new DefaultLogger();
            if( Boolean.getBoolean(DO_SYSLOG_KEY) ) {
                logger.set(DO_SYSLOG_KEY, new Boolean(true));
                logger.set(SYSLOG_PORT_KEY,
                           new Integer(netdb_props.getProperty(SYSLOG_PORT_KEY, "514")));
                logger.set(SYSLOG_HOST_KEY,
                           netdb_props.getProperty(SYSLOG_HOST_KEY, "localhost"));
                logger.set(SYSLOG_FACILITY_KEY,
                           netdb_props.getProperty(SYSLOG_FACILITY_KEY, "local3"));
                logger.set(SYSLOG_PRIORITY_KEY,
                           netdb_props.getProperty(SYSLOG_PRIORITY_KEY, "info"));
            }

            // db_props
            db_props.put("user",
                         netdb_props.getProperty(DBCONNECTION_USER_KEY, "netdb_client"));
            db_props.put("password",
                         netdb_props.getProperty(DBCONNECTION_PASSWORD_KEY, "password"));
            db_props.put("USE_METADATA", "false");
            db_props.put("CANCEL_ALL", "false");
            db_props.put("IGNORE_DONE_IN_PROC", "true");
            db_props.put("APPLICATIONNAME", "ctisql"); // don't remember what it is for

            // dbs_props
            dbs_props.put("user",
                          netdb_props.getProperty(DBCONNECTION_SEARCH_USER_KEY, "netdb_search"));
            dbs_props.put("password",
                          netdb_props.getProperty(DBCONNECTION_SEARCH_PASSWORD_KEY, "password"));
            dbs_props.put("USE_METADATA", "false");
            dbs_props.put("CANCEL_ALL", "true");
            dbs_props.put("QUERY_TIMEOUT_CANCELS_ALL", "true");
            dbs_props.put("IGNORE_DONE_IN_PROC", "true");
            dbs_props.put("APPLICATIONNAME", "ctisql"); // don't remember what it is for

            db_url = netdb_props.getProperty(DBCONNECTION_URL_KEY, "");
            logger.info("config loaded");

            String dirProviderClassName = System.getProperty(DIR_PROVIDER_CLASS_KEY);
            if( dirProviderClassName != null ) {
                dirProviderClass = Class.forName(dirProviderClassName);
                ((DirectoryProvider)dirProviderClass.newInstance()).init(logger);
            }

            ptr_pref = Utils.parsePTRprefConfigString();
        }
    } // end of reloadConfig()

    // does nothing, exists just to satisfy Logger interface
    public void set (String what, Object value) {
    }

    /**
     * establishes db connection as a regular (netdb_client) user and returns NetDBConnection,
     * which is a wrapping class above java.sql.Connection
     */
    NetDBConnection getConnection() throws Exception {

        Connection con = null;
        if( !use_connection_pool )
            con = DriverManager.getConnection(db_url, db_props);
        else
            throw new Exception("db connection pool is not implemented yet.");
        return new NetDBConnection(this, con, db_url, db_props);
    } // end of getConnection()

    /**
     * establishes db connection as a search (netdb_search) user and returns NetDBConnection,
     * which is a wrapping class above java.sql.Connection
     */
    NetDBConnection getSearchConnection() throws Exception {
        Connection con = null;
        if( !use_connection_pool )
            con = DriverManager.getConnection(db_url, dbs_props);
        else
            throw new Exception("db connection pool is not implemented yet.");
        return new NetDBConnection(this, con, db_url, dbs_props);
    } // end of getSearchConnection()

    public synchronized boolean isTimeout() {
        try {
            // 15 min. default
            long timeout = Long.parseLong(netdb_props.getProperty(TIMEOUT_KEY, "900000"));
            if( (Calendar.getInstance().getTime().getTime() - last_call) > timeout )
                return true;
            else
                return false;
        } catch (Exception ex) {
            return false;
        }
    } // end of isTimeout()

    private synchronized void recordTime() {
        last_call = Calendar.getInstance().getTime().getTime();
        StackTraceElement[] st = (new Throwable()).getStackTrace();
        String call = Utils.nStr(st[1].getMethodName());
        if( !call.equals("getState") ) {
            last_method_call = call;
            last_method_call_at = Calendar.getInstance().getTime().getTime();
        }
    }

    /**
     * check that 'what' lies withing acceptable limit and return db
     * connection for normal (netdb_cilent) user or throw exception.
     */
    NetDBConnection checkWhat(int what)
        throws Exception
    {
        switch( what ) {
        case NODE:
        case MX:
        case USER:
        case NETWORK:
        case NET_ONLY:
        case GROUP:
        case ATEAM:
        case DOMAIN:
        case OS:
        case MAKE:
        case MODEL:
        case MODEL_TYPE:
        case DEPARTMENT:
        case LOCATION:
        case IPADDRSPACE:
        case PERSON:
        case DHCP_OPTIONS:
        case STATE:
            return getConnection();
        default:
            throw new Exception("NetDB::checkWhat(...) - unsupported 'what' type.");
        }
    } // end of checkWhat(...)

    /**
     * check that 'what' lies withing acceptable limit and return db
     * connection for normal (netdb_cilent) user or throw exception.
     */
    NetDBConnection checkQuickSearchWhat(int what)
        throws Exception, InternalException {
        switch( what ) {
        case NODE:
        case MX:
        case USER:
        case NETWORK:
        case NET_ONLY:
        case GROUP:
        case ATEAM:
        case DOMAIN:
            return getSearchConnection();
        default:
            throw new InternalException("NetDB::checkQuickSearchWhat(...) - unsupported 'what' type.");
        }
    } // end of checkQuickSearchWhat(...)

    /** used for debugging */
    public static String const2str(int num) {
        switch( num ) {
        case NODE:              return "NODE";
        case NETWORK:           return "NETWORK";
        case USER:              return "USER";
        case GROUP:             return "GROUP";
        case ATEAM:             return "ATEAM";
        case DOMAIN:            return "DOMAIN";
        case OS:                return "OS";
        case MAKE:              return "MAKE";
        case MODEL:             return "MODEL";
        case DEPARTMENT:        return "DEPARTMENT";
        case LOCATION:          return "LOCATION";
        default:
            return "*unknown*";
        }
    }

    /**
     * converts obj.get("object") which is a string to internal integer representation,
     * like 'node' -> NODE, etc
     */
    public static int obj2what(String objectStr) throws Exception {
        if(       objectStr.equals("node") || objectStr.equals("nodes") )
            return NODE;
        else if ( objectStr.equals("network") )
            return NETWORK;
        else if ( objectStr.equals("domain") )
            return DOMAIN;
        else if ( objectStr.equals("user") )
            return USER;
        else if ( objectStr.equals("group") )
            return GROUP;
        else if ( objectStr.equals("ateam") )
            return ATEAM;
        else if ( objectStr.equals("os") )
            return OS;
        else if ( objectStr.equals("make") )
            return MAKE;
        else if ( objectStr.equals("model") )
            return MODEL;
        else if ( objectStr.equals("location") )
            return LOCATION;
        else if ( objectStr.equals("department") )
            return DEPARTMENT;
        else if ( objectStr.equals("person") )
            return PERSON;
        else if ( objectStr.equals("state") )
            return STATE;
        else
            throw new Exception("Uknown object type");
    }

    void checkForDBErrors(String headerMsg, Hashtable dbResult) throws Exception, NetDBException {
        // check for errors
        Vector error_set = (Vector)dbResult.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            Vector errors = new Vector();
            for( int i = 0; i < error_set.size(); i++ )
                errors.addElement(error_set.elementAt(i).toString().trim());
            throw new NetDBException(headerMsg, errors);
        }
    } // end of checkForDBErrors(...)

    public void removeReservedName (Hashtable name) {
        synchronized (reservedNames) {
            try {
                String nameH = (String)reservedNames.remove(name);
                if( nameH != null )
                    debug("removed name "+name.get("fqdn")+" from list of reserved");
                else
                    debug("failed to remove name "+name.get("fqdn")+" from list of reserved");
            } catch (Exception ex) {
                debug("failed to remove name "+name.get("fqdn")+" from list of reserved");
            }
        }
    }

    /** logs debug message */
    public void debug(Object msg) {
        doLog(Constants.DEBUG, msg.toString());
    }

    /** logs warning message */
    public void warn(Object msg) {
        doLog(Constants.WARN, msg.toString());
    }

    /** logs error message */
    public void error(Object msg) {
        doLog(Constants.ERROR, msg.toString());
    }

    /** logs info message */
    public void info(Object msg) {
        doLog(Constants.INFO, msg.toString());
    }

    /** Does actuall logging */
    private void doLog(int level, String msg) {
        StackTraceElement[] st = (new Throwable()).getStackTrace();
        switch( level ) {
        case Constants.DEBUG:
            logger.debug(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                         st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.WARN:
            logger.warn(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                        st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.ERROR:
            logger.error(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                         st[2].getLineNumber()+" | "+msg);
            break;

        case Constants.INFO:
            logger.info(timestamp+" "+st[2].getClassName()+"::"+st[2].getMethodName()+"() "+
                        st[2].getLineNumber()+" |  "+msg);
            break;
        }
    }

    public SystemParameter selectSystemParameter(String paramName)
        throws Exception, SQLException, InvocationTargetException, IllegalAccessException
    {
        NetDBConnection con = checkWhat(NODE);
        SystemParameterCommand call = null;

        try {
            call = new SystemParameterCommand(this, paramName);
            return call.packResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }
    } // end of selectSystemParameter(...)


    public void register_lock(java.sql.Timestamp tm, Integer what, String handle, Integer id) {
        synchronized (locks) {
            Hashtable lock = new Hashtable();
            lock.put("lock",   tm);
            lock.put("what",   what);
            String key = "";
            if( handle != null ) {
                lock.put("handle", handle);
                key = "handle="+handle;
            }
            if( id != null ) {
                lock.put("id", id);
                key += "id="+id;
            }

            locks.put(key+" "+what, lock);
        }
    } // end of register_lock(...)

    // clean registered locks of given lock
    public void unregister_lock(java.sql.Timestamp tm, Integer what) {
        debug("unregister_lock("+tm+", "+what+")");
        Vector keys_to_remove = new Vector();
        java.sql.Timestamp one_day_ago =
            new java.sql.Timestamp(Calendar.getInstance().getTime().getTime() - 86400000);
        synchronized (locks) {
            Enumeration enm = locks.keys();
            while( enm.hasMoreElements() ) {
                Object key = enm.nextElement();
                Hashtable lock = (Hashtable)locks.get(key);
                java.sql.Timestamp lock_tm = (java.sql.Timestamp)lock.get("lock");
                if( lock_tm.equals(tm) && lock.get("what").equals(what) )
                    keys_to_remove.addElement(key);
                else if( lock_tm.before(one_day_ago) ) {
                    // remove stale locks, note that time zone of
                    // java server should be the same as a db server
                    // or bad things may happen
                    keys_to_remove.addElement(key);
                }
            }

            for( int i = 0; i < keys_to_remove.size(); i++ )
                locks.remove(keys_to_remove.elementAt(i));
        }
    } // end of unregister_lock(...)

    // ensure proper amount of node tags (aka. custom fields)
    private void check_node_tags(Hashtable node, Integer custom_fields_total) throws NetDBException {
        if( Utils.nHash(node.get("tags")).size()  > custom_fields_total.intValue() )
            throw (new NetDBException("Not a valid object. See attached error Vector for details.")).
                add("Node may not have more than "+custom_fields_total+" tags");
    } // end of check_node_tags(...)

    // set ptr pref values associated with ip addresses to default value where applicable
    private void set_ptr_prefs(Hashtable node) {
        // set ptr pref to default value on all ipc ip addresses
        Iterator it = Utils.nHash(node.get("ipc_addresses")).values().iterator();
        while( it.hasNext() )
            ((Hashtable)it.next()).
                put("PTRpref", System.getProperty("DefaultPTRpref", "0"));

        // we do not alter ptr pref values if node is advanced
        if( Utils.nHash(node.get("types")).get("Advanced") != null )
            return;

        // if node is not advanced then all ptr prefs are set to default value
        it = Utils.nHash(node.get("interfaces")).values().iterator();
        while( it.hasNext() ) {
            Hashtable intfc = (Hashtable)it.next();
            Iterator ips = Utils.nHash(intfc.get("ips")).values().iterator();
            while( ips.hasNext() )
                ((Hashtable)ips.next()).
                    put("PTRpref", System.getProperty("DefaultPTRpref", "0"));
        }
    } // end of set_ptr_prefs(...)

    // called in node info and fills in abbreviated ptr pref name
    private Hashtable fill_in_ptr_pref_strings(Hashtable node) {
        Iterator it = Utils.nHash(node.get("interfaces")).values().iterator();
        while( it.hasNext() ) {
            Hashtable intfc = (Hashtable)it.next();
            Iterator ips = Utils.nHash(intfc.get("ips")).values().iterator();
            while( ips.hasNext() ) {
                Hashtable ip = (Hashtable)ips.next();
                ip.put("PTRprefAbbr", Utils.nStr(Utils.nHash(ptr_pref.get(Utils.nStr(ip.get("PTRpref")))).get("abbreviation")));
            }
        }
        return node;
    } // end of fill_in_ptr_pref_strings

    protected Integer verify_or_die (int what, String handle) throws Exception, NetDBException
    {
        recordTime();
        DBCall call = new UtilVerifyOrDieCommand (this, handle, what);
        NetDBConnection con = getConnection();
        Hashtable result = new Hashtable();
        try {
            result = call.processResultSet(con.callDB(call));
        } finally {
            try {
                con.releaseConnection();
            } catch (Exception ex) {}
        }

        Vector error_set = (Vector)result.get("errors");
        if( error_set != null && error_set.size() > 0 ) {
            String ex_msg = "";
            for( int i = 0; i < error_set.size(); i++ )
                ex_msg += error_set.elementAt(i).toString().trim()+" ";
            throw new NetDBException(ex_msg, error_set);
        }

        return new Integer (((BigDecimal)result.get (RETURN_VALUE)).intValue ());
    } // end of verify_or_die(...)
}
