/************************************************************************
 *
 * NetDBHook.java - this is the only publicly accessible class.
 *
 * $Id: NetDBHook.java,v 1.4 2007-01-04 00:55:30 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.lang.*;
import stanford.netdb.utils.*;

/**
 * This class is used to create an instance of NetDBpi (public interface) which
 * works as a handle to the database. There is no other way to obtain instance
 * of NetDBpi.
 * <br><br>
 * Author: Dmitri Priimak<br>
 * Version: $Id: NetDBHook.java,v 1.4 2007-01-04 00:55:30 priimak Exp $
 */
public class NetDBHook {

    /**
     * Creating instance of NetDBpi involves loading user profile from the database and
     * verifying that it is an active user, if so then instance of NetDBpi is created and
     * returned to the caller. With each instance of NetDB associated timestamp which is
     * used for logging and can be used for debugging and enumeration purposes. You may
     * retrieve it by calling {@link NetDBpi#getState() NetDBpi.getState()}.
     * You should hold and reuse this handle throughout your
     * session ( if you maintain session at all ). Returned instance of NetDBpi will keep
     * track of names and ips reservations and when destroyed will cancel reserved but not
     * accepted names or/and ip addresses. Note, that as of now it will not unlock
     * previously locked objects at the end of the session.
     *
     * @param NetID network id of the netdb user, in case of Stanford it is SUNetID.
     * @param god the value of this parameter does not have any implications in this
     * library and if set to true results only in placing in a user Hashtable value
     * Boolean(true) under the key "god" and you are free to use it the way you like.
     * Corresponding copy of user hashtable could be obtained by calling
     * <a href=NetDBpi.html#getCurrentUser()>NetDBpi.getCurrentUser()</a>
     * @throws Exception in case if user does not exist in a database or is not
     * active; appropriate message stored in a thrown Exception
     * @see stanford.netdb.lib.NetDBpi
     */
    public static NetDBpi createInstance(String NetID, boolean god) throws Exception {
        NetDB.reloadConfigInt(true);
        NetDB netdb;
        try {
            netdb = new NetDB();
            netdb.NetID = NetID;
            netdb.user = netdb.login(NetID);
        } catch (Exception ex) {
            throw new Exception(ex+"<br><font class='attn'>If you see this message please submit HelpSU ticket.</font>");
        }
        if( netdb.user == null || netdb.user.get("NetID") == null )
            throw new Exception("Person '"+NetID+"' is not a valid NetDB user.");
        if( !((Boolean)netdb.user.get("active?")).booleanValue() )
            throw new Exception("Person '"+NetID+"' is not an active NetDB user.");
        if( god )
            netdb.user.put("god", new Boolean(true));
        netdb.defs = netdb.loadDefaults();
        netdb.custom_fields_total = new Integer(Constants.MAX_CUSTOM_FIELDS);
        try {
            if( netdb.defs.get("MaxNodeCustomFields") != null )
                netdb.custom_fields_total = (Integer)netdb.defs.get("MaxNodeCustomFields");
        } catch (Exception exx) { }
        return netdb;
    }
}
