/************************************************************************
 *
 * NewDirProvider.java -
 *
 * $Id: NewDirProvider.java,v 1.2 2006-09-26 17:50:33 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.util.*;
import java.lang.*;
import java.util.regex.*;
import stanford.netdb.utils.*;
import javax.security.auth.kerberos.*;
import javax.security.auth.login.*;
import javax.security.auth.Subject;
import javax.naming.directory.*;
import javax.naming.*;

class NewDirProvider implements DirectoryProvider {

    private static Logger common_logger = null;
    private Logger logger = null;

    private static LoginContext lc = null;
    private static String lock = "";
    private static String dn = "cn=People,dc=Stanford,dc=edu";

    public static void main(String[] cmd) {
        try {
            if( cmd.length < 2 )
                System.out.println("Usage: $ NewDirProvider (-n NetID|-r RegID|-l Lookup)");
            else {
                NewDirProvider dir = new NewDirProvider();
                dir.init(new DefaultLogger());
                if( cmd[0].equals("-n") )
                    Utils.printHash(dir.getByUID(cmd[1]));
                else if( cmd[0].equals("-r") )
                    Utils.printHash(dir.getByKey(cmd[1]));
                else if( cmd[0].equals("-l") )
                    Utils.printHash(dir.lookup(cmd[1]));
                else
                    System.out.println("Usage: $ NewDirProvider (-n NetID|-r RegID|-l Lookup)");
            }
        } catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    public NewDirProvider() { }

    public boolean init(Logger logger) {
        common_logger = logger;

        try {
            lc = new LoginContext("NewDirProvider");
            return true;
        } catch (Exception ex) {
            error(ex.getMessage());
        }
        return false;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public boolean connect(NetDBpi netdb) {
        return true;
    }

    public boolean disconnect() {
        return true;
    }

    public Hashtable lookup(String searchString) throws Exception {
        return doLookup(searchString, "lookup");
    }

    public Hashtable getByKey(String key) throws Exception {
        return doLookup(key, "byKey");
    }

    public Hashtable getByUID(String uid) throws Exception {
        return doLookup(uid, "byUID");
    }

    public Hashtable verify(Hashtable person)
        throws Exception, IllegalArgumentException, DirectoryProvider.DirectoryException {
        if( person == null )
            throw new IllegalArgumentException("Argument must not be null");

        Object regid = person.get("RegID");
        if( regid == null )
            throw new IllegalArgumentException("Persons hashtale must contain key 'RegID'.");

        if( !(regid instanceof String) )
            throw new IllegalArgumentException("Value of 'RegID' is not a String.");

        String regID = ((String)regid).trim();
        if( regID.length() == 0 )
            throw new IllegalArgumentException("Key 'RegID' must have non empty value.");

        Hashtable retval = getByKey(regID);
        if( retval.size() == 0 ) {
            Hashtable return_person = new Hashtable();
            return_person.put("warning", "Did not find person in a directory.");
            return return_person;
        }

        if( retval.size() > 1 ) {
            Hashtable return_person = new Hashtable();
            return_person.put("warning", "Found more then one person in a directory.");
            return return_person;
        }

        return Utils.nHash(retval.get(retval.keys().nextElement()));
    } // end of verify(...)

    private Hashtable doLookup(String searchString, String sType) throws Exception {
        String filter = "()";
        if( sType.equals("byUID") )
            filter = "(|(suSUNetID="+searchString.trim()+")(uid="+searchString.trim()+"))";
        else if( sType.equals("byKey") )
            filter = "(suRegID="+searchString.trim()+")";
        else if( sType.equals("lookup") ) {
            filter = makeLookupFilter(searchString.trim());
        } else
            throw new Exception("Illegal search type");

        synchronized(lock) {
            if( lc == null )
                relogin("LoginContext is null");
            Subject subj = lc.getSubject();
            if( subj == null )
                relogin("Subject is null");
            subj = lc.getSubject();
            Set cr = subj.getPrivateCredentials();
            KerberosTicket tk = null;
            Iterator it = cr.iterator();
            while( it.hasNext() ) {
                Object obj = it.next();
                if( obj instanceof javax.security.auth.kerberos.KerberosTicket ) {
                    tk = (javax.security.auth.kerberos.KerberosTicket)obj;
                    break;
                }
            }
            Date end = tk.getEndTime();
            if( end.before(new Date()) )
                relogin("Ticked expired");
        }

        NamingEnumeration answer =
            (NamingEnumeration)Subject.doAs(lc.getSubject(),
                                            new JndiSearchAction(filter));

        if (answer == null)
            return new Hashtable();

        Hashtable people = new Hashtable();
        try {
            while (answer != null && answer.hasMore()) {
                String displayName = null;
                String suDisplayNameLF = null;
                String regID = null;
                String lastName = "";
                String firstName = "";
                Hashtable entry = new Hashtable();
                SearchResult sr = (SearchResult)answer.next();
                Attributes attrs = sr.getAttributes();

                String dirID = null;
                boolean visible = false;
                boolean active = false;
                Hashtable groups = new Hashtable();
                for (NamingEnumeration ae = attrs.getAll(); ae.hasMore();) {
                    javax.naming.directory.Attribute attr =
                        (javax.naming.directory.Attribute)ae.next();
                    String key = attr.getID();
                    NamingEnumeration enm = attr.getAll();
                    while( enm.hasMore() ) {
                        String value = ((String)enm.next()).trim();
                        if( key.equals("displayName") ) {
                            displayName = value;
                            entry.put("name", value);
                        } else if( key.equals("suRegID") ) {
                            regID = value;
                            entry.put("RegID", regID);
                        } else if( key.equals("telephoneNumber") )
                            entry.put("phone", value);
                        else if( key.equals("mail") )
                            entry.put("email", value);
                        else if( key.equals("uid") )
                            entry.put("NetID", value);
                        else if( key.equals("suUniqueIdentifier") )
                            dirID = value;
                        else if( key.equals("givenName") )
                            firstName = value;
                        else if( key.equals("suPrivilegeGroup") ) {
                            if( value.equals("stanford:stanford") ||
                                value.equals("stanford:administrative") )
                                active = true;
                            groups.put(value, "");
                        } else if( key.equals("suVisibIdentity") ) {
                            value = value.toLowerCase();
                            if( value.equals("world") || value.equals("stanford") )
                                visible = true;
                        } else if( key.equals("ou") )
                            entry.put("department", value);
                        else if( key.equals("sn") ) {
                            if( entry.get("lastname") != null )
                                continue;
                            lastName = value;
                            String fc = lastName.substring(0, 1).toUpperCase();
                            String rest = "";
                            if( lastName.length() > 1 )
                                rest = lastName.substring(1);
                            entry.put("lastname", fc+rest);
                        } else if( key.equals("suDisplayNameLF") )
                            suDisplayNameLF = value;
                    }
                }

                entry.put("groups", groups);
                entry.put("active?", new Boolean(active));

                if( visible )
                    entry.put("visible", "true");

                if( dirID != null && visible )
                    entry.put("DirID", dirID);

                if( regID != null ) {
                    if( suDisplayNameLF != null )
                        people.put(suDisplayNameLF.toLowerCase()+" "+regID.trim(), entry);
                    else
                        people.put(lastName+" "+firstName+" "+regID.trim(), entry);
                }
            }
        } catch (Exception ex) {
            error(ex.getMessage());
        }

        // prefill 'object' -> 'person'
        Enumeration enm = people.keys();
        while( enm.hasMoreElements() ) {
            Object person = people.get(enm.nextElement());
            if( person instanceof Hashtable ) {
                ((Hashtable)person).put("object", "person");
            }
        }

        return people;
    } // end of doLookup(...)

    private String makeLookupFilter(String s) {

        String first, last;

        if (s.indexOf("(") != -1 || s.indexOf(")") != -1) return("");

        // rotate about the first comma if necessay
        if (s.indexOf(",") != -1) s = s.substring(s.indexOf(",")+1,s.length())
                                    + " " + s.substring(0,s.indexOf(","));

        // trim
        s = s.trim();

        // return filter for 'fisrt last' or just 'string'
        if (s.indexOf(" ") != -1) {
            first = s.substring(0,s.indexOf(" "));
            last  = s.substring(s.indexOf(" "),s.length()).trim();
            return("(|(&(sn=" +last+ ")(gn=" +first+ "))" +
                   "(displayName=" +first+ " " +last+ ")" +
                   "(suOtherName=" +first+ " " +last+ "))");
        } else
            return("(|(uid=" +s+ ")(suSUNetID=" +s+ ")(suGeneralID=" +s+ ")(sn=" +s+ "))");
    } // end of makeLookupFilter(...)

    private void relogin(String msg) {
        debug("relogin: "+msg);
        // try three times
        for( int i = 0; i < 3; i++ ) {
            try {
                lc = new LoginContext("NewDirProvider");
                lc.login();
                break;
            } catch (Exception ex) {
                error(ex.getMessage());
            }
        }
    }

    private void error(Object msg) {
        if( logger == null ) {
            synchronized( common_logger ) {
                common_logger.error(msg);
            }
        } else
            logger.error(msg);
    }

    private void debug(Object msg) {
        if( logger == null ) {
            synchronized( common_logger ) {
                common_logger.debug(msg);
            }
        } else
            logger.debug(msg);
    }

    class JndiSearchAction implements java.security.PrivilegedExceptionAction {
        private String filter;

        public JndiSearchAction(String filter) {
            this.filter = filter;
        }

        public Object run() throws Exception{
            return performJndiOperation();
        }

        private synchronized NamingEnumeration performJndiOperation() throws Exception {
            NamingEnumeration answer = null;
            DirContext ctx = null;
            for( int i = 0; i < 5; i++) {
                try {
                    Hashtable local = new Hashtable();
                    local.put(Context.INITIAL_CONTEXT_FACTORY,
                              "com.sun.jndi.ldap.LdapCtxFactory");
                    local.put(Context.PROVIDER_URL,
                              urlConvert(System.getProperty("dir.provider.url")));
                    local.put(Context.SECURITY_AUTHENTICATION, "GSSAPI");
                    local.put("javax.security.sasl.qop", "auth-conf");

                    ctx = new InitialDirContext(local);
                    debug("dir search = ["+filter+"]");
                    answer = ctx.search(dn, filter, new SearchControls());
                    ctx.close();

                }  catch (NamingException ex) {
                    debug(ex.toString());
                    throw ex;

                } finally {
                    if( ctx != null ) {
                        try {
                            ctx.close();
                        } catch (Exception exx) {
                            error(exx.getMessage());
                        }
                    }
                }
                return answer;
            }
            return null;
        }

        private String urlConvert(String url) {
            Matcher m = Pattern.compile("(^.*://)(.*)(:.*)").matcher(url);
            if( !m.matches() ) {
                return url;
            }

            try {
                return m.group(1)+
                    org.xbill.DNS.Address.getByName(m.group(2)).getHostAddress()+
                    m.group(3);
            } catch (Exception ex) {
                error(ex.getMessage());
            }
            return url;
        }
    } // end of class JndiSearchAction
}
