/************************************************************************
 *
 * Node2ModifyCommand.java -
 *
 * $Id: Node2ModifyCommand.java,v 1.2 2008-03-15 02:56:12 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2007, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import stanford.netdb.lib.*;
import java.util.*;
import java.lang.*;
import java.sql.*;
import su.netdb.parser.*;

/** modify two nodes in the same transaction */
class Node2ModifyCommand extends DBCall implements ComplexDBCall {

    private NetDBii netdb;

    private Integer op_uid;

    private Hashtable oldNode1;
    private Hashtable newNode1;
    private Hashtable oldNode2;
    private Hashtable newNode2;

    private String    idNode1;
    private String    idNode2;

    public  Hashtable acceptedIPs   = new Hashtable();
    public  Hashtable acceptedNames = new Hashtable();

    public Node2ModifyCommand(NetDBii netdb,
                              Hashtable oldNode1, Hashtable newNode1,
                              Hashtable oldNode2, Hashtable newNode2,
                              Integer op_uid)
        throws Exception
    {
        super((stanford.netdb.utils.Logger)netdb);

        this.netdb    = netdb;
        this.op_uid   = op_uid;

        this.oldNode1 = oldNode1;
        this.newNode1 = newNode1;

        this.oldNode2 = oldNode2;
        this.newNode2 = newNode2;

        idNode1 = (String)this.newNode1.get("id");
        idNode2 = (String)this.newNode2.get("id");
    }

    // This class shouldn't inherit from DBCall, but it does...
    public String getSQL() {
        return "";
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        boolean commit_finally = false;
        BeginTransaction(con.con);

        try {
            // first node
            NodeModifyCommand nmc =
                new NodeModifyCommand(netdb, oldNode1, newNode1, op_uid);

            for( Hashtable name : (Set<Hashtable>)nmc.acceptedNames.keySet() )
                acceptedNames.put(name, "");
            Utils.appendHash(acceptedIPs, nmc.acceptedIPs);

            checkErrors("Failed to update node", nmc.doIt(con));

            if( !nmc.commit_finally )
                throw new Exception("Failed to update node");

            // second node
            nmc = new NodeModifyCommand(netdb, oldNode2, newNode2, op_uid);

            for( Hashtable name : (Set<Hashtable>)nmc.acceptedNames.keySet() )
                acceptedNames.put(name, "");
            Utils.appendHash(acceptedIPs, nmc.acceptedIPs);

            checkErrors("Failed to update node", nmc.doIt(con));

            if( !nmc.commit_finally )
                throw new Exception("Failed to update node");

            commit_finally = true;

        } finally {
            if( commit_finally ) {
                try {
                    logger.info("modifying nodes id "+idNode1+", "+idNode2+" done");
                    logger.debug("CommitTransaction");
                    CommitTransaction(con.con);
                } catch(Exception exx) { }

                // remove accepted ips from list of reserved
                Enumeration enm = acceptedIPs.keys();
                while( enm.hasMoreElements() )
                    try {
                        netdb.unregister_reserved_ip(((String)enm.nextElement()).trim());
                    } catch (Exception ex) {}

            } else
                try {
                    logger.info("modifying nodes id "+idNode1+", "+idNode2+" failed");
                    logger.debug("RollbackTransaction");
                    RollbackTransaction(con.con);
                } catch(Exception exx) { }
        }

        return new Hashtable();
    }
}
