/************************************************************************
 *
 * OSModifyCommand.java - db call NetDB_OS.master 'update'
 *
 * $Id: OSModifyCommand.java,v 1.4 2008-03-15 02:56:17 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;

/** db call NetDB_OS.master 'update' */
class OSModifyCommand extends DBCall {

    private static final int OP_USER_INDEX     = 0; // p_op_uid
    private static final int OS_NAME_INDEX     = 1; // p_os
    private static final int OS_NEW_NAME_INDEX = 2; // p_os_new
    private static final int DESCRIPTION_INDEX = 3; // p_description

    private String oldName;
    private String newName;

    public OSModifyCommand(stanford.netdb.utils.Logger logger,
                           Hashtable oldOS, Hashtable newOS,
                           Integer op_uid)
    {
        super(logger);

        oldName = (String)oldOS.get ("name");
        newName = (String)newOS.get ("name");

        if (oldName.equals (newName)) // update
        {
            db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_OS, Function_Map.UPDATE);

            values = new Object[4];
            values[OP_USER_INDEX]     = op_uid;
            values[OS_NAME_INDEX]     = oldOS.get("name");
            values[OS_NEW_NAME_INDEX] = newOS.get("name");
            values[DESCRIPTION_INDEX] = Utils.nStr(newOS, "comment").trim();
        }
        else // supplant
        {
            db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_OS, Function_Map.SUPPLANT);

            values = new Object[3];
            values[OP_USER_INDEX]     = op_uid;
            values[OS_NAME_INDEX]     = oldOS.get("name");
            values[OS_NEW_NAME_INDEX] = newOS.get("name");
        }
    }

    public String getSQL() {
        int function = Function_Map.UPDATE;
        if (! oldName.equals (newName))
            function = Function_Map.SUPPLANT;

        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_OS, function));
    }
}
