/************************************************************************
2 *
 * ObjectGroupCommand.java - inteface to db call NetDB_Owner.add/remove
 *
 * $Id: ObjectGroupCommand.java,v 1.2 2006-10-25 04:18:35 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** inteface to db call NetDB_Owner.add/remove */
class ObjectGroupCommand extends DBCall implements ComplexDBCall {

    public static final String ADD_GROUPS     = "add";
    public static final String REMOVE_GROUPS  = "remove";

    private static final int ID_OP_USER_INDEX = 0; // p_op_uid
    private static final int LOCK_INDEX       = 1; // p_lock
    private static final int GROUP_ID_INDEX   = 2; // p_owner_id
    private static final int GROUP_NAME_INDEX = 3; // p_owner_name
    private static final int OBJECT_ID_INDEX  = 4; // p_record_id
    private static final int TYPE_INDEX       = 5; // p_record_type

    public static final String NODE           = "node";
    public static final String USER           = "user";
    public static final String NETWORK        = "network";
    public static final String ATEAM          = "adminteam";
    public static final String DOMAIN         = "domain" ;
    public static final String DOMAIN_ASSIGN  = "domain_assign" ;
    public static final String DOMAIN_USE     = "domain_use" ;
    public static final String IPADDRSPACE    = "addressspace";

    private Vector groups;
    private String action;

    /**
     * Constructor:
     * @param action - String = { "insert", "delete" }, it is recommended to use
     * predefined constats ADD_GROUPS and REMOVE_GROUPS.
     * @param oid    - id of the object we are manipulating with
     * @param type   - ObjectGroupCommand.{NODE, USER, NETWORK, ATEAM, DOMAIN, IPADDRSPACE}
     * @param lock   - lock (timestamp) of parent object
     * @param groups - vector of group labels
     * @param op_uid - id of operating user.
     *
     */
    public ObjectGroupCommand(stanford.netdb.utils.Logger logger, String action,
                              int oid, String type, Timestamp lock, Vector groups,
                              Integer op_uid)
    throws InternalException, Exception {
        super(logger);
        this.groups              = groups;
        this.action              = action;
        db_call_type             = ORA_FUN_IGNORE_RET_CALL;

        values                   = new Object[6];
        values[ID_OP_USER_INDEX] = op_uid;
        values[LOCK_INDEX]       = lock;
        values[TYPE_INDEX]       = type;
        values[OBJECT_ID_INDEX]  = new Integer(oid);
        values[GROUP_ID_INDEX]   = new Null(Types.NUMERIC);
    }

    public String getSQL() {
        String dbschema = System.getProperty(Constants.DBSCHEMA_KEY);
        dbschema = (dbschema==null||dbschema.trim().length()==0)?"":(dbschema.trim()+".");
        return "{call "+dbschema+"NetDB_Owner."+action+"(?, ?, ?, ?, ?, ?)}";
    }

    public Hashtable doIt(NetDBConnection con) throws NetDBException, Exception {
        for( int i = 0; i < groups.size(); i++ ) {
            values[GROUP_NAME_INDEX] = groups.elementAt(i);

            Hashtable result = con.callDB(this);

            // check for errors
            Vector error_set = (Vector)result.get("errors");
            if( error_set != null && error_set.size() > 0 )
                return result;
        }
        return new Hashtable();
    }
}
