/************************************************************************
 *
 * PersonInsertUpdateCommand.java - db call NetDB_Person.master 'insert'
 *
 * $Id: PersonInsertUpdateCommand.java,v 1.3 2008-04-24 05:32:08 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.lang.*;
import java.util.*;
import java.sql.*;

/** db call NetDB_Person.master 'insert' */
class PersonInsertUpdateCommand extends DBCall {

    private static final int PERSON_ID_INDEX       = 1; // p_person_id
    private static final int REG_ID_PERSON_INDEX   = 2; // p_person_regid
    private static final int NAME_INDEX            = 3; // p_name
    private static final int NET_ID_INDEX          = 4; // p_netid
    private static final int EMAIL_INDEX           = 5; // p_email
    private static final int PHONE_INDEX           = 6; // p_phone
    private static final int ACTIVE_INDEX          = 7; // p_active
    private static final int LOWER_LAST_NAME_INDEX = 8; // p_lastname
    private static final int DIR_ID_INDEX          = 9; // p_rirectoryid

    public PersonInsertUpdateCommand(stanford.netdb.utils.Logger logger, Hashtable person, Integer op_uid) {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_PERSON, Function_Map.INSERT);

        values = new Object[10];
        values[0]                   = ref_cursor;
        values[PERSON_ID_INDEX]     = o2n(null, Types.NUMERIC);
        values[REG_ID_PERSON_INDEX] = person.get("RegID"); // person *must* have RegID

        if( person.get("name") != null )
            values[NAME_INDEX] = ((String)person.get("name")).trim();
        else
            values[NAME_INDEX] = new Null (Types.VARCHAR);

        if( person.get("NetID") != null )
            values[NET_ID_INDEX] = ((String)person.get("NetID")).trim();
        else
            values[NET_ID_INDEX] = new Null (Types.VARCHAR);

        if( person.get("email") != null )
            values[EMAIL_INDEX] = ((String)person.get("email")).trim();
        else
            values[EMAIL_INDEX] = new Null (Types.VARCHAR);

        if( person.get("phone") != null )
            values[PHONE_INDEX] = ((String)person.get("phone")).trim();
        else
            values[PHONE_INDEX] = new Null (Types.VARCHAR);

        if( person.get("active?") == null )
            values[ACTIVE_INDEX] = new Short((short)0); // presume inactive
        else
            values[ACTIVE_INDEX] = new Short( ((Boolean)person.get("active?")).booleanValue()?(short)1:(short)0 );

        if( person.get("lastname") != null )
            values[LOWER_LAST_NAME_INDEX] = ((String)person.get("lastname")).trim().toLowerCase();
        else
            values[LOWER_LAST_NAME_INDEX] = new Null (Types.VARCHAR);

        if( person.get("DirID") != null )
            values[DIR_ID_INDEX] = ((String)person.get("DirID")).trim();
        else
            values[DIR_ID_INDEX] = new Null (Types.VARCHAR);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_PERSON, Function_Map.INSERT));
    }
}
