/************************************************************************
 *
 * RecordsByOwnerFullSearchCommand.java -
 *
 * $Id: RecordsByOwnerFullSearchCommand.java,v 1.2 2007-10-26 21:01:17 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2007, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.text.*;
import java.util.*;

class RecordsByOwnerFullSearchCommand extends FullSearchCommand {

    private static String[] views =
    { "FS_Related_Owner_AdminTeam", "FS_Related_Owner_Domain",
      "FS_Related_Owner_User",      "FS_Related_Owner_Node",
      "FS_Related_Owner_Network",   "FS_Related_Owner_ResName" };

    public RecordsByOwnerFullSearchCommand(stanford.netdb.utils.Logger logger,
                                           Hashtable search_params) {
        super(logger, search_params);
    }

    public String getSearchSQL(String view) throws Exception {
        search_params.put("fs.table",  view);

        return "SELECT id \"ID\", name \"handle\", type"+
            " FROM "+view+" WHERE ID IN ( SELECT unique(id) FROM "+
            assmCommonSQL(search_params) + " ) ";
    }

    public String getSearchSQL() throws Exception {
        String sql = "";

        for( String view : views )
            sql += ( sql.length() > 0 ? "\nUNION ALL\n" : "" ) +
                getSearchSQL(view);

        return sql;
    }

    // this search needs custom picker
    public Hashtable processResultSet(Hashtable res) throws Exception {
        if( res.get("errors") != null )
            return Utils.hash("errors", res.get("errors"));

        Vector column_names = (Vector)res.get("column_names");

        Hashtable result =
            Utils.hash("Admin Teams", new Hashtable(),
                       "Domains",     new Hashtable(),
                       "Users",       new Hashtable(),
                       "Nodes",       new Hashtable(),
                       "Networks",    new Hashtable(),
                       "ResNames",    new Hashtable());
        Vector result_set = (Vector)res.get("resultSet");

        for( Vector row : (Vector<Vector>)result_set ) {
            String id = row.elementAt(0).toString();
            String type = row.elementAt(2).toString();

            Utils.get_hash_or_create(Utils.get_hash_or_create(result, type), id).
                put(row.elementAt(1).toString(), "");
        }

        // now go second time and prune uneeded names
        Hashtable new_res = new Hashtable();
        for( Map.Entry records_ent : (Set<Map.Entry>)result.entrySet() ) {
            Object type       = records_ent.getKey();
            Hashtable records = (Hashtable)records_ent.getValue();

            Hashtable new_records = new Hashtable();
            new_res.put(type, new_records);

            for( Hashtable record : (Collection<Hashtable>)records.values() ) {
                if( record.size() == 1 )
                    new_records.put(record.keys().nextElement(), "");
                else {
                    Iterator it = Utils.sortedKeys(record, string_cmp);
                    while( it.hasNext() ) { new_records.put(it.next(), ""); break; }
                }
            }
        }
        result.clear();

        return new_res;
    }
}