/************************************************************************
 *
 * SampleDirProvider.java -
 *
 * $Id: SampleDirProvider.java,v 1.5 2008-05-23 18:52:35 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.util.regex.*;
import stanford.netdb.utils.*;

public class SampleDirProvider implements DirectoryProvider {

    private Logger logger = null;

    public SampleDirProvider() { }

    public static void main(String[] cmd) { }

    public boolean init(Logger logger) throws Exception, DirectoryProvider.DirectoryException {
        return true;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public boolean connect(NetDBpi netdb) throws Exception, DirectoryProvider.DirectoryException {
        return true;
    }

    public boolean disconnect() throws Exception, DirectoryProvider.DirectoryException {
        return true;
    }

    public Hashtable lookup(String searchString) throws Exception, DirectoryProvider.DirectoryException {
        return lookup(null, null, searchString);
    }

    public Hashtable getByKey(String key) throws Exception, DirectoryProvider.DirectoryException {
        return lookup(key, null, null);
    }

    public Hashtable getByUID(String uid) throws Exception, DirectoryProvider.DirectoryException {
        return lookup(null, uid, null);
    }

    public Hashtable verify(Hashtable person)
        throws Exception, IllegalArgumentException, DirectoryProvider.DirectoryException {
        if( person == null )
            throw new IllegalArgumentException("Argument must not be null");

        Object regid = person.get("RegID");
        if( regid == null )
            throw new IllegalArgumentException("Persons hashtale must contain key 'RegID'.");

        if( !(regid instanceof String) )
            throw new IllegalArgumentException("Value of 'RegID' is not a String.");

        String regID = ((String)regid).trim();
        if( regID.length() == 0 )
            throw new IllegalArgumentException("Key 'RegID' must have non empty value.");

        Hashtable retval = getByKey(regID);
        if( retval.size() == 0 ) {
            Hashtable return_person = new Hashtable();
            return_person.put("warning", "Did not find person in the directory.");
            return return_person;
        }

        if( retval.size() > 1 ) {
            Hashtable return_person = new Hashtable();
            return_person.put("warning", "Found more then one person in the directory.");
            return return_person;
        }

        return Utils.nHash(retval.get(retval.keys().nextElement()));
    } // end of verify(...)

    private Hashtable lookup(String key, String uid, String searchString) {
        int index = 0;
        Hashtable result = new Hashtable();
        BufferedReader file = null;
        try{
            Pattern p = Pattern.compile("^(\\S+):(.*)$");

            Hashtable person = new Hashtable();
            file = new BufferedReader( new FileReader(System.getProperty("directory.file")) );
            String line;
            boolean body = false;
            while( ( line = file.readLine() ) != null ){
                line = line.trim();
                if( line.equals("begin") ) {
                    body = true;
                    continue;
                }

                if( !body ) continue;

                if( line.equals("person") ) {
                    if( person.size() > 0 && match(person, key, uid, searchString) ) {
                        person.put("object", "person");
                        result.put(Utils.nStr(person.get("NetID"))+" "+
                                   Utils.nStr(person.get("RegID")), person);
                        if( key != null || uid != null )
                            return result;
                    }
                    person = new Hashtable();
                    continue;
                }

                if( line.equals("end") ) {
                    if( person.size() > 0 && match(person, key, uid, searchString) ) {
                        person.put("object", "person");
                        result.put(Utils.nStr(person.get("NetID"))+" "+
                                   Utils.nStr(person.get("RegID")), person);
                    }
                    body = false;
                    break;
                }

                Matcher m = p.matcher(line);
                if( !m.matches() ) continue;

                String pkey = m.group(1);
                if( pkey.endsWith("?") )
                    person.put(pkey, new Boolean(m.group(2).trim()));
                else
                    person.put(pkey, m.group(2).trim());
            }

        } catch (Exception ex) {
            ex.printStackTrace();

        } finally {
            try {
                file.close();
            } catch (Exception ex) {
            }
        }

        return result;
    }

    private boolean match(Hashtable person, String key, String uid, String searchString) {
        if( key != null && Utils.nStr(person.get("RegID")).equals(key) ) return true;
        if( uid != null && Utils.nStr(person.get("NetID")).equals(uid) ) return true;

        if( searchString != null && searchString.trim().length() > 0 &&
            Utils.nStr(person.get("name")).toLowerCase().matches("^.*"+searchString.trim().toLowerCase()+".*$") )
            return true;

        return false;
    }
}
