/************************************************************************
 *
 * StateModifyCommand.java - db call NetDB_State.master 'modify'
 *
 *
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2005, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import stanford.netdb.lib.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** db call NetDB_State.master 'modify' */
class StateModifyCommand extends DBCall {

    private static final int ID_OP_USER_INDEX     = 0;  // p_op_uid
    private static final int STATE_NAME_INDEX     = 1;  // p_state
    private static final int STATE_NEW_NAME_INDEX = 2;  // p_state_new
    private static final int STATE_RANK_INDEX     = 3;  // p_rank
    private static final int DENY_DHCP_INDEX      = 4;  // p_denyDHCP
    private static final int DENY_DNS_INDEX       = 5;  // p_denyDNS
    private static final int DESCRIPTION_INDEX    = 6;  // p_description
    private static final int ACCESS_GROUP_INDEX   = 7;  // p_accessgroup

    /**
     * Constructor for StateModifyCommand class.
     *
     * @param oldState   old state object to be modified
     * @param newState   new state object
     * @param op_uid     id of operating user
     */
    public StateModifyCommand(stanford.netdb.utils.Logger logger, Hashtable oldState,
                              Hashtable newState, Integer op_uid) {
        super(logger);

        db_call_type = ORA_FUN_IGNORE_RET_CALL;
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_STATE, Function_Map.UPDATE);

        values = new Object[8];
        values[ID_OP_USER_INDEX]        = op_uid;
        values[STATE_NAME_INDEX]        = oldState.get("name");
        values[STATE_NEW_NAME_INDEX]    = newState.get("name");
        if( newState.get("rank") != null )
            values[STATE_RANK_INDEX]    = newState.get("rank");

        if( newState.get("denyDHCP") != null )
            values[DENY_DHCP_INDEX]     = new Short( ((Boolean)newState.get("denyDHCP")).booleanValue()?"1":"0" );
        else
            values[DENY_DHCP_INDEX]     = 0;
            
        if( newState.get("denyDNS") != null )
            values[DENY_DNS_INDEX]      = new Short( ((Boolean)newState.get("denyDNS")).booleanValue()?"1":"0" );
        else
            values[DENY_DNS_INDEX]      = 0;

        if((newState.get("description")!=null) &&
           ((String)newState.get("description")).trim().length() > 0 )
            values[DESCRIPTION_INDEX]   = newState.get("description");
        else
            values[DESCRIPTION_INDEX]   = o2v(null);

        if((newState.get("access")!=null) &&
          ((String)newState.get("access")).trim().length() > 0 )
            values[ACCESS_GROUP_INDEX]  = newState.get("access");
        else
            values[ACCESS_GROUP_INDEX]  = new Null (Types.VARCHAR);
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_STATE, Function_Map.UPDATE));
    }
}
