/************************************************************************
 *
 * ATeamDepartmentsCommand.java - db call NetDB_AdminTeam.org
 *
 * $Id: UtilVerifyOrDieCommand.java,v 1.1 2008-03-15 02:56:21 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.lib;

import stanford.netdb.utils.*;
import java.util.*;
import java.lang.*;
import java.sql.*;

/** This class is used to add and remove departments to/from admin team. */
class UtilVerifyOrDieCommand extends DBCall {

    private static final int ID_INDEX             = 1; // p_record_id
    private static final int HANDLE_INDEX         = 2; // p_handle
    private static final int RECORD_TYPE_INDEX    = 3; // p_record_type
    private static final int NAME_TYPE_INDEX      = 4; // p_name_type
    private static final int CHECK_RESERVED_INDEX = 5; // check_reserved

    public UtilVerifyOrDieCommand(stanford.netdb.utils.Logger logger,
                                  Integer ID, String  handle, Integer record_type) throws Exception, IllegalArgumentException {
        this(logger, ID, handle, record_type, null, null);
    }

    public UtilVerifyOrDieCommand(stanford.netdb.utils.Logger logger,
                                  String  handle, Integer record_type) throws Exception, IllegalArgumentException {
        this(logger, null, handle, record_type, null, null);
    }

    public UtilVerifyOrDieCommand(stanford.netdb.utils.Logger logger,
                                  Integer ID, Integer record_type) throws Exception, IllegalArgumentException {
        this(logger, ID, null, record_type, null, null);
    }

    public UtilVerifyOrDieCommand(stanford.netdb.utils.Logger logger,
                                  Integer ID, String  handle, Integer record_type,
                                  String  name_type, Boolean check_reserved) throws Exception, IllegalArgumentException {
        super(logger);
        db_call_type = Function_Map_Controller.get_function_type (Function_Map.PACKAGE_UTIL, Function_Map.VERIFY_OR_DIE);

        values = new Object[6];
        values[0]                    = new OutParam(Types.NUMERIC);
        values[ID_INDEX]             = o2n(ID, Types.NUMERIC);
        values[HANDLE_INDEX]         = o2n(handle, Types.VARCHAR);
        values[NAME_TYPE_INDEX]      = o2n(name_type, Types.VARCHAR);
        values[CHECK_RESERVED_INDEX] = o2n(check_reserved, Types.TINYINT);

        switch (record_type)
        {
            case NetDB.ADVANCED_NODE:       values[RECORD_TYPE_INDEX] = "node";         break;
            case NetDB.ATEAM:               values[RECORD_TYPE_INDEX] = "adminteam";    break;
            case NetDB.DEPARTMENT:          values[RECORD_TYPE_INDEX] = "organization"; break;
            case NetDB.DHCP_OPTIONS:        values[RECORD_TYPE_INDEX] = "dhcpoption";   break;
            case NetDB.DHCP_SERVICE:        values[RECORD_TYPE_INDEX] = "dhcpservice" ; break;
            case NetDB.DOMAIN:              values[RECORD_TYPE_INDEX] = "domain";       break;
            case NetDB.GROUP:               values[RECORD_TYPE_INDEX] = "owner";        break;
            case NetDB.INTERFACE:           values[RECORD_TYPE_INDEX] = "interface";    break;
            case NetDB.IP:                  values[RECORD_TYPE_INDEX] = "address";      break;
            case NetDB.IPADDRSPACE:         values[RECORD_TYPE_INDEX] = "addressspace"; break;
            case NetDB.LOCATION:            values[RECORD_TYPE_INDEX] = "location";     break;
            case NetDB.MAKE:                values[RECORD_TYPE_INDEX] = "make";         break;
            case NetDB.MX:                  values[RECORD_TYPE_INDEX] = "mx";           break;
            case NetDB.NETWORK:             values[RECORD_TYPE_INDEX] = "network";      break;
            case NetDB.NODE:                values[RECORD_TYPE_INDEX] = "node";         break;
            case NetDB.OS:                  values[RECORD_TYPE_INDEX] = "os";           break;
            case NetDB.PERSON:              values[RECORD_TYPE_INDEX] = "person";       break;
            case NetDB.STATE:               values[RECORD_TYPE_INDEX] = "state";        break;
            case NetDB.USER:                values[RECORD_TYPE_INDEX] = "user";         break;
            default:
                throw new Exception("Unknown record_type");
        }
    }

    public String getSQL() {
        return proc_call_sql(Function_Map_Controller.get_function_name (Function_Map.PACKAGE_UTIL, Function_Map.VERIFY_OR_DIE));
    }
}
