/************************************************************************
 *
 * CNVParser.java - parser for custom node view datafiles
 *
 * $Id: CNVParser.java,v 1.15 2008-04-03 18:24:41 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.text.*;
import java.util.regex.*;
import java.util.*;
import java.io.*;

public class CNVParser extends Object {

    private static boolean printout_resuling_hash = false;
    private static Hashtable data                 = null;

    public static void main(String[] args) {
        if( args.length == 0 ) {
            System.out.
                println("Custom Node View Parser\n"+
                        "Usage: $ ./cnv_parser [-h] <view_data_file_1> <view_data_file_2> ... <view_data_file_N>\n"+
                        "-h - will result in printout of generated hash data structure\n\n");
            System.exit(1);
        }

        String data_files = "";
        for( int i = 0; i < args.length; i++ ) {
            if( args[i].equals("-h") ) {
                printout_resuling_hash = true;
                continue;
            }
            data_files += (data_files.length()>0?" ":"")+args[i].trim();
        }
        System.setProperty("custom.node.modify.views", data_files);

        try {
            Hashtable result = (new CNVParser()).loadCustomNodeViewsData();
            if( result.get("error") != null )
                throw new Exception((String)result.get("error"));
            if( printout_resuling_hash )
                Utils.printHash(result);
            else
                System.out.println("Parsing complete.");
            System.exit(0);


        } catch (Exception ex) {
            System.out.println("ERROR: "+ex.getMessage()+"\n");
            System.exit(1);
        }
    }

    public Hashtable loadCustomNodeViewsData() throws FileNotFoundException, Exception {
        String[] data_files = System.getProperty("custom.node.modify.views", "").split("\\s+");

        data = new Hashtable();

        Pattern section_def_p       = Pattern.compile("^\\s*(\\S+)\\s*(\\S.*)?$");
        Pattern selector_p          = Pattern.compile("^\\s*selector\\{\\s*$");
        Pattern size_p              = Pattern.compile("^\\s*size:\\s*(\\d+)\\s*$");
        Pattern multiple_p          = Pattern.compile("^\\s*multiple:\\s*([yY][eE][sS]|[nN][oO])\\s*$");
        Pattern allow_p             = Pattern.compile("^\\s*allow-node-types:(.*)$");
        Pattern disallow_p          = Pattern.compile("^\\s*disallow-node-types:(.*)$");
        Pattern section_p           = Pattern.compile("^\\s*section:(.+)$");
        Pattern name_p              = Pattern.compile("^\\s*name:(.+)$");
        Pattern separator_p         = Pattern.compile("^\\s*separator:(.+)$");
        Pattern value_p             = Pattern.compile("^\\s*value:\\s*(\\S+)\\s+([\\S\\s]+?)\\s*$");
        Pattern validate_p          = Pattern.compile("^\\s*validate:(.+)$");
        Pattern default_value_p     = Pattern.compile("^\\s*default-value:(.+)$");
        Pattern save_default_p      = Pattern.compile("^\\s*save-default-value:\\s*([yY][eE][sS]|[nN][oO])\\s*$");
        Pattern html_element_p      = Pattern.compile("^\\s*html-element:\\s*(drop-down|text-field|radio|checkbox|label|selector\\..*)\\s*$");
        Pattern field_length_p      = Pattern.compile("^\\s*field-length:\\s*(\\d+)\\s*$");
        Pattern element_inject_p    = Pattern.compile("^\\s*html-inject:(.+)$");
        Pattern display_label_p     = Pattern.compile("^\\s*display-label:(.+)$");
        Pattern info_screen_label_p = Pattern.compile("^\\s*info-screen-label:(.+)$");
        Pattern display_on_info_p   = Pattern.compile("^\\s*display-on-info:\\s*([yY][eE][sS]|[nN][oO])\\s*$");
        Pattern regex_p             = Pattern.compile("^\\s*regex:(.+)$");
        Pattern block_p             = Pattern.compile("^\\s*([^{}\\s]+)\\s*\\{\\s*$");

        Pattern heredoc_p           = Pattern.compile("^(.*)<<(\\S+)\\s*$");

        for( int i = 0; i < data_files.length; i++ ) {
            BufferedReader file = null;
            try {
                file = new BufferedReader(new FileReader(data_files[i]));
                String line;

                int line_num            = 0;
                Hashtable entry         = new Hashtable();
                Hashtable tdata         = new Hashtable();
                Hashtable sections      = null;
                Hashtable section       = null;
                Hashtable item          = null;

                Utils.get_hash_or_create(tdata, "allow node types");
                Utils.get_hash_or_create(tdata, "disallow node types");

                Hashtable selector_data = new Hashtable();
                Hashtable entries       = new Hashtable();
                tdata.put("entries", entries);

                String  block = "top";
                String  hdoc_label      = null;
                String  hdoc_partial    = null;
                boolean found_view_name = false;
                boolean found_allow     = false;
                boolean found_disallow  = false;

                while( ( line = file.readLine() ) != null ) {
                    line = line.trim();
                    line_num++;

                    if( hdoc_label != null ) {
                        if( line.startsWith(hdoc_label) && line.trim().equals(hdoc_label) ) {
                            line       = hdoc_partial;
                            hdoc_label = null;
                        } else {
                            hdoc_partial += line;
                            continue;
                        }
                    }

                    Matcher heredoc_m = heredoc_p.matcher(line);
                    if( heredoc_m.matches() && hdoc_label == null ) {
                        hdoc_label   = heredoc_m.group(2);
                        hdoc_partial = heredoc_m.group(1).trim();
                    }

                    if( line.equals("") || line.matches("^\\s*#.*$") ) // skip comments and empty lines
                        continue;

                    if( !block.equals("top") && line.matches("^\\s*\\}\\s*$") ) { // end of the block
                        if( block.equals("section") ) {
                            sections.put(sections.size()+"", section);
                            block = "sections";
                            continue;
                        }

                        if( block.equals("selector") ) {
                            block = "top";
                            continue;
                        }

                        if( block.equals("item") ) {
                            block = "selector";
                            continue;
                        }

                        if( block.equals("record") ) {
                            entry.put("id", ""+entries.size());
                            entries.put(""+entries.size(), entry);

                            // give name to html elements of type 'label'
                            if( Utils.nStr(entry.get("html element")).equals("label") )
                                entry.put("name", Utils.nStr(entry.get("display label")));

                            // check entry for all proper parts been present
                            if( entry.get("section") == null ||
                                     entry.get("section").toString().trim().length() == 0 )
                                data.put("error", "Missing \"section:\". Line "+line_num+
                                         " in "+data_files[i]+" data file.");
                            else if( entry.get("name") == null ||
                                     entry.get("name").toString().trim().length() == 0 )
                                data.put("error", "Missing \"name:\". Line "+line_num+
                                         " in "+data_files[i]+" data file.");
                            else if( entry.get("display label") == null ||
                                     entry.get("display label").toString().trim().length() == 0 )
                                data.put("error", "Missing \"display-label:\". Line "+line_num+
                                         " in "+data_files[i]+" data file.");

                            entry = new Hashtable();
                        }
                        block = "top"; // return to the "top"
                        continue;
                    }

                    if( block.equals("top") ) {
                        Matcher allow_m = allow_p.matcher(line);
                        if( allow_m.matches() ) {
                            String[] allows = allow_m.group(1).trim().split("\\s+");
                            Hashtable allows_node_types = Utils.get_hash_or_create(tdata, "allow node types");
                            for( int j = 0; j < allows.length; j++ )
                                if( allows[j].length() != 0 )
                                    allows_node_types.put(allows[j], "");
                            continue;
                        }

                        Matcher disallow_m = disallow_p.matcher(line);
                        if( disallow_m.matches() ) {
                            String[] disallows = disallow_m.group(1).trim().split("\\s+");
                            Hashtable disallow_node_types = Utils.get_hash_or_create(tdata, "disallow node types");
                            for( int j = 0; j < disallows.length; j++ )
                                if( disallows[j].length() != 0 )
                                    disallow_node_types.put(disallows[j], "");
                            continue;
                        }

                        if( line.matches("^\\s*view-name:\\s+[^_]+$") ) {
                            found_view_name = true;
                            data.put(line.substring(line.indexOf("view-name:")+11).trim(), tdata);
                            continue;
                        }

                        if( line.matches("^\\s*help-url:.*$") ) {
                            String url = Utils.nStr(line.substring(line.indexOf("help-url:")+9)).trim();
                            if( url.length() > 0 ) tdata.put("help url", url);
                            continue;
                        }

                        Matcher block_m = block_p.matcher(line);
                        if( block_m.matches() ) {
                            block = block_m.group(1).trim();
                            if( block.equals("selector") ) {
                                selector_data = Utils.hash("separator", ":", "html element", "selector");
                                entries.put(entries.size()+"", selector_data);
                            }
                            continue;
                        }

                        // we are here if at the top level of parsing we encountered unknown element
                        data.put("error", "Unable to parse line "+line_num+" in "+data_files[i]+" data file.");
                        break;

                    } else if( block.equals("selector") ) {
                        Matcher display_label_m = display_label_p.matcher(line);
                        Matcher size_m          = size_p.matcher(line);
                        Matcher multiple_m      = multiple_p.matcher(line);
                        Matcher section_m       = section_p.matcher(line);
                        Matcher name_m          = name_p.matcher(line);
                        Matcher separator_m     = separator_p.matcher(line);
                        if( display_label_m.matches() )
                            selector_data.put("display label", display_label_m.group(1).trim());
                        else if( name_m.matches() )
                            selector_data.put("name", name_m.group(1).trim());
                        else if( separator_m.matches() )
                            selector_data.put("separator", separator_m.group(1).trim());
                        else if( size_m.matches() )
                            selector_data.put("size", size_m.group(1).trim());
                        else if( multiple_m.matches() )
                            selector_data.put("multiple", multiple_m.group(1).trim());
                        else if( section_m.matches() )
                            selector_data.put("section", section_m.group(1).trim());
                        else {
                            Matcher block_m = block_p.matcher(line);
                            if( block_m.matches() ) {
                                block = block_m.group(1).trim();
                                if( !block.equals("item") ) {
                                    data.put("error", "Unable to parse line "+line_num+
                                             " in "+data_files[i]+" data file.");
                                    break;
                                }
                                Hashtable items = Utils.get_hash_or_create(selector_data, "items");
                                item = new Hashtable();
                                items.put(items.size()+"", item);
                                continue;
                            }
                            data.put("error", "Unable to parse line "+line_num+
                                      " in "+data_files[i]+" data file.");
                            break;
                        }

                    } else if( block.equals("record") ) {
                        Matcher section_m           = section_p.matcher(line);
                        Matcher name_m              = name_p.matcher(line);
                        Matcher value_m             = value_p.matcher(line);
                        Matcher validate_m          = validate_p.matcher(line);
                        Matcher default_value_m     = default_value_p.matcher(line);
                        Matcher save_default_m      = save_default_p.matcher(line);
                        Matcher html_element_m      = html_element_p.matcher(line);
                        Matcher element_inject_m    = element_inject_p.matcher(line);
                        Matcher field_length_m      = field_length_p.matcher(line);
                        Matcher display_label_m     = display_label_p.matcher(line);
                        Matcher info_screen_label_m = info_screen_label_p.matcher(line);
                        Matcher display_on_info_m   = display_on_info_p.matcher(line);

                        if(      section_m.matches() )
                            entry.put("section", section_m.group(1).trim());
                        else if( name_m.matches() )
                            entry.put("name", name_m.group(1).trim());
                        else if( value_m.matches() ) {
                            Hashtable values = Utils.get_hash_or_create(entry, "values");
                            values.put(values.size()+"",
                                       Utils.hash("name",  value_m.group(1).trim(),
                                                  "label", value_m.group(2).trim()));
                        } else if( validate_m.matches() )
                            Utils.get_hash_or_create(entry, "validate").put(validate_m.group(1).trim(), "");
                        else if( default_value_m.matches() )
                            entry.put("default value", default_value_m.group(1).trim());
                        else if( save_default_m.matches() )
                            entry.put("save default",  save_default_m.group(1).trim());
                        else if( html_element_m.matches() ) {
                            String html_element = html_element_m.group(1).trim();
                            entry.put("html element", html_element);
                            if( html_element.startsWith("selector.") )
                                entry.put("selector", html_element.substring(9));
                        } else if( field_length_m.matches() )
                            entry.put("field length", field_length_m.group(1).trim());
                        else if( element_inject_m.matches() )
                            entry.put("inject", element_inject_m.group(1).trim());
                        else if( display_label_m.matches() )
                            entry.put("display label", display_label_m.group(1).trim());
                        else if( info_screen_label_m.matches() )
                            entry.put("info label", info_screen_label_m.group(1).trim());
                        else if( display_on_info_m.matches() )
                            entry.put("display on info screen", display_on_info_m.group(1).trim());
                        else {
                            data.put("error", "Unable to parse line "+line_num+
                                      " in "+data_files[i]+" data file.");
                            break;
                        }
                        continue;

                    } else if( block.equals("section") ) {
                        Matcher name_m          = name_p.matcher(line);
                        Matcher display_label_m = display_label_p.matcher(line);
                        Matcher regex_m         = regex_p.matcher(line);

                        if( name_m.matches() )
                            section.put("name", name_m.group(1).trim());
                        else if( display_label_m.matches() )
                            section.put("display label", display_label_m.group(1).trim());
                        else if( regex_m.matches() )
                            section.put("regex", regex_m.group(1).trim());

                        continue;

                    } else if( block.equals("item") ) {
                         Matcher name_m          = name_p.matcher(line);
                         Matcher display_label_m = display_label_p.matcher(line);
                         Matcher default_value_m = default_value_p.matcher(line);

                         if( name_m.matches() )
                             item.put("name", name_m.group(1).trim());
                         else if( display_label_m.matches() )
                             item.put("display label", display_label_m.group(1).trim());
                         else if( default_value_m.matches() )
                             item.put("default value", default_value_m.group(1).trim());

                         continue;

                    } else if( block.equals("sections") ) {
                        sections = Utils.get_hash_or_create(tdata, "sections");

                        if( line.matches("^\\s*section\\s*\\{\\s*$") ) {
                            section = new Hashtable();
                            block = "section";
                        }
                        continue;

                    } else {
                        data.put("error", "Unable to parse line "+line_num+
                                 " in "+data_files[i]+" data file.");
                        break;
                    }
                }

                if( !found_view_name )
                    data.put("error", "Failed to find view name in file \""+data_files[i]+"\".");

            } catch (Exception ex) {
                ex.printStackTrace();

            } finally {
                try {
                    file.close();
                } catch (Exception ex) {
                }
            }
        }

        try {
            validate(data);
        } catch (Exception ex) {
            ex.printStackTrace();
            data.put("error", Utils.n2s(ex.getMessage()));
        }

        if( data.get("error") != null ) {
            Object error = data.get("error");
            data = new Hashtable();
            data.put("error", error);
        }
        return data;
    } // end of loadCustomNodeViewsData()

    // ensure valid view data structure
    private void validate(Hashtable cnv) throws Exception {
        if( cnv.get("error") != null )
            return;

        Enumeration view_enm = cnv.keys();
        while( view_enm.hasMoreElements() ) {
            String view_name = (String)view_enm.nextElement();
            Object obj = cnv.get(view_name);
            Hashtable view = (Hashtable)cnv.get(view_name);

            // sections
            Hashtable sections = Utils.nHash(view.get("sections"));
            if( sections.size() == 0 )
                throw new Exception("view \""+view_name+"\" must have at least one section.");

            // selectors
            Hashtable selectors = Utils.nHash(view.get("selectors"));
            Enumeration selector_enm = selectors.keys();
            while( selector_enm.hasMoreElements() ) {
                String selector_name = (String)selector_enm.nextElement();
                Hashtable selector = (Hashtable)selectors.get(selector_name);
                String[] directives = {"size", "size:", "multiple", "multiple:",
                                       "section", "section:", "display label", "display-label:"};
                for( int i = 0; i < directives.length; i += 2 ) {
                    if( selector.get(directives[i]) == null )
                        throw new Exception("selector \""+selector_name+"\" in view \""+
                                            view_name+"\" missing \""+directives[i+1]+"\" directive.");
                }
            }

            // entries
            Hashtable entries = Utils.nHash(view.get("entries"));
            Enumeration entries_enm = entries.keys();
            while( entries_enm.hasMoreElements() ) {
                String entry_num = (String)entries_enm.nextElement();
                Hashtable entry = (Hashtable)entries.get(entry_num);

                // html-element:
                String html_element = Utils.nStr(entry.get("html element")).trim();
                if( html_element.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" found entry without html-element: directive.");

                // name:
                String name = Utils.nStr(entry.get("name")).trim();
                if( !html_element.equals("selector") && name.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" found entry without name.");

                // section:
                String section = Utils.nStr(entry.get("section")).trim();
                if( section.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" does not have \"section:\" directive.");
                else if( !foundSection(sections, section) )
                    throw new Exception("in view \""+view_name+"\" section \""+section+
                                        "\" associated with entry \""+name+"\" does not exist.");

                // display-label: is not required for selector elements
                String display_label = Utils.nStr(entry.get("display label")).trim();
                if( display_label.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" does not have \"display-label:\" directive.");

                // selector
                String selector = Utils.nStr(entry.get("selector")).trim();
                if( html_element.startsWith("selector.") ) {
                    if( selector.length() == 0 )
                        throw new Exception("in view \""+view_name+"\" entry \""+name+
                                            "\" does not have selector name.");
                    else if( selectors.get(selector) == null ) // ensure selector exists
                        throw new Exception("in view \""+view_name+"\" selector \""+selector+
                                            "\" associated with entry \""+name+"\" does not exist.");
                }

                // default-value:
                String default_value = Utils.nStr(entry.get("default value")).trim();
                if( !html_element.equals("selector") && !html_element.equals("label") &&
                    default_value.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" does not have \"default-value:\" directive.");


                // field-length:
                String field_length = Utils.nStr(entry.get("field length")).trim();
                if( html_element.equals("text-field") &&
                    field_length.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" does not have \"field-length:\" directive.");

                // save-default-value:
                String save_default = Utils.nStr(entry.get("save default")).trim();
                if( !html_element.equals("checkbox") &&
                    !html_element.equals("selector") &&
                    !html_element.equals("label")    &&
                    save_default.length() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" does not have \"save-default-value:\" directive.");

                // value:
                Hashtable values = Utils.nHash(entry.get("values"));
                if( ( html_element.equals("drop-down") || html_element.equals("radio") ) &&
                    values.size() == 0 )
                    throw new Exception("in view \""+view_name+"\" entry \""+name+
                                        "\" must have at least one \"value:\" directive.");

            }
        }
    } // end of validate(...)

    // this function looks into values for each key and tries
    // to find matching value if found it returns true, othervise false
    public static boolean foundSection(Hashtable sections, Object value) {
        Enumeration enm = sections.keys();
        while( enm.hasMoreElements() ) {
            Hashtable section = (Hashtable)sections.get(enm.nextElement());
            String regex = Utils.nStr(section.get("regex")).trim();
            if( regex.length() > 0 ) {
                if( Pattern.compile(regex).matcher(value.toString()).matches() )
                    return true;
            } else if( Utils.n2s(section.get("name")).equals(value) )
                return true;
        }
        return false;
    } // end of foundSection

    public static Hashtable getSection(Hashtable sections, Object section_name) {
        Enumeration enm = sections.keys();
        while( enm.hasMoreElements() ) {
            Hashtable section = (Hashtable)sections.get(enm.nextElement());
            String regex = Utils.nStr(section.get("regex")).trim();
            if( regex.length() > 0 ) {
                if( Pattern.compile(regex).matcher(section_name.toString()).matches() )
                    return section;
            } else if( Utils.n2s(section.get("name")).equals(section_name) )
                return section;
        }
        return null;
    } // end of foundSection

    // finds canonical section name, which is needed when regex is used to recognize sections
    public static String canonicalSection(Hashtable tags_definition, String name) {
        Hashtable sections = Utils.nHash(tags_definition.get("sections"));
        for( Hashtable section : (Collection<Hashtable>)sections.values() ) {
            String regex = Utils.nStr(section.get("regex")).trim();
            if( regex.length() > 0 ) {
                if( Pattern.compile(regex).matcher(name).matches() )
                    return (String)section.get("name");
            } else if( Utils.n2s(section.get("name")).equals(name) )
                return (String)section.get("name");
        }
        return null;
    } // end of foundSection

    public static String savedSection(Hashtable tags_definition, String name) {
        String canonical_section_name = canonicalSection(tags_definition, name);

        Hashtable sections =
            Utils.nHash(Utils.nHash(tags_definition.get("sections")));

        Hashtable section = new Hashtable();
        for( Hashtable section_h : (Collection<Hashtable>)sections.values() ) {
            if( section_h.get("name").equals(canonical_section_name) ) {
                section = section_h;
                break;
            }
        }

        String saved_name = Utils.nStr(section.get("saved name")).trim();
        if( saved_name.length() > 0 ) return saved_name;
        return Utils.nStr(section.get("name"));
    }
}
