/************************************************************************
 *
 * Hash.java - This is an utility class provided for convenience of
 *             programmers utilizing functions in netdb.jar
 *
 * $Id: Hash.java,v 1.1.1.1 2006-06-30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2001 Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.util.*;
import java.lang.*;

/**
 * This is an utility class provided for convenience of programmers
 * utilizing functions in netdb.jar
 */
public class Hash extends Hashtable{

    /**
     * Prior to putting obj into the hashtale this method will do a lot
     * of checking for data to be valid. For example, for key 'active?'
     * object must be an instance of Boolean. If object or key are not
     * valid the Exception will be thrown with appropriate mesage.
     */
    public void put(String key, Object obj) throws Exception {
        if( key == null )
            throw new Exception("Hash::put(key, obj) - 'key' is null.");

        if( obj == null )
            throw new Exception("Hash::put(key, obj) - 'obj' is null.");

        // 'active?' -> (Boolean)
        // 'roaming' -> (Boolean)
        // 'use-dhcp' -> (Boolean)
        if( key.equals("active?") ||
            key.equals("roaming") ||
            key.equals("use-dhcp") ) {
            if( !( obj instanceof Boolean ) )
                throw new Exception("Hash::put(key, obj) - for key = '"+key+"'"+
                                    " object must be instance of Boolean");
        } else
            throw new Exception("Hash::put(key, obj) - invalid key.");

        super.put(key, obj);
    } // end of put(...)

    /**
     * Returns String object for 'key', throws Exception if object is
     * not a String.
     */
    public String getStr(String key) throws Exception {
        Object obj = get(key);
        if( !( obj instanceof String ) )
            throw new Exception("Object under this key is not an instance of String");
        return (String)obj;
    } // end of getStr(...)

    /**
     * Returns Vector object for 'key', throws Exception if object is
     * not a Vector.
     */
    public Vector getVector(String key) throws Exception {
        Object obj = get(key);
        if( !( obj instanceof Vector ) )
            throw new Exception("Object under this key is not an instance of Vector");
        return (Vector)obj;
    } // end of getVector(...)

    /**
     * Returns Hashtable object for 'key', throws Exception if object is
     * not a Hashtable.
     */
    public Hashtable getHashtable(String key) throws Exception {
        Object obj = get(key);
        if( !( obj instanceof Hashtable ) )
            throw new Exception("Object under this key is not an instance of Hashtable");
        return (Hashtable)obj;
    } // end of getHashtable(...)

    /**
     * Returns Hash object for 'key', throws Exception if object is
     * not a Hash.
     */
    public Hash getHash(String key) throws Exception {
        Object obj = get(key);
        if( !( obj instanceof Hash ) )
            throw new Exception("Object under this key is not an instance of Hash");
        return (Hash)obj;
    } // end of getHash(...)
}
