/************************************************************************
 *
 * IPAddressSQLBuilder.java -
 *
 * $Id: IPAddressSQLBuilder.java,v 1.1 2007-08-03 06:26:31 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2007, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.util.*;
import su.netdb.parser.*;

public class IPAddressSQLBuilder extends FieldPostFilter {

    private String full_field_name;

    public IPAddressSQLBuilder(String full_field_name) {
        super();
        this.full_field_name = full_field_name;
    }

    public String filter(String table, String column, String value) throws Exception {
        String prefix = "";
        //" ( SELECT id FROM "+table+" WHERE ";

        try {
            Hashtable parsing_result = (new Parser()).parse(value);
            // check if it was parsed as an ip address
            String ip_low = (String)parsing_result.get("ip_low");
            if( ip_low == null )
                throw new Exception("\""+Utils.htmlize(value)+"\" is not an ip address or a range");
            String ip_high = (String)parsing_result.get("ip_high");
            if( ip_high != null ) {
                // special handling for address space search
                if( full_field_name.equals("fs.FS_AddressSpace.ip.address") )
                    return " IP.normalize('"+value+
                        "') between FS_AddressSpace.address and IP.normalize(IP.broadcast(FS_AddressSpace.address_txt))";

                if( ip_low.equals(ip_high) ) {
                    return column+" = IP.normalize('"+ip_low+"') ";
                } else // it is an ip range
                    return column+" between IP.normalize('"+ip_low+
                        "') and IP.normalize('"+ip_high+"') ";
            } else // wildcard search
                return column+"_txt like '"+ip_low+"' ";

        } catch (IllegalArgumentException ex) {
            ex.printStackTrace();
            throw ex;

        } catch (Exception ex) {
            ex.printStackTrace();
            if( value.matches("^[\\*%\\?_\\.0123456789]*$") )
                return column+"_txt like '"+value.replace('?', '_').replace('*', '%')+"' ";
            else
                throw ex;
        }
    }
}
