/************************************************************************
 *
 * Posix.java -
 *
 * $Id: Posix.java,v 1.1.1.1 2006-06-30 23:38:10 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.utils;

import java.io.*;
import java.lang.*;

public class Posix extends Object {

    private static SigTERMHandler handler = null;

    public Posix() {}

    /**
     * This function calls sigaction to set dummy
     * signal handler for SIGHUP.
     */
    public static native void disableSIGHUP();

    public native void sigterm();

    /** see 'man seteuid' */
    public static native int setuid(int uid);

    /** see 'man setegid' */
    public static native int setgid(int gid);

    /** see 'man getpid' */
    public static native int getpid();

    /** see 'man fork' */
    public static native int fork();

    /**
     * This is analog of standard unix function seteuid, but it accepts
     * user name rather then gid.
     */
    public static void setuid(String user) throws Exception {
        if( setuid(getid("/etc/passwd", user)) != 0)
            throw new Exception("failed to set euid '"+user+"'");
    }

    /**
     * This is analog of standard unix function setegid, but it accepts
     * group name rather then gid.
     */
    public static void setgid(String group) throws Exception {
        if( setgid(getid("/etc/group", group)) != 0)
            throw new Exception("failed to set gid '"+group+"'");
    }

    /**
     * get the id from /etc/passwd or /etc/group by 'name'
     * throws Exception if something goes wrong.
     */
    private static int getid(String file_name, String name) throws Exception {
        BufferedReader file = null;
        try {
            file = new BufferedReader(new FileReader(file_name));
            if( file == null )
                throw new Exception("Failed to open file "+file_name);

            String line = null;
            String ename = name+":";
            while( (line = file.readLine()) != null ) {
                if( line.startsWith(ename) ) {
                    int index = line.indexOf(':');
                    if( index < 0 )
                        throw new Exception("doesn't look like a password file");
                    index = line.indexOf(':', index+1);
                    if( index < 0 )
                        throw new Exception("doesn't look like a password file");
                    int end_index = line.indexOf(':', index+1);
                    if( end_index < 0 )
                        throw new Exception("doesn't look like a password file");
                    String uid = line.substring(index+1, end_index);
                    return Integer.parseInt(uid);
                }
            }
            throw new Exception("no such user/group '"+name+"'");
        } finally {
            try {
                file.close();
            } catch(Exception ex) { }
        }
    }

    static {
        System.loadLibrary("JPosix");
    }

    public void registerSigTERMaction(SigTERMHandler h) {
        handler = h;
    }

    private static void callback() {
        handler.handleSigTERM();
    }
}
