/************************************************************************
 *
 * PosixImpl.c -
 *
 * $Id: PosixImpl.c,v 1.2 2006-08-30 23:42:12 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
#include <jni.h>
#include "stanford_netdb_utils_Posix.h"
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>

static struct sigaction action;
static struct sigaction saction;

static struct sigaction saction;
static struct sigaction ssaction;
static jclass aobj;
static JavaVM *vm;

static void handle(int sig);

static void shandle(int sig) {
  JNIEnv *jnv;
  jint retval;
  jclass cls;
  jmethodID mid;

  if( vm == NULL )
    return;

  retval = (*vm)->AttachCurrentThread(vm, &jnv, (void *)NULL);
  if( retval != 0 )
    return;

  cls = (*jnv)->FindClass(jnv, "stanford/netdb/utils/Posix");
  if( cls == 0 )
    if ((*jnv)->ExceptionOccurred(jnv))
      (*jnv)->ExceptionDescribe(jnv);

  mid = (*jnv)->GetStaticMethodID(jnv, cls, "callback", "()V");
  if( mid == 0 )
    if ((*jnv)->ExceptionOccurred(jnv))
      (*jnv)->ExceptionDescribe(jnv);

  if( mid != 0 ) {
    (*jnv)->CallStaticVoidMethod(jnv, aobj, mid);
  }
  if ((*jnv)->ExceptionOccurred(jnv))
    (*jnv)->ExceptionDescribe(jnv);
}

JNIEXPORT void JNICALL Java_stanford_netdb_utils_Posix_disableSIGHUP
(JNIEnv *env, jclass obj) {
  // do not check that call succeeded
  // we are just trying to mask sig HUP anyway
  sigemptyset(&action.sa_mask);
  action.sa_flags = 0;
  action.sa_handler = handle;
  sigaction(SIGHUP, &action, (struct sigaction *)NULL);
}

JNIEXPORT void JNICALL Java_stanford_netdb_utils_Posix_sigterm
(JNIEnv *env, jclass obj) {
  aobj = obj;

  if( ((*env)->GetJavaVM(env, &vm)) != 0 )
    vm = NULL;

  if ((*env)->ExceptionOccurred(env)) {
    (*env)->ExceptionDescribe(env);
  }
  sigemptyset(&ssaction.sa_mask);
  ssaction.sa_flags = 0;
  ssaction.sa_handler = shandle;
  sigaction(SIGTERM, &ssaction, (struct sigaction *)NULL);
}

JNIEXPORT jint JNICALL Java_stanford_netdb_utils_Posix_setuid
(JNIEnv *env, jclass obj, jint uid) {
  return setuid(uid);
}

JNIEXPORT jint JNICALL Java_stanford_netdb_utils_Posix_setgid
(JNIEnv *env, jclass obj, jint gid) {
  return setgid(gid);
}

JNIEXPORT jint JNICALL Java_stanford_netdb_utils_Posix_getpid
(JNIEnv *env, jclass obj) {
  return getpid();
}

JNIEXPORT jint JNICALL Java_stanford_netdb_utils_Posix_fork
(JNIEnv *env, jclass obj) {
  return fork();
}

// just a dummy handler
static void handle(int sig) {
  printf("Got SIGHUP.\n");
}
