/************************************************************************
 *
 * CommonPage.java -
 *
 * $Id: CommonPage.java,v 1.21 2007-10-31 17:58:01 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public abstract class CommonPage extends HttpServlet {

    private static String nextKey = "0";

    protected static Comparator state_cmp;
    protected static Comparator order_cmp;
    protected static Comparator name_cmp;
    protected static Comparator records_cmp;
    protected static Comparator person_cmp;
    protected static Comparator ipentry_cmp;
    protected static Comparator ip_cmp;
    protected static Comparator ip_str_cmp;
    protected static Comparator num_cmp;
    protected static Comparator intfc_cmp       = null;
    protected static SimpleDateFormat sdf;
    protected static SimpleDateFormat csv_df;

    public static Comparator long_cmp           = null;

    protected static String os_msg  = "<br><b><em>&nbsp;The OS field is used for central site licensing purposes.<br>"+
        "&nbsp;Please try to use a general OS instead of creating a specific OS (i.e.  \"Mac OS X\" instead of \"Mac OS X 10.4.8\").<br>"+
        "&nbsp;Note that the operating systems do get consolidated regularly to keep the list manageable.<br>"+
        "&nbsp;If this data is vital, please use Custom Fields instead.</em></b><br><br>";

    protected static String model_msg  = "<br><b><em>&nbsp;The model field is used for central site licensing purposes.<br>"+
        "&nbsp;Please try to use a general model instead of creating a specific model (i.e.  \"HP Laserjet\" instead of \"HP Laserjet 5100n\").<br>"+
        "&nbsp;Note that the models do get consolidated regularly to keep the list manageable.<br>"+
        "&nbsp;If this data is vital, please use Custom Fields instead.";

    protected static String model_msg_full = model_msg + " For new Makes, please submit a HelpSU ticket.";


    static String synLock = "lock"; // syncronization lock

    public static Hashtable ptr_pref = new Hashtable();

    static {
        // complete messages
        model_msg      += "</em></b><br><br>";
        model_msg_full += "</em></b><br><br>";

        // use for ordering states
        state_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable state1 = (Hashtable)((Map.Entry)o1).getValue();
                    String name1 = Utils.nStr(state1.get("name"));
                    Integer rank1 = (Integer)state1.get("rank");
                    if( rank1 == null ) rank1 = new Integer(0);

                    Hashtable state2 = (Hashtable)((Map.Entry)o2).getValue();
                    String name2 = Utils.nStr(state2.get("name"));
                    Integer rank2 = (Integer)state2.get("rank");
                    if( rank2 == null ) rank2 = new Integer(0);

                    if( rank1.compareTo(rank2) == 0 )
                        return name1.compareToIgnoreCase(name2);

                    return rank1.compareTo(rank2);
                }

                public boolean equals(Object obj) {
                    return true;
                }
            };

        // this one is used in ordering of columns of full search results
        order_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable column1 = (Hashtable)o1;
                    Hashtable column2 = (Hashtable)o2;

                    // special treatment of Interface# column
                    String column_name1 = (String)column1.get("name");
                    String column_name2 = (String)column2.get("name");

                    if( column_name1.equals(column_name2) )
                        return 0;


                    // special handling of LowRes and HighRes columns in network results
                    if( column_name1.equals("LowRes") && column_name2.equals("HighRes") )
                        return -1;

                    if( column_name2.equals("LowRes") && column_name1.equals("HighRes") )
                        return 1;

                    if( ( column_name1.equals("LowRes") || column_name1.equals("HighRes") ) &&
                        column_name2.equals("AddressSpace") )
                        return 1;

                    if( ( column_name2.equals("LowRes") || column_name2.equals("HighRes") ) &&
                        column_name1.equals("AddressSpace") )
                        return -1;

                    if( column_name1.equals("All Types") && column_name2.equals("Record Type") )
                        return -1;

                    if( column_name2.equals("All Types") && column_name1.equals("Record Type") )
                        return 1;

                    int requested_place1 = Integer.parseInt((String)column1.get("requested_place"));
                    int def_place1 = Integer.parseInt((String)column1.get("def_place"));
                    int requested_place2 = Integer.parseInt((String)column2.get("requested_place"));
                    int def_place2 = Integer.parseInt((String)column2.get("def_place"));

                    if( requested_place1 == requested_place2 )
                        return def_place2<def_place1?1:-1;

                    if( requested_place1 > 99 && requested_place2 < 100 ) {
                        if( requested_place1 - 100 == requested_place2 )
                            return 1;
                        requested_place1 -= 100;
                    }

                    if( requested_place2 > 99 && requested_place1 < 100 ) {
                        if( requested_place2 - 100 == requested_place1 )
                            return -1;
                        requested_place2 -= 100;
                    }

                    return requested_place2<requested_place1?1:-1;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        intfc_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable intfc1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable intfc2 = (Hashtable)((Map.Entry)o2).getValue();

                    boolean hasIPs1 = false;
                    boolean hasIPs2 = false;
                    long ip1 = 0;
                    long ip2 = 0;

                    Hashtable ips = nHash(intfc1.get("ips"));
                    for( String IP : (Set<String>)ips.keySet() ) {
                        try {
                            long ip = Utils.ipstr2long(IP);
                            if( hasIPs1 )
                                ip1 = ip<ip1?ip:ip1;
                            else {
                                hasIPs1 = true;
                                ip1 = ip;
                            }
                        } catch (Exception ex) { }
                    }

                    ips = nHash(intfc2.get("ips"));
                    for( String IP : (Set<String>)ips.keySet() ) {
                        try {
                            long ip = Utils.ipstr2long(IP);
                            if( hasIPs2 )
                                ip2 = ip<ip2?ip:ip2;
                            else {
                                hasIPs2 = true;
                                ip2 = ip;
                            }
                        } catch (Exception ex) { }
                    }

                    if( !hasIPs1 && hasIPs2 )
                        return -1;

                    if( hasIPs1 && !hasIPs2 )
                        return 1;

                    if( !hasIPs1 && !hasIPs2 ) {
                        String i1 = (String)((Map.Entry)o1).getKey();
                        String i2 = (String)((Map.Entry)o2).getKey();

                        long v1 = 0;
                        boolean number1 = true;
                        try {
                            v1 = (new Long((String)i1)).longValue();
                        } catch (Exception ex) {
                            number1 = false;
                            try {
                                v1 = (new Long( ((String)i1).substring(((String)i1).indexOf("_")+2) )).longValue();
                            } catch(Exception exx) {
                                v1 = 0;
                            }
                        }

                        long v2 = 0;
                        boolean number2 = true;
                        try {
                            v2 = (new Long((String)i2)).longValue();
                        } catch (Exception ex) {
                            number2 = false;
                            try {
                                v2 = (new Long( ((String)i2).substring(((String)i2).indexOf("_")+2) )).longValue();
                            } catch(Exception exx) {
                                v2 = 0;
                            }
                        }

                        if( number1 && !number2 )
                            return 1;

                        if( !number1 && number2 )
                            return -1;

                        try {
                            long cmp_val =  v2 - v1;
                            return cmp_val>0?1:(cmp_val<0?-1:0);
                        } catch (Exception ex) {}

                        return 0;
                    }

                    try {
                        long cmp_val =  ip1 - ip2;
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    } catch (Exception ex) {}

                    return 0;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    return ((String)o1).compareToIgnoreCase((String)o2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        records_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    String r1 = (String)o1;
                    String r2 = (String)o2;
                    r1 = r1.trim().equals("Owner")?"Group":r1;
                    r2 = r2.trim().equals("Owner")?"Group":r2;
                    return r1.compareToIgnoreCase(r2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        person_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Object n1 = ((Hashtable)o1).get("name");
                    Object n2 = ((Hashtable)o2).get("name");
                    return ((String)n1).compareToIgnoreCase((String)n2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ipentry_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    java.util.Map.Entry m1 = (Map.Entry)o1;
                    Hashtable h1 = (Hashtable)m1.getValue();
                    String i1 = (String)h1.get("ip");

                    java.util.Map.Entry m2 = (Map.Entry)o2;
                    Hashtable h2 = (Hashtable)m2.getValue();
                    String i2 = (String)h2.get("ip");

                    return Utils.ipstr2num(i1) - Utils.ipstr2num(i2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ip_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    long cmp_val =
                        Utils.ipstr2long((String)o1) - Utils.ipstr2long((String)o2);
                    return cmp_val>0?1:(cmp_val<0?-1:0);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ip_str_cmp = new Comparator() {
                public int compare(Object ip1, Object ip2) {
                    return Utils.ip2norm((String)ip1).compareToIgnoreCase(Utils.ip2norm((String)ip2));
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        num_cmp = new Comparator() {
                public int compare(Object num1, Object num2) {
                    return (new Integer((String)num1)).compareTo(new Integer((String)num2));
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        // define format for displaying Timestamp dates
        sdf    = new SimpleDateFormat("MMM-dd-yyyy hh:mm a");
        csv_df = new SimpleDateFormat("M/dd/yy HH:mm");
    }

    public HttpSession begin(HttpServletRequest request, HttpServletResponse response, Hashtable store)
        throws ServletException, IOException
    {
        return begin(request, response, "text/html", store);
    }


    public HttpSession begin(HttpServletRequest request, HttpServletResponse response,
                         String contextType, Hashtable store)
        throws ServletException, IOException
    {
        response.setContentType(contextType);

        // no session? relogin user at home page
        HttpSession session = request.getSession(false);
        if (session == null)
        {
            response.sendRedirect("/home");
            return null;
        }

        if( store != null ) {
            if( store.get("no session store") == null ) {
                // create session store
                String sessionKey = "store "+getNextKey();
                session.setAttribute(sessionKey, store);
                store.put("sessionKey", sessionKey);
            }

            // encode twice!!!
            store.put("newFromPage",
                          URLEncoder.encode(URLEncoder.encode(request.getRequestURI()+"?"+
                                                              request.getQueryString(), "UTF-8"),
                                            "UTF-8"));

            String fromPage =
                URLDecoder.decode((String)Utils.n2s(request.getParameter("fromPage")),
                                  "UTF-8");

            if( fromPage == null || fromPage.trim().length() == 0 )
                fromPage = "/home";
            store.put("fromPage", fromPage);

            String fromFromPage = request.getParameter("fromFromPage");
            if( fromFromPage != null ) {
                fromFromPage = URLDecoder.decode((String)Utils.n2s(fromFromPage), "UTF-8");
                store.put("fromFromPage", fromFromPage);
            }
        }

        return session;
    }

    public static String commonHtmlBegin(String title) {
        return
            "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
            "<HTML>\n <HEAD>\n"+
            "  <META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
            "  <LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"/netdb.css\">"+
            "  <TITLE>"+title+"</TITLE>\n"+
            " </HEAD>\n\n"+
            "<BODY>\n";
    }

    public static String encode(String src) throws java.io.UnsupportedEncodingException {
        return URLEncoder.encode(src, "UTF-8");
    }

    public static String decode(String src) throws java.io.UnsupportedEncodingException {
        return URLDecoder.decode(src, "UTF-8");
    }

    public static final boolean SHOW_HOME_LINK = true;
    public static final boolean NO_HOME_LINK = false;
    public static final boolean SHOW_PROFILE_LINK = true;
    public static final boolean NO_PROFILE_LINK = false;
    public static final boolean SHOW_LOGOUT_LINK = true;
    public static final boolean NO_LOGOUT_LINK = false;

    public String commonHeader(HttpSession session, boolean homeLink, boolean profileLink, boolean logoutLink,
                               String minor_title, String major_title, String helpPage) throws Exception
    {
        return commonHeader(session, "/home", homeLink, profileLink, logoutLink,
                            minor_title, major_title, helpPage);
    }

    public String commonHeader(HttpSession session, String homeLinkURL, boolean homeLink, boolean profileLink,
                               boolean logoutLink, String minor_title, String major_title,
                               String helpPage) throws Exception
    {
        return commonHeader(session, homeLinkURL, homeLink, profileLink, logoutLink,
                                   minor_title, major_title, helpPage, "");
    }

    public String commonHeader(HttpSession session, String homeLinkURL, boolean homeLink, boolean profileLink,
                               boolean logoutLink, String minor_title, String major_title,
                               String helpPage, String fromPage) throws Exception
    {
        String user_name = null;
        String netID = null;
        if( session != null ) {
            user_name = (String)session.getAttribute("user_name");
            netID = (String)session.getAttribute("NetID");
        }
        String helpLabel = "NetDB Help";
        if( helpPage.equals("hlpateam.html") )
            helpLabel = "Admin Team Help";
        else if( helpPage.equals("hlpdomain.html") )
            helpLabel = "Domain Help";
        else if( helpPage.equals("hlpgroup.html") )
            helpLabel = "Group Help";
        else if( helpPage.equals("hlplsearch.html") )
            helpLabel = "Log Search Help";
        else if( helpPage.equals("hlpfsearch.html") )
            helpLabel = "Full Search Help";
        else if( helpPage.equals("hlpnetdb.html") )
            helpLabel = "NetDB Help";
        else if( helpPage.equals("hlpnetwork.html") )
            helpLabel = "Network Help";
        else if( helpPage.equals("hlpnode.html") )
            helpLabel = "Node Help";
        else if( helpPage.equals("hlpqsearch.html") )
            helpLabel = "Quick Search Help";
        else if( helpPage.equals("hlpuser.html") )
            helpLabel = "User Help";
        else if( helpPage.equals("hlpuserprofile.html") )
            helpLabel = "Profile Help";

        return
            "<DIV id=\"pageHeader\">\n"+
            "<TABLE WIDTH='100%' BORDER='0' CELLPADDING='0'>\n"+
            "<TR>\n"+
            "  <TD VALIGN='TOP' WIDTH='17%'>"+(homeLink?"<A ID='HomeLink' HREF='"+homeLinkURL+"'>":"")+
            "<IMG ALT='NetDB Logo' SRC='/images/NetDBlogo.gif' BORDER=0>"+(homeLink?"</A>":"")+"</TD>\n"+
            "  <TD VALIGN='MIDDLE' ALIGN='CENTER'>"+
            (minor_title.trim().length()>0?("<H2>"+minor_title+"</H2><br>"):"")+
            (major_title.trim().length()>0?("    <H1>"+major_title+"</H1>"):"")+"</TD>\n"+
            "  <TD WIDTH='17%' VALIGN='MIDDLE' ALIGN='RIGHT'>\n"+
            (profileLink?("    <A HREF='/Profile?fromPage="+encode(fromPage)+"'>My Profile</A><BR>\n"):"")+
            (helpPage!=null?("    <A HREF='"+System.getProperty(Constants.HELPPAGESROOT_URL_KEY)+
                             "/"+helpPage+"'>"+helpLabel+"</A><BR>\n"):"")+
            (logoutLink?("    <A HREF='/logout'>Logout</A>"):"")+(user_name==null?"":"<BR>"+user_name+"")+"</TD>\n"+
            "</TR>\n"+
            "</TABLE>\n"+
            "</DIV>\n";
    }

    public void end(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    }

    public void handleException(HttpServletResponse response, Exception ex) {
        // display error page
        try {
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            errMsg = (errMsg == null ? "???" : errMsg);
            response.getWriter().println( getCommonErrorPage(errMsg) );
        } catch(Exception exx) {
        }
    }

    protected void exec(Executor executor) throws Exception {
        try {
            executor.method();

        } catch (NetDBException ex) {
            Vector errors = ex.getMessages();
            String msg = "";
            int esize = errors.size();
            for( int i = 0; i < esize; i++ )
                msg += "Error: "+errors.elementAt(i).toString().trim()+(i<esize-1?"\n":"");
            throw new Exception(msg);
        }
    }

    public String msgPanels(HttpSession session) {
        if( session == null )
            return "";

        String error_msg = (String)session.getAttribute("Error.errorMsg");
        String info_msg = (String)session.getAttribute("info_msg");
        String warning = (String)session.getAttribute("Warning");

        try {
            session.removeAttribute("Error.errorMsg");
        } catch (Exception ex) { }

        try {
            session.removeAttribute("info_msg");
        } catch (Exception ex) { }

        try {
            session.removeAttribute("Warning");
        } catch (Exception ex) { }

        return (error_msg != null ?
                "<DIV id=\"error_panel\">"+
                "<TABLE WIDTH='100%' BORDER='0' CELLPADDING='0'>\n"+
                "<TR>\n"+
                "    <TD WIDTH='100%'>"+Utils.htmlize(error_msg)+"</TD>\n"+
                "</TR>\n"+
                "</TABLE>\n</DIV>\n":"")+
            (info_msg != null ?
             "<DIV id=\"info_panel\">"+
             "<TABLE WIDTH='100%' BORDER='0' CELLPADDING='0'>\n"+
             "<TR>\n"+
             "    <TD WIDTH='100%'>"+Utils.htmlize(info_msg)+"</TD>\n"+
             "</TR>\n"+
             "</TABLE>\n</DIV>\n":"")+
            (warning != null ?
             "<DIV id=\"warning_panel\">"+
             "<TABLE WIDTH='100%' BORDER='0' CELLPADDING='0'>\n"+
             "<TR>\n"+
             "    <TD WIDTH='100%'>"+Utils.htmlize(warning)+"</TD>\n"+
             "</TR>\n"+
             "</TABLE>\n</DIV>\n":"");
    }

    public static Hashtable nHash(Object src) {
        return Utils.nHash(src);
    }

    public static boolean activep(Hashtable person) {
        Boolean active = (Boolean)person.get("active?");

        if (active == null) {
            active = new Boolean (false); }

        return active.booleanValue ();
    }

    public static String getNextKey() {
        String retKey = null;
        synchronized(nextKey) {
            retKey = new String(nextKey);
            nextKey = (new Long(Long.parseLong(nextKey) + (long)1)).toString();
        }
        return retKey;
    }

    public static String makePersonLink( Hashtable person )
    {
        // constructs a StanfordWho link if <PERSON> has directory id
        String dirID = (String)person.get("DirID");
        return (dirID == null ? null : System.getProperty(Constants.STANFORDWHO_URL_KEY)+dirID);
    }

    public static String compressNames( Hashtable names, String default_domain, String sep_string )
    {
        // formats <NAME>s (ignoring aliases and mxs) by stripping default_domain
        // if possible and joining multiple names with sep_string

        String result = "";
        if (names == null) return result;
        if (sep_string == null) sep_string = ", ";

        Object[] oa = names.keySet().toArray();  // unsorted
        for (int i = 0; i < oa.length; ++i)
        {
            String key = (String)oa[i];  // fqdn
            Hashtable ht = (Hashtable)names.get(key);
            String name = (String)ht.get("name");
            String domain = (String)ht.get("domain");
            if (! domain.equalsIgnoreCase(default_domain))
                name = name + "." + domain;
            if (result.length() > 0)
                result += sep_string;  // separate if multiples
            result += name;
        }

        return result;
    }

    public static String compressName( String name, String default_domain )
    {
        // formats name by stripping default_domain if possible

        if (name == null) return "";
        if (default_domain == null) return name;
        int ix = name.indexOf(".");
        if (ix < 0) return name;
        String prefix = name.substring(0, ix);
        String suffix = name.substring(ix+1);
        if (suffix.equalsIgnoreCase(default_domain)) return prefix;
        return name;
    }

    public static String getCommonErrorPage( String errMsg )
    {
        // displays a page showing a fatal NetDB error
        return
            "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
            "<HTML>\n<HEAD>"+
            "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
            "<LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"/netdb.css\">"+
            "<TITLE>NetDB Error</TITLE>\n"+
            "</HEAD>\n\n"+
            "<BODY>\n"+
            "<DIV id=\"ErrorPage\">\n"+
            "<BR><H2 ALIGN=\"center\">NetDB Error</H2>\n"+
            "<H3 ALIGN=\"center\">ERROR: "+Utils.htmlize(errMsg)+"</H3>\n"+
            "</DIV>\n</BODY>\n</HTML>";
    }

    public static String getCommonHeaderTable( String minor_title, String major_title,
                                               String user_id, String fromPage)
        throws Exception
    {
        return getCommonHeaderTable(minor_title, major_title, user_id, null, true, fromPage);
    }

    public static String getCommonHeaderTable( String minor_title, String major_title,
                                               String user_id, String user_name, String fromPage)
        throws Exception
    {
        return getCommonHeaderTable(minor_title, major_title, user_id, user_name, false, fromPage);
    }

    private static String getCommonHeaderTable( String minor_title, String major_title,
                                                String user_id, String user_name, boolean home, String fromPage)
        throws Exception
    {
        // displays the common header table for most pages
        return
            "<DIV id=\"pageHeader\">\n"+
            "<TABLE WIDTH='100%' BORDER='0' CELLPADDING='0'>\n"+
            "<TR>\n"+
            "  <TD VALIGN='TOP' WIDTH='17%'>"+(!home?"<A ID='HomeLink' HREF='/home'>":"")+
            "<IMG ALT='NetDB Logo' SRC='/images/NetDBlogo.gif' BORDER=0>"+(!home?"</A>":"")+"</TD>\n"+
            "  <TD VALIGN='MIDDLE' ALIGN='CENTER'>"+
            (minor_title.trim().length()>0?("<H2>"+minor_title+"</H2><br>"):"")+
            "    <H1>"+major_title+"</H1></TD>\n"+
            "  <TD WIDTH='17%' VALIGN='MIDDLE' ALIGN='RIGHT'>\n"+
            "    <A HREF='/Profile?fromPage="+encode(fromPage)+"'>My Profile</A><BR>\n"+
            "    <A HREF='"+System.getProperty(Constants.HELPPAGESROOT_URL_KEY)+
            "/helpnetdb.html'>NetDB Help</A><BR>\n"+
            "    <A HREF='/logout'>Logout</A>"+(user_name==null?"":"<BR>"+user_name+"")+"</TD>\n"+
            "</TR>\n"+
            "</TABLE>\n"+
             "</DIV>\n";
    }

    public static String getCommonQSearchForm(String search_string, String search_object, String fromPage)
    {
        if( search_object == null || search_object.trim().length() == 0 )
            search_object = "Nodes";

        // displays the quicksearch form
        return
            "<DIV id=\"QSForm\">\n"+
            "<FORM METHOD='POST' action='/qsearch'>\n"+
            "<TABLE WIDTH='90%' ALIGN='CENTER' BORDER='1' CELLSPACING='0' CELLPADDING='3'>\n"+
            "<TR><TD>\n"+
            "    <TABLE ALIGN='CENTER' WIDTH='100%' CELLPADDING='2' BORDER='0' CELLPADDING='0'>\n"+
            "    <TR>\n"+
            "      <TD ALIGN='LEFT' WIDTH='20%'><B><A NAME='quicksearch'></A>Quick Search</B>&nbsp;&nbsp;</TD>\n"+
            "      <TD ALIGN='LEFT'>Enter a name, hardware address or IP address: </TD>\n"+
            "      <TD ALIGN='RIGHT'><A TABINDEX='2' HREF='"+System.getProperty(Constants.HELPPAGESROOT_URL_KEY)+"/hlpqsearch.html'>"+"Quick Search Help</A>&nbsp;</TD>\n"+
            "    </TR>\n"+
            "    <TR>\n"+
            "      <TD></TD>\n"+
            "      <TD ALIGN='LEFT' NOWRAP><INPUT TABINDEX='1' TYPE='TEXT' NAME='search_string' SIZE='20' value='"+Utils.htmlize(search_string)+"'>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n"+
            "        Record&nbsp;type:"+
            "        <SELECT TABINDEX='1' SIZE=1 NAME='search_object'>\n"+
            "          <OPTION VALUE='All Types' "+(search_object.equals("All Types")?"SELECTED":"")+">All Types\n"+
            "          <OPTION VALUE='Admin Teams' "+(search_object.equals("Admin Teams")?"SELECTED":"")+">Admin Teams\n"+
            "          <OPTION VALUE='Domains' "+(search_object.equals("Domains")?"SELECTED":"")+">Domains\n"+
            "          <OPTION VALUE='Groups' "+(search_object.equals("Groups")?"SELECTED":"")+">Groups\n"+
            //            "          <OPTION VALUE='MXes' "+(search_object.equals("MXes")?"SELECTED":"")+">MXes\n"+
            "          <OPTION VALUE='Networks' "+(search_object.equals("Networks")?"SELECTED":"")+">Networks\n"+
            "          <OPTION VALUE='Nodes' "+(search_object.equals("Nodes")?"SELECTED":"")+">Nodes\n"+
            "          <OPTION VALUE='Users' "+(search_object.equals("Users")?"SELECTED":"")+">Users\n"+
            //"          <OPTION VALUE='Owned' "+(search_object.equals("Owned")?"SELECTED":"")+">Owned\n"+
            "        </SELECT>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n"+
            "        <INPUT ID='SearchButton' TABINDEX='1' TYPE='SUBMIT' VALUE='Search'>\n"+
            "      </TD>\n"+
            "    </TR>\n"+
            "    <TR>\n"+
            "      <TD ALIGN='LEFT'><B>Full Search</B></TD>\n"+
            "      <TD ALIGN='LEFT'>\n"+
            "        <A ID='Link_FS_Node' HREF='/FullSearchForm?mode=simple&searchtype=Node'>Node</A>&nbsp;&nbsp;\n"+
            "        <A ID='Link_FS_User' HREF='/FullSearchForm?mode=simple&searchtype=User'>User</A>&nbsp;&nbsp;\n"+
            "        <A ID='Link_FS_Network' HREF='/FullSearchForm?mode=simple&searchtype=Network'>Network</A>&nbsp;&nbsp;\n"+
            "        <A ID='Link_FS_Group' HREF='/FullSearchForm?mode=simple&searchtype=Group'>Group</A>&nbsp;&nbsp;\n"+
            "        <A ID='Link_FS_ATeam' HREF='/FullSearchForm?mode=simple&searchtype=ATeam'>Admin&nbsp;Team</A>&nbsp;&nbsp;\n"+
            "        <A ID='Link_FS_Domain' HREF='/FullSearchForm?mode=simple&searchtype=Domain'>Domain</A></TD>\n"+
            "      <TD ALIGN='RIGHT'>\n"+
            "        <A ID='Link_FS_Log' HREF='/FullSearchForm?mode=simple&searchtype=Log&fromPage="+fromPage+"'>Log&nbsp;Search</A></TD>\n"+
            "    </TR>\n"+
            "    </TABLE></TD>\n"+
            "</TR>\n"+
            "</TABLE>\n"+
            "</FORM>\n"+
            "</DIV>\n";
    }

    public static String getCommonBottomLabel() {
        return getCommonBottomLabel("helpnetdb.html");
    }

    public static String getCommonBottomLabel(String hlpURL)
    {
        // it should be configurable from netdb.properties file !!!
        String label =
            "<DIV id=\"bottom_panel\">\n"+
            "<P ALIGN='CENTER'>"+
            "<A HREF=\""+System.getProperty(Constants.HELP_REQUEST_URL_KEY)+"\">Feedback</A> || "+
            "<A HREF=\""+System.getProperty(Constants.HELPPAGESROOT_URL_KEY)+"/"+hlpURL+"\">Help</A>"+
            System.getProperty(Constants.DEPT_STR_KEY,  "").trim()+
            System.getProperty(Constants.ORG_STR_KEY,   "").trim()+
            "<BR>";

        if( !Constants.RELEASE_VERSION.equals("") || !Constants.RELEASE_NAME.equals("") ) {
            label += "Version: ";
            if( !Constants.RELEASE_VERSION.equals("") ) {
                String version_url = System.getProperty(Constants.RELEASE_VERSION_URL_KEY);
                if( version_url != null && version_url.trim().length() > 0 )
                    label += "<A HREF=\""+version_url+"\">"+Utils.htmlize(Constants.RELEASE_VERSION)+"</A> ";
                else
                    label += Utils.htmlize(Constants.RELEASE_VERSION);
            }

            if( !Constants.RELEASE_NAME.equals("") ) {
                String version_url = System.getProperty(Constants.RELEASE_NAME_URL_KEY);
                label += label.length()>0?" ":"";
                if( version_url != null && version_url.trim().length() > 0 )
                    label += "<A HREF=\""+version_url+"\">\""+Utils.htmlize(Constants.RELEASE_NAME)+"\"</A> ";
                else
                    label += "\""+Utils.htmlize(Constants.RELEASE_NAME)+"\"";
            }

            if( !Constants.BUILD_DATE.equals("") )
                label += (label.length()>0?" ":"")+"("+Constants.BUILD_DATE+")";
        }
        return label+
            "<BR>Copyright 2000-2007 Stanford University. All Rights Reserved. "+
            "</P>\n"+
            "</DIV>\n";
    }

    public static String getCommonHtmlBegin(String title) {
        return getCommonHtmlBegin(title, "");
    }

    public static String getCommonHtmlBegin(String title, String extra) {
        return getCommonHtmlBegin(title, extra, "0", "0");
    }

    public static String getCommonHtmlBegin(String title, String pos_x, String pos_y) {
        return getCommonHtmlBegin(title, "", pos_x, pos_y);
    }

    public static String getCommonHtmlBegin(String title, String extra, String pos_x, String pos_y) {
        if( pos_x == null || pos_x.trim().length() == 0 ) pos_x = "0";
        if( pos_y == null || pos_y.trim().length() == 0 ) pos_y = "0";

        return
            "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
            "<HTML>\n<HEAD>\n"+
            "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
            "<LINK REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"/netdb.css\">"+
            "<TITLE>"+title+"</TITLE>\n"+extra+
            "</HEAD>\n\n"+
            "<BODY onLoad=\"javascript:self.scrollTo("+pos_x+", "+pos_y+");\">\n";
    }

    /**
     * This method is used only to print out exceptions on a page which
     * do not suppose to happen in a first place, which warrants user
     * unfriendly stack trace to be displayed.
     */
    public void printCommonErrorPage(HttpServletResponse response, Exception ex) {
        try {
            ex.printStackTrace();
            // display error page
            String errMsg = ex.getMessage();
            if( errMsg == null ) {
                StringWriter sw = new StringWriter();
                ex.printStackTrace(new PrintWriter(sw, true));
                sw.close();
                errMsg = "" + sw.toString();
            }
            response.getWriter().println( getCommonErrorPage(errMsg) );
        } catch (Exception exx) {
            // it is an outrage if we end up here
            ex.printStackTrace();
        }
    }

    public static Object[] sorted_keys(Hashtable src, Comparator cmp) {
        return Utils.sorted_keys(src, cmp);
    }

    protected static Object getData(HttpSession session, String label) {
        String dataReady = (String)session.getAttribute("dataReady");
        while( dataReady == null ) {
            try { Thread.currentThread().sleep(100); } catch (Exception ex) {}
                dataReady = (String)session.getAttribute("dataReady");
        }
        synchronized(synLock) {
            return session.getAttribute(label);
        }
    }

    protected static void setData(HttpSession session, String label, Object data) {
        synchronized(synLock) {
            session.setAttribute(label, data);
        }
    }

    // using tags definition we try parse current tags (custom fields) in node hash
    // result of parsing is a new hash added to a node under key <"custom tags "+view_name>
    // which basically a copy of tags_definition hash 'value' field added, which indicates
    // current value for a given entry
    public void parse_tags_for_custom_view(Hashtable node,   Hashtable tags_definition,
                                           String view_name, int custom_fields_total,
                                           boolean ensure_sections_present)
        throws Exception
    {
        Hashtable tags_def       = Utils.copyHash(tags_definition);
        String error_msg         = "Unable to parse custom fields for "+view_name+".";
        Hashtable custom_tags    = new Hashtable();

        Hashtable tags           = Utils.get_hash_or_create(node, "tags");
        Hashtable tags_separated = nHash(node.get("tags_separated"));

        Hashtable sections_def   = nHash(tags_definition.get("sections"));
        int parsed_sections      = 0;

        if( tags_separated.size() == 0 ) {
            // using regular tags
            for( String section : (Set<String>)tags.keySet() ) {
                Hashtable sec = CNVParser.getSection(sections_def, section);
                if( sec == null ) continue;
                sec.put("saved name", section);

                parsed_sections++;
                Hashtable tags_data = extract_tags_data((String)tags.get(section), error_msg);
                parse_tag_section_for_custom_view(node, tags_data, tags_def, section, error_msg);
            }

            if( ensure_sections_present && parsed_sections < sections_def.size() )
                throw new Exception(error_msg);

            parsed_sections = parsed_sections + custom_fields_total - tags.size();

        } else {
            // using tags separated
            int tags_separated_size = 0;
            for( Hashtable tag : (Collection<Hashtable>)tags_separated.values() ) {
                String section = (String)Utils.n2s(tag.get("tag"));
                if( section.trim().length() > 0 ) tags_separated_size++;

                Hashtable sec = CNVParser.getSection(sections_def, section);
                if( sec == null ) continue;
                sec.put("saved name", section);

                parsed_sections++;
                Hashtable tags_data = extract_tags_data((String)Utils.n2s(tag.get("value")), error_msg);
                parse_tag_section_for_custom_view(node, tags_data, tags_def, section, error_msg);
            }

            if( ensure_sections_present && parsed_sections < sections_def.size() )
                throw new Exception(error_msg);

            parsed_sections = parsed_sections + custom_fields_total - tags_separated_size;
        }

        if( parsed_sections < sections_def.size() )
            throw new Exception(error_msg+" "+parsed_sections+" out of "+
                                sections_def.size()+" required sections were found or available.");

        node.put("custom tags "+view_name, tags_def);
    } // end of parse_tags_for_custom_view(...)

    // parse a particular section of custom node view
    private void parse_tag_section_for_custom_view(Hashtable node, Hashtable tags_data, Hashtable tags_def,
                                                   String section, String error_msg)
        throws Exception
    {
        for( String name : (Set<String>)tags_data.keySet() ) {
            String value = (String)tags_data.get(name);
            Hashtable entry_def = foundEntry(tags_def, section, name, value);
            if( entry_def == null ) {
                if( value.length() == 0 )
                    throw new Exception(error_msg+" Offending element is \""+name+"\".");
                else
                    throw new Exception(error_msg+" Offending name=value pair is \""+name+"="+value+"\".");
            }

            if( Utils.n2s(entry_def.get("html element")).equals("checkbox") )
                value = "on";

            else if( Utils.n2s(entry_def.get("html element")).equals("selector") ) {
                // this is a name=values pair selector
                String[] values = value.split(java.util.regex.Pattern.quote(Utils.nStr(entry_def.get("separator"))));
                for( String val : values ) {
                    for( Hashtable item : (Collection<Hashtable>)Utils.nHash(entry_def.get("items")).values() )
                        if( Utils.nStr(item.get("name")).equals(val) )
                            item.put("value", "selected");
                }
                continue;

            } else if( Utils.nStr(entry_def.get("html element")).length() == 0 ) // this is a selector without a name
                value = "selected";

            entry_def.put("value", value);
        }
        node.put("parsed section "+section, "yes");
    } // end of parse_tag_section_for_custom_view(...)

    // given section, name and value search for appropriate entry in tags definition
    // this function is exclusivly for use building custom node views
    private Hashtable foundEntry(Hashtable tags_definition, Object section, Object name, String value) {
        section = CNVParser.canonicalSection(tags_definition, section.toString());

        Hashtable entries = nHash(tags_definition.get("entries"));
        for( Hashtable entry : (Collection<Hashtable>)entries.values() ) {

            if( entry.get("html element").equals("selector") &&
                Utils.n2s(entry.get("section")).equals(section) ) { // dive in

                if( Utils.nStr(entry.get("name")).equals(name) ) return entry;

                // otherwise try return item
                for( Hashtable item : (Collection<Hashtable>)nHash(entry.get("items")).values() )
                    if( name.equals(item.get("name")) ) return item;
                continue;
            }

            if( Utils.n2s(entry.get("name")).equals(name) &&
                Utils.n2s(entry.get("section")).equals(section) ) {
                String html_element = (String)Utils.n2s(entry.get("html element"));

                if( html_element.equals("text-field") ) {
                    for( String key : (Set<String>)nHash(entry.get("validate")).keySet() )
                        if( value.matches(key) ) return entry;

                } else if( html_element.equals("checkbox") || html_element.startsWith("selector.") )
                    return entry;

                else {
                    for( Hashtable value_def : (Collection<Hashtable>)nHash(entry.get("values")).values() )
                        if( value.equals(value_def.get("name")) ) return entry;
                }
            }
        }
        return null;
    } // end of foundEntry(...)

    protected Hashtable findSelector(Hashtable tags_definition, String selector_label) {
        for( Hashtable entry : (Collection<Hashtable>)nHash(tags_definition.get("entries")).values() )
            if( entry.get("html element").equals("selector") && entry.get("display label").equals(selector_label) )
                return entry;
        return new Hashtable();
    }

    private Hashtable extract_tags_data(String src, String error_msg) throws Exception {
        Hashtable data = new Hashtable();
        String[] parts = src.split(",");
        for( int i = 0; i < parts.length; i++ ) {
            String[] name_value_pair = parts[i].split("=");
            if( name_value_pair.length == 0 || name_value_pair[0].trim().length() == 0 ) continue;
            if( name_value_pair.length > 2 )
                throw new Exception(error_msg+" Offending name=value pair is \""+parts[i].trim()+"\".");
            data.put(name_value_pair[0].trim(), name_value_pair.length==2?name_value_pair[1].trim():"");
        }
        return data;
    } // end of extract_tags_data(...)

    protected static String build_html_element(Hashtable node, int counter, Hashtable entry, String view_name)
        throws Exception
    {
        return build_html_element(node, counter, entry, view_name, false);
    }

    // given entry hashtable builds html element, currently used only for custom views
    protected static String build_html_element(Hashtable node,
                                               int counter,
                                               Hashtable entry,
                                               String view_name,
                                               boolean info)
        throws Exception
    {

        String ret_val = "";
        String html_element = Utils.nStr(entry.get("html element"));
        Object inject       = entry.get("inject");
        String value        = Utils.nStr(entry.get("value")).trim();
        String default_value = ((String)Utils.n2s(entry.get("default value"))).trim();
        if( value.length() == 0 ) value = default_value;
        if( value.equals("NIL") ) value = "";

        String field_length = (String)entry.get("field length");
        if( field_length == null || field_length.trim().length() == 0 )
            field_length = "40";

        if( html_element.equals("selector") ) {
            if( !info )
                ret_val += "<SELECT ID='Selector_Tag_"+entry.get("display label")+"' NAME='"+
                    encode(view_name+"_Tag_"+counter+"_TagSL_"+entry.get("section")+"_"+entry.get("display label"))+
                    "' SIZE='"+entry.get("size")+"'"+(inject==null?"":(" "+inject+" "))+
                    (((String)Utils.n2s(entry.get("multiple"))).equalsIgnoreCase("yes")?" MULTIPLE":"")+
                    ">";

            Hashtable items = Utils.nHash(entry.get("items"));
            Iterator it = Utils.sortedKeys(items, num_cmp);
            while( it.hasNext() ) {
                Hashtable item = (Hashtable)items.get(it.next());
                String ivalue = ((String)Utils.n2s(item.get("value"))).trim();

                // setting default value should happen only once
                if( ivalue.equals("") && node.get("looked view "+view_name) == null &&
                    node.get("parsed section "+entry.get("section")) == null )
                    ivalue = ((String)Utils.n2s(item.get("default value"))).equalsIgnoreCase("on")?"selected":"";

                item.put("selected", ivalue);

                if( info ) {
                    if( Utils.nStr(item.get("selected")).equals("selected") )
                        ret_val += (ret_val.length()>0?", ":"") +
                            Utils.htmlize(Utils.nStr(item.get("display label")), false);
                } else
                    ret_val += "<OPTION VALUE='"+
                        encode(Utils.nStr(item.get("name")))+"' "+
                        Utils.nStr(item.get("selected"))+">"+
                        Utils.htmlize(Utils.nStr(item.get("display label")), false);
            }
            ret_val += (!info?"</SELECT>":"")+"<BR><BR>";

        } else if( html_element.equals("text-field") ) {
            return info?value:("<INPUT ID='"+view_name+"_Tag_"+entry.get("name")+
                "' TYPE='TEXT' SIZE='"+field_length+"' NAME='"+
                encode(view_name+"_Tag_"+counter+"_Tag_"+entry.get("section")+"_"+entry.get("name"))+
                "' "+(inject==null?"":(" "+inject+" "))+"value=\""+Utils.htmlize(value, false)+"\">");

        } else if( html_element.equals("drop-down") ) {
            if( !info )
                ret_val += "<SELECT ID='"+view_name+"_Tag_"+entry.get("section")+
                    "_"+entry.get("name")+"' SIZE=1 NAME='"+
                    encode(view_name+"_Tag_"+counter+"_Tag_"+entry.get("section")+"_"+entry.get("name"))+
                    (inject==null?"":(" "+inject+" "))+"'>\n";
            Hashtable values = nHash(entry.get("values"));
            Iterator it = Utils.sortedKeys(values, num_cmp);
            while( it.hasNext() ) {
                Hashtable value_item = (Hashtable)values.get(it.next());
                String val = Utils.nStr(value_item.get("name")).trim();
                if( info )
                    ret_val +=
                        (val.equals(value==null?default_value:value)?
                         ((ret_val.length()>0?", ":"")+
                          Utils.htmlize((String)value_item.get("label"), false)):"");
                else
                    ret_val += "<OPTION VALUE='"+encode(val)+"'"+
                        (val.equals(value==null?default_value:value)?" SELECTED":"")+">"+
                        Utils.htmlize((String)value_item.get("label"), false);
            }
            if( !info ) ret_val += "</SELECT>";

        } else if( html_element.equals("radio") ) {
            Hashtable values = nHash(entry.get("values"));
            Iterator it = Utils.sortedKeys(values, num_cmp);
            while( it.hasNext() ) {
                Hashtable value_item = (Hashtable)values.get(it.next());
                String val = Utils.nStr(value_item.get("name")).trim();
                if( info )
                    ret_val +=
                        val.equals(value==null?default_value:value)?
                        Utils.htmlize((String)value_item.get("label"), false):"";
                else {
                    ret_val += "<INPUT ID='"+view_name+"_Tag_"+entry.get("section")+
                        "_"+entry.get("name")+"' TYPE=RADIO NAME='"+
                        encode(view_name+"_Tag_"+counter+"_Tag_"+entry.get("section")+"_"+entry.get("name"))+
                        "' VALUE='"+encode(val)+"'"+(inject==null?"":(" "+inject+" "))+
                        (val.equals(value==null?default_value:value)?" CHECKED":"")+">"+
                        Utils.htmlize((String)value_item.get("label"), false);
                }
            }

        } else if( html_element.equals("checkbox") ) {
            if( info )
                ret_val += !value.equals("")&&!value.equalsIgnoreCase("off")?"Yes":"No";
            else
                ret_val += "<INPUT ID='"+view_name+"_TagCB_"+entry.get("section")+
                    "_"+entry.get("name")+"' TYPE='checkbox' NAME='"+
                    encode(view_name+"_Tag_"+counter+"_TagCB_"+entry.get("section")+"_"+entry.get("name"))+
                    "' VALUE='"+encode((String)Utils.n2s(entry.get("name")))+"'"+
                    (inject==null?"":(" "+inject+" "))+
                    (!value.equals("")&&!value.equalsIgnoreCase("off")?" CHECKED":"")+">";
        }

        return ret_val;
    } // build_html_element(...)
}
