/************************************************************************
 *
 * Delete.java -
 *
 * $Id: Delete.java,v 1.5 2007-10-26 21:10:26 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Delete extends ModifyPage
{

    public Delete() {}

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable oHash = null;
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        int what          = -1;
        String handle     = null;
        Timestamp lockTm  = null;
        NetDBpi netdb     = null;
        String fromPage   = "";
        String action     = request.getParameter("Action");
        String show_owned = null;
        try {
            fromPage = request.getParameter("fromPage");
            if( fromPage == null )
                fromPage = "/home";

            PrintWriter out = response.getWriter();

            netdb = (NetDBpi)session.getAttribute("netdb");
            String object = request.getParameter("object");
            handle = decode(request.getParameter("handle"));

            Hashtable store = new Hashtable();
            String help_link = resolveWhat(object, store);
            what = ((Integer)store.get("what")).intValue();
            lockTm = new Timestamp(Long.parseLong(request.getParameter("lockTm")));

            if( action.equals("Cancel") ) {
                try {
                    try {
                        if( ( what == Constants.NODE || what == Constants.NETWORK ) &&
                            Utils.isNumber(handle) )
                            netdb.unlock(what, Integer.parseInt(handle), lockTm);
                        else
                            netdb.unlock(what, handle, lockTm);
                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+errors.elementAt(i).toString().trim()+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    session.setAttribute("Warning", ex.getMessage());
                }
                response.sendRedirect(fromPage);
                return;
            }

            if( action.equals("Delete") ) {

                oHash = netdb.getInfo(what, handle);
                oHash.put("locked", lockTm);
                try {

                    netdb.delete(oHash);

                    if( ((String)Utils.n2s(oHash.get("object"))).equals("group") ) {
                        // reload global list of groups
                        setData(session, "myGroups", netdb.listMyGroups("*"));
                    }

                } catch (NetDBException ex) {
                    Vector errors = ex.getMessages();
                    String msg = "";
                    int esize = errors.size();
                    for( int i = 0; i < esize; i++ ) {
                        String emsg = errors.elementAt(i).toString().trim();
                        if( Utils.n2s(oHash.get("object")).equals("group") && emsg.indexOf("still in use") > -1 ) {
                            show_owned = Utils.nStr(oHash.get("name"));
                            emsg = emsg.substring(0, emsg.length() - 1) + " by the following records.";
                        }
                        msg += "Error: "+emsg+(i<esize-1?"\n":"");

                    }
                    throw new Exception(msg);
                }

                session.setAttribute("info_msg", object+" '"+handle+
                                     "' has been deleted.");

                // repack mxSlaves
                Hashtable mxSlavesH = new Hashtable();
                Enumeration enm = request.getParameterNames();
                while( enm.hasMoreElements() ) {
                    String param = (String)enm.nextElement();
                    if( !param.startsWith("mxSlave") )
                        continue;
                    String value = URLDecoder.decode(request.getParameter(param), "UTF-8");
                    String[] mxParams = value.split("\\ +");
                    if( mxParams != null && mxParams.length > 2 ) {
                        Hashtable mxingNode = (Hashtable)mxSlavesH.get(mxParams[0].trim());
                        if( mxingNode == null ) {
                            mxingNode = new Hashtable();
                            mxSlavesH.put(mxParams[0].trim(), mxingNode);
                        }

                        Hashtable mxingNodeName = (Hashtable)mxingNode.get(mxParams[1].trim());
                        if( mxingNodeName == null ) {
                            mxingNodeName = new Hashtable();
                            mxingNode.put(mxParams[1].trim(), mxingNodeName);
                        }

                        mxingNodeName.put(mxParams[2].trim(), "");
                    }
                }

                // now remove mxes
                String warning = "";
                for( String nodeID : (Set<String>)mxSlavesH.keySet() ) {
                    int idNode = (new Integer(nodeID)).intValue();
                    Hashtable mxingNode = (Hashtable)mxSlavesH.get(nodeID);;

                    Timestamp mxingNodeLockTm = null;
                    try {
                        mxingNodeLockTm = netdb.lock(Constants.NODE, idNode);
                        Hashtable oldMXingNode = netdb.getInfo(Constants.NODE, idNode);
                        Hashtable newMXingNode = Utils.copyHash(oldMXingNode);
                        oldMXingNode.put("locked", mxingNodeLockTm);
                        newMXingNode.put("locked", mxingNodeLockTm);
                        removeMXes(newMXingNode, mxingNode);
                        netdb.update(oldMXingNode, newMXingNode);
                        mxingNodeLockTm = null;

                    } catch (Exception ex) {
                        for( String mxingNodeName : (Set<String>)mxingNode.keySet() ) {
                            Hashtable mxNames = (Hashtable)mxingNode.get(mxingNodeName);
                            for( Object mx_name : mxNames.keySet() )
                                warning += (warning.length()>0?"<br>\n":"")+
                                    "Warning: failed to remove mx(es) "+mx_name+
                                    " from node "+mxingNodeName;
                        }

                    } finally {
                        try {
                            if( mxingNodeLockTm != null )
                                netdb.unlock(Constants.NODE, idNode, mxingNodeLockTm);
                        } catch (Exception ex) { }
                    }
                }
                if( warning.length() > 0 )
                    session.setAttribute("Warning", warning);

                if( fromPage.startsWith("/Info" ) ) {
                    if( URLDecoder.decode(fromPage, "UTF-8").indexOf("/qresults") > -1 )
                        response.sendRedirect("/qresults");
                    else if( URLDecoder.decode(fromPage, "UTF-8").indexOf("/FS") > -1)
                        response.sendRedirect("/FS");
                    else
                        response.sendRedirect("/home");
                } else
                    response.sendRedirect(fromPage);
                return;
            }

        } catch (Exception exx) {
            session.setAttribute("Error.errorMsg", exx.getMessage());
            try {
                try {
                    if( netdb != null && !action.equals("Delete") )
                        netdb.unlock(what, handle, lockTm);
                } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+errors.elementAt(i).toString().trim()+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                }
            } catch (Exception exxx) {
                session.setAttribute("Warning", exxx.getMessage());
            }
            if( show_owned == null )
                response.sendRedirect(fromPage);
            else {
                session.setAttribute("search_string", "\""+show_owned+"\"");
                response.sendRedirect("/owned");
            }
        }
    } // end of doPost(...)

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable oHash = null;
        HttpSession session = null;
        try {
            Hashtable store = new Hashtable();
            session = begin(request, response, store);
            if( session == null )
                return;

            String sessionKey = (String)store.get("sessionKey");
            String fromPage = (String)Utils.n2s(store.get("fromPage"));

            PrintWriter out = response.getWriter();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String object = request.getParameter("object");
            String handle = request.getParameter("handle");
            store.put("handle", handle);
            Timestamp lockTm = null;

            Hashtable what_store = new Hashtable();
            String help_link = resolveWhat(object, what_store);
            int what = ((Integer)what_store.get("what")).intValue();

            // Let see if object's handle is number
            int oid = -1;
            boolean numeric_handle = false;
            if( what == Constants.NODE || what == Constants.NETWORK )
                try {
                    oid = Integer.parseInt(handle);
                    numeric_handle = true;
                } catch(Exception ex) { };

            try {
                try {
                    if( numeric_handle )
                        lockTm = netdb.lock(what, oid);
                    else
                        lockTm = netdb.lock(what, handle);

                    if( lockTm == null )
                        throw new Exception("Failed to lock "+object.toLowerCase());
                    store.put("lockTm", lockTm);
                } catch (NetDBException ex) {
                    Vector errors = ex.getMessages();
                    String msg = "";
                    int esize = errors.size();
                    for( int i = 0; i < esize; i++ )
                        msg += "Error: "+errors.elementAt(i).toString().trim()+(i<esize-1?"\n":"");
                    throw new Exception(msg);
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                session.setAttribute("Error.errorMsg", ex.getMessage());
                response.sendRedirect(fromPage);
                return;
            }

            if( numeric_handle )
                oHash = netdb.getInfo(what, oid);
            else
                oHash = netdb.getInfo(what, handle);

            // extact name with original case
            if( what == Constants.NODE || what == Constants.NETWORK ) {
                Hashtable names = Utils.nHash(oHash.get("names"));
                for( String key : (Set<String>)names.keySet() ) {
                    String name = key.trim().toLowerCase();
                    Hashtable nameH = (Hashtable)names.get(key);
                    handle = nameH.get("name")+"."+nameH.get("domain");
                    store.put("handle", handle);
                    break;
                }
            }

            if (oHash == null)
                throw new Exception("No info found for "+object.toLowerCase()+
                                    " <em>"+handle+"</em>");

            out.println(commonHtmlBegin("Delete "+object+" : "+Utils.htmlize(handle, false))+
                        commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object="+object+
                                     "&sessionKey="+encode(sessionKey),
                                     SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                     "", "Confirm "+object+" Deletion<br>"+Utils.htmlize(handle, false), help_link)+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        (nHash(oHash.get("types")).get("Router")!=null?
                         "<CENTER><FONT CLASS='big_warning_msg'>You are about to delete an entire router."+
                         "<BR>Are you sure?</FONT></CENTER>":"")+
                        "<FORM METHOD=\"POST\" ACTION='/Delete'>"+
                        "<P CLASS=\"form_buttons\">\n"+
                        "<INPUT ID='Button_Delete' ACCESSKEY=\"D\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Delete\"> "+
                        "<INPUT ID='Button_Cancel' ACCESSKEY=\"C\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                        "<INPUT TYPE=\"Hidden\" NAME=\"fromPage\" VALUE=\""+fromPage+"\">"+
                        "<INPUT TYPE=\"Hidden\" NAME=\"object\" VALUE=\""+object+"\">"+
                        "<INPUT TYPE=\"Hidden\" NAME=\"handle\" VALUE=\""+encode(handle)+"\">"+
                        "<INPUT TYPE=\"Hidden\" NAME=\"lockTm\" VALUE=\""+lockTm.getTime()+"\">"+
                        "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                        "</P>\n");

            if( what == Constants.NODE ) {
                Hashtable mxSlaves = Utils.nHash(oHash.get("mxSlaves"));
                if( mxSlaves.size() > 0 ) {
                    out.println("&nbsp;&nbsp;To stop directing mail to these mail servers, "+
                                "check the appropriate boxes.<br>");
                    int counter = 0;
                    for( String nodeID : (Set<String>)mxSlaves.keySet() ) {
                        Hashtable node_names = (Hashtable)mxSlaves.get(nodeID);

                        java.util.List list = Arrays.asList(node_names.keySet().toArray());
                        Collections.sort(list, name_cmp);
                        for( Object node_name : Utils.sort(node_names.keySet(), name_cmp) ) {
                            for( Object mx_name : Utils.sort(Utils.nHash(node_names.get(node_name)).keySet(), name_cmp) )
                                out.println("&nbsp;&nbsp;&nbsp;&nbsp;<input type=\"checkbox\" name=\"mxSlave"+
                                            (counter++)+"\" value=\""+
                                            URLEncoder.encode(nodeID+" "+node_name.toString().trim()+" "+
                                            mx_name.toString().trim(), "UTF-8")+
                                            "\"/><b>"+node_name.toString().trim()+
                                            "</b> getting mail for <b>"+mx_name.toString().trim()+"</b><br>");
                        }
                    }
                }
            }
            out.println("<HR NOSHADE SIZE=2>\n");

            // call info method
            Class infoClass = Class.forName("stanford.netdb.html.Info"+object);
            java.lang.reflect.Method printOutInfo =
                infoClass.getMethod("printOutInfo",
                                    new Class[] {
                                        Class.forName("javax.servlet.http.HttpSession"),
                                        Class.forName("javax.servlet.http.HttpServletResponse"),
                                        Class.forName("java.io.PrintWriter"),
                                        Class.forName("java.util.Hashtable"),
                                        Class.forName("java.lang.String")
                                    });
            oHash.put("no inner form", "");
            printOutInfo.invoke(null, new Object[] {session, response, out, oHash, (String)Utils.n2s(store.get("newFromPage"))} );

            out.println("<HR NOSHADE SIZE=2>\n"+
                        "<P CLASS=\"form_buttons\">\n"+
                        "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Delete\"> "+
                        "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                        "</P>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }

    } // end of doGet(...)

    private String resolveWhat(String object, Hashtable store) {
        if( object.equals("Node") ) {
            store.put("what", new Integer(Constants.NODE));
            return "hlpnode.html";

        } else if( object.equals("AdminTeam") ) {
            store.put("what", new Integer(Constants.ATEAM));
            return "hlpateam.html";

        } else if( object.equals("Network") ) {
            store.put("what", new Integer(Constants.NETWORK));
            return "hlpnetwork.html";

        } else if( object.equals("User") ) {
            store.put("what", new Integer(Constants.USER));
            return "hlpuser.html";

        } else if( object.equals("Group") ) {
            store.put("what", new Integer(Constants.GROUP));
            return "hlpgroup.html";

        } else if( object.equals("Domain") ) {
            store.put("what", new Integer(Constants.DOMAIN));
            return "hlpdomain.html";
        }
        store.put("what", new Integer(-1));
        return "";
    }

    private void removeMXes(Hashtable mxingNode, Hashtable mxes) {
        Hashtable names = Utils.nHash(mxingNode.get("names"));
        for( String key : (Set<String>)names.keySet() ) {
            String nameFQDN = key.toLowerCase().trim();

            Hashtable mxs = null;
            for( String name : (Set<String>)mxes.keySet() ) {
                if( name.toLowerCase().equals(nameFQDN) ) {
                    mxs = (Hashtable)mxes.get(name);
                    break;
                }
            }
            if( mxs == null )
                continue;

            Vector keysToRemove = new Vector();
            Hashtable name = (Hashtable)names.get(key);
            Hashtable mxNames = Utils.nHash(name.get("mxs"));

            for( String mxKey : (Set<String>)mxNames.keySet() ) {
                for( String mxName : (Set<String>)mxs.keySet() ) {
                    if( mxKey.toLowerCase().trim().equals(mxName.toLowerCase().trim()) ) {
                        keysToRemove.addElement(mxKey);
                        break;
                    }
                }
            }

            // now remove keysToRemove
            for( int i = 0; i < keysToRemove.size(); i++ )
                mxNames.remove(keysToRemove.elementAt(i));
        }
    } // end of removeMXes(...)
}
