/************************************************************************
 *
 * FullSearch.java -
 *
 * $Id: FullSearch.java,v 1.6 2008-04-04 21:25:43 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.util.regex.*;
import javax.servlet.*;
import javax.servlet.http.*;
import su.netdb.parser.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class FullSearch extends ModifyPage {

    public FullSearch() {}

    // No search is performed on HTTP GET, it only displays the last performed
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();
        HttpSession session =
            begin(request, response,
                  rootURL.equals("/results.csv")?"text/plain":"text/html", null);
        if( session == null )
            return;

        try {

            if( request.getParameter("sort") != null )
                session.setAttribute("sort column", decode(request.getParameter("sort")));
            if( request.getParameter("dir") != null )
                session.setAttribute("last direction sort", decode(request.getParameter("dir")));

            handleSearchResults(session, out, rootURL);

        } catch (Exception ex) {
            session.setAttribute("Error.errorMsg", "Error: "+ex.getMessage());
            response.sendRedirect("/home");
            return;
        }
    } // end of doGet(...)

    // Does the search and displays it.
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        String searchType = "";

        PrintWriter out = response.getWriter();

        Hashtable store = new Hashtable();
        store.put("no session store", "");
        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        // Go over the passed parameters, check them and fill in searchData Hashtable
        String searchID                 = null;
        boolean have_data               = false; // indicates if any fields have some data in it.
        boolean have_fields_for_display = false; // indicates if nothing was selected for display
        boolean gotErrors               = false;
        try {
            searchType = request.getParameter("searchtype");

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            if( ((String)Utils.n2s(request.getParameter("ACTION"))).equals("Cancel") ) {
                session.removeAttribute("full_search_data");
                response.sendRedirect(decode((String)store.get("fromFromPage")));
                return;
            }

            if( ((String)Utils.n2s(request.getParameter("ACTION"))).equals("Reset") ) {
                session.removeAttribute("full_search_data");
                response.sendRedirect(decode((String)store.get("fromPage")));
                return;
            }

            Hashtable searchData = new Hashtable();
            Enumeration enm =  request.getParameterNames();
            while( enm.hasMoreElements() ) {
                String key = (String)enm.nextElement();
                String decodedKey = key;

                String value = request.getParameter(key);

                if( !decodedKey.endsWith(" error") &&
                    decodedKey.matches("^(fs\\.[^\\.]+\\.)?(txt|num|tst|ip|hw)\\..*$") ) {
                    try {
                        if( value.trim().length() > 0 )
                            have_data = true;
                        String field = decodedKey;
                        if( field.startsWith("fs.") ) {
                            Matcher m = Pattern.compile("^fs\\.[^\\.]+\\.(.+)$").matcher(decodedKey);
                            if( m.matches() ) // field name contains table name as well
                                field = m.group(1);
                        }
                        if( field.equals("txt.customFields") ||
                            field.equals("txt.DHCPoptions") )
                            Utils.parseField(new Hashtable(), decodedKey, field,
                                             value.replaceAll("&", "&amp;").
                                             replaceAll("\\\\\\\\", "&bsl;").
                                             replaceAll("\\\\=", "&eq;"),
                                             "dummy", "column");
                        else
                            Utils.parseField(new Hashtable(), decodedKey, field, value, "dummy", "column");
                        searchData.remove(decodedKey+" error");
                    } catch(Exception ex) {
                        gotErrors = true;
                        if (ex.getMessage().equals("bad IP range, low > high"))
                            addSessionMsg(session, "Error.errorMsg", "Error: Bad IP range. The "+
                                "starting value for the range is greater than the ending value.");
                        else
                            addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());

                        searchData.put(decodedKey+" error", "");
                    }

                } else if( decodedKey.equals("cb.i.f.InMyGroups") && value.length() > 0 )
                    // checking in a node full search page that user wants to search in his groups
                    // is sufficient to perform search.
                    have_data = true;

                searchData.put(decodedKey, value);

                if( !decodedKey.equals("displayOrder.Actions") &&
                    decodedKey.startsWith("displayOrder.") && !value.equals("-1") )
                    have_fields_for_display = true;

            }

            if( Utils.n2s(searchData.get("searchtype")).equals("User") )
                // always do user full search
                have_data = true;

            session.setAttribute("full_search_data", searchData);

            if( ((String)Utils.n2s(searchData.get("fs.FS_NameNode.txt.fullname_lc"))).trim().length() > 0 &&
                searchData.get("cb.i.Name:Alias")     == null &&
                searchData.get("cb.i.Name:Interface") == null &&
                searchData.get("cb.i.Name:IP")        == null &&
                searchData.get("cb.i.Name:IPC")       == null &&
                searchData.get("cb.i.Name:Node")      == null ) {
                gotErrors = true;
                addSessionMsg(session, "Error.errorMsg",
                              "Name search requires selection of one or more Name Types.");
            }


            // check error conditions
            if( !have_data ) {
                addSessionMsg(session, "Error.errorMsg",
                              "Error: Please enter some search criteria.");
                gotErrors = true;
            }

            if( !have_fields_for_display ) {
                addSessionMsg(session, "Error.errorMsg",
                              "Error: Please select at least one field to be displayed.");
                gotErrors = true;
            }

            if( gotErrors ) {
                response.sendRedirect(decode((String)store.get("fromPage")));
                return;
            }

            // register new search id and cancel other searches in this session
            searchID = netdb.createSearchID();
            synchronized(synLock) {
                String anotherSID = (String)session.getAttribute("full search id");

                if( anotherSID != null ) // cancel another search
                    netdb.cancelSearch(anotherSID);

                session.setAttribute("full search id", searchID);
            }

            Hashtable sr = null; // search result
            String searchTypeLabel = "";

            // do the search
            try {
                if( searchType.equals("Domain") ) {
                    searchTypeLabel = "Domains";
                    sr = netdb.fullSearch(Constants.DOMAIN, searchData, searchID);

                } else if( searchType.equals("AdminTeam") ) {
                    searchTypeLabel = "AdminTeams";
                    sr = netdb.fullSearch(Constants.ATEAM, searchData, searchID);

                } else if( searchType.equals("Group") ) {
                    searchTypeLabel = "Groups";
                    sr = netdb.fullSearch(Constants.GROUP, searchData, searchID);

                } else if( searchType.equals("Network") ) {
                    searchTypeLabel = "Networks";
                    sr = netdb.fullSearch(Constants.NETWORK, searchData, searchID);

                } else if( searchType.equals("Node") ) {
                    searchTypeLabel = "Nodes";
                    sr = netdb.fullSearch(Constants.NODE, searchData, searchID);

                } else if( searchType.equals("User") ) {
                    searchTypeLabel = "Users";
                    sr = netdb.fullSearch(Constants.USER, searchData, searchID);

                } else if( searchType.equals("Log") ) {
                    searchTypeLabel = "Log";
                    sr = netdb.fullSearch(Constants.LOG, searchData, searchID);

                } else
                    throw new Exception("Unrecognized search type.");

                session.setAttribute("searchTypeLabel", searchTypeLabel);

            } finally {
                // search is done, remove 'full search id'
                session.removeAttribute("full search id");
            }

            if( sr == null ) { // search was canceled
                session.setAttribute("Warning",
                            "Warning: Your search was canceled. Perhaps by another search.");
                response.sendRedirect("/FullSearchForm?searchtype="+searchType);
                return;
            }

            if( sr.get("errors") != null ) {
                Vector errors = (Vector)sr.get("errors");
                String error = "";
                for( int i = 0; i<errors.size(); i++ )
                    error += "Error: "+errors.elementAt(i);

                addSessionMsg(session, "Error.errorMsg", error);
            }

            session.setAttribute("full search results", sr);

            handleSearchResults(session, out, "");

            session.removeAttribute("full_search_data");

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
            response.sendRedirect("/FullSearchForm?searchtype="+searchType);
        }
    } // end of doPost(...)

    // display search results
    private void handleSearchResults(HttpSession session, PrintWriter out, String rootURL)
        throws ArrayIndexOutOfBoundsException, Exception
    {
        Hashtable sr            = Utils.nHash(session.getAttribute("full search results"));
        String search_type      = (String)sr.get("search type");
        Hashtable result        = Utils.nHash(sr.get("result"));
        boolean actions         = Utils.n2s(sr.get("show actions")).equals("true");
        String output_format    = (String)sr.get("output_format");
        boolean isSafeFormat    = output_format.equals("plain")?true:false;
        String object           = (String)sr.get("object");

        // record_type is used in a log full search to display proper
        // note to the user about ip addresses
        String record_type      = (String)Utils.n2s(sr.get("recordtype"));

        Vector displayOrder = (Vector)sr.get("displayOrder");

        // remove columns which will not be displayed
        for(;;) {
            boolean removed = false;
            for( int i = 0; i < displayOrder.size(); i++ ) {
                Hashtable column = (Hashtable)displayOrder.elementAt(i);
                if( column.get("requested_place").equals("-1") ) {
                    displayOrder.removeElementAt(i);
                    removed = true;
                    break;
                }
            }
            if( !removed ) break;
        }

        String sort_column = "";
        if( session.getAttribute("sort column") != null )
            sort_column = (String)session.getAttribute("sort column");
        String last_direction = (String)session.getAttribute("last direction sort");
        if( last_direction == null )
            last_direction = "down";

        // sort it
        java.util.List alist = Arrays.asList(displayOrder.toArray());
        Collections.sort(alist, order_cmp);
        Object[] columns = alist.toArray();

        if( sort_column.equals("") ) {
            // extract sorted column if we don't know it yet.
            for (int i = 0; i < columns.length; ++i) {
                sort_column = (String)Utils.n2s(((Hashtable)columns[i]).get("name"));
                break;
            }
        }

        // Build display_columns Vector
        Vector display_columns = new Vector();
        boolean intfc_column_added = false;
        for (int i = 0; i < columns.length; ++i) {
            Hashtable column = (Hashtable)columns[i];
            if( ((String)Utils.n2s(column.get("requested_place"))).equals("-1") )
                continue;
            String name = (String)Utils.n2s(column.get("name"));
            if( !intfc_column_added && search_type.equals("Nodes") &&
                ( name.equals("Name:Interface")   ||
                  name.equals("Name:IP")          ||
                  name.equals("Name:IPC")         ||
                  name.equals("Address")          ||
                  name.equals("PTR Pref")         ||
                  name.equals("Active Address")   ||
                  name.equals("Hardware Address") ||
                  name.equals("DHCP")             ||
                  name.equals("DHCP Option")      ||
                  name.equals("Roaming") ) ) {
                // add Interface# column
                Hashtable intfc_column = new Hashtable();
                intfc_column.put("requested_place", ""+i);
                intfc_column.put("def_place", ""+i);
                intfc_column.put("name", "Interface#");
                intfc_column.put("type", "NoSortString");
                display_columns.addElement(intfc_column);
                intfc_column_added = true;
            }

            display_columns.addElement(column);
        }

        if( !rootURL.equals("/results.csv") )
            out.println(getCommonHtmlBegin("Full Search Results")+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Search Result", search_type.equals("Log Records")?"hlplsearch.html":"hlpfsearch.html",
                                     "/FS")+
                        msgPanels(session)+
                        "<DIV id=\"full_search_results\"><BR>"+
                        "<CENTER CLASS=\"new_search_links\">New Search:&nbsp;&nbsp;"+
                        "<a href=\"/FullSearchForm?searchtype=Node\">Node</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=User\">User</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=Network\">Network</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=Group\">Group</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=ATeam\">Admin Team</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=Domain\">Domain</a>"+
                        "&nbsp;&nbsp;<a href=\"/FullSearchForm?searchtype=Log\">Log</a></b>"+
                        "</CENTER>"+
                        "<BR>\n"+
                        (search_type.equals("Log Records")&&(record_type.equals("'node'")||record_type.equals("'network'"))?
                         "&nbsp;&nbsp;Note that results do not include IPC or dynamic DHCP addresses.<BR>\n":"")+
                        "<DIV id=\"total_label\">&nbsp;"+search_type+"("+result.size()+"). "+
                        "Sorted by column \""+sort_column+"\" ["+
                        (last_direction.equals("down")?"ascending":"descending")+"]</DIV>"+
                        "<TABLE WIDTH='1%' CELLSPACING='0' CELLPADDING='0' BORDER='0'>\n"+
                        "<THEAD frame=bellow><TR>"+
                        "<TD>&nbsp;&nbsp;&nbsp;#&nbsp;</TD>");

        if( !rootURL.equals("/results.csv") && actions )
            out.println("<TD>Actions&nbsp;</TD>");

        String sort_column_type = "";
        for (int i = 0; i < display_columns.size(); ++i) {
            Hashtable column = (Hashtable)display_columns.elementAt(i);
            String name = (String)Utils.n2s(column.get("name"));
            if( sort_column.equals("") ) {
                sort_column = name;
                sort_column_type = (String)column.get("type");
            }
            String dir = "down";
            if( sort_column.equals(name) ) {
                dir = last_direction.equals("down")?"up":"down";
                sort_column_type = (String)column.get("type");
            }
            if( rootURL.equals("/results.csv") )
                out.print(name+(i+1<display_columns.size()?",":""));
            else
                out.println("<TD NOWRAP><A HREF=\"/FS?sort="+encode(name)+"&dir="+encode(dir)+"\">"+
                            Utils.htmlize(name)+"</A>&nbsp;&nbsp;</TD>");
        }
        if( rootURL.equals("/results.csv") )
            out.println("");
        else
            out.println("<TR></THEAD>\n<TBODY>");

        int color = 0;
        FullSearchSort fss = new FullSearchSort(result, sort_column, sort_column_type, last_direction);
        Object[] data = fss.doSort();
        for (int i = 0; i < data.length; i++) {
            Hashtable row = (Hashtable)((Map.Entry)data[i]).getValue();
            String handle = URLEncoder.encode((String)((Vector)row.get("handle")).elementAt(0), "UTF-8");
            if( !rootURL.equals("/results.csv") )
                out.println("<TR CLASS=\""+(color==0?"full_search_results_dark_row":"full_search_results_light_row")+
                            "\"><TD VALIGN=\"TOP\">&nbsp;&nbsp;&nbsp;"+
                            (search_type.equals("Log Records")?"":"<A ID=\"Link_Info_"+i+"\" HREF=\"/Dispatch?Action=info&object="+object+"&fromPage=/FS&handle="+handle+"\">")+
                            (i+1)+
                            (search_type.equals("Log Records")?"":"</A>")+"&nbsp;&nbsp;</TD>");
            color = color==1?0:1;
            if( !rootURL.equals("/results.csv") && actions ) {
                out.println("<TD VALIGN=\"TOP\" NOWRAP>"+
                            "<A ID=\"Link_Mod_"+i+"\" HREF=\"/Dispatch?Action=modify&object="+object+
                            "&fromPage=/FS&handle="+handle+"\">Mod</A>&nbsp;&nbsp;"+
                            "<A ID=\"Link_Tmpl_"+i+"\" HREF=\"/Dispatch?Action=use&object="+object+
                            "&fromPage=/FS&handle="+handle+"\">Tmpl</A>&nbsp;&nbsp;"+
                            "<A ID=\"Link_Del_"+i+"\" HREF=\"/Dispatch?Action=delete&object="+object+
                            "&fromPage=/FS&handle="+handle+
                            "\">Del</A>&nbsp;&nbsp;&nbsp;&nbsp;"+
                            "</TD>");
            }
            for( int j = 0; j < display_columns.size(); j++ ) {
                if( !rootURL.equals("/results.csv") )
                    out.println("<TD VALIGN=\"TOP\" NOWRAP>");
                Hashtable column = (Hashtable)display_columns.elementAt(j);
                Vector cell = (Vector)row.get(column.get("name"));
                java.util.List clist = Arrays.asList(cell.toArray());
                String type = (String)column.get("type");
                if( type.equals("String") )
                    Collections.sort(clist, name_cmp);
                else if( type.equals("ip") )
                    Collections.sort(clist, ip_str_cmp);
                // no soring is required if it is timestamp
                Object[] cdata = clist.toArray();
                boolean csv_element_exists = false;
                for (int k = 0; k < cdata.length; k++) {
                    String display_data = type.equals("Timestamp")?sdf.format(cdata[k]):cdata[k].toString();
                    if( rootURL.equals("/results.csv") )
                        display_data = type.equals("Timestamp")?csv_df.format(cdata[k]):cdata[k].toString();
                    if( rootURL.equals("/results.csv") ) {
                        if( display_data.trim().length() > 0 ) {
                            if( csv_element_exists ) out.print("|");
                            csv_element_exists = true;

                            boolean enclose = false;
                            if( display_data.indexOf(",")  > -1 ||
                                display_data.indexOf("\"") > -1 )
                                enclose = true;

                            display_data = display_data.replaceAll("\"", "\"\"");
                            if( enclose ) display_data = "\""+display_data+"\"";

                            out.print(display_data);
                        }
                    } else
                        out.println("    "+Utils.htmlize(display_data, false)+
                                    "&nbsp;&nbsp;"+(k+1<cdata.length?"<BR>":""));
                }
                if( rootURL.equals("/results.csv") )
                    out.print((j+1<display_columns.size()?",":""));
                else
                    out.println("</TD>");
            }
            if( rootURL.equals("/results.csv") )
                out.println("");
            else
                out.println("</TR>");
        }

        if( !rootURL.equals("/results.csv") )
            out.println("</TBODY>\n</TABLE>\n<BR>\n"+
                        "<DIV id=\"csv_link\">"+
                        "<A HREF=\"/results.csv\">Generate result (of the last search) in CSV form</A>"+
                        "</DIV><BR>"+
                        getCommonQSearchForm("", "Nodes", encode("/FS"))+
                        getCommonBottomLabel(search_type.equals("Log Records")?"hlplsearch.html":"hlpfsearch.html")+
                        "</DIV>"+
                        "</BODY>\n"+
                        "</HTML>\n");
    } // end of handleSearchResults()
}
