/************************************************************************
 *
 * InfoNetwork.java - displays info for a network
 *
 * $Id: InfoNetwork.java,v 1.2 2006-09-14 17:40:30 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoNetwork extends InfoPage
{
    public InfoNetwork() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain network key attribute from session
            String object = (String)request.getParameter("handle");
            String handle = new String(object);

            // obtain network access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasNetworkObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());

            int oid = -1;
            boolean numeric_handle = false;
            try {
                oid = Integer.parseInt(object);
                numeric_handle = true;
            } catch(Exception ex) { }

            // lookup network info
            Hashtable info = null;

            if( numeric_handle )
                info = netdb.getInfo(Constants.NETWORK, oid);
            else
                info = netdb.getInfo(Constants.NETWORK, object);

            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "Error: No info found for network "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            // check group access
            if( hasAccess && !netdb.checkGroupAccess(info) )
                hasAccess = false;

            // extact name with original case
            Hashtable names = Utils.nHash(info.get("names"));
            Object[] name = sorted_keys(names, name_cmp);
            for (int i = 0; i < name.length; ++i) {
                Hashtable nameH = (Hashtable)names.get(name[i]);

                if( numeric_handle ) { // first name will be handle
                    object = nameH.get("name")+"."+nameH.get("domain");
                    break;

                } else if( ((String)name[i]).toLowerCase().equals(object.trim().toLowerCase()) ) {
                    // not numeric handle
                    object = nameH.get("name")+"."+nameH.get("domain");
                    break;
                }
            }

            // compress network name by stripping default domain
            object = compressName(object, default_domain);

            // info header and function bar
            out.println(
                        commonHtmlBegin("NetworkInfo: "+object)+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Network Info<br>"+object, "hlpnetwork.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        commonLinksPanel("Network", hasAccess, handle, store, true));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));

            // info function bar, qsearch panel, footer
            out.println(
                        commonLinksPanel("Network", hasAccess, handle, store, true)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n"
            );

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    } // end of doGet(...)

    public static void printOutInfo(HttpSession session, HttpServletResponse response, PrintWriter out,
                                    Hashtable info, String newFromPage)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String o_groups = "";
            Hashtable ht = (Hashtable)info.get("groups");
            if (ht != null)
            {
                java.util.List alist = Arrays.asList(ht.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                {
                    if (o_groups.length() > 0)
                        o_groups += ", ";
                    o_groups += Utils.htmlize((String)oa[i], false);
                }
            }

            String o_comment = (String)info.get("comment");
            o_comment = (o_comment == null ? "" : o_comment);

            String o_dhcp_service = (String)info.get("dhcp_service");
            o_dhcp_service = (o_dhcp_service == null ? "" : o_dhcp_service);

            // format dhcp_options as unsorted rows of 'key: value'
            String o_dhcp_options = "";
            ht = (Hashtable)info.get("dhcp_options");
            if (ht != null)
            {
                java.util.List list1 = Arrays.asList(ht.keySet().toArray());
                Collections.sort(list1, name_cmp);
                Object[] optsArray = list1.toArray();
                for( int j = 0; j < optsArray.length; j++ ) {
                    Hashtable optionH = (Hashtable)ht.get(optsArray[j]);
                    if (o_dhcp_options.length() > 0)
                        o_dhcp_options += "<BR>";
                    o_dhcp_options +=
                        Utils.htmlize(Utils.n2s(optionH.get("option"))+"="+Utils.n2s(optionH.get("value")),
                                      false);
                }
            }

            // network info content
            out.println(
                "<TABLE WIDTH='100%'>\n"+
                "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n");

            Hashtable namesH = (Hashtable)info.get("names");
            java.util.List list = Arrays.asList(namesH.keySet().toArray());
            Collections.sort(list, name_cmp);
            Object[] namesArray = list.toArray();
            for (int i = 0; i < namesArray.length; ++i) {
                Hashtable nameH = (Hashtable)namesH.get(namesArray[i]);
                out.println("<TR>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Network Name</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'><B>"+
                            Utils.fullDisplayName((String)nameH.get("name"),
                                                  (String)nameH.get("domain"),
                                                  default_domain)+
                            "</B></TD>\n"+
                            "</TR>\n");
            }

            // locations
            Hashtable locations = Utils.nHash(info.get("locations"));
            out.println("<TR><TD VALIGN='TOP' WIDTH='10%' NOWRAP>Locations</TD><TD></TD><TD>");
            if( locations.size() > 0 ) {
                Object[] loc = sorted_keys(locations, name_cmp);
                for (int i = 0; i < loc.length; ++i) {
                    Hashtable location = (Hashtable)locations.get(loc[i]);
                    out.println((i>0?"<BR>":"")+
                                "<B>"+Utils.htmlize(location.get("name")+
                                              " ("+location.get("site")+"-"+
                                              location.get("code")+")", false)+"</B>");
                }
            }
            out.println("</TD></TR>");

            out.println(
                "<TR>\n"+
                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Groups</TD>\n"+
                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                "  <TD VALIGN='TOP'><B>"+o_groups+"</B></TD>\n"+
                "</TR>\n"+
                (o_comment.length() > 0 ?
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Comment</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_comment)+"</B></TD>\n"+
                    "</TR>\n":"")+
                "<TR>\n"+
                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>DHCP Service</TD>\n"+
                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                "  <TD VALIGN='TOP'><B>"+o_dhcp_service+"</B></TD>\n"+
                "</TR>\n"+
                "<TR>\n"+
                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>DHCP Options</TD>\n"+
                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                "  <TD VALIGN='TOP'><B>"+o_dhcp_options+"</B></TD>\n"+
                "</TR>\n"
            );

            // format address spaces sections
            ht = (Hashtable)info.get("address_spaces");
            if (ht != null && ht.size() > 0)
            {
                // header for address space section
                out.println(
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "<TR BGCOLOR='#CCCCCC'>\n"+
                    "  <TD COLSPAN=3>IP Address Space(s)</TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>&nbsp;</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><TABLE WIDTH='100%'>\n"+
                    "  <TR>\n"+
                    "    <TD VALIGN='TOP' WIDTH='40%'>Address Space</TD>\n"+
                    "    <TD VALIGN='TOP' WIDTH='30%'>Low Reserve</TD>\n"+
                    "    <TD VALIGN='TOP' WIDTH='30%'>High Reserve</TD>\n"+
                    "  </TR>\n"
                );

                java.util.List xlist = Arrays.asList(ht.keySet().toArray());
                Collections.sort(xlist, ip_cmp);
                Object[] oa = xlist.toArray();
                for (int i = 0; i < oa.length; ++i)
                {
                    String a_index = String.valueOf(i+1);
                    String a_key = (String)oa[i];
                    Hashtable address = (Hashtable)ht.get(a_key);

                    Integer ival = (Integer)address.get("prefix_length");
                    int prefixLength = (ival == null ? 0 : ival.intValue());  // range 0..32
                    int prefixMask = ~(0xffffffff << (32 - prefixLength));

                    ival = (Integer)address.get("low_res");
                    int low_res = (ival == null ? 0 : ival.intValue());
                    String a_lowres = String.valueOf(low_res);

                    ival = (Integer)address.get("high_res");
                    int high_res = (ival == null ? 0 : ival.intValue());
                    String a_highres = String.valueOf(high_res);

                    String a_comment = (String)address.get("comment");
                    a_comment = (a_comment == null ? "" : a_comment);

                    String a_groups = "";
                    Hashtable groups = (Hashtable)address.get("groups");
                    if (groups != null)
                    {
                        java.util.List alist = Arrays.asList(groups.keySet().toArray());
                        Collections.sort(alist, name_cmp);
                        Object[] oa2 = alist.toArray();
                        for (int j = 0; j < oa2.length; ++j)
                        {
                            if (a_groups.length() > 0)
                                a_groups += ", ";
                            a_groups += Utils.htmlize((String)oa2[j], false);
                        }
                    }

                    // format dhcp_addresses
                    String a_dhcp_addrs = "";
                    Hashtable addrs = (Hashtable)address.get("dhcp_addresses");
                    if (addrs != null)
                    {
                        java.util.List alist = Arrays.asList(addrs.keySet().toArray());
                        Collections.sort(alist, ip_cmp);
                        Object[] oa2 = alist.toArray();
                        for (int j = 0; j < oa2.length; ++j)
                        {
                            String key = (String)oa2[j];  // ipaddr
                            Hashtable vals = (Hashtable)addrs.get(key);
                            Boolean b = (Boolean)vals.get("active?");
                            String a_active = (b != null && b.booleanValue() ? "" : "&nbsp;&nbsp;<em>inactive</em>");
                            Hashtable names = (Hashtable)vals.get("names");
                            String a_names = compressNames(names, default_domain, ", ");
                            String row = "<B>"+key +
                                (a_names.length() > 0 ? "&nbsp;&nbsp;&nbsp;("+a_names+")" : "") + "</B>"+
                                a_active;

                            if (a_dhcp_addrs.length() > 0)
                                a_dhcp_addrs += "<BR>";
                            a_dhcp_addrs += row;
                        }
                    }

                    // format dhcp_options as unsorted rows of 'key: value'
                    String a_dhcp_options = "";
                    Hashtable options = nHash(address.get("dhcp_options"));
                    if (options.size() > 0)
                    {
                        java.util.List list1 = Arrays.asList(options.keySet().toArray());
                        Collections.sort(list1, name_cmp);
                        Object[] optsArray = list1.toArray();
                        for( int j = 0; j < optsArray.length; j++ ) {
                            Hashtable optionH = (Hashtable)options.get(optsArray[j]);
                            if (a_dhcp_options.length() > 0)
                                a_dhcp_options += "<BR>";
                            a_dhcp_options +=
                                Utils.htmlize(Utils.n2s(optionH.get("option"))+"="+Utils.n2s(optionH.get("value")),
                                              false);
                        }
                    }

                    // address space info
                    out.println(
                        "  <TR>\n"+
                        "    <TD COLSPAN='3'><HR NOSHADE SIZE='1'></TD>\n"+
                        "  </TR>\n"+
                        "  <TR>\n"+
                        "    <TD VALIGN='TOP' WIDTH='20%'><B>"+a_key+"</B></TD>\n"+
                        "    <TD VALIGN='TOP' WIDTH='40%'><B>"+a_lowres+"</B></TD>\n"+
                        "    <TD VALIGN='TOP' WIDTH='40%'><B>"+a_highres+"</B></TD>\n"+
                        "  </TR>\n"+
                        "  <TR>\n"+
                        "    <TD VALIGN='TOP' WIDTH='20%'>DHCP Options</TD>\n"+
                        "    <TD VALIGN='TOP' COLSPAN='2'><B>"+a_dhcp_options+"</B></TD>\n"+
                        "  </TR>\n"+
                        "  <TR>\n"+
                        "    <TD VALIGN='TOP' WIDTH='20%'>Groups</TD>\n"+
                        "    <TD VALIGN='TOP' COLSPAN='2'><B>"+a_groups+"</B></TD>\n"+
                        "  </TR>\n"+
                        (a_comment.length() > 0 ?
                            "  <TR>\n"+
                            "    <TD VALIGN='TOP' WIDTH='20%'>Comment</TD>\n"+
                            "    <TD VALIGN='TOP' COLSPAN='2'><B>"+Utils.htmlize(a_comment)+"</B></TD>\n"+
                            "  </TR>\n" : "")+
                        "  <TR>\n"+
                        "    <TD VALIGN='TOP' WIDTH='20%'>Dynamic DHCP Addresses</TD>\n"+
                        "    <TD VALIGN='TOP' COLSPAN='2'>"+a_dhcp_addrs+"</TD>\n"+
                        "  </TR>\n"
                    );
                }

                // footer for address space section
                out.println(
                    "  </TABLE></TD>\n"+
                    "</TR>\n"
                );
            }

            Hashtable person = (Hashtable)info.get("created.by");
            String o_clink = makePersonLink(person);
            String o_cdate = sdf.format(info.get("created"));
            String o_cacct = (String)person.get("NetID");
            o_cacct = (o_cacct == null ? "" : o_cacct);
            String o_create =
                (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
                (String)person.get("name")+
                " ("+o_cacct+")" +
                (o_clink != null ? "</A>" : "")+
                "&nbsp; "+o_cdate;

            person = (Hashtable)info.get("mod.by");
            String o_mlink = makePersonLink(person);
            String o_mdate = sdf.format(info.get("modified"));
            String o_macct = (String)person.get("NetID");
            o_macct = (o_macct == null ? "" : o_macct);
            String o_modify =
                (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
                (String)person.get("name") +
                " ("+o_macct+")" +
                (o_mlink != null ? "</A>" : "")+
                "&nbsp; "+o_mdate;

            out.println(
                "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                "<TR>\n"+
                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Created By</TD>\n"+
                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                "</TR>\n"+
                "<TR>\n"+
                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Modified By</TD>\n"+
                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                "</TR>\n"+
                "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                "</TABLE>\n"
            );
    } // end of printOutInfo(...)
}
