/************************************************************************
 *
 * InfoNode.java - displays info for a node
 *
 * $Id: InfoNode.java,v 1.22 2007-10-31 17:48:45 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoNode extends InfoPage
{
    public InfoNode() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        loadData(request, response, session, store);
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        loadData(request, response, session, store);
    }

    private void loadData(HttpServletRequest  request,
                          HttpServletResponse response,
                          HttpSession         session,
                          Hashtable           store)
    {
        try {
            PrintWriter out = response.getWriter();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain node key attribute from session
            String object = request.getParameter("handle");
            String handle = new String(object);

            // obtain node access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasNodeObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());

            // First see if object's handle is number
            int oid = -1;
            boolean numeric_handle = false;
            try {
                oid = Integer.parseInt(object);
                numeric_handle = true;
            } catch(Exception ex) { }

            // lookup node info
            Hashtable info = null;

            if( numeric_handle )
                info = netdb.getInfo(Constants.NODE, oid);
            else
                info = netdb.getInfo(Constants.NODE, object);

            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "Error: No info found for node "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            info.put("handle", handle);

            // try to parse custom fields for custom views
            int custom_fields_total     = ((Integer)getData(session, "custom_fields_total")).intValue();
            Hashtable custom_node_views = nHash(getData(session, "custom.node.views"));
            Hashtable parsed_views      = new Hashtable();

            for( Object view_name : custom_node_views.keySet() ) {
                try {
                    parse_tags_for_custom_view(info, nHash(custom_node_views.get(view_name.toString())),
                                               view_name.toString(), custom_fields_total, true);
                    parsed_views.put(view_name, "");
                } catch (Exception ex) { }
            }

            info.put("parsed views", parsed_views);
            info.put("fromPage", Utils.nStr(store.get("fromPage")));

            String enabled_view = decode(Utils.nStr(request.getParameter("custom_view")));
            if( enabled_view.length() > 0 && !enabled_view.equals("--") &&
                parsed_views.get(enabled_view) == null )
                session.setAttribute("Error.errorMsg",
                                     "Error: Unable to present requested view "+
                                     "as node has changed.");
            else
                info.put("enabled view", enabled_view);

            // check access
            Boolean can_modify = (Boolean)info.get("can_modify?");
            if( can_modify == null ) can_modify = Boolean.FALSE;
            hasAccess = can_modify.booleanValue();

            // extact name with original case
            Hashtable names = nHash(info.get("names"));
            Object[] name = sorted_keys(names, name_cmp);
            for (int i = 0; i < name.length; ++i) {
                String nameLc = ((String)name[i]).toLowerCase();
                Hashtable nameH = (Hashtable)names.get(name[i]);
                if( numeric_handle ) { // first name will be handle
                    object = nameH.get("name")+"."+nameH.get("domain");
                    break;
                }
                // not numeric handle
                if( nameLc.equals(object.trim().toLowerCase()) ) {
                    object = nameH.get("name")+"."+nameH.get("domain");
                    break;
                }
            }

            // compress node name by stripping default domain
            object = compressName(object, default_domain);

            // info header and function bar
            String pos_x = request.getParameter("PosX");
            String pos_y = request.getParameter("PosY");

            if( Utils.nStr(session.getAttribute("Error.errorMsg")).trim().length() > 0  ) pos_x = pos_y = "0";

            String move_interfaces_node_types = System.getProperty("move.interfaces.node.types", "^none$");

            for( Map.Entry ent : (Set<Map.Entry>)nHash(info.get("types")).entrySet() ) {
                if( ((String)((Hashtable)ent.getValue()).get("type")).matches(move_interfaces_node_types) ) {
                    store.put("show move interfaces", "please do so");
                    break;
                }
            }

            out.println(
                        getCommonHtmlBegin("NodeInfo: "+object, pos_x, pos_y)+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Node Info<br>"+object, "hlpnode.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"modifyPage\">\n"+
                        commonLinksPanel("Node", hasAccess, handle, store));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));

            // info function bar, qsearch panel, footer
            out.println(commonLinksPanel("Node", hasAccess, handle, store)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n"
            );

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    }

    public static void printOutInfo(HttpSession session, HttpServletResponse response, PrintWriter out,
                                    Hashtable info, String newFromPage)
        throws Exception
    {
        int custom_fields_total = ((Integer)getData(session, "custom_fields_total")).intValue();

        String default_domain = (String)session.getAttribute("default_domain");
        // node content info
        if( info.get("no inner form") == null )
            out.println("<FORM NAME=\"MainForm\" METHOD=\"POST\" "+
                        "onSubmit=\"document.MainForm.PosX.value = (document.all)?document.body.scrollLeft:window.pageXOffset; "+
                        "document.MainForm.PosY.value = (document.all)?document.body.scrollTop:window.pageYOffset;\" ACTION='/InfoNode?handle="+
                        Utils.nStr(info.get("handle"))+
                        "&fromPage="+encode(Utils.nStr(info.get("fromPage")))+"'>\n"+
                        "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosX'>"+
                        "<INPUT TYPE='HIDDEN' VALUE='' NAME='PosY'>");

        out.println("<TABLE WIDTH='100%'>\n<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n");

        // names section

        ////String o_id = (String)info.get("id");
        ////o_id = (o_id == null ? "" : o_id);

        Hashtable ht = (Hashtable)info.get("names");
        if (ht != null)
        {
            java.util.List alist = Arrays.asList(ht.keySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                String a_index = String.valueOf(i+1);
                String key = (String)oa[i];  // fqdn
                Hashtable ht2 = (Hashtable)ht.get(key);
                out.println(
                            "<TR>\n"+
                            //"  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Name["+a_index+"]</TD>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Name</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'><B>"+
                            Utils.fullDisplayName((String)ht2.get("name"),
                                                  (String)ht2.get("domain"),
                                                  default_domain)+
                            "</B></TD>\n"+
                            "</TR>\n"
                            );

                String a_aliases = "";
                Hashtable aliases = (Hashtable)ht2.get("aliases");
                if (aliases != null)
                {
                    java.util.List aalist = Arrays.asList(aliases.keySet().toArray());
                    Collections.sort(aalist, name_cmp);
                    Object[] oa2 = aalist.toArray();
                    if (oa2.length > 0)
                    {
                        for (int j = 0; j < oa2.length; ++j)
                        {
                            String key2 = (String)oa2[j];
                            Hashtable ht3 = (Hashtable)aliases.get(key2);
                            String name2 = (String)ht3.get("name");
                            String domain2 = (String)ht3.get("domain");
                            if (! domain2.equalsIgnoreCase(default_domain))
                                name2 = name2 + "." + domain2;
                            if (a_aliases.length() > 0)
                                a_aliases += "<BR>";
                            a_aliases += "&nbsp;&nbsp;&nbsp;"+name2;
                        }
                        out.println(
                                    "<TR>\n"+
                                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>&nbsp;&nbsp;&nbsp;Aliases</TD>\n"+
                                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                                    "  <TD VALIGN='TOP'><B>"+a_aliases+"</B></TD>\n"+
                                    "</TR>\n"
                                    );
                    }
                }

                String a_mxs = "";
                Hashtable mxs = (Hashtable)ht2.get("mxs");
                if (mxs != null)
                {
                    java.util.List aalist = Arrays.asList(mxs.keySet().toArray());
                    Collections.sort(aalist, name_cmp);
                    Object[] oa2 = aalist.toArray();
                    if (oa2.length > 0)
                    {
                        for (int j = 0; j < oa2.length; ++j)
                        {
                            String key2 = (String)oa2[j];
                            Hashtable ht3 = (Hashtable)mxs.get(key2);
                            String name2 = (String)ht3.get("name");
                            String domain2 = (String)ht3.get("domain");
                            String pref2 = (String)ht3.get("pref");
                            pref2 = (pref2 == null ? "" : "&nbsp;&nbsp;&nbsp;Pref:<B>"+pref2+"</B>");
                            if (! domain2.equalsIgnoreCase(default_domain))
                                name2 = name2 + "." + domain2;
                            if (a_mxs.length() > 0)
                                a_mxs += "<BR>";
                            a_mxs += "&nbsp;&nbsp;&nbsp;<B>"+name2+"</B>"+pref2;
                        }
                        out.println(
                                    "<TR>\n"+
                                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>&nbsp;&nbsp;&nbsp;Receive mail for</TD>\n"+
                                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                                    "  <TD VALIGN='TOP'>"+a_mxs+"</TD>\n"+
                                    "</TR>\n"
                                    );
                    }
                }

                out.println("<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n");
            }
        }

        String o_department = (String)info.get("dept");
        o_department = (o_department == null ? "" : o_department);

        String o_location = "<B>"+
            Utils.htmlize(info.get("location")+" ("+info.get("site")+"-"+info.get("code")+")", false)+"</B>";
        String o_room = (String)info.get("room");
        o_room = (o_room == null ? "" : "Room: <B>"+Utils.htmlize(o_room)+"</B>");
        o_location = (o_location.length() == 0 ? o_room : o_location+"&nbsp;&nbsp;&nbsp;&nbsp;"+o_room);

        String o_make = (String)info.get("make");
        o_make = (o_make == null ? "" : o_make + " ");
        String o_model = (String)info.get("model");
        o_model = (o_model == null ? "" : o_model);
        o_make = o_make + o_model;


        String o_types = "";
        Hashtable node_types = nHash(info.get("types"));

        for( Map.Entry ent : (java.util.List<Map.Entry>)Utils.sort(node_types.entrySet(), Utils.node_type_cmp) )
            o_types += (o_types.length()>0?", ":"")+((Hashtable)ent.getValue()).get("type");

        String o_oses = "";
        ht = (Hashtable)info.get("oses");
        if (ht != null)
        {
            java.util.List alist = Arrays.asList(ht.keySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                if (o_oses.length() > 0)
                    o_oses += ", ";
                o_oses += (String)oa[i];
            }
        }

        String o_admins = "";
        ht = (Hashtable)info.get("admins");
        if (ht != null)
        {
            Hashtable ateams = (Hashtable)ht.get("ateams");
            if (ateams != null)
            {
                java.util.List alist = Arrays.asList(ateams.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                {
                    String name = (String)oa[i];
                    String noid = URLEncoder.encode(name, "UTF-8");
                    name += " (admin team)";
                    String row = "<A HREF=\"/Dispatch?Action=info&object=AdminTeam&fromPage="+
                        response.encodeURL(newFromPage)+"&handle="+noid+"\">"+name+"</A>";
                    if (o_admins.length() > 0)
                        o_admins += "<BR>";
                    o_admins += row;
                }
            }
            Hashtable admins = (Hashtable)ht.get("admins");
            if (admins != null)
            {
                // We're given the ID as the key. It make considerably more sense to sort
                // on the name. To do so, we need to first GET the names to sort them.
                java.util.List id_list    = Arrays.asList(admins.keySet().toArray());
                Hashtable      admin_name = new Hashtable ();

                for (int i = 0; i < id_list.size (); i++)
                {
                    Hashtable person = (Hashtable)admins.get(id_list.get (i));

                    admin_name.put((String)person.get("name") + (String)person.get("id"), id_list.get(i));
                }

                // Now we have a shiny new list of names mapped to their ID.
                java.util.List alist = Arrays.asList(admin_name.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i)
                {
                    String key = (String)admin_name.get(oa[i]);
                    Hashtable person = (Hashtable)admins.get(key);
                    String wlink = makePersonLink(person);
                    String row =
                        (wlink != null ? "<A HREF='"+wlink+"'>" : "")+
                        (String)person.get("name") +
                        (wlink != null ? "</A>" : "");
                    if (o_admins.length() > 0)
                        o_admins += "<BR>";
                    o_admins += row;
                }
            }
        }

        String o_users = "";
        ht = (Hashtable)info.get("users");
        if (ht != null)
        {
            // We're given the ID as the key. It make considerably more sense to sort
            // on the name. To do so, we need to first GET the names to sort them.
            java.util.List id_list   = Arrays.asList(ht.keySet().toArray());
            Hashtable      user_name = new Hashtable ();

            for (int i = 0; i < id_list.size (); i++)
            {
                Hashtable person = (Hashtable)ht.get(id_list.get (i));

                user_name.put((String)person.get("name") + (String)person.get("id"), id_list.get(i));
            }

            // Now we have a shiny new list of names mapped to their ID.
            java.util.List alist = Arrays.asList(user_name.keySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                String key = (String)user_name.get(oa[i]);
                Hashtable person = (Hashtable)ht.get(key);
                String wlink = makePersonLink(person);
                String row =
                    (wlink != null ? "<A HREF='"+wlink+"'>" : "")+
                    (String)person.get("name") +
                    (wlink != null ? "</A>" : "");
                if (o_users.length() > 0)
                    o_users += "<BR>";
                o_users += row;
            }
        }

        if( o_types.length() > 0 )
            out.println("<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Node Types</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_types)+"</B></TD>\n"+
                        "</TR>");

        if( info.get("move interfaces form") == null ) {
            out.println("<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>State</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+Utils.htmlize((String)Utils.n2s(info.get("state")))+"</B></TD>\n"+
                        "</TR>\n"+
                        "<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Department</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_department)+"</B></TD>\n"+
                        "</TR>\n"+
                        "<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Location</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'>"+o_location+"</TD>\n"+
                        "</TR>\n"+
                        "<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Make and Model</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_make)+"</B></TD>\n"+
                        "</TR>\n"+
                        "<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>OS</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_oses)+"</B></TD>\n"+
                        "</TR>\n"+
                        "<TR>\n"+
                        "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Administrators</TD>\n"+
                        "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                        "  <TD VALIGN='TOP'><B>"+o_admins+"</B></TD>\n"+
                        "</TR>");
            if( o_users.length() > 0 )
                out.println("<TR>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Users</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'><B>"+o_users+"</B></TD>\n"+
                            "</TR>");

            // format custom fields as 'fieldname  fieldvalue' rows
            ht = (Hashtable)info.get("tags");
            if (ht != null && ht.size() > 0 ) {
                Hashtable parsed_views = nHash(info.get("parsed views"));
                String enabled_view = Utils.nStr(info.get("enabled view"));
                String help_url = "";
                if( enabled_view.length() > 0 && !enabled_view.equals("--") )
                    help_url = Utils.nStr(nHash(info.get("custom tags "+enabled_view)).get("help url")).trim();

                out.println("<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                            "<TR CLASS='interfaceHeader'>\n"+
                            " <TD COLSPAN='3' VALIGN='TOP' WIDTH='10%' NOWRAP>"+
                            "  <TABLE CELLSPACING='0' WIDTH='100%'><TR><TD NOWRAP>"+
                            "Custom Fields"+
                            (enabled_view.length()>0&&!enabled_view.equals("--")?
                             (" in "+
                              (help_url.length()>0?("<a href=\""+help_url+"\">"+enabled_view+"</a>"):
                               enabled_view)+
                              " view"):"")+
                            "</TD><TD WIDTH='100%'>&nbsp;</TD>\n"+
                            "<TD ALIGN='RIGHT' NOWRAP>");

                if( parsed_views.size() > 0 && info.get("no inner form") == null ) {
                    out.println("<SELECT ID='Selector_Tag_Custom_View' "+
                                "NAME='custom_view' SIZE='1' "+
                                "onChange='document.MainForm.PosX.value = (document.all)?document.body.scrollLeft:window.pageXOffset; "+
                                "document.MainForm.PosY.value = (document.all)?document.body.scrollTop:window.pageYOffset; "+
                                "document.MainForm.submit()'>"+
                                "<OPTION VALUE='"+encode("--")+"'>--");

                    for( String view_name : (java.util.List<String>)Utils.sort(parsed_views.keySet(), name_cmp) )
                        out.println("<OPTION VALUE='"+encode(view_name)+"'"+
                                    (enabled_view.equals(view_name)?" SELECTED":"")+">"+
                                    Utils.htmlize(view_name, false));

                    out.println("</SELECT>&nbsp;<NOSCRIPT><INPUT ID=\"Button_Go_View\" TYPE=\"SUBMIT\" "+
                                "NAME=\"Action\" VALUE=\"Go\">&nbsp;</NOSCRIPT></FORM>");
                }

                out.println("</TD></TR></TABLE></TD></TR>");

                Hashtable sections = new Hashtable();
                if( enabled_view.length() > 0 && !enabled_view.equals("--") ) {
                    // display custom view
                    out.println("<TR><TD></TD><TD></TD>\n"+
                                "<TD VALIGN='TOP'><TABLE CELLSPACING='0' CELLPADDING='0'>");

                    Hashtable custom_tags = nHash(info.get("custom tags "+enabled_view));
                    Hashtable tags = nHash(custom_tags.get("entries"));
                    sections = nHash(custom_tags.get("sections"));

                    for( Object key : Utils.sort(sections.keySet(), num_cmp) ) {
                        Hashtable section_h = (Hashtable)sections.get(key);
                        String section      = Utils.nStr(section_h.get("name"));
                        String section_dsp  = Utils.nStr(section_h.get("display label")).trim();
                        if( section_dsp.length() > 0 )
                            out.println("<TR><TD COLSPAN='2'><B>"+section_dsp+"</B></TD></TR>");

                        for( Object entr_key : Utils.sort(tags.keySet(), num_cmp) ) {
                            Hashtable entry = (Hashtable)tags.get(entr_key);
                            if( !entry.get("section").equals(section) ) continue;
                            String display_label = Utils.nStr(entry.get("display label"));
                            if( !((String)Utils.n2s(entry.get("html element"))).startsWith("selector.") ) { // selectors are build separately
                                if( Utils.nStr(entry.get("html element")).equals("label") ) {
                                    String info_label = (String)entry.get("info label");
                                    if( Utils.nStr(entry.get("display on info screen")).matches("^[yY][eE][sS]$") )
                                        out.println("<TR><TD VALIGN='TOP' COLSPAN='2'>"+
                                                    (info_label==null?display_label:info_label)+"</TD></TR>");
                                } else
                                    out.println("<TR><TD>"+display_label+
                                                "&nbsp;&nbsp;<BR>&nbsp;</TD><TD VALIGN='TOP'><B>"+
                                                build_html_element(info, 0, entry, enabled_view, true)+
                                                "</B></TD></TR>");
                            }
                        }

                        // build selectors
                        Hashtable selectors = nHash(custom_tags.get("selectors"));
                        for( Object entr_key : Utils.sort(tags.keySet(), num_cmp) ) {
                            Hashtable entry = (Hashtable)tags.get(entr_key);
                            String selector = (String)entry.get("selector");
                            if( selector == null ) continue;
                            Hashtable selector_def = (Hashtable)selectors.get(selector);
                            if( selector_def == null ) continue;
                            Hashtable data = Utils.get_hash_or_create(selector_def, "data");
                            String value = ((String)Utils.n2s(entry.get("value"))).trim();
                            if( value.equals("") )
                                value = ((String)Utils.n2s(entry.get("default value"))).equalsIgnoreCase("on")?"selected":"";
                            entry.put("selected", value);
                            data.put(entr_key, entry);
                        }

                        for( String selector_name : (java.util.List<String>)Utils.sort(selectors.keySet(), name_cmp) ) {
                            Hashtable selector_def = (Hashtable)selectors.get(selector_name);

                            // skip selector which belon to different section
                            if( !Utils.nStr(selector_def.get("section")).equals(section) ) continue;

                            out.println("<TR><TD VALIGN='TOP'>"+selector_def.get("display label")+
                                        "&nbsp;&nbsp;</TD><TD>");

                            Hashtable selector_data = nHash(selector_def.get("data"));

                            String list = "";
                            for( Object entr_key : Utils.sort(selector_data.keySet(), num_cmp) ) {
                                Hashtable entry = (Hashtable)selector_data.get(entr_key);
                                if( Utils.nStr(entry.get("selected")).equals("selected") )
                                    list += (list.length()>0?", ":"")+"<B>"+
                                        Utils.htmlize(Utils.nStr(entry.get("display label")), false)+"</B>";
                            }
                            out.println(list+"</TD></TR>");
                        }
                    }
                    out.println("</TABLE></TD></TR>");
                    if( ht.size() > sections.size() )
                        out.println("<TR CLASS='interfaceHeader'>\n"+
                                    "<TD COLSPAN='3' VALIGN='TOP' WIDTH='10%' NOWRAP>"+
                                    "Remaining Custom Fields</TD></TR>");
                }

                java.util.List alist = Arrays.asList(ht.keySet().toArray());
                Collections.sort(alist, name_cmp);
                Object[] oa = alist.toArray();
                for (int i = 0; i < oa.length; ++i) {
                    String key = (String)oa[i];
                    if( CNVParser.foundSection(sections, key) ) continue;
                    String val = (String)ht.get(key);
                    out.println("<TR>\n"+
                                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>"+Utils.htmlize(key)+"</TD>\n"+
                                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                                "  <TD VALIGN='TOP'><B>"+Utils.htmlize(val)+"</B></TD>\n"+
                                "</TR>\n"
                                );
                }
            }
        }

        // interfaces section
        ht = (Hashtable)info.get("interfaces");
        Hashtable troubled_hws = nHash(info.get("troubled_hws"));
        if (ht != null)
        {
            java.util.List alist = Arrays.asList(ht.entrySet().toArray());
            Collections.sort(alist, intfc_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                String i_index = String.valueOf(i+1);

                Map.Entry entry = (Map.Entry)oa[i];
                Hashtable iface = (Hashtable)entry.getValue();

                String intfcID = (String)entry.getKey();
                Boolean b = (Boolean)iface.get("roaming?");
                boolean isRoaming = (b != null && b.booleanValue());
                String i_roaming = (isRoaming ? "Yes" : "No");
                b = (Boolean)iface.get("use-dhcp?");
                boolean useDHCP = (b != null && b.booleanValue());
                String i_usedhcp = (useDHCP ? "Yes" : "No");
                String i_hardware = (String)iface.get("hw");
                i_hardware =
                    (info.get("move interfaces form")!=null?
                     ("<INPUT ID='new.hw."+i+"' "+
                      "TYPE='TEXT' SIZE='15' NAME='"+encode("intfc_hw_"+intfcID)+"' VALUE='"+
                      Utils.htmlize(Utils.hw2display((String)(iface.get("hw new")==null?iface.get("hw"):iface.get("hw new"))))+"'"+
                      (troubled_hws.get(Utils.nStr(iface.get("hw new")==null?iface.get("hw"):iface.get("hw new")).trim().toLowerCase())!=null?" CLASS='error_form_font'":"")+"> "):
                     (i_hardware == null ? "" :
                      ("<b>"+(i_hardware.substring(0, 4)+"."+
                              i_hardware.substring(4, 8)+"."+
                              i_hardware.substring(8)).toLowerCase())+"</b>"));

                i_hardware +=
                    (i_hardware.length() > 0 ? "&nbsp;&nbsp;&nbsp;&nbsp;" : "")+
                    "DHCP/BootP: <b>"+i_usedhcp+"</b>&nbsp;&nbsp;&nbsp;&nbsp;"+
                    "Roaming: <b>"+i_roaming+"</b>";

                String i_dhcp_options = "";
                Hashtable ht2 = (Hashtable)iface.get("dhcp-options");
                if (ht2 != null) {
                    java.util.List alist2 = Arrays.asList(ht2.keySet().toArray());
                    Collections.sort(alist2, name_cmp);

                    Object[] oa2 = ht2.keySet().toArray();
                    for (int j = 0; j < oa2.length; ++j) {
                        Hashtable dhcp_option_H = (Hashtable)ht2.get(oa2[j]);

                        String opt = (String)dhcp_option_H.get("option");
                        String val = (String)dhcp_option_H.get("value");
                        if (i_dhcp_options.length() > 0)
                            i_dhcp_options += "<BR>";
                        i_dhcp_options += Utils.htmlize(opt + " = " + val, false);
                    }
                }

                String i_ips = "";
                ht2 = (Hashtable)iface.get("ips");
                if (ht2 != null)
                {
                    java.util.List alist2 = Arrays.asList(ht2.keySet().toArray());
                    Collections.sort(alist2, ip_str_cmp);
                    Object[] oa2 = alist2.toArray();
                    for (int j = 0; j < oa2.length; ++j)
                        {
                            String key2 = (String)oa2[j];  // ipaddr
                            Hashtable vals = (Hashtable)ht2.get(key2);
                            Boolean b2 = (Boolean)vals.get("active?");
                            String i_active = (b2 != null && b2.booleanValue() ? "" : "&nbsp;&nbsp;&nbsp;&nbsp;Inactive");
                            Hashtable names = (Hashtable)vals.get("names");
                            String i_names = compressNames(names, default_domain, ", ");
                            String row = "<b>"+key2+"</b>"+
                                (i_names.length() > 0 ? "&nbsp;&nbsp;&nbsp;(<b>"+i_names+"</b>)" : "") +
                                (nHash(info.get("types")).get("Advanced")!=null?
                                 ("&nbsp;&nbsp;"+Utils.nStr(nHash(CommonPage.ptr_pref.get(Utils.nStr(vals.get("PTRpref")))).get("info name"))):"")+
                                i_active;

                            if (i_ips.length() > 0)
                                i_ips += "<BR>";
                            i_ips += row;
                        }
                }

                String i_names = "";
                ht2 = (Hashtable)iface.get("names");
                if (ht2 != null)
                {
                    i_names = compressNames(ht2, default_domain, ", ");
                }

                out.println(
                            "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                            "<TR>\n"+
                            //"  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Interface["+i_index+"]</TD>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>"+(info.get("move interfaces form")!=null?
                             (" <input id=\"interface.cb."+i+"\""+" type=checkbox name=\""+
                              "interface.to.move.cb\" value=\""+encode(intfcID)+"\""+(iface.get("to_move")!=null?" CHECKED":"")+">&nbsp;"):"")+
                            "Interface "+i_index+

                            "</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'><B>"+i_names+"</B></TD>\n"+
                            "</TR>\n"+
                            "<TR>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Hardware Address</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'>"+i_hardware+"</TD>\n"+
                            "</TR>\n"+
                            "<TR>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>IP Addresses</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'>"+i_ips+"</TD>\n"+
                            "</TR>");
                if( i_dhcp_options.length() > 0 )
                    out.println("<TR>\n"+
                                "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>DHCP Options</TD>\n"+
                                "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                                "  <TD VALIGN='TOP'><B>"+i_dhcp_options+"</B></TD>\n"+
                                "</TR>\n"
                                );
            }
        }

        // ipc addresses section

        String o_ipc_addrs = "";
        Hashtable addrs = (Hashtable)info.get("ipc_addresses");
        if (addrs != null)
        {
            java.util.List alist = Arrays.asList(addrs.keySet().toArray());
            Collections.sort(alist, ip_str_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                String key = (String)oa[i];  // ipaddr
                Hashtable vals = (Hashtable)addrs.get(key);
                Boolean b = (Boolean)vals.get("active?");
                String a_active = (b != null && b.booleanValue() ? "" : "&nbsp;&nbsp;&nbsp;&nbsp;Inactive");
                Hashtable names = (Hashtable)vals.get("names");
                String a_names = compressNames(names, default_domain, ", ");
                String row = key +
                    (a_names.length() > 0 ? "&nbsp;&nbsp;&nbsp;("+a_names+")" : "") +
                    a_active;

                if (o_ipc_addrs.length() > 0)
                    o_ipc_addrs += "<BR>";
                o_ipc_addrs += row;
            }
        }

        // comment section
        SimpleDateFormat sdf2 = new SimpleDateFormat("MM/dd/yyyy");
        String o_expdate = (String)Utils.n2s(info.get("exp.date") == null ? "" : sdf2.format(info.get("exp.date")));

        String o_groups = "";
        ht = (Hashtable)info.get("groups");
        if (ht != null)
        {
            java.util.List alist = Arrays.asList(ht.keySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i)
            {
                if (o_groups.length() > 0)
                    o_groups += ", ";
                o_groups += Utils.htmlize((String)oa[i], false);
            }
        }

        String o_comment = (String)info.get("comment");
        o_comment = (o_comment == null ? "" : o_comment);

        Hashtable person = (Hashtable)info.get("created.by");
        String o_clink = makePersonLink(person);
        String o_cdate = sdf.format(info.get("created"));
        String o_cacct = (String)person.get("NetID");
        o_cacct = (o_cacct == null ? "" : o_cacct);
        String o_create =
            (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
            (String)person.get("name")+
            " ("+o_cacct+")" +
            (o_clink != null ? "</A>" : "")+
            "&nbsp; "+o_cdate;

        person = (Hashtable)info.get("mod.by");
        String o_mlink = makePersonLink(person);
        String o_mdate = sdf.format(info.get("modified"));
        String o_macct = (String)person.get("NetID");
        o_macct = (o_macct == null ? "" : o_macct);
        String o_modify =
            (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
            (String)person.get("name") +
            " ("+o_macct+")" +
            (o_mlink != null ? "</A>" : "")+
            "&nbsp; "+o_mdate;

        if( info.get("move interfaces form") == null ) {
            if( nHash(info.get("types")).get("Template") != null )
                out.println("<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                            "<TR><TD WIDTH='10%' NOWRAP>Address Space</TD>"+
                            "<TD WIDTH='5%'>&nbsp;</TD><TD><B>"+Utils.n2s(info.get("addr_space"))+"</B></TD></TR>");

            out.println((o_ipc_addrs.length() > 0 ?
                         "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                         "<TR>\n"+
                         "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>IP Addresses<br>(IPC provider)</TD>\n"+
                         "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                         "  <TD VALIGN='TOP'><B>"+o_ipc_addrs+"</B></TD>\n"+
                         "</TR>\n" : "")+
                        "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>");

            if( o_expdate.length() > 0 )
                out.println("<TR>\n"+
                            "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Expiration Date</TD>\n"+
                            "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                            "  <TD VALIGN='TOP'><B>"+o_expdate+"</B></TD>\n"+
                            "</TR>");
        } else
            out.println("<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>");

        out.println("<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Groups</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_groups+"</B></TD>\n"+
                    "</TR>\n"+
                    (o_comment.length() > 0 ?
                     "<TR>\n"+
                     "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Comment</TD>\n"+
                     "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                     "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_comment)+"</B></TD>\n"+
                     "</TR>\n":"")+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Record ID</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+Utils.htmlize(((String)Utils.n2s((String)info.get("id"))).trim(), false)+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Created By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Modified By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "</TABLE>\n");

        if( info.get("move interfaces form") !=null )
            out.println("<INPUT TYPE=\"Hidden\" NAME=\"ready\" VALUE=\"yes\">");

        if( info.get("no inner form") == null )
            out.println("</FORM>");
    } // end of printOutInfo(...)
}
