/************************************************************************
 *
 * InfoUser.java - displays info for an admin-team
 *
 * $Id: InfoUser.java,v 1.2 2006-10-31 04:01:07 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class InfoUser extends InfoPage
{
    public InfoUser() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        try {
            HttpSession session = begin(request, response, store);
            if( session == null )
                return;

            PrintWriter out = response.getWriter();
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");

            String object = request.getParameter("handle");
            String handle = new String(object);

            // obtain admin team access flag from session
            Boolean checkValue = (Boolean)session.getAttribute("hasUserObjectAccess?");
            boolean hasAccess = (checkValue != null && checkValue.booleanValue());

            // lookup user info
            Hashtable info = netdb.getInfo(Constants.USER, object);
            if (info == null || info.get("id") == null ) {
                session.setAttribute("Error.errorMsg",
                                     "Error: No info found for user "+object);
                response.sendRedirect((String)store.get("fromPage"));
                return;
            }

            if( hasAccess && !netdb.checkGroupAccess(info) )
                hasAccess = false;

            // info header and function bar
            out.println(commonHtmlBegin("UserInfo: "+object)+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "User Info<br>"+object, "hlpuser.html",
                                     decode((String)store.get("newFromPage")))+
                        msgPanels(session)+
                        "<DIV id=\"infoPage\">\n"+
                        commonLinksPanel("User", hasAccess, handle, store));

            printOutInfo(session, response, out, info, (String)store.get("newFromPage"));

            out.println(commonLinksPanel("User", hasAccess, handle, store)+
                        "<HR NOSHADE SIZE=1>\n"+
                        getCommonQSearchForm("", "Nodes", (String)store.get("newFromPage"))+
                        "<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    } // end of doGet(...)

    public static void printOutInfo(HttpSession session,
                                    HttpServletResponse response,
                                    PrintWriter out, Hashtable info,
                                    String newFromPage)
        throws Exception
    {
        String o_name = (String)info.get("name");
        o_name = (o_name == null ? "???" : o_name);

        // make StanfordWho link if has directory id
        String dirid = (String)info.get("DirID");
        String o_nlink = (dirid != null ?
                          System.getProperty(Constants.STANFORDWHO_URL_KEY)+dirid : null);

        o_name =
            (o_nlink != null ? "<A HREF='"+o_nlink+"'>" : "")+
            o_name+
            (o_nlink != null ? "</A>" : "");

        String o_netid = (String)info.get("NetID");
        o_netid = (o_netid == null ? "" : o_netid);

        String o_email = (String)info.get("email");
        o_email = Utils.htmlize(o_email == null ? "" : o_email, false);

        String o_phone = (String)info.get("phone");
        o_phone = (o_phone == null ? "" : o_phone);
        if (o_phone.startsWith("+1 "))
            o_phone = o_phone.substring(3);  // strip "+1 " prefix

        Boolean b = (Boolean)info.get("active?");
        String o_active = (b == null ? "Inactive" : (b.booleanValue() ? "Active" : "Inactive"));

        String o_defdomain = (String)info.get("default_domain");
        o_defdomain = (o_defdomain == null ? "" : o_defdomain);

        String o_defaddr = (String)info.get("default_addr_space");
        o_defaddr = (o_defaddr == null ? "" : o_defaddr);

        // format dept/consultant lines
        Vector official_depts = new Vector();
        Vector other_depts = new Vector();
        Hashtable ht = (Hashtable)info.get("departments");
        if (ht != null) {
            Object[] oa = ht.keySet().toArray();
            for (int i = 0; i < oa.length; ++i) {
                String d_key = (String)oa[i];
                Hashtable ht2 = (Hashtable)ht.get(d_key);
                b = (Boolean)ht2.get("official?");
                boolean isOfficial = (b != null && b.booleanValue());
                String curr_dept = "";
                Hashtable ht3 = (Hashtable)ht2.get("consultants");
                if (ht3 != null) {
                    Object[] oa2 = ht3.keySet().toArray();
                    for (int j = 0; j < oa2.length; ++j) {
                        String c_key = (String)oa2[j];
                        Hashtable person = (Hashtable)ht3.get(c_key);
                        String c_wlink = makePersonLink(person);
                        String c_name =
                            (c_wlink != null ? "<A HREF='"+c_wlink+"'>" : "")+
                            (String)person.get("name") +
                            (c_wlink != null ? "</A>" : "");

                        if (curr_dept.length() > 0)
                            curr_dept += ", ";
                        curr_dept += c_name;
                    }
                    if (curr_dept.length() > 0)
                        curr_dept = "( " + curr_dept + " )";
                }
                curr_dept = Utils.htmlize(d_key, false) + " " + curr_dept;

                if (isOfficial)
                    official_depts.addElement(curr_dept);
                else
                    other_depts.addElement(curr_dept);
            }
        }

        // sort and format official depts
        String o_officials = "";
        if (official_depts.size() > 0) {
            java.util.List alist = Arrays.asList(official_depts.toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_officials.length() > 0)
                    o_officials += "<BR>";
                o_officials += (String)oa[i];
            }
        }

        // sort and format other depts
        String o_others = "";
        if (other_depts.size() > 0) {
            java.util.List alist = Arrays.asList(other_depts.toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_others.length() > 0)
                    o_others += "<BR>";
                o_others += (String)oa[i];
            }
        }

        // groups section

        String o_defgroup = (String)info.get("default_group");
        o_defgroup = Utils.htmlize(o_defgroup == null ? "" : o_defgroup, false);

        b = (Boolean)info.get("all_groups?");
        boolean allgroups = (b != null && b.booleanValue());

        String o_groups = (allgroups ? "All groups" : "");
        ht = (Hashtable)info.get("groups");
        if (ht != null) {
            java.util.List alist = Arrays.asList(ht.keySet().toArray());
            Collections.sort(alist, name_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_groups.length() > 0)
                    o_groups += "<BR>";
                o_groups += Utils.htmlize((String)oa[i], false);
            }
        }

        // records section

        b = (Boolean)info.get("all_objects?");
        boolean allobjects = (b != null && b.booleanValue());

        String o_records = (allobjects ? "All records" : "");
        ht = (Hashtable)info.get("records");
        if (ht != null) {
            ////Object[] sa = ht.values().toArray();
            Object[] sa = ht.keySet().toArray();
            java.util.List alist = Arrays.asList(sa);
            Collections.sort(alist, records_cmp);
            Object[] oa = alist.toArray();
            for (int i = 0; i < oa.length; ++i) {
                if (o_records.length() > 0)
                    o_records += "<BR>";
                String rec = (String)oa[i];
                if( rec.equals("Owner") )
                    o_records += "Group";
                else
                    o_records += rec;
            }
        }

        // comment section

        String o_comment = (String)info.get("comment");
        o_comment = (o_comment == null ? "" : o_comment);

        String o_lastlogin = (info.get("last_login") == null ? "" : sdf.format(info.get("last_login")));

        Hashtable person = (Hashtable)info.get("created.by");
        String o_clink = makePersonLink(person);
        String o_cdate = sdf.format(info.get("created"));
        String o_cacct = (String)person.get("NetID");
        o_cacct = (o_cacct == null ? "" : o_cacct);
        String o_create =
            (o_clink != null ? "<A HREF='"+o_clink+"'>" : "")+
            (String)person.get("name")+
            " ("+o_cacct+")" +
            (o_clink != null ? "</A>" : "")+
            "&nbsp; "+o_cdate;

        person = (Hashtable)info.get("mod.by");
        String o_mlink = makePersonLink(person);
        String o_mdate = sdf.format(info.get("modified"));
        String o_macct = (String)person.get("NetID");
        o_macct = (o_macct == null ? "" : o_macct);
        String o_modify =
            (o_mlink != null ? "<A HREF='"+o_mlink+"'>" : "")+
            (String)person.get("name") +
            " ("+o_macct+")" +
            (o_mlink != null ? "</A>" : "")+
            "&nbsp; "+o_mdate;

        // user info content
        out.println(
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Name</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_name+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>SUNet ID</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_netid+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Email</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_email+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Phone</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_phone+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Account</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_active+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Default Domain</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_defdomain+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Default Address Space</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_defaddr+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>LNA departments<BR>(Network Consultants)</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_officials+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Other departments<BR>(Network Consultants)</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_others+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Default Group</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_defgroup+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Member Groups</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_groups+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Records</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_records+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3'><HR NOSHADE SIZE=1></TD></TR>\n"+
                    (o_comment.length() > 0 ?
                     "<TR>\n"+
                     "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Comment</TD>\n"+
                     "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                     "  <TD VALIGN='TOP'><B>"+Utils.htmlize(o_comment)+"</B></TD>\n"+
                     "</TR>\n":"")+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Login</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_lastlogin+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Created By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_create+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR>\n"+
                    "  <TD VALIGN='TOP' WIDTH='10%' NOWRAP>Last Modified By</TD>\n"+
                    "  <TD WIDTH='5%'>&nbsp;</TD>\n"+
                    "  <TD VALIGN='TOP'><B>"+o_modify+"</B></TD>\n"+
                    "</TR>\n"+
                    "<TR><TD COLSPAN='3' HEIGHT='5'></TD></TR>\n"+
                    "</TABLE>\n"
                    );

    } // end of printOutInfo
}
