/************************************************************************
 *
 * List.java -
 *
 * $Id: List.java,v 1.3 2006-09-26 23:15:09 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class List extends CommonPage {

    private String searchType = "";

    public List() {}

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        response.sendRedirect("/home");
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        try {
            HttpSession session = begin(request, response, null);
            if( session == null )
                return;

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String what = URLDecoder.decode((String)Utils.n2s(request.getParameter("what")), "UTF-8");

            Hashtable data = null;


            if( what.equals("Groups") )
                data = netdb.simpleSearch(Constants.GROUP, "*");

            else if( what.equals("Locations") )
                data = netdb.getInfo(Constants.LOCATION, "");

            else if( what.equals("Departments") )
                data = netdb.getInfo(Constants.DEPARTMENT, "");

            else if( what.equals("OSes") )
                data = netdb.getInfo(Constants.OS, "");

            else if( what.equals("States") )
                data = netdb.listStates();

            else if( what.equals("make_and_model") ) {
                what = "Makes And Models";
                data = netdb.getInfo(Constants.MODEL, "");
                // re-shuffle
                Hashtable tmp_data = new Hashtable();
                for( Hashtable model : (Collection<Hashtable>)data.values() )
                    tmp_data.put(model.get("make")+" "+model.get("model"), "");

                data = tmp_data;

            } else if( what.equals("dhcp_options") ) {
                what = "DHCP Options";
                data = netdb.getInfo(Constants.DHCP_OPTIONS, "");

            } else if( what.equals("dhcp_services") ) {
                what = "DHCP Services";
                data = netdb.listDHCPServices();

            } else
                throw new Exception("Unsupported list type");

            PrintWriter out = response.getWriter();
            out.println(
                        commonHtmlBegin("List Of "+what)+
                        commonHeader(session, NO_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                     "", what, "helpnetdb.html")+
                        "<DIV id=\"list_page\"><BR>");
            if( what.equals("States") ) {
                java.util.List list = Arrays.asList(data.entrySet().toArray());
                Collections.sort(list, state_cmp);
                Iterator it = list.iterator();
                while( it.hasNext() )
                    out.println(Utils.htmlize(Utils.nStr(((Hashtable)((Map.Entry)it.next()).getValue()).get("name")), false)+"<BR>");

            } else {
                java.util.List list = Arrays.asList(data.keySet().toArray());
                Collections.sort(list, name_cmp);
                Object[] dataArray = list.toArray();
                for( int i = 0; i < dataArray.length; i++ )
                    out.println(Utils.htmlize(dataArray[i].toString().trim(), false)+"<BR>");
            }

            out.println("<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } catch (Exception ex) {
            printCommonErrorPage(response, ex);
            return;
        }
    } // end of doGet(...)
}
