/************************************************************************
 *
 * Location.java -
 *
 * $Id: Location.java,v 1.3 2006-09-26 23:40:29 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Location extends ModifyPage {

    public Location() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Location");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();


            if( rootURL.equals("/Location") ) {
                try {
                    Hashtable locations = netdb.getInfo(Constants.LOCATION, "");
                    store.put("locations", locations);
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of locations.");
                }

            } else if( rootURL.equals("/LocationsList") ) {
                try {
                    Hashtable locations = (Hashtable)store.get("locations_for_sort");
                    if( locations == null ) {
                        locations = netdb.getInfo(Constants.LOCATION, "");
                        for( Hashtable location : (Collection<Hashtable>)locations.values() ) 
                            location.put("siteandcode", location.get("site")+"-"+location.get("code"));

                        store.put("locations_for_sort", locations);
                    }
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of locations.");
                }

            } else {
                try {
                    try {
                        Hashtable oldLocation = new Hashtable();
                        String location = ((String)Utils.n2s(request.getParameter("location"))).trim();
                        if( location.length() > 0 ) {
                            oldLocation = netdb.getInfo(Constants.LOCATION, "");
                            oldLocation = nHash(oldLocation.get(location));
                            if( oldLocation.size() == 0 )
                                throw new Exception("Location \""+location+"\" does not exist.");
                        } else {
                            // creating new location
                            oldLocation.put("location", "");
                            oldLocation.put("site",     "");
                            oldLocation.put("code",     "");
                        };
                        oldLocation.put("object", "location");
                        store.put("oldLocation", oldLocation);
                        store.put("newLocation", Utils.copyHash(oldLocation));

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    response.sendRedirect("/Location");
                    return;
                }
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, request, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable locations = (Hashtable)store.get("locations");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddLocation") )
                    action = "Create Location";
                else if( rootURL.equals("/ModifyLocation") )
                    action = "Modify Location";
            }

            if( rootURL.equals("/Location") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("locations");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select location you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one location you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                Hashtable location = new Hashtable();
                                location.put("object", "location");
                                location.put("location", decode(values[0]));
                                netdb.delete(location);
                                setData(session, "locations", netdb.getInfo(Constants.LOCATION, ""));
                                session.setAttribute("info_msg", "SUCCESS: Location '"+decode(values[0])+"' has been deleted");

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());

                        } finally {
                            try {
                                locations = netdb.getInfo(Constants.LOCATION, "");
                                store.put("locations", locations);
                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            }
                        }
                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select location you wish to delete.");

            } else if( rootURL.equals("/Location") && action.equals("Add") ) {
                response.sendRedirect("/AddLocation");
                return;

            } else if( rootURL.equals("/Location") && action.equals("Modify") ) {
                String[] values1 = request.getParameterValues("locations");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select location you wish to modify.");
                else if( values1.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one location to modify.");
                else {
                    Hashtable location = (Hashtable)locations.get(decode(values1[0]));
                    if( location == null || location.size() == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Location \""+decode(values1[0])+"\" does not exist.");
                    else
                        response.sendRedirect("/ModifyLocation?location="+encode((String)Utils.n2s(location.get("location"))));
                    return;
                }

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/Location");
                return;

            } else if( action.equals("Create Location") || action.equals("Modify Location") ) {
                Hashtable newLocation = new Hashtable();
                newLocation.put("object", "location");
                newLocation.put("location", ((String)Utils.n2s(request.getParameter("location"))).trim());
                newLocation.put("site", ((String)Utils.n2s(request.getParameter("site"))).trim());
                newLocation.put("code", ((String)Utils.n2s(request.getParameter("code"))).trim());
                store.put("newLocation", newLocation);

                try {
                    try {
                        Utils.validate(Constants.LOCATION, newLocation, Constants.NARROW);

                        if( action.equals("Create Location") ) {
                            netdb.insert(newLocation);
                            session.setAttribute("info_msg", "SUCCESS: Location has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldLocation"), newLocation);
                            session.setAttribute("info_msg", "SUCCESS: Location has been modified.");
                        }

                        setData(session, "locations", netdb.getInfo(Constants.LOCATION, ""));

                        response.sendRedirect("/Location");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "ERROR: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, request, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, HttpServletRequest request, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/Location")?"Manage Locations":
                                    (rootURL.equals("/ModifyLocation")?"Modify Location":
                                     (rootURL.equals("/LocationsList")?"List of Locations":"Create Location")))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/Location")?"Manage Locations":
                                 (rootURL.equals("/ModifyLocation")?"Modify Location":
                                  (rootURL.equals("/LocationsList")?"List of Locations":"Create Location")),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    (!rootURL.equals("/LocationsList")?
                     ("<DIV class='modHeader'>\n"+
                      "<TABLE WIDTH='100%'>\n"+
                      "<TR><TD NOWRAP>"+
                      (rootURL.equals("/Location")?"Select locations(s) to modify.":
                       (rootURL.equals("/ModifyLocation")?"Modify an existing location.":"Create new location."))+
                      "</TD></TR></TABLE></DIV>"):""));

        if( rootURL.equals("/Location") ) {
            Hashtable locations = (Hashtable)store.get("locations");
            out.println("<BLOCKQUOTE>Sortable list of <a href='/LocationsList' target='_blank'>locations</a>."+
                        "<TABLE WIDTH='100%'><TR><TD WIDTH='1%'>&nbsp;<BR>"+
                        "<SELECT ID='Selector_Locations' SIZE=30 NAME='locations' "+
                        "ondblclick=\"document.Form.Button_Modify.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.Button_Modify.click();\" MULTIPLE>");
            java.util.List list = Arrays.asList(locations.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String key = (String)it.next();
                Hashtable location = (Hashtable)locations.get(key);
                String location_name = (String)Utils.n2s(location.get("location"));
                String site          = (String)Utils.n2s(location.get("site"));
                String code          = (String)Utils.n2s(location.get("code"));
                out.println("<OPTION VALUE='"+encode(key)+"'>"+
                            Utils.htmlize(location_name+" ("+site+"-"+code+")", false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Add' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='Button_Modify' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='Button_Delete' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD>");

            out.println("</TD></TR>"+
                        "</TABLE></BLOCKQUOTE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else if( rootURL.equals("/LocationsList") ) {
            // sortable list of locations
            String sorted_column = request.getParameter("sort");
            if( sorted_column == null || sorted_column.trim().length() == 0 )
                sorted_column = "siteandcode";
            String sort_dir = request.getParameter("dir");
            if( sort_dir == null || sort_dir.trim().length() == 0 )
                sort_dir = "down";
            String dir_site_and_code = sorted_column.equals("siteandcode")?(sort_dir.equals("up")?"down":"up"):"down";
            String dir_location = sorted_column.equals("location")?(sort_dir.equals("up")?"down":"up"):"down";

            out.println("<BLOCKQUOTE><TABLE><THEAD frame=bellow><TR>"+
                        "<TD NOWRAP><A HREF=\"/LocationsList?sort=siteandcode&dir="+encode(dir_site_and_code)+"\">"+
                        "Site-Code</A>&nbsp;&nbsp;</TD>"+
                        "<TD NOWRAP><A HREF=\"/LocationsList?sort=location&dir="+encode(dir_location)+"\">"+
                        "Name</A>&nbsp;&nbsp;</TD>"+
                        "</TR></THEAD>\n<TBODY>");

            Object[] locations =
                TableHashSort.doSort((Hashtable)store.get("locations_for_sort"), sorted_column,
                                     TableHashSort.STRING, sort_dir.equals("down")?TableHashSort.DOWN:TableHashSort.UP);
            for( int i = 0; i < locations.length; i++ ) {
                out.println("<TR><TD NOWRAP>"+Utils.htmlize((String)((Hashtable)((Map.Entry)locations[i]).getValue()).get("siteandcode"), false)+
                            "</TD><TD NOWRAP>"+Utils.htmlize((String)((Hashtable)((Map.Entry)locations[i]).getValue()).get("location"), false)+
                            "</TD></TR>");
            }

            out.println(""+
                        "</TABLE></BLOCKQUOTE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying location
            Hashtable newLocation = (Hashtable)store.get("newLocation");
            out.println("<TABLE WIDTH='100%'><TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Location</TD>"+
                        "<TD><INPUT ID='TextField_Location' TYPE='TEXT' SIZE='21' NAME='location' value=\""+
                        Utils.htmlize((String)Utils.n2s(newLocation.get("location")), false)+
                        "\"></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Site</TD>"+
                        "<TD><INPUT ID='TextField_Site' TYPE='TEXT' SIZE='21' NAME='site' value=\""+
                        Utils.htmlize((String)Utils.n2s(newLocation.get("site")), false)+
                        "\"></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Code</TD>"+
                        "<TD><INPUT ID='TextField_Code' TYPE='TEXT' SIZE='21' NAME='code' value=\""+
                        Utils.htmlize((String)Utils.n2s(newLocation.get("code")), false)+
                        "\"></TD></TR>\n"+
                        "<TR COLSPAN='2'><TD><INPUT ID='Button_Create_Or_Modify' TYPE='SUBMIT' NAME='action' VALUE=\""+
                        (rootURL.equals("/AddLocation")?"Create Location":"Modify Location")+
                        "\"> <INPUT ID='Button_Cancel' TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    } // end of printOutHtml(...)
}
