/************************************************************************
 *
 * LoginServlet.java - initial http session creation and loggin in
 *
 * $Id: LoginServlet.java,v 1.7 2008-05-23 18:51:39 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2002, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class LoginServlet extends ModifyPage
{
    private static Comparator people_cmp = new Comparator() {
            public int compare(Object o1, Object o2) {
                Hashtable person1 = (Hashtable)((Map.Entry)o1).getValue();
                Hashtable person2 = (Hashtable)((Map.Entry)o2).getValue();
                return (Utils.nStr(person1.get("lastname"))+" "+Utils.nStr(person1.get("NetID"))).
                    compareToIgnoreCase(Utils.nStr(person2.get("lastname"))+" "+
                                        Utils.nStr(person2.get("NetID")));
                    }
            public boolean equals(Object obj) {
                return true;
            }
        };

    public LoginServlet() {}

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        String netID = decode(Utils.nStr(request.getParameter("login_as_person")));
        response.sendRedirect("/login?uid="+netID);
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        response.setContentType("text/html");

        HttpSession session = request.getSession(false);
        if( session != null ) {
            session.invalidate();
        }
        session = request.getSession(true);

        Hashtable store = new Hashtable();
        String sessionKey = "store "+getNextKey();
        session.setAttribute(sessionKey, store);
        store.put("sessionKey", sessionKey);

        System.gc();

        // enfore session timeout. def. value 15 minutes
        session.setMaxInactiveInterval(Integer.parseInt(System.getProperty(Constants.TIMEOUT_KEY, "900000"))/1000);

        String netID = request.getHeader(System.getProperty(Constants.AUTH_USER_KEY, "X-WebAuth-User"));

        // following lines added for running netdb in test/debug/demo mode
        if( netID == null ) {
            if( System.getProperty(Constants.DEMO_AUTH_KEY, "").trim().equals("true") &&
                request.getParameter("uid") == null ) {
                try {
                    printOutAuthHTML(session, response.getWriter(), store);
                } catch (Exception ex) {
                    response.sendRedirect("/LoginFailed?error_msg="+encode(ex.getMessage()));
                }
                return;

            } else {
                if( System.getProperty(Constants.CHEAT_USER_KEY, "").trim().length() > 0 ) {
                    netID = System.getProperty(Constants.CHEAT_USER_KEY);
                    if( request.getParameter("uid") != null )
                        netID = request.getParameter("uid");
                }
            }
        }

        if (netID == null || netID.trim().length() == 0) {
            response.getWriter().println(
                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
                "<HTML>\n<HEAD>"+
                "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
                "<TITLE>Auth failed</TITLE>\n"+
                "</HEAD>\n\n"+
                "<BODY BGCOLOR=\"#FFFFFF\">\n"+
                "<BR><H2 ALIGN=\"center\"><FONT COLOR=\"#AA0000\">Authentication failed</FONT></H2>\n"+
                "</BODY>\n</HTML>"
            );
            return;
        }
        session.setAttribute("NetID", netID.trim());

        try {
            NetDBpi netdb = NetDBHook.createInstance(netID.trim(), false);
            session.setAttribute("sid", (String)(nHash(netdb.getState())).get("timestamp"));
            session.setAttribute("netdb", netdb);
            session.setAttribute("user", netdb.getCurrentUser());
            session.setAttribute("X-Forwarded-For", Utils.n2s(request.getHeader("X-Forwarded-For")));
            (new LoadData(session, netdb)).start();
            ((stanford.netdb.utils.Logger)netdb).info("http session started ["+session.getId()+"]");
            response.sendRedirect("/home");

        } catch (Exception ex) {
            response.sendRedirect("/LoginFailed?error_msg="+encode(ex.getMessage()));
            return;
        }

        session.setAttribute("master", new NetDBMaster());
    }

    class LoadData extends Thread {
        private HttpSession session;
        private NetDBpi netdb;

        public LoadData(HttpSession session, NetDBpi netdb) {
            this.session = session;
            this.netdb = netdb;
        }

        public void run() {
            try {
                session.setAttribute("departments",       netdb.getInfo(Constants.DEPARTMENT, ""));
                session.setAttribute("locations",         netdb.getInfo(Constants.LOCATION, ""));
                session.setAttribute("dhcp_options",      netdb.getInfo(Constants.DHCP_OPTIONS, ""));
                session.setAttribute("models",            netdb.getInfo(Constants.MODEL, 0));
                session.setAttribute("model_types",       netdb.getInfo(Constants.MODEL_TYPE, 0));
                session.setAttribute("oses",              netdb.getInfo(Constants.OS, 0));
                session.setAttribute("myGroups",          netdb.listMyGroups("*"));
                session.setAttribute("myAddrSpaces",      netdb.listMyAddrSpaces());
                session.setAttribute("dhcp_services",     netdb.listDHCPServices());
                session.setAttribute("domains",           netdb.listMyDomains());
                session.setAttribute("allRecords",        netdb.listRecords());
                session.setAttribute("states",            netdb.listMyStates(null));
                session.setAttribute("custom.node.views", (new CNVParser()).loadCustomNodeViewsData());
                session.setAttribute("node types",        netdb.list(Constants.NODE_TYPE, null, false));
                session.setAttribute("my node types",     netdb.list(Constants.NODE_TYPE, null, true));

                Hashtable global_defaults = netdb.getGlobalDefaults();
                session.setAttribute("global.defaults",   global_defaults);

                Integer custom_fields_total = new Integer(Constants.MAX_CUSTOM_FIELDS);
                try {
                    if( global_defaults.get("MaxNodeCustomFields") != null )
                        custom_fields_total = (Integer)global_defaults.get("MaxNodeCustomFields");
                } catch (Exception exx) { }
                session.setAttribute("custom_fields_total", custom_fields_total);

                session.setAttribute("dataReady", "");
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    // This is auth interface, which is to be used only
    public void printOutAuthHTML(HttpSession session, PrintWriter out, Hashtable store)
        throws Exception
    {
        out.println(commonHtmlBegin("NetDB Demo Login"));

        Hashtable people = new Hashtable();
        Hashtable users  = new Hashtable();
        try {
            SampleDirProvider dp = new SampleDirProvider();
            people = dp.lookup(".*");

            NetDBpi netdb = NetDBHook.createInstance(System.getProperty(Constants.DEMO_PREAUTH_NETID_KEY, "").trim(), false);
            users = netdb.simpleSearch(Constants.USER, "*");
            Hashtable netids = new Hashtable();
            for(Hashtable user : (Collection<Hashtable>)users.values() )
                netids.put(user.get("NetID"), "");

            for(Hashtable person : (Collection<Hashtable>)people.values() ) {
                if( netids.get(person.get("NetID")) == null )
                    person.put("nauser", "");
            }
        } catch (Exception ex) {
        }

        out.println("<FORM NAME='DemoLogin' METHOD='POST' ACTION='/login'>"+
                    "<CENTER><BR><BR><BR><BR>"+
                    "<TABLE CELLPADDING='3px' WIDTH='1%' STYLE='border-style: solid; border-color: #900; border-width: 2px;}'>"+
                    "<TR><TD STYLE='text-align: center; background-color: #900; color: #ffffff; font-size: 200%;'>NetDB Demo Login</TD></TR>"+
                    "<TR><TD><TABLE><TR><TD><IMG SRC='/images/NetDBlogo.gif'></TD>"+
                    "<TD NOWRAP='nowrap'>Login as:&nbsp;&nbsp;&nbsp;</TD><TD><SELECT ID='Selector_Login_As_Person' SIZE=1 NAME='login_as_person'>");

        for( Iterator it = Utils.sortedEntries(people, people_cmp); it.hasNext(); ) {
            Hashtable person = (Hashtable)((Map.Entry)it.next()).getValue();
            if( person.get("nauser") == null )
                out.println("<OPTION VALUE='"+encode(Utils.nStr(person.get("NetID")))+"'>"+
                            Utils.htmlize(person.get("name")+" ("+person.get("NetID")+")", false));
        }

        out.println("</SELECT></TD></TR>"+
                    "<TR><TD COLSPAN='2'>&nbsp;</TD><TD><INPUT TYPE='SUBMIT' NAME='Action' VALUE='   Login   '><BR><BR></TD></TR>"+
                    "</TR></TR></TABLE><TR>"+
                    "<TD ALIGN='justify' STYLE='border-top-style: solid; border-top-color: #900; border-top-width: 2px;'>"+
                    "NetDB uses people for user accounts, node users, node "+
                    "administrators, admin team members, and department "+
                    "consultants. NetDB searches an external directory for "+
                    "people. The directory used for this demo version contains "+
                    "the following people:"+
                    "</TD></TR>"+
                    "<TR><TD ALIGN='center'><TABLE><THEAD><TR STYLE='background-color:lightgray; '><TD>Name</TD><TD>NetID</TD></TR></THEAD><TBODY>");

        for( Iterator it = Utils.sortedEntries(people, people_cmp); it.hasNext(); ) {
            Hashtable person = (Hashtable)((Map.Entry)it.next()).getValue();
            out.println("<TR><TD>"+person.get("name")+"&nbsp;&nbsp;&nbsp;</TD><TD>"+person.get("NetID")+"</TD></TR>");
        }
        out.println("</TBODY></TABLE></TD></TR>"+
                    "</TABLE></CENTER></FORM>"+
                    "</BODY>\n"+
                    "</HTML>\n");
    }
}
