/************************************************************************
 *
 * Make.java -
 *
 * $Id: Make.java,v 1.2 2006-09-08 18:01:32 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class Make extends ModifyPage {

    public Make() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Make");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();

            if( rootURL.equals("/Make") ) {
                try {
                    Hashtable makes = netdb.getInfo(Constants.MAKE, "");
                    store.put("makes", makes);
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of makes.");
                }

            } else {

                try {
                    try {
                        Hashtable oldMake = new Hashtable();
                        String make = ((String)Utils.n2s(request.getParameter("make"))).trim();
                        if( make.length() > 0 ) {
                            oldMake = nHash(netdb.getInfo(Constants.MAKE, ""));
                            if( oldMake.get(make) == null )
                                throw new Exception("Make \""+make+"\" does not exist.");
                            oldMake.put("name", make);
                        } else {
                            // creating new make
                            oldMake.put("name", "");
                        };
                        oldMake.put("object", "make");
                        store.put("oldMake", oldMake);
                        store.put("newMake", Utils.copyHash(oldMake));

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }

                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    response.sendRedirect("/Make");
                    return;
                }
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable makes = (Hashtable)store.get("makes");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddMake") )
                    action = "Create Make";
                else if( rootURL.equals("/ModifyMake") )
                    action = "Modify Make";
            }

            if( rootURL.equals("/Make") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("makes");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select make you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one make you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                Hashtable make = new Hashtable();
                                make.put("object", "make");
                                make.put("name", decode(values[0]));
                                netdb.delete(make);
                                session.setAttribute("info_msg", "SUCCESS: Make '"+decode(values[0])+"' has been deleted");

                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());

                        } finally {
                            try {
                                makes = netdb.getInfo(Constants.MAKE, "");
                                store.put("makes", makes);
                            } catch (Exception ex) {
                                addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            }
                        }

                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select make you wish to delete.");

            } else if( rootURL.equals("/Make") && action.equals("Add") ) {
                response.sendRedirect("/AddMake");
                return;

            } else if( rootURL.equals("/Make") && action.equals("Modify") ) {
                String[] values1 = request.getParameterValues("makes");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select make you wish to modify.");
                else if( values1.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one make to modify.");
                else {
                    if( makes.get(decode(values1[0])) == null )
                        addSessionMsg(session, "Error.errorMsg", "Error: Make \""+decode(values1[0])+"\" does not exist.");
                    else
                        response.sendRedirect("/ModifyMake?make="+values1[0]);
                    return;
                }

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/Make");
                return;

            } else if( action.equals("Create Make") || action.equals("Modify Make") ) {
                Hashtable newMake = new Hashtable();
                newMake.put("object", "make");
                newMake.put("name", ((String)Utils.n2s(request.getParameter("make"))).trim());
                store.put("newMake", newMake);

                try {
                    try {
                        Utils.validate(Constants.MAKE, newMake, Constants.NARROW);

                        if( action.equals("Create Make") ) {
                            netdb.insert(newMake);
                            session.setAttribute("info_msg", "SUCCESS: Make has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldMake"), newMake);
                            session.setAttribute("info_msg", "SUCCESS: Make has been modified.");
                        }

                        response.sendRedirect("/Make");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/Make")?"Manage Makes":(rootURL.equals("/ModifyMake")?"Modify Make":"Create Make"))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/Make")?"Manage Makes":(rootURL.equals("/ModifyMake")?"Modify Make":"Create Make"),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>"+
                    (rootURL.equals("/Make")?"Select Makes(s) to modify.":
                     (rootURL.equals("/ModifyMake")?"Modify an existing Make.":"Create a new Make."))+
                    "</TD></TR></TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        if( rootURL.equals("/Make") ) {
            Hashtable makes = (Hashtable)store.get("makes");
            out.println("<TR><TD WIDTH='1%'>&nbsp;<BR><SELECT ID='Selector_Makes' SIZE=30 NAME='makes' "+
                        "ondblclick=\"document.Form.Button_Modify.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.Button_Modify.click();\" MULTIPLE>");
            java.util.List list = Arrays.asList(makes.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String make = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(make)+"'>"+Utils.htmlize(make, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Add' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='Button_Modify' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='Button_Delete' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD>");

            out.println("</TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying make
            Hashtable newMake = (Hashtable)store.get("newMake");
            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Make</TD>"+
                        "<TD><INPUT ID='TextField_Make' TYPE='TEXT' SIZE='21' NAME='make' value=\""+
                        Utils.htmlize((String)Utils.n2s(newMake.get("name")), false)+
                        "\"></TD></TR>\n"+
                        "<TR COLSPAN='2'><TD><INPUT ID='Button_Create_Or_Modify' TYPE='SUBMIT' NAME='action' VALUE='"+
                        (rootURL.equals("/AddMake")?"Create Make":"Modify Make")+
                        "'> <INPUT ID='Button_Cancel' TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    } // end of printOutHtml(...)
}
