/************************************************************************
 *
 * ModifyDomain.java -
 *
 * $Id: ModifyDomain.java,v 1.8 2007-06-26 00:29:28 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class ModifyDomain extends ModifyPage {

    public ModifyDomain() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Domain");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain key domain attributes from request
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            if( object != null ) {
                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyDomain") ) {
                    // lock
                    try {
                        lockTm = netdb.lock(Constants.DOMAIN, handle);
                        if( lockTm == null )
                            throw new Exception("Failed to lock domain "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldDomain = null;
            if( rootURL.equals("/CreateDomain") ) {

                if( !netdb.checkAccess(Constants.DOMAIN) )
                    throw new Exception("User '"+session.getAttribute("NetID")+"' can not create domains.");

                if( handle == null ) { // creating clean domain
                    oldDomain =
                        Utils.hash("id", "",
                                   "object",      "domain",
                                   "groups",      new Hashtable(),
                                   "useAsName",   new Hashtable(),
                                   "assign",      new Hashtable(),
                                   "comment",     "",
                                   "domain",      "",
                                   "limited?",    Boolean.TRUE);

                } else { // creating domain from template
                    oldDomain = netdb.getInfo(Constants.DOMAIN, object);
                    if (oldDomain == null || oldDomain.get("id") == null ) {
                        session.setAttribute("Error.errorMsg",
                                             "Error: No info found for domain "+object);
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    handle = null;
                    // we inherit only few parts of original domain
                    oldDomain.put("domain", "");
                    oldDomain.put("comment", "");
                    oldDomain.remove("created.by");
                }

            } else { // domain modify
                oldDomain = netdb.getInfo(Constants.DOMAIN, object);
                if (oldDomain == null || oldDomain.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for domain "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
            }

            if( lockTm != null )
                oldDomain.put("locked", lockTm);

            Hashtable newDomain = Utils.copyHash(oldDomain);
            store.put("oldDomain", oldDomain);
            store.put("newDomain", newDomain);
            store.put("oldNewDomain", Utils.copyHash(newDomain));

            // compress domain name by stripping default domain
            object = compressName(object, default_domain);
            store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            session.setAttribute("Error.errorMsg", errMsg == null ? "???" : errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();

        String sessionKey = decode(request.getParameter("sessionKey"));
        String default_domain = (String)session.getAttribute("default_domain");
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
        String handle = (String)store.get("handle");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newDomain = (Hashtable)store.get("newDomain");
        Hashtable oldDomain = (Hashtable)store.get("oldDomain");
        Hashtable oldNewDomain = (Hashtable)store.get("oldNewDomain");

        String action = Utils.nStr((String)request.getParameter("Action"));

        try {
            if( !action.equals("Reset") && !action.equals("Cancel") ) {
                // pick up domain name
                newDomain.put("domain",
                              (Utils.nStr((String)request.getParameter("domain_name"))).trim());

                // pick up limited?
                newDomain.put("limited?",
                              new Boolean((Utils.nStr(request.getParameter("limited"))).equals("Yes")));

                // pick up comment
                newDomain.put("comment",
                               (Utils.nStr((String)request.getParameter("comment"))).trim());

                // pick up useAsName groups
                Hashtable useAsNameGroups = new Hashtable();
                newDomain.put("useAsName", useAsNameGroups);

                String[] newGroups = request.getParameterValues("use_as_name_groups");
                if( newGroups != null ) {
                    for( int i = 0; i < newGroups.length; i++ )
                        useAsNameGroups.put(decode(newGroups[i]), "");
                }

                // pick up assign groups
                Hashtable assign_groups = new Hashtable();
                newDomain.put("assign", assign_groups);

                newGroups = request.getParameterValues("assign_groups");
                if( newGroups != null ) {
                    for( int i = 0; i < newGroups.length; i++ )
                        assign_groups.put(decode(newGroups[i]), "");
                }

                // pick up groups
                Hashtable groups = new Hashtable();
                newDomain.put("groups", groups);

                newGroups = request.getParameterValues("groups");
                if( newGroups != null ) {
                    for( int i = 0; i < newGroups.length; i++ )
                        groups.put(decode(newGroups[i]), "");
                }

                String newDomainName = (Utils.nStr(newDomain.get("domain"))).trim();
                String oldDomainName = (Utils.nStr(oldNewDomain.get("domain"))).trim();

                if( oldDomainName.length() > 0 && newDomainName.length() > 0 &&
                    !newDomainName.toLowerCase().equals(oldDomainName.toLowerCase()) ) {
                    addSessionMsg(session, "Error.errorMsg", "Error: You can only change case of domain's name");
                    newDomain.put("domain_name_error", "");
                } else
                    newDomain.remove("domain_name_error");
            } // end of 'if( !action.equals("Reset") && !action.equals("Cancel") )'


            if( action.equals("Verify") || action.equals("Save") ) {
                try {
                    Utils.validate(Constants.DOMAIN, newDomain, Constants.NARROW);

                    try {
                        String newDomainName = (Utils.nStr(newDomain.get("domain"))).trim();
                        String oldDomainName = (Utils.nStr(oldNewDomain.get("domain"))).trim();
                        if( oldDomainName.length() == 0 && newDomainName.length() > 0 ) {
                            String r_name = netdb.reserveName(newDomainName, Constants.DOMAIN_TYPE);
                            if( r_name != null && r_name.trim().length() > 0 ) {
                                Hashtable reserved_names = Utils.get_hash_or_create(store, "reserved_names");
                                synchronized(nextKey) {
                                    Hashtable nameStore = new Hashtable();
                                    nameStore.put("fqdn", r_name);
                                    nameStore.put("type", Constants.DOMAIN_TYPE);
                                    reserved_names.put(Constants.DOMAIN_TYPE+" "+r_name, nameStore);
                                }

                                newDomain.put("domain_name_reserved", "");
                                oldNewDomain.put("domain", r_name);
                                newDomain.put("domain", r_name);
                                addSessionMsg(session, "info_msg", "Domain name has been verified.");
                            } else
                                throw new Exception("Error: Failed to reserve name '"+newDomainName+"'");
                        }
                        newDomain.remove("domain_name_error");

                    } catch (NetDBException exx) {
                        Vector messages = exx.getMessages();
                        if( messages != null && messages.size() > 0 ) {
                            for( int i = 0; i < messages.size(); i++ )
                                addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                        }
                        newDomain.put("domain_name_error", "");

                    } catch (Exception exx) {
                        addSessionMsg(session, "Error.errorMsg", "Error: "+exx.getMessage());
                        newDomain.put("domain_name_error", "");
                    }

                } catch (NetDBException ex) {
                    Vector messages = ex.getMessages();
                    if( messages != null && messages.size() > 0 ) {
                        for( int i = 0; i < messages.size(); i++ )
                            addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                }
            } // end of 'if( action.equals("Verify") || action.equals("Save") )'

            if( action.equals("Cancel") ) {
                try {
                    try {
                        if( lockTm != null )
                            netdb.unlock(Constants.DOMAIN, handle, lockTm);

                    } catch (NetDBException ex) {
                        /*
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                        */
                    }

                    for( Hashtable nameStore : (Collection<Hashtable>)nHash(store.get("reserved_names")).values() ) {
                        try {
                            netdb.cancelName((String)nameStore.get("fqdn"), (String)nameStore.get("type"));
                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int i = 0; i < esize; i++ )
                                msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                            throw new Exception(msg);
                        }
                    }

                } catch (Exception ex) {
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                }
                response.sendRedirect((String)store.get("fromPage"));
                return;

            } else if( action.equals("Reset") ) {
                newDomain = Utils.copyHash((Hashtable)store.get("oldDomain"));
                store.put("newDomain", newDomain);
                store.put("oldNewDomain", Utils.copyHash(newDomain));
            }

            if( action.equals("Save") &&
                session.getAttribute("Error.errorMsg") == null &&
                session.getAttribute("Warning") == null ) {
                session.removeAttribute("info_msg");
                session.removeAttribute("Error.errorMsg");
                session.removeAttribute("Warning");
                try {
                    try {
                        handle = (Utils.nStr(newDomain.get("domain"))).trim();

                        if( rootURL.equals("/ModifyDomain") ) {
                            netdb.update(oldDomain, newDomain);
                            session.setAttribute("info_msg", "SUCCESS: Domain '"+handle+"' has been updated.");
                        } else {
                            netdb.insert(newDomain);
                            session.setAttribute("info_msg", "SUCCESS: Domain '"+handle+"' has been created.");
                        }

                        // reload domains
                        setData(session, "domains", netdb.simpleSearch(Constants.DOMAIN, "*"));

                        String fromFromPage = (String)store.get("fromFromPage");
                        response.sendRedirect("/Dispatch?Action=info&object=Domain&fromPage="+
                                              encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                              "&handle="+encode(handle));
                        return;

                    } catch (NetDBException ex) {
                        ex.printStackTrace();
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            } // end of 'if( action.equals("Save") ... )'

            printOutModifyHTML(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            session.setAttribute("Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store, String rootURL)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String sessionKey = (String)store.get("sessionKey");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newDomain = (Hashtable)store.get("newDomain");
        Hashtable oldDomain = (Hashtable)store.get("oldDomain");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");

        modTop(session, out, object, handle, "Domain", "Domain", "hlpdomain.html", sessionKey, rootURL);

        // domain name
        String domain = (Utils.nStr(newDomain.get("domain"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Domain</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='TextField_Domain_Name' "+
                    (newDomain.get("domain_name_error")!=null?"CLASS='error_form_font' ":
                     (newDomain.get("domain_name_reserved")!=null?"CLASS='reserved_form_font' ":""))+
                    "TYPE='TEXT' SIZE='21' NAME='domain_name' value='"+Utils.htmlize(domain)+"'>"+
                    "</TD></TR>\n");

        // limited?
        boolean limited = ((Boolean)newDomain.get("limited?")).booleanValue();
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Limited</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<INPUT ID='Radio_Limited_Yes' TYPE='radio' NAME='limited' VALUE='Yes'"+(limited?" CHECKED":"")+"> Yes"+
                    "<INPUT ID='Radio_Limited_No' TYPE='radio' NAME='limited' VALUE='No'"+(!limited?" CHECKED":"")+"> No"+
                    "</TD></TR><TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");


        // assign groups
        int index = 0;
        Hashtable assign = Utils.get_hash_or_create(newDomain, "assign");
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Create names in this domain<table cellpadding=10><tr><td><em>"+
                    "users in these groups may create names in this domain.</em></td></tr></table></TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        for( String group : (java.util.List<String>)Utils.sort(assign.keySet(), name_cmp) )
            out.println("<INPUT ID='CheckBox_AssignGroup_"+(index++)+"' TYPE='checkbox' CHECKED NAME='assign_groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");

        if( assign.size() > 0 ) out.println("<BR>");

        out.println("<SELECT ID='Selector_AssignGroups' NAME='assign_groups' multiple size='6'>");
        Hashtable myGroups = (Hashtable)getData(session, "myGroups");
        for( String group : (java.util.List<String>)Utils.sort(myGroups.keySet(), name_cmp) )
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // use as name groups
        index = 0;
        Hashtable useAsName = Utils.get_hash_or_create(newDomain, "useAsName");
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Use domain as name<table cellpadding=10><tr><td><em>"+
                    "users in these groups may use this domain as a node or MX name.</em></td></tr></table></TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        for( String group : (java.util.List<String>)Utils.sort(useAsName.keySet(), name_cmp) )
            out.println("<INPUT ID='CheckBox_UseAsNameGroup_"+(index++)+"' TYPE='checkbox' CHECKED NAME='use_as_name_groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");

        if( useAsName.size() > 0 ) out.println("<BR>");

        out.println("<SELECT ID='Selector_UseAsNameGroups' NAME='use_as_name_groups' multiple size='6'>");
        for( String group : (java.util.List<String>)Utils.sort(myGroups.keySet(), name_cmp) )
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // comment
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Comment' NAME='comment' ROWS='3' COLS='60'>"+
                    Utils.htmlize(Utils.nStr((String)newDomain.get("comment")).trim(), false)+
                    "</TEXTAREA>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // NetDB Groups
        index = 0;
        Hashtable groups = Utils.get_hash_or_create(newDomain, "groups");
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font>&nbsp;"+
                    "Groups<br>&nbsp;(1 or more)</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");

        for( String group : (java.util.List<String>)Utils.sort(groups.keySet(), name_cmp) )
            out.println("<INPUT ID='CheckBox_Group_"+(index++)+"' TYPE='checkbox' CHECKED NAME='groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");
        if( groups.size() > 0 ) out.println("<BR>");

        out.println("<SELECT ID='Selector_Groups' NAME='groups' multiple size='6'>");
        for( String group : (java.util.List<String>)Utils.sort(myGroups.keySet(), name_cmp) )
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>");

        Hashtable person = (Hashtable)newDomain.get("created.by");
        if( person != null ) {
            String dirID = (String)person.get("DirID");
            boolean active = activep(person);

            String byStr = (dirID != null?"<A HREF=\""+System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (dirID != null?"</A>":"")+" ("+newDomain.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newDomain.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (dirID != null?"<A HREF=\""+System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (dirID != null?"</A>":"")+" ("+newDomain.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        modBottom(out);
    } // printOutModifyHTML(...)
}
