/************************************************************************
 *
 * ModifyUser.java -
 *
 * $Id: ModifyUser.java,v 1.5 2007-06-26 00:29:29 jlidtke Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2004, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;  // SimpleDateFormat
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class ModifyUser extends ModifyPage {

    public ModifyUser() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "User");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

            // obtain key user attributes from request
            String object = request.getParameter("handle");
            String handle = null;
            Timestamp lockTm = null;
            if( object != null ) {
                handle = new String(object);
                store.put("handle", handle);
                if( rootURL.equals("/ModifyUser") ) {
                    // lock
                    try {
                        lockTm = netdb.lock(Constants.USER, handle);
                        if( lockTm == null )
                            throw new Exception("Failed to lock user "+handle);

                    } catch (Exception ex) {
                        ex.printStackTrace();
                        session.setAttribute("Error.errorMsg", ex.getMessage());
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    store.put("lockTm", lockTm);
                }
            }

            Hashtable oldUser = null;
            if( rootURL.equals("/CreateUser") ) {

                if( !netdb.checkAccess(Constants.USER) )
                    throw new Exception("User '"+session.getAttribute("NetID")+"' can not create users.");

                if( handle == null ) { // creating clean user
                    oldUser = new Hashtable();
                    oldUser.put("id", "");
                    oldUser.put("object", "user");
                    oldUser.put("NetID", "");
                    oldUser.put("RegID", "");
                    oldUser.put("active?", Boolean.TRUE);
                    oldUser.put("all_objects?", Boolean.FALSE);
                    oldUser.put("all_groups?", Boolean.FALSE);
                    oldUser.put("default_domain", (String)Utils.n2s(session.getAttribute("default_domain")));
                    oldUser.put("default_group", "");

                    oldUser.put("groups", new Hashtable());
                    oldUser.put("departments", new Hashtable());
                    oldUser.put("records", new Hashtable());

                    oldUser.put("comment", "");
                } else { // creating user from template
                    Hashtable tmpOldUser = netdb.getInfo(Constants.USER, object);
                    if ( tmpOldUser == null || tmpOldUser.get("id") == null ) {
                        session.setAttribute("Error.errorMsg",
                                             "Error: No info found for user "+object);
                        response.sendRedirect((String)store.get("fromPage"));
                        return;
                    }
                    handle = null;
                    // we inherit only few parts of original user
                    oldUser = new Hashtable();
                    oldUser.put("id", "");
                    oldUser.put("NetID", "");
                    oldUser.put("RegID", "");
                    oldUser.put("object", "user");
                    oldUser.put("active?", tmpOldUser.get("active?"));
                    oldUser.put("default_domain", tmpOldUser.get("default_domain"));
                    oldUser.put("default_addr_space", (String)Utils.n2s(tmpOldUser.get("default_addr_space")));
                    oldUser.put("departments", nHash(tmpOldUser.get("departments")));
                    oldUser.put("default_group", tmpOldUser.get("default_group"));
                    oldUser.put("groups", nHash(tmpOldUser.get("groups")));
                    oldUser.put("records", nHash(tmpOldUser.get("records")));
                    oldUser.put("all_objects?", tmpOldUser.get("all_objects?"));
                    oldUser.put("all_groups?", tmpOldUser.get("all_groups?"));
                    oldUser.put("comment", "");
                }

            } else { // user modify
                oldUser = netdb.getInfo(Constants.USER, object);
                if (oldUser == null || oldUser.get("id") == null ) {
                    session.setAttribute("Error.errorMsg",
                                         "Error: No info found for user "+object);
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
            }

            if( lockTm != null )
                oldUser.put("locked", lockTm);

            Hashtable newUser = Utils.copyHash(oldUser);
            store.put("oldUser", oldUser);
            store.put("newUser", newUser);
            store.put("oldNewUser", Utils.copyHash(newUser));

            if( object != null )
                store.put("object", object);

            printOutModifyHTML(session, out, store, rootURL, "0", "0");

        } catch (Exception ex) {
            // display error page
            ex.printStackTrace();
            String errMsg = ex.getMessage();
            errMsg = (errMsg == null ? "???" : errMsg);
            session.setAttribute("Error.errorMsg", errMsg);
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL = request.getServletPath();

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
        String handle = (String)store.get("handle");
        Timestamp lockTm = (Timestamp)store.get("lockTm");
        Hashtable newUser = (Hashtable)store.get("newUser");
        Hashtable oldUser = (Hashtable)store.get("oldUser");
        Hashtable oldNewUser = (Hashtable)store.get("oldNewUser");

        String action = (String)Utils.n2s((String)request.getParameter("Action"));

        if( request.getParameter("verifyNetID") != null )
            action = "verify_netid";

        try {

            if( !action.equals("Reset") && !action.equals("Cancel") ) {
                // pick up new data
                String netid = request.getParameter("netID");
                if( netid != null )
                    newUser.put("NetID", netid);

                // pick up active?
                newUser.put("active?",
                            new Boolean(((String)Utils.n2s(request.getParameter("active"))).
                                        trim().equals("YES")));

                // pick up def. domain
                newUser.put("default_domain", ((String)Utils.n2s(request.getParameter("defdomain"))).trim());

                // pick up def. addr. space
                String new_addr_space = decode(((String)Utils.n2s(request.getParameter("def_as")))).trim();
                if( new_addr_space.equals("---") )
                    newUser.remove("default_addr_space");
                else
                    newUser.put("default_addr_space", new_addr_space);

                // pick up LNA Departments
                Hashtable departments = new Hashtable();
                newUser.put("departments", departments);

                String[] newDeps = request.getParameterValues("lnadepartments");
                if( newDeps != null ) {
                    for( int i = 0; i < newDeps.length; i++ ) {
                        Hashtable dep = new Hashtable();
                        dep.put("official?", Boolean.TRUE);
                        String key = decode(newDeps[i]);
                        if( !key.equals("--None--") )
                            departments.put(key, dep);
                    }
                }

                // other departments
                newDeps = request.getParameterValues("otherdepartments");
                if( newDeps != null ) {
                    for( int i = 0; i < newDeps.length; i++ ) {
                        Hashtable dep = new Hashtable();
                        dep.put("official?", Boolean.FALSE);
                        String key = decode(newDeps[i]);
                        if( !key.equals("--None--") )
                            departments.put(key, dep);
                    }
                }

                // def. group
                Hashtable groups = new Hashtable();
                newUser.put("groups", groups);
                String def_group = decode(((String)Utils.n2s(request.getParameter("def_group")))).trim();
                if( !def_group.equals("--None--") ) {
                    newUser.put("default_group", def_group);

                    // also add this def. group to the list of member groups
                    groups.put(def_group, "");
                } else
                    newUser.remove("default_group");

                // pick up all_groups?
                newUser.put("all_groups?",
                            new Boolean(((String)Utils.n2s(request.getParameter("allGroups"))).
                                        trim().equals("YES")));

                // pick up member groups
                String[] newGroups = request.getParameterValues("groups");
                if( newGroups != null ) {
                    for( int i = 0; i < newGroups.length; i++ ) {
                        String key = decode(newGroups[i]);
                        if( !key.equals("--None--") )
                            groups.put(key, "");
                    }
                }

                // pick up all_objects?
                newUser.put("all_objects?",
                            new Boolean(((String)Utils.n2s(request.getParameter("allRecords"))).
                                        trim().equals("YES")));

                // pick up records
                Hashtable records = new Hashtable();
                newUser.put("records", records);
                String[] newRecords = request.getParameterValues("records");
                if( newRecords != null ) {
                    for( int i = 0; i < newRecords.length; i++ ) {
                        String rec = decode(newRecords[i]);
                        records.put(rec, rec);
                    }
                }

                // pick up comment
                newUser.put("comment", ((String)Utils.n2s(request.getParameter("comment"))).trim());
            } // end of 'if( !action.equals("Reset") && !action.equals("Cancel") )'

            if( action.equals("verify_netid") || action.equals("Verify") || action.equals("Save") ) {
                try {
                    store.put("oldNewUser", Utils.copyHash(newUser));

                } catch (NetDBException ex) {
                    Vector messages = ex.getMessages();
                    if( messages != null && messages.size() > 0 ) {
                        for( int i = 0; i < messages.size(); i++ )
                            addSessionMsg(session, "Error.errorMsg", "Error: "+messages.elementAt(i));
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", "Error: "+ex.getMessage());
                }
            } // end of 'if( action.equals("Verify") || action.equals("Save") )'

            if( action.equals("Cancel") ) {
                try {
                    try {
                        if( lockTm != null )
                            netdb.unlock(Constants.USER, handle, lockTm);

                    } catch (NetDBException ex) {
                        /*
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int i = 0; i < esize; i++ )
                            msg += "Error: "+(String)errors.elementAt(i)+(i<esize-1?"\n":"");
                        throw new Exception(msg);
                        */
                    }

                } catch (Exception ex) {
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                }
                response.sendRedirect((String)store.get("fromPage"));
                return;

            } else if( action.equals("Reset") ) {
                newUser = Utils.copyHash((Hashtable)store.get("oldUser"));
                store.put("newUser", newUser);
                store.put("oldNewUser", Utils.copyHash(newUser));

            }

            if( action.equals("verify_netid") ||
                ( ( action.equals("Verify") || action.equals("Save") ) && rootURL.equals("/CreateUser") ) ) {
                String netid = ((String)Utils.n2s(request.getParameter("netID"))).trim();
                String oldNetID = (String)oldNewUser.get("NetID");
                if( !oldNetID.trim().equals(netid) || newUser.get("no such person") != null ) {
                    try {
                        try {
                            newUser.remove("no such person");
                            Hashtable foundPeople = netdb.dirSearch(Constants.NETID, netid);

                            if( foundPeople == null || foundPeople.size() != 1 ) {
                                addSessionMsg(session, "Error.errorMsg", "NetID has no match");
                                newUser.put("name",           "");
                                newUser.put("email",          "");
                                newUser.put("no such person", "");
                            } else {
                                Hashtable person = (Hashtable)foundPeople.get(foundPeople.keys().nextElement());

                                if( !person.get("NetID").toString().equalsIgnoreCase(netid) )
                                    addSessionMsg(session, "Warning",
                                                  "WARNING: Using principal NetID \""+
                                                  person.get("NetID")+
                                                  "\" instead of entered NetID alias \""+netid+"\".");
                                newUser.put("NetID", person.get("NetID"));
                                Object obj = person.get("DirID");
                                if( obj != null )
                                    newUser.put("DirID", obj);

                                obj = person.get("RegID");
                                if( obj != null )
                                    newUser.put("RegID", obj);

                                obj = person.get("email");
                                if( obj != null )
                                    newUser.put("email", obj);

                                obj = person.get("name");
                                if( obj != null )
                                    newUser.put("name", obj);

                                obj = person.get("active?");
                                if( obj != null )
                                    newUser.put("dirActive?", obj);

                                obj = person.get("lastname");
                                if( obj != null )
                                    newUser.put("lastname", obj);
                            }

                        } catch (NetDBException ex) {
                            Vector errors = ex.getMessages();
                            String msg = "";
                            int esize = errors.size();
                            for( int j = 0; j < esize; j++ )
                                msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                            throw new Exception(msg);
                        }

                    } catch (Exception ex) {
                        addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                    }
                }
            }

            if( action.equals("verify_netid") || action.equals("Verify") ) {
                oldNewUser = Utils.copyHash(newUser);
                store.put("oldNewUser", oldNewUser);
            }

            if( action.equals("Save") &&
                session.getAttribute("Error.errorMsg") == null ) {
                session.removeAttribute("info_msg");
                session.removeAttribute("Error.errorMsg");

                try {
                    try {
                        String displayName = handle = ((String)Utils.n2s(newUser.get("name"))).trim();
                        handle = ((String)Utils.n2s(newUser.get("NetID"))).trim();
                        if( displayName.length() == 0 )
                            displayName = handle;

                        Utils.validate(Constants.USER, newUser, Constants.NARROW);

                        if( rootURL.equals("/ModifyUser") ) {
                            netdb.update(oldUser, newUser);
                            session.setAttribute("info_msg", "SUCCESS: User '"+displayName+"' has been updated.");
                        } else {

                            Hashtable persons = new Hashtable();
                            Hashtable person = new Hashtable();
                            persons.put(Utils.n2s(newUser.get("RegID")), person);

                            person.put("object", "person");
                            if( newUser.get("RegID") != null )
                                person.put("RegID", newUser.get("RegID"));

                            if( newUser.get("NetID") != null )
                                person.put("NetID", newUser.get("NetID"));

                            if( newUser.get("name") != null )
                                person.put("name", newUser.get("name"));

                            if( newUser.get("email") != null )
                                person.put("email", newUser.get("email"));

                            if( newUser.get("phone") != null )
                                person.put("phone", newUser.get("phone"));

                            if( newUser.get("dirActive?") != null )
                                person.put("active?", newUser.get("dirActive?"));

                            if( newUser.get("DirID") != null )
                                person.put("DirID", newUser.get("DirID"));

                            if( newUser.get("lastname") != null )
                                person.put("lastname", newUser.get("lastname"));

                            updatePersons(netdb, persons);
                            newUser.put("id", person.get("id"));;
                            netdb.insert(newUser);
                            session.setAttribute("info_msg", "SUCCESS: User '"+displayName+"' has been created.");
                        }

                        String fromFromPage = (String)store.get("fromFromPage");
                        response.sendRedirect("/Dispatch?Action=info&object=User&fromPage="+
                                              encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                              "&handle="+encode(handle));
                        return;

                    } catch (NetDBException ex) {
                        ex.printStackTrace();
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            } // end of 'if( action.equals("Save") ... )'

            printOutModifyHTML(session, out, store, rootURL,
                               request.getParameter("PosX"),
                               request.getParameter("PosY"));

        } catch (Exception ex) {
            ex.printStackTrace();
            session.setAttribute("Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost(...)

    public void printOutModifyHTML(HttpSession session, PrintWriter out, Hashtable store,
                                   String rootURL, String pos_x, String pos_y)
        throws Exception
    {
        String sessionKey = (String)store.get("sessionKey");
        Hashtable newUser = (Hashtable)store.get("newUser");
        String handle = (String)store.get("handle");
        String object = (String)store.get("object");

        if( Utils.nStr(session.getAttribute("Error.errorMsg")).trim().length() > 0  ) pos_x = pos_y = "0";

        store.put("PosX", pos_x);
        store.put("PosY", pos_y);

        modTop(session, out, object, handle, "User", "User", "hlpuser.html", sessionKey, rootURL);

        // Name
        String name = ((String)Utils.n2s(newUser.get("name"))).trim();
        if( name.length() > 0 )
            out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Name</TD>\n"+
                        "<TD VALIGN='TOP'>"+Utils.htmlize(name, false)+
                        "</TD></TR>\n");

        // NetID
        String netID = ((String)Utils.n2s(newUser.get("NetID"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> NetID</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    (rootURL.equals("/ModifyUser")?Utils.htmlize(netID, false):"<INPUT ID='TextField_NetID' "+
                     (newUser.get("netid_error")!=null?"CLASS='error_form_font' ":"")+
                     "TYPE='TEXT' SIZE='21' NAME='netID' value='"+Utils.htmlize(netID, false)+"'>"+
                     " <INPUT ID='Button_Verify_NetID' TYPE='SUBMIT' NAME='verifyNetID' VALUE='Verify'>")+
                    "</TD></TR>\n");

        // Name
        String email = ((String)Utils.n2s(newUser.get("email"))).trim();
        if( email.length() > 0 )
            out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Email</TD>\n"+
                        "<TD VALIGN='TOP'>"+Utils.htmlize(email, false)+
                        "</TD></TR>\n");

        // active?
        boolean activeB = ((Boolean)newUser.get("active?")).booleanValue();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Active Account?</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<INPUT ID='Radio_Active' TYPE='RADIO' NAME='active' VALUE='YES'"+(activeB?" checked":"")+">Yes"+
                    "<INPUT ID='Radio_Inactive' TYPE='RADIO' NAME='active' VALUE='NO'"+(activeB?"":" checked")+">No"+
                    "</TD></TR>\n");

        // def. domain
        String def_domain = ((String)Utils.n2s(newUser.get("default_domain"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>&nbsp;<font class='star'>*</font> Default Domain</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<SELECT ID='Selector_Def_Domain' NAME='defdomain' SIZE='1'>");

        Hashtable domains = (Hashtable)getData(session, "domains");
        java.util.List list = Arrays.asList(domains.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();
        while( it.hasNext() ) {
            String domain = (String)it.next();
            out.println("<OPTION VALUE='"+encode(domain)+"'"+
                        (domain.equals(def_domain)?" SELECTED":"")+">"+domain.trim()+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n");

        // Default Address Space
        String def_as = ((String)Utils.n2s(newUser.get("default_addr_space"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%' NOWRAP>&nbsp;Default Address Space</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<SELECT ID='Selector_Def_Address_Space' NAME='def_as' SIZE='1'>");
        out.println("<OPTION VALUE='"+
                    encode("---")+"'"+
                    (def_as.length()==0?" SELECTED":"")+">---");
        Hashtable myAddrSpaces = (Hashtable)getData(session, "myAddrSpaces");
        list = Arrays.asList(myAddrSpaces.keySet().toArray());
        Collections.sort(list, ip_str_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String prefix = (String)((Hashtable)myAddrSpaces.get(it.next())).get("prefix");
            out.println("<OPTION VALUE='"+
                        encode(prefix)+"'"+
                        (prefix.equals(def_as)?" SELECTED":"")+">"+prefix);
        }
        out.println("</SELECT></TD></TR>\n");

        out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");

        // LNA Departments
        Hashtable depts = (Hashtable)newUser.get("departments");
        if( depts == null ) {
            depts = new Hashtable();
            newUser.put("departments", depts);
        }

        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;LNA Departments</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(depts.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        int index = 0;
        while( it.hasNext() ) {
            String dep = (String)it.next();
            Hashtable depH = (Hashtable)depts.get(dep);
            if( ((Boolean)depH.get("official?")).booleanValue() )
                out.println("<INPUT ID='CheckBox_LNA_Department_"+(index++)+
                            "' TYPE='checkbox' CHECKED NAME='lnadepartments' value='"+
                            encode(dep)+"'>"+Utils.htmlize(dep.trim(), false)+"<BR>");
        }
        if( depts.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_LNA_Departments' NAME='lnadepartments' MULTIPLE SIZE='6'>"+
                    "<OPTION VALUE='"+encode("--None--")+"'>--None--</OPTION>");
        Hashtable departments = (Hashtable)getData(session, "departments");
        list = Arrays.asList(departments.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String dep = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(dep)+"'>"+Utils.htmlize(dep, false)+"</OPTION>");
        }
        out.println("</SELECT>\n</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");

        // Other Departments
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Other Departments</TD>\n"+
                    "<TD VALIGN='TOP'>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(depts.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            String dep = (String)it.next();
            Hashtable depH = (Hashtable)depts.get(dep);
            if( !((Boolean)depH.get("official?")).booleanValue() )
                out.println("<INPUT ID='CheckBox_Other_Department_"+(index++)+
                            "' TYPE='checkbox' CHECKED NAME='otherdepartments' value='"+
                            encode(dep)+"'>"+Utils.htmlize(dep.trim(), false)+"<BR>");
        }
        if( depts.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_Other_Departments' NAME='otherdepartments' MULTIPLE SIZE='6'>"+
                    "<OPTION VALUE='"+encode("--None--")+"'>--None--</OPTION>");
        list = Arrays.asList(departments.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String dep = ((String)it.next()).trim();
            out.println("<OPTION VALUE='"+encode(dep)+"'>"+Utils.htmlize(dep, false)+"</OPTION>");
        }
        out.println("</SELECT>\n</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");

        // def. group
        String def_group = ((String)Utils.n2s(newUser.get("default_group"))).trim();
        out.println("<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n"+
                    "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Default Group</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<SELECT ID='Selector_Def_Group' NAME='def_group' SIZE='1'>"+
                    "<OPTION VALUE='"+encode("--None--")+"'"+
                    (def_group.length()==0?" SELECTED":"")+">--None--</OPTION>");

        Hashtable myGroups = (Hashtable)getData(session, "myGroups");

        // If the default groups isn't one the editing user has access to, we need to excplicitly add it.
        // Otherwise it will be lost to the great digital void.
        if (!def_group.equals ("") && ! myGroups.containsKey (def_group)) {
            myGroups.put(def_group, (String)"The current default group."); }

        list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'"+
                        (group.equals(def_group)?" SELECTED":"")+">"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        // member groups
        Hashtable groups = (Hashtable)newUser.get("groups");
        if( groups == null ) {
            groups = new Hashtable();
            newUser.put("groups", groups);
        }
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> "+
                    "Member Groups</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='CheckBox_All_Groups' TYPE='CHECKBOX' NAME='allGroups' VALUE='YES'"+
                    (((Boolean)newUser.get("all_groups?")).booleanValue()?" CHECKED":"")+
                    "> All Groups<BR><BR>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(groups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<INPUT ID='CheckBox_Group_"+(index++)+
                        "' TYPE='checkbox' CHECKED NAME='groups' value='"+
                        encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"<BR>");
        }
        if( groups.size() > 0 )
            out.println("<BR>");
        out.println("<SELECT ID='Selector_Groups' NAME='groups' MULTIPLE SIZE='6'>");
        list = Arrays.asList(myGroups.keySet().toArray());
        Collections.sort(list, name_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String group = (String)it.next();
            out.println("<OPTION VALUE='"+encode(group)+"'>"+Utils.htmlize(group.trim(), false)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");

        // Records
        Hashtable user = (Hashtable)session.getAttribute("user");
        Hashtable allRecords = nHash(user.get("records"));

        if( ((Boolean)user.get("all_objects?")).booleanValue() )
            allRecords = (Hashtable)getData(session, "allRecords");

        Hashtable records = (Hashtable)newUser.get("records");
        if( records == null ) {
            records = new Hashtable();
            newUser.put("records", records);
        }

        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Records</TD>\n"+
                    "<TD VALIGN='TOP'><INPUT ID='CheckBox_All_Records' TYPE='CHECKBOX' "+
                    "NAME='allRecords' VALUE='YES'"+
                    (((Boolean)newUser.get("all_objects?")).booleanValue()?" CHECKED":"")+
                    "> All Records<BR><BR>Selected (<em>uncheck to remove</em>):<BR>");
        list = Arrays.asList(records.keySet().toArray());
        Collections.sort(list, records_cmp);
        it = list.iterator();
        index = 0;
        while( it.hasNext() ) {
            String record = (String)it.next();
            String rec = record.trim();
            out.println("<INPUT ID='CheckBox_Record_"+(index++)+"' TYPE='checkbox' CHECKED NAME='records' value='"+
                        encode(record)+"'>"+(rec.equals("Owner")?"Group":rec)+"<BR>");
        }

        if( groups.size() > 0 ) // XXX what is that? I'll have to think about it.
            out.println("<BR>");

        out.println("<SELECT ID='Selector_Records' NAME='records' MULTIPLE SIZE='6'>");
        list = Arrays.asList(allRecords.keySet().toArray());
        Collections.sort(list, records_cmp);
        it = list.iterator();
        while( it.hasNext() ) {
            String record = (String)it.next();
            String rec = record.trim();
            out.println("<OPTION VALUE='"+encode(record)+"'>"+(rec.equals("Owner")?"Group":rec)+"</OPTION>");
        }
        out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>\n");

        // comment
        String comment = (String)Utils.n2s((String)newUser.get("comment"));
        out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD>\n"+
                    "<TD VALIGN='TOP'>"+
                    "<TEXTAREA ID='TextArea_Comment' NAME='comment' ROWS='3' COLS='60'>"+
                    Utils.htmlize(comment.trim(), false)+"</TEXTAREA>"+
                    "</TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");

        Hashtable person = (Hashtable)newUser.get("created.by");
        if( person != null ) {
            out.println("<TR><TD COLSPAN='2'><HR NOSHADE SIZE=1></TD></TR>");

            // last login
            if( newUser.get("last_login") != null )
                out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Login</TD>\n"+
                            "<TD VALIGN='TOP'>"+newUser.get("last_login").toString().substring(0, 16)+
                            "</TD></TR>");

            String dirID = (String)person.get("DirID");
            boolean active = activep(person);

            String byStr = (dirID != null?"<A HREF=\""+System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (dirID != null?"</A>":"")+" ("+newUser.get("created").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Created By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");

            // modified by
            person = (Hashtable)newUser.get("mod.by");
            dirID = (String)person.get("DirID");
            active = activep(person);

            byStr = (dirID != null?"<A HREF=\""+System.getProperty("stanfordwho.url")+dirID.trim()+"\">":"");
            if( person.get("name") != null )
                byStr += person.get("name")+" ";
            if( person.get("NetID") != null )
                byStr += "("+person.get("NetID")+") ";
            byStr += (dirID != null?"</A>":"")+" ("+newUser.get("modified").toString().substring(0, 16)+")";
            out.println("<TR><TD VALIGN='TOP'>&nbsp;Last Modified By</TD>\n"+
                        "<TD VALIGN='TOP'>"+
                        byStr+"</TD></TR>");
        }

        modBottom(out);
    } // end of printOutModifyHTML(...)
}
