/************************************************************************
 *
 * MoveInterfaces.java -
 *
 * $Id: MoveInterfaces.java,v 1.4 2008-02-04 23:46:22 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2007, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class MoveInterfaces extends ModifyPage {

    public MoveInterfaces() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Node");
        Timestamp lockTm = null;
        String handle    = null;
        NetDBpi netdb    = null;

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            PrintWriter out = response.getWriter();
            String rootURL = request.getServletPath();

            netdb = (NetDBpi)session.getAttribute("netdb");
            String netID = (String)session.getAttribute("NetID");
            String default_domain = (String)session.getAttribute("default_domain");

            // obtain key node attributes from request
            String object = request.getParameter("handle");
            Hashtable source_node = null;

            if( object != null ) {
                try {
                    int oid = Integer.parseInt(object);
                    source_node = netdb.getInfo (Constants.NODE, oid);

                    if ( source_node == null || source_node.get("id") == null )
                        throw new Exception("Error: No info found for node "+object);

                    object = (String)nHash(source_node.get("names")).keys().nextElement();
                } catch(Exception ex) { }
                handle = new String(object);
                store.put("handle", handle);
                // lock
                try {
                    lockTm = netdb.lock(Constants.NODE, handle);
                    if( lockTm == null )
                        throw new Exception("Failed to lock node "+handle);

                } catch (Exception ex) {
                    ex.printStackTrace();
                    session.setAttribute("Error.errorMsg", ex.getMessage());
                    response.sendRedirect((String)store.get("fromPage"));
                    return;
                }
                store.put("lockTm", lockTm);
            }

            if (source_node == null)
                source_node = netdb.getInfo(Constants.NODE, object);

            if ( source_node == null || source_node.get("id") == null )
                throw new Exception("Error: No info found for network "+object);

            if( lockTm != null )
                source_node.put("locked", lockTm);

            // default destination page
            source_node.put("destination_page", "dst");

            for( Hashtable intfc : (Collection<Hashtable>)(nHash(source_node.get("interfaces")).values() ))
                intfc.put("replace hw", "true");

            store.put("source_node", source_node);
            store.put("original_node", Utils.copyHash(source_node));

            // extact name with original case
            object = extractName(source_node, object);

            // compress network name by stripping default domain
            object = compressName(object, default_domain);
            store.put("object", object);

            printOutModifyHTML(session, response, out, store, rootURL);

        } catch (Exception ex) {
            try {
                if( lockTm != null && handle != null && netdb != null )
                    netdb.unlock(Constants.NODE, handle, lockTm);
            } catch (Exception exx) {}
            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }

    } // end of doGet

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();
        String rootURL  = request.getServletPath();

        String sessionKey      = decode(request.getParameter("sessionKey"));
        String default_domain  = (String)session.getAttribute("default_domain");
        Hashtable store        = (Hashtable)session.getAttribute(sessionKey);
        NetDBpi netdb          = (NetDBpi)session.getAttribute("netdb");
        String handle          = (String)store.get("handle");
        Timestamp lockTm       = (Timestamp)store.get("lockTm");
        Hashtable source_node  = (Hashtable)store.get("source_node");
        Hashtable troubled_hws = Utils.get_hash_or_create(source_node, "troubled_hws");
        troubled_hws.clear();

        String action          = Utils.nStr((String)request.getParameter("Action"));

        try {
            if( request.getParameter("ready") != null ) {
                if( action.equals("Cancel") ) {
                    try {
                        if( lockTm != null ) {
                            if( Utils.isNumber(handle) )
                                netdb.unlock(Constants.NODE, Integer.parseInt(handle), lockTm);
                            else
                                netdb.unlock(Constants.NODE, handle, lockTm);
                        }
                    } catch (NetDBException ex) { }

                    response.sendRedirect((String)store.get("fromPage"));
                    return;

                } else if ( action.equals("Reset") ) {
                    source_node = Utils.copyHash((Hashtable)store.get("original_node"));
                    store.put("source_node", source_node);

                } else if ( action.equals("Move Interfaces") ) {

                    Hashtable interfaces = nHash(source_node.get("interfaces"));
                    for( Hashtable intfc : (Collection<Hashtable>)interfaces.values() ) {
                        intfc.remove("to_move");
                        intfc.remove("replace hw");
                        intfc.remove("keep dhcp");
                    }

                    // pick destination page
                    String destination_page = Utils.nStr(request.getParameter("destination_page"));
                    source_node.put("destination_page", destination_page);

                    // pick hw addresses
                    String[] new_hw_addresses = getParameterNamesStaringWith(request, "intfc_hw_");
                    if( new_hw_addresses == null ) new_hw_addresses = new String[0];
                    for( String new_hw_addr : new_hw_addresses ) {
                        nHash(interfaces.get(new_hw_addr.substring(9))).
                            put("hw new", Utils.nStr(request.getParameter(new_hw_addr)).trim());
                    }


                    // mark appropriate interface to move
                    String[] interfaces_to_move = request.getParameterValues("interface.to.move.cb");
                    if( interfaces_to_move == null ) interfaces_to_move = new String[0];
                    for( String intfc_to_move : interfaces_to_move )
                        nHash(interfaces.get(intfc_to_move)).put("to_move", "");

                    // try to lock destination node
                    String destination_node_name = request.getParameter("destination_node").trim();
                    if( destination_node_name.length() > 0 ) {
                        destination_node_name = Utils.name(destination_node_name)+"."+
                            Utils.domain(destination_node_name, default_domain);
                    }
                    source_node.put("destination_node_name", destination_node_name);


                    Timestamp dlockTm = null;
                    Vector errors = new Vector();
                    try {
                        Hashtable destination_node = new Hashtable();

                        if( !destination_node_name.equals("") ) {
                            try {
                                dlockTm = netdb.lock(Constants.NODE, destination_node_name);
                                if( dlockTm == null )
                                    errors.addElement("Failed to lock destination node "+destination_node_name);
                                else {
                                    destination_node = netdb.getInfo (Constants.NODE, destination_node_name);

                                    if ( destination_node == null ||
                                         destination_node.get("id") == null )
                                    errors.addElement("Destination node \""+
                                                      destination_node_name+"\" does not exist.");
                                    else if( dlockTm != null ) {
                                        destination_node.put("locked", dlockTm);
                                        store.put("dlockTm", dlockTm);
                                    }
                                }
                            } catch (NetDBException ex) {
                                for( Object msg : ex.getMessages() )
                                    errors.addElement(msg.toString());
                            }
                        }

                        Hashtable destination_node_new = Utils.copyHash(nHash(destination_node));

                        // remove interface from the source node. we have to work on the copy here if
                        // we have to roll back operation
                        Hashtable source_node_altered = Utils.copyHash(source_node);

                        interfaces = nHash(source_node_altered.get("interfaces"));

                        // copy intrerfaces to a new node
                        Vector intfcs_to_move = new Vector();
                        for( String intfc_id : (Set<String>)interfaces.keySet() ) {
                            Hashtable intfc = (Hashtable)interfaces.get(intfc_id);
                            if( intfc.get("to_move") != null )
                                intfcs_to_move.addElement(intfc_id);

                            String new_hw = Utils.nStr(request.getParameter("intfc_hw_"+intfc_id)).trim();
                            if( new_hw.length() > 0 ) {
                                String parsedHW = null;
                                try {
                                    parsedHW = (String)(new Parser()).parse(new_hw).get("hw");
                                } catch (Exception exx) {
                                    troubled_hws.put(new_hw.trim().toLowerCase(), "");
                                }
                                if( parsedHW == null )
                                    troubled_hws.put(new_hw.trim().toLowerCase(), "");
                                else {
                                    troubled_hws.remove(new_hw.trim().toLowerCase());
                                    new_hw = parsedHW;
                                }

                                if( intfc.get("to_move") != null )
                                    intfc.put("hw", new_hw);

                            } else if( intfc.get("to_move") != null ) {

                                intfc.put("hw", "");
                                intfc.put("roaming?",  Boolean.FALSE);
                                intfc.put("use-dhcp?", Boolean.FALSE);
                            }


                            if( intfc.get("to_move") != null ) {
                                // add interface to the destination node
                                Utils.get_hash_or_create(destination_node_new, "interfaces").
                                    put("moved interface "+intfc_id, intfc);
                            }
                        }

                        // now remove copied interfaces from the old node
                        for( Object key : intfcs_to_move )
                            interfaces.remove(key);

                        // now build hash for modification
                        Hashtable oldNodes =
                            Utils.hash("object", "nodes",
                                       "nodes",  Utils.hash("1", source_node,
                                                            "2", destination_node));

                        Hashtable newNodes =
                            Utils.hash("object", "nodes",
                                       "nodes",  Utils.hash("1", source_node_altered,
                                                            "2", destination_node_new));

                        // see if we got any errors
                        if( destination_node_name.equals("") )
                            errors.addElement("Empty destination node.");

                        if( troubled_hws.size() > 0 ) {
                            for( Object hw : troubled_hws.keySet() )
                                errors.addElement("Incorrect hardware address format \""+hw+"\".");
                        }

                        if( errors.size() > 0 )
                            throw new NetDBException("", errors);

                        Hashtable results = nHash(netdb.update(oldNodes, newNodes));

                        Vector warnings = (Vector)results.get("warnings");
                        if( warnings != null ) {
                            for( int i = 0; i < warnings.size(); i++ )
                                addSessionMsg(session, "Warning", ((String)warnings.elementAt(i)).trim());
                        }

                        String fromFromPage = (String)store.get("fromFromPage");
                        response.sendRedirect("/Dispatch?Action=info&object=Node&fromPage="+
                                              encode(fromFromPage!=null?fromFromPage:(String)store.get("fromPage"))+
                                              "&handle="+(destination_page.equals("src")?handle:destination_node_name));

                    } catch (NetDBException ex) {
                        String emsg = "";
                        for( Object msg : ex.getMessages() )
                            emsg += "Error: "+msg+"\n";
                        session.setAttribute("Error.errorMsg", emsg);

                    } catch (Exception ex) {
                        session.setAttribute("Error.errorMsg", ex.getMessage());

                    } finally {
                        try { // try to unlock destination node
                            if( dlockTm != null )
                                netdb.unlock(Constants.NODE, destination_node_name, dlockTm);
                        } catch (Exception ex) { }
                    }

                } else
                    System.out.println("UKNOWN ACTION ["+action+"]");

            } else
                System.out.println("NOT READY");

            printOutModifyHTML(session, response, out, store, rootURL);

        } catch (Exception ex) {
            // if we are here something unexpected happened
            // we should never end up here
            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
        }
    } // end of doPost

    public void printOutModifyHTML(HttpSession session, HttpServletResponse response,
                                   PrintWriter out,     Hashtable store,
                                   String rootURL)
        throws Exception
    {
        String default_domain = (String)session.getAttribute("default_domain");

        String sessionKey     = (String)store.get("sessionKey");
        Timestamp lockTm      = (Timestamp)store.get("lockTm");
        Hashtable source_node = (Hashtable)store.get("source_node");
        String handle         = (String)store.get("handle");
        String object         = (String)store.get("object");

        out.println(commonHtmlBegin("Move interface(s) in node : "+Utils.htmlize(handle, false))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home&pre=unlock&object=Node&sessionKey="+
                                 encode(sessionKey),
                                 SHOW_HOME_LINK, NO_PROFILE_LINK, NO_LOGOUT_LINK,
                                 "", "Move interface(s)<br>"+Utils.htmlize(handle, false), "hlpnode.html")+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" ACTION='"+rootURL+"'>"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "<INPUT ID='Button_Move' ACCESSKEY=\"M\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Move Interfaces\"> "+
                    "<INPUT ID='Button_Reset' ACCESSKEY=\"M\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Reset\"> "+
                    "<INPUT ID='Button_Cancel' ACCESSKEY=\"C\" TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"fromPage\" VALUE=\""+rootURL+"\">"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"object\" VALUE=\""+object+"\">"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"handle\" VALUE=\""+encode(handle)+"\">"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"lockTm\" VALUE=\""+lockTm.getTime()+"\">"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<BR><BR>After moving interfaces display <INPUT ID='Destination_Src' TYPE='radio' name='destination_page' VALUE='src'"+
                    (source_node.get("destination_page").equals("src")?" CHECKED":"")+
                    "> source or <INPUT ID='Destination_Src' TYPE='radio' name='destination_page' VALUE='dst'"+
                    (source_node.get("destination_page").equals("dst")?" CHECKED":"")+"> destination node info page."+
                    "</P><HR>&nbsp;&nbsp;Destination Node <INPUT ID='TextField_Destination' "+
                    "TYPE='TEXT' SIZE='27' NAME='destination_node' VALUE='"+
                    Utils.htmlize(Utils.nStr(source_node.get("destination_node_name")), false)+"'><HR>");

        Class infoClass = Class.forName("stanford.netdb.html.InfoNode");
        java.lang.reflect.Method printOutInfo =
            infoClass.getMethod("printOutInfo",
                                new Class[] {
                                    Class.forName("javax.servlet.http.HttpSession"),
                                    Class.forName("javax.servlet.http.HttpServletResponse"),
                                    Class.forName("java.io.PrintWriter"),
                                    Class.forName("java.util.Hashtable"),
                                    Class.forName("java.lang.String")
                                });
        source_node.put("no inner form", "");
        source_node.put("move interfaces form", "true");
        printOutInfo.invoke(null, new Object[] {session, response, out, source_node, Utils.nStr(store.get("fromPage"))} );

        out.println("<HR NOSHADE SIZE=2>\n"+
                    "<P CLASS=\"form_buttons\">\n"+
                    "&nbsp;<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Move Interfaces\"> "+
                    "<INPUT TYPE=\"SUBMIT\" NAME=\"Action\" VALUE=\"Cancel\">\n"+
                    "</P>\n"+
                    "</FORM>\n<BR>\n"+
                    getCommonBottomLabel()+
                    "</DIV>\n"+
                    "</BODY>\n"+
                    "</HTML>\n");

    }
}
