/************************************************************************
 *
 * OS.java -
 *
 * $Id: OS.java,v 1.5 2007-09-28 20:04:56 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

import static stanford.netdb.utils.Utils.hash;

public class OS extends ModifyPage {

    public OS() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "Os");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();

            if( rootURL.equals("/OS") ) {
                try {
                    Hashtable osList = netdb.getInfo(Constants.OS, "");
                    store.put("osList", osList);
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of OSes.");
                }

            } else {
                Hashtable oldOS = new Hashtable();
                String osName = request.getParameter("osName");
                if( osName == null )
                    oldOS.put("name", "");
                else
                    oldOS.put("name", decode(osName));

                String osComment = request.getParameter("osComment");
                if( osComment == null )
                    oldOS.put("comment", "");
                else
                    oldOS.put("comment", osComment);
                oldOS.put("object", "os");

                store.put("oldOS", oldOS);
                store.put("newOS", Utils.copyHash(oldOS));
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable osList = (Hashtable)store.get("osList");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddOS") )
                    action = "Create OS";
                else if( rootURL.equals("/ModifyOS") )
                    action = "Modify OS";
            }

            if( rootURL.equals("/OS") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("oslist1");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select an OS you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one OS you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                netdb.delete(hash("object", "os", "name", decode(values[0])));
                                session.setAttribute("info_msg", "SUCCESS: OS '"+decode(values[0])+"' has been deleted");

                                osList = netdb.getInfo(Constants.OS, "");
                                store.put("osList", osList);
                                setData(session, "oses", osList);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select an OS you wish to delete.");

            } else if( rootURL.equals("/OS") && action.equals("Replace") ) {
                String[] values1 = request.getParameterValues("oslist1");
                String[] values2 = request.getParameterValues("oslist2");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select one or more OSes to replace.");
                else if( values1 == null || values2.length != 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select an OS to replace with.");
                else {
                    Hashtable os2 = hash("object", "os",
                                         "replace", "",
                                         "name",    decode(values2[0]),
                                         "comment", Utils.n2s(osList.get(decode(values2[0]))));

                    boolean gotErrors = false;
                    for( int i = 0; i < values1.length; i++ ) {
                        try {
                            try {
                                Hashtable os1 = hash("object", "os",
                                                     "replace", "",
                                                     "name",    decode(values1[i]),
                                                     "comment", Utils.n2s(osList.get(decode(values1[i]))));
                                netdb.update(os1, os2);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                            gotErrors = true;
                        }
                    }

                    if( !gotErrors )
                        session.setAttribute("info_msg", "SUCCESS: OSes have been replaced");
                    try {
                        osList = netdb.getInfo(Constants.OS, "");
                        store.put("osList", osList);
                    } catch(Exception exx) {
                        throw new Exception("Failed to load list of OSes.");
                    }
                }

            } else if( rootURL.equals("/OS") && action.equals("Add") ) {
                response.sendRedirect("/AddOS");
                return;

            } else if( rootURL.equals("/OS") && action.equals("Modify") ) {
                String[] values1 = request.getParameterValues("oslist1");
                if( values1 == null || values1.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select an OS you wish to modify.");
                else if( values1.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one OS to modify.");
                else {
                    response.sendRedirect("/ModifyOS?osName="+encode(values1[0])+
                                          "&osComment="+
                                          encode(Utils.nStr(osList.get(decode(values1[0])))));
                    return;
                }

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/OS");
                return;

            } else if( action.equals("Create OS") || action.equals("Modify OS") ) {
                Hashtable newOS = hash("object", "os",
                                       "name",    (Utils.nStr(request.getParameter("osName"))).trim(),
                                       "comment", (Utils.nStr(request.getParameter("osComment"))).trim());
                store.put("newOS", newOS);
                try {
                    try {
                        Utils.validate(Constants.OS, newOS, Constants.NARROW);

                        if( action.equals("Create OS") ) {
                            netdb.insert(newOS);
                            session.setAttribute("info_msg", "SUCCESS: OS has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldOS"), newOS);
                            session.setAttribute("info_msg", "SUCCESS: OS has been modified.");
                        }
                        setData(session, "oses", netdb.getInfo(Constants.OS, 0));
                        response.sendRedirect("/OS");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/OS")?"Manage OSes":
                                    (rootURL.equals("/ModifyOS")?"Modify OS":"Create OS"))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/OS")?"Manage OSes":
                                 (rootURL.equals("/ModifyOS")?"Modify OS":"Create OS"),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>"+
                    (rootURL.equals("/OS")?"Select OS(s) to repoint from the left. Select the OS to point to, from the right.":
                     (rootURL.equals("/ModifyOS")?"Modify an existing OS.":"Create new OS."))+
                    "</TD></TR></TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        if( rootURL.equals("/OS") ) {
            Hashtable osList = (Hashtable)store.get("osList");
            out.println("<TR><TD WIDTH='1%'>&nbsp;<BR><SELECT ID='os_list' SIZE=30 NAME='oslist1' "+
                        "ondblclick=\"document.Form.ModifyButton.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.ModifyButton.click();\" MULTIPLE>");
            java.util.List list = Arrays.asList(osList.keySet().toArray());
            Collections.sort(list, name_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String os = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(os)+"'>"+Utils.htmlize(os, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='AddButton' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='ModifyButton' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='DeleteButton' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD>");

            out.println("<TD WIDTH='1%'> Becomes </TD><TD>&nbsp;<BR><SELECT ID='replacement_list' SIZE=30 NAME='oslist2'>");
            list = Arrays.asList(osList.keySet().toArray());
            Collections.sort(list, name_cmp);
            it = list.iterator();
            while( it.hasNext() ) {
                String os = ((String)it.next()).trim();
                out.println("<OPTION VALUE='"+encode(os)+"'>"+Utils.htmlize(os, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='ReplaceButton' TYPE='SUBMIT' NAME='action' VALUE='Replace'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying os
            Hashtable newOS = (Hashtable)store.get("newOS");
            out.println("<TR><TD VALIGN='TOP' WIDTH='1%' COLSPAN='2'>"+os_msg+"</TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> OS</TD>"+
                        "<TD><INPUT ID='os_name_field' TYPE='TEXT' SIZE='21' NAME='osName' value=\""+
                        Utils.htmlize(Utils.nStr(newOS.get("name")), false)+
                        "\"></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD><TD>"+
                        "<TEXTAREA ID='comment_text_area' NAME='osComment' ROWS='2' COLS='40'>"+
                        Utils.htmlize(Utils.nStr(newOS.get("comment")), false)+
                        "</TEXTAREA></TD></TR>");

            out.println("<TR COLSPAN='2'><TD><INPUT ID='AddModifyOSButton' TYPE='SUBMIT' NAME='action' VALUE='"+
                        (rootURL.equals("/AddOS")?"Create OS":"Modify OS")+
                        "'> <INPUT TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    }
}
