/************************************************************************
 *
 * RelatedOwnerSearch.java -
 *
 * $Id: RelatedOwnerSearch.java,v 1.2 2008-04-03 18:27:22 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2007, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import java.text.*;
import java.util.regex.*;
import javax.servlet.*;
import javax.servlet.http.*;
import su.netdb.parser.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class RelatedOwnerSearch extends ModifyPage {

    public RelatedOwnerSearch() {}

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null ) return;
        session.setAttribute("Error.errorMsg", "Error: HTTP POST unsupported.");
        response.sendRedirect("/home");
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = Utils.hash("no session store", "");
        HttpSession session = begin(request, response, store);
        if( session == null ) return;

        PrintWriter out = response.getWriter();

        Hashtable sr    = null;
        String searchID = null;
        try {
            String search_string = Utils.nStr(session.getAttribute("search_string"));
            out.println(getCommonHtmlBegin("Quick Search Results")+
                        commonHeader(session, "/home", SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                     "", "Search Result", "hlpqsearch.html",
                                     "/owned")+
                        "&nbsp;Search results for records related to group \"<B>"+Utils.htmlize(search_string)+"</B>\"<BR><BR>\n");

            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            searchID = netdb.createSearchID();
            sr = netdb.fullSearch(Constants.ALL,
                                  Utils.hash("txt.lower(owner)", search_string),
                                  searchID);

            if( sr == null ) { // search was canceled
                session.setAttribute("Warning",
                                     "Warning: Your search was canceled. Perhaps by another search.");
                response.sendRedirect("/home");
                return;
            }

            out.println(msgPanels(session)+"<DIV id=\"qs_results\">\n"+
                    "<TABLE WIDTH='100%' CELLSPACING='1' CELLPADDING='0' BORDER='0'>\n");
            String[] types = {"Nodes", "Users", "Networks", "Admin Teams", "Domains", "ResNames"};
            int global_index = 1;
            for( String type : types ) {
                Hashtable result = nHash(sr.get(type));
                if( type.equals("ResNames") && result.size() == 0 ) continue;
                out.println("<TR CLASS=\"qs_table_header\">\n"+
                            "  <TD NOWRAP COLSPAN='7'>"+type+" ( "+result.size()+" )</TD>\n"+
                            "</TR>\n"+
                            "<TR CLASS=\"qs_table_sub_header\">\n"+
                            "  <TD WIDTH='1%' ALIGN='RIGHT' NOWRAP>#&nbsp;&nbsp;</TD>"+
                            "  <TD NOWRAP>Name</TD>"+
                            "  <TD NOWRAP COLSPAN=\"5\">Actions</TD>"+
                            "</TR>\n"+
                            "<TR>\n");

                String object     = "NoSuchObject";
                boolean hasAccess = false;
                if(      type.equals("Nodes") ) {
                    hasAccess = ((Boolean)session.getAttribute("hasNodeObjectAccess?")).booleanValue();
                    object = "Node";
                } else if( type.equals("Users") ) {
                    hasAccess = ((Boolean)session.getAttribute("hasUserObjectAccess?")).booleanValue();
                    object = "User";
                } else if( type.equals("Networks") ) {
                    hasAccess = ((Boolean)session.getAttribute("hasNetworkObjectAccess?")).booleanValue();
                    object = "Network";
                } else if( type.equals("Admin Teams") ) {
                    hasAccess = ((Boolean)session.getAttribute("hasATeamObjectAccess?")).booleanValue();
                    object = "AdminTeam";
                } else if( type.equals("Domains") ) {
                    hasAccess = ((Boolean)session.getAttribute("hasDomainObjectAccess?")).booleanValue();
                    object = "Domain";
                }

                int index = 1;
                for( Object name : sorted_keys(result, name_cmp) ) {
                    out.println("<TR>\n"+
                                "<TD WIDTH='1%' ALIGN='right' NOWRAP>"+(index++)+"&nbsp;&nbsp;</TD>\n"+
                                "<TD WIDTH='1%' NOWRAP><A ID='Link_Info_"+global_index+"' HREF='/Dispatch?Action=info&object="+object+"&fromPage=/owned&handle="+encode((String)name)+"'>"+
                                Utils.htmlize((String)name)+"</A>&nbsp;</TD>\n"+
                                "<TD WIDTH='98%' NOWRAP>"+
                                (hasAccess?"  <A ID='Link_Modify_"+global_index+"' HREF='/Dispatch?Action=modify&object="+object+"&fromPage=/owned&handle="+encode((String)name)+"'>Mod</A>":"")+
                                "&nbsp;&nbsp;\n"+
                                (hasAccess?"  <A ID='Link_Template_"+global_index+"' HREF='/Dispatch?Action=use&object="+object+"&fromPage=/owned&handle="+encode((String)name)+"'>Tmpl</A>":"")+
                                "&nbsp;&nbsp;\n"+
                                 (hasAccess?"  <A ID='Link_Delete_"+global_index+"' HREF='/Dispatch?Action=delete&object="+object+"&fromPage=/owned&handle="+encode((String)name)+"'>Del</A>":"")+
                                "&nbsp;&nbsp;&nbsp;&nbsp;</TD>\n"+
                                "</TR>\n");
                    global_index++;

                }

                out.println("  <TD COLSPAN=\"7\"><HR></TD>\n"+
                            "</TR>\n");
            }


        } catch (Exception ex) {
            session.setAttribute("Error.errorMsg", "Error: "+ex.getMessage());
            response.sendRedirect("/home");
        }

        // display quicksearch panel and footer
        out.println("</TABLE>\n</DIV>\n"+
                    "<BR>\n"+
                    getCommonBottomLabel("hlpqsearch.html")+
                    "</BODY>\n"+
                    "</HTML>\n");
    }
}