/************************************************************************
 *
 * Stat.java -
 *
 * $Id: Stat.java,v 1.2 2007-11-13 00:30:30 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2005, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.sql.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

public class Stat extends InfoPage {

    public Stat() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("no session store", "");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        PrintWriter out = response.getWriter();

        try {
            String sort = ((String)Utils.n2s(request.getParameter("sort"))).trim();
            if( sort.length() == 0 )
                sort = "sid";

            String dir = ((String)Utils.n2s(request.getParameter("dir"))).trim();
            if( dir.length() == 0 )
                dir = "down";

            String to_kill = ((String)Utils.n2s(request.getParameter("to_kill"))).trim();

            String dt = ((String)Utils.n2s(request.getParameter("dt"))).trim();
            if( dt.length() == 0 )
                dt = "0";
            try {
                if( Integer.parseInt(dt) < 1 )
                    dt = null;
            } catch (Exception ex) {
                dt = null;
            }

            String netID = (String)session.getAttribute("NetID");
            Hashtable acl = Utils.readACL();
            Hashtable entry = (Hashtable)acl.get(netID);
            if( entry == null || entry.get("god") == null )
                throw new Exception("Access denied.");

            out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"+
                        "<html>\n"+
                        " <head>\n"+
                        "  <meta HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=ISO-8859-1\">\n"+
                        (dt!=null?("  <meta http-equiv=\"refresh\" content=\""+dt+"\">\n"):"")+
                        "<link REL=\"stylesheet\" TYPE=\"text/css\" HREF=\"/netdb.css\">\n"+
                        "<title>System Stats.</title>\n"+
                        "</head>\n"+
                        "<TABLE WIDTH='100%' CELLSPACING='1' CELLPADDING='0' BORDER='0'>"+
                        "<THEAD>");

            synchronized ( NetDBMaster.statStore ) {
                java.util.List alist = Arrays.asList(NetDBMaster.statStore.entrySet().toArray());
                Object[] oa = alist.toArray();
                out.println("<tr CLASS='qs_table_header'><td NOWRAP COLSPAN=5>Users ("+oa.length+
                            ")</td><td align='right'>"+Calendar.getInstance().getTime()+
                            "&nbsp;</td></tr>"+
                            "<TR CLASS='qs_table_sub_header'><TD><A HREF=\"/stat?sort=sid&dir="+
                            (dir.equals("down")?"up":"down")+"&dt="+Utils.n2s(dt)+"\">SID</A></TD>"+
                            "<TD NOWRAP>REMOTE HOST</TD>"+
                            "<TD NOWRAP>STARTED AT</TD><TD><A HREF=\"/stat?sort=uid&dir="+
                            (dir.equals("down")?"up":"down")+"&dt="+Utils.n2s(dt)+
                            "\">UID</A></TD><TD NOWRAP>LAST CALL</TD>"+
                            "<TD NOWRAP><A HREF=\"/stat?sort=lca&dir="+
                            (dir.equals("down")?"up":"down")+"&dt="+Utils.n2s(dt)+"\">LAST CALL AT</A></TD>"+
                            "</TR></THEAD></TBODY>");


                String do_kill = ((String)Utils.n2s(request.getParameter("do_kill"))).trim();
                String kill_sid = null;
                try {
                    for (int i = 0; i < oa.length; ++i) {
                        HttpSession sess = (HttpSession)((Map.Entry)oa[i]).getValue();
                        NetDBpi netdb = (NetDBpi)sess.getAttribute("netdb");
                        Hashtable state = nHash(netdb.getState());
                        String sid = (String)state.get("timestamp");
                        if( do_kill.equals(sid) ) {
                            kill_sid = (String)((Map.Entry)oa[i]).getKey();
                            break;
                        }
                    }

                    if( kill_sid != null ) {
                        HttpSession sess = (HttpSession)NetDBMaster.statStore.remove(kill_sid);
                        if( sess != null ) {
                            NetDBpi netdb = (NetDBpi)sess.getAttribute("netdb");
                            ((stanford.netdb.utils.Logger)netdb).info("session killed ["+kill_sid+"]");
                            sess.invalidate();
                            System.gc();
                            System.runFinalization();
                        }
                    }
                } catch (Exception ex) { }

                alist = Arrays.asList(NetDBMaster.statStore.entrySet().toArray());
                Collections.sort(alist, new StatCmp(sort, dir));
                oa = alist.toArray();

                for (int i = 0; i < oa.length; ++i) {

                    HttpSession sess = (HttpSession)((Map.Entry)oa[i]).getValue();
                    NetDBpi netdb = (NetDBpi)sess.getAttribute("netdb");
                    try {
                        Hashtable state = nHash(netdb.getState());
                        String sid = (String)state.get("timestamp");
                        Calendar cal = Calendar.getInstance();
                        cal.setTimeInMillis(Long.parseLong(sid));

                        String last_call_at = Utils.nStr(state.get("last_call_at"));
                        Calendar lcal = Calendar.getInstance();
                        lcal.setTimeInMillis(Long.parseLong(last_call_at));

                        out.println("<TR"+(to_kill.equals(sid)?" CLASS='attn_row'":"")+"><TD>"+sid+
                                    " [<a href=\"/stat?sort="+sort+"&dir="+dir+
                                    "&dt="+Utils.n2s(dt)+"&"+(to_kill.equals(sid)?"do":"to")+"_kill="+
                                    sid+"\">kill</a>]</TD>"+
                                    "<TD>"+Utils.n2s(sess.getAttribute("X-Forwarded-For"))+"</TD><TD>"+
                                    cal.getTime()+"</TD>"+
                                    "<TD>"+netdb.getUser().get("NetID")+"</TD><TD>"+
                                    state.get("last_method_call")+"</TD><TD>"+
                                    lcal.getTime()+
                                    "</TD></TR>\n");
                    } catch (Exception ex) {
                        // netdb may get destroyed right after we get it from session but
                        // before we call getState(...)
                    }
                }
            }

            out.println("</TBODY></TABLE></DIV></BODY>\n</HTML>\n");

        } catch (Exception ex) {
            ex.printStackTrace();
            session.setAttribute("Error.errorMsg", ex.getMessage());
            response.sendRedirect((String)store.get("fromPage"));
            return;
        }
    }

    private class StatCmp implements Comparator {

        String sort = null;
        int dir = 1;

        public StatCmp(String sort, String dir) {
            this.sort = sort;
            this.dir = dir.equals("down")?1:-1;
        }

        public int compare(Object o1, Object o2) {
            HttpSession sess1 = (HttpSession)((Map.Entry)o1).getValue();
            HttpSession sess2 = (HttpSession)((Map.Entry)o2).getValue();

            if( sort.equals("sid") || sort.equals("lca") ) {
                try {

                    String key = sort.equals("sid")?"timestamp":"last_call_at";
                    long diff = Long.parseLong((String)((NetDBpi)sess1.getAttribute("netdb")).getState().get(key)) -
                        Long.parseLong((String)((NetDBpi)sess2.getAttribute("netdb")).getState().get(key));

                    return dir*(diff>0?1:(diff<0?-1:0));

                } catch (Exception ex) {
                    return 0;
                }
            }

            if( sort.equals("uid") ) {
                try {

                    String netid1 = (String)((NetDBpi)sess1.getAttribute("netdb")).getUser().get("NetID");
                    String netid2 = (String)((NetDBpi)sess2.getAttribute("netdb")).getUser().get("NetID");

                    return dir*netid1.compareToIgnoreCase(netid2);

                } catch (Exception ex) {
                    return 0;
                }
            }

            return 0;
        }

        public boolean equals(Object obj) {
            return true;
        }
    }
}
