/************************************************************************
 *
 * State.java -
 *
 *
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2003, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.html;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;

public class State extends ModifyPage {

    public State() {}

    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        Hashtable store = new Hashtable();
        store.put("type", "state");

        HttpSession session = begin(request, response, store);
        if( session == null )
            return;

        try {
            String netID = (String)session.getAttribute("NetID");
            NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");
            String rootURL = request.getServletPath();

            if( rootURL.equals("/State") ) {
                try {
                    store.put("stateList", netdb.listMyStates(null));
                } catch(Exception exx) {
                    throw new Exception("Failed to load list of States.");
                }

            } else {
                Hashtable oldState = new Hashtable();
                String stateName = request.getParameter("stateName");
                if( stateName == null )
                    oldState.put("name", "");
                else
                    oldState.put("name", stateName);

                String rank = Utils.nStr(request.getParameter("stateRank"));
                oldState.put("stateRank", rank);
                try { oldState.put("rank", new Integer(rank)); } catch (Exception ex) { }

                String stateComment = request.getParameter("stateComment");
                if( stateComment == null )
                    oldState.put("description", "");
                else
                    oldState.put("description", stateComment);

                String denyDHCP = request.getParameter("denyDHCP");
                if( denyDHCP == null )
                    oldState.put("denyDHCP", Boolean.FALSE);
                else
                    oldState.put("denyDHCP", new Boolean(denyDHCP));

                String denyDNS = request.getParameter("denyDNS");
                if( denyDNS == null )
                    oldState.put("denyDNS", Boolean.FALSE);
                else
                    oldState.put("denyDNS", new Boolean(denyDNS));

                String stateAccess = request.getParameter("stateAccess");
                if( stateAccess != null )
                    oldState.put("access", stateAccess);

                oldState.put("object", "state");

                store.put("oldState", oldState);
                store.put("newState", Utils.copyHash(oldState));
            }

            PrintWriter out = response.getWriter();
            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doGet(...)

    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
        HttpSession session = begin(request, response, null);
        if( session == null )
            return;

        String rootURL = request.getServletPath();

        NetDBpi netdb = (NetDBpi)session.getAttribute("netdb");

        String sessionKey = decode(request.getParameter("sessionKey"));
        Hashtable store = (Hashtable)session.getAttribute(sessionKey);
        Hashtable stateList = (Hashtable)store.get("stateList");

        try {
            PrintWriter out = response.getWriter();
            String action = request.getParameter("action");
            if( action == null || action.trim().length() == 0 ) {
                if( rootURL.equals("/AddState") )
                    action = "Create State";
                else if( rootURL.equals("/ModifyState") )
                    action = "Modify State";
            }

            if( rootURL.equals("/State") && action.equals("Delete") ) {
                String[] values = request.getParameterValues("statelist");
                if( values != null ) {
                    if( values.length == 0 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select a State you wish to delete.");
                    else if( values.length > 1 )
                        addSessionMsg(session, "Error.errorMsg", "Error: Please select only one State you wish to delete.");
                    else {
                        // delete
                        try {
                            try {
                                Hashtable state = new Hashtable();
                                state.put("object", "state");
                                state.put("name", decode(values[0]));
                                state.put("rank", new Integer(0));
                                netdb.delete(state);
                                session.setAttribute("info_msg", "SUCCESS: State '"+decode(values[0])+"' has been deleted");

                                stateList = netdb.listMyStates(null);
                                store.put("stateList", stateList);
                                setData(session, "states", stateList);
                            } catch (NetDBException ex) {
                                Vector errors = ex.getMessages();
                                String msg = "";
                                int esize = errors.size();
                                for( int j = 0; j < esize; j++ )
                                    msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                                throw new Exception(msg);
                            }
                        } catch (Exception ex) {
                            addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                        }
                    }
                } else
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select a State you wish to delete.");

            } else if( rootURL.equals("/State") && action.equals("Add") ) {
                response.sendRedirect("/AddState");
                return;

            } else if( rootURL.equals("/State") && action.equals("Modify") ) {
                String[] values = request.getParameterValues("statelist");
                if( values == null || values.length == 0 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select a State you wish to modify.");
                else if( values.length > 1 )
                    addSessionMsg(session, "Error.errorMsg", "Error: Please select only one State to modify.");
                else {
                    Hashtable state = (Hashtable) stateList.get(decode(values[0]));
                    Object rank = state.get("rank");
                    if( rank == null ) rank = "";
                    response.sendRedirect("/ModifyState?stateName="+encode((String)state.get("name"))+
                                          "&denyDNS="+state.get("denyDNS")+
                                          "&denyDHCP="+state.get("denyDHCP")+
                                          "&stateComment="+
                                          encode((String)Utils.n2s(state.get("description")))+
                                          "&stateRank="+encode(rank.toString())+
                                          "&stateAccess="+
                                          encode((String)Utils.n2s(state.get("access"))));
                    return;
                }

            } else if( action.equals("Cancel") ) {
                response.sendRedirect("/State");
                return;

            } else if( action.equals("Create State") || action.equals("Modify State") ) {
                Hashtable newState = new Hashtable();
                newState.put("object", "state");
                newState.put("name", ((String)Utils.n2s(request.getParameter("stateName"))).trim());
                newState.put("denyDNS",  new Boolean((String)Utils.n2s(request.getParameter("denyDNS"))));
                newState.put("denyDHCP", new Boolean((String)Utils.n2s(request.getParameter("denyDHCP"))));
                String rank_str = ((String)Utils.n2s(request.getParameter("stateRank"))).trim();
                newState.put("stateRank", rank_str);
                try {
                    newState.put("rank", new Integer(rank_str));
                } catch (Exception ex) { }
                newState.put("description", ((String)Utils.n2s(request.getParameter("stateComment"))).trim());
                String tmpAccess = ((String)Utils.n2s(request.getParameter("stateAccess"))).trim();
                if(!tmpAccess.equals(""))
                    newState.put("access", tmpAccess.equals("Group")?"Owner":tmpAccess);
                store.put("newState", newState);
                try {
                    try {
                        Utils.validate(Constants.STATE, newState, Constants.NARROW);

                        if( action.equals("Create State") ) {
                            netdb.insert(newState);
                            session.setAttribute("info_msg", "SUCCESS: State has been created");
                        } else {
                            netdb.update((Hashtable)store.get("oldState"), newState);
                            session.setAttribute("info_msg", "SUCCESS: State has been modified.");
                        }
                        setData(session, "states", netdb.listMyStates(null));
                        response.sendRedirect("/State");
                        return;

                    } catch (NetDBException ex) {
                        Vector errors = ex.getMessages();
                        String msg = "";
                        int esize = errors.size();
                        for( int j = 0; j < esize; j++ )
                            msg += "Error: "+(String)errors.elementAt(j)+(j<esize-1?"\n":"");
                        throw new Exception(msg);
                    }
                } catch (Exception ex) {
                    addSessionMsg(session, "Error.errorMsg", ex.getMessage());
                }
            }

            printOutHtml(session, out, store, rootURL);

        } catch (Exception ex) {
            ex.printStackTrace();
            addSessionMsg(session, "Error.errorMsg", "Error: "+ex);
            response.sendRedirect(store!=null?(String)store.get("fromPage"):"/home");
        }
    } // end of doPost(...)

    public void printOutHtml(HttpSession session, PrintWriter out, Hashtable store, String rootURL) throws Exception {
        String sessionKey = (String)store.get("sessionKey");
        out.println(commonHtmlBegin(rootURL.equals("/State")?"Manage Node States":
                                    (rootURL.equals("/ModifyState")?"Modify State":"Create State"))+
                    commonHeader(session, "/Dispatch?Action=redirect&to=/home",
                                 SHOW_HOME_LINK, SHOW_PROFILE_LINK, SHOW_LOGOUT_LINK,
                                 "", rootURL.equals("/State")?"Manage States":
                                 (rootURL.equals("/ModifyState")?"Modify State":"Create State"),
                                 "hlpnetdb.html", decode((String)store.get("newFromPage")))+
                    msgPanels(session)+
                    "<DIV id=\"infoPage\">\n"+
                    "<FORM METHOD=\"POST\" NAME=\"Form\" ACTION='"+rootURL+"'>"+
                    "<INPUT TYPE=\"Hidden\" NAME=\"sessionKey\" VALUE=\""+encode(sessionKey)+"\">"+
                    "<DIV class='modHeader'>\n"+
                    "<TABLE WIDTH='100%'>\n"+
                    "<TR><TD NOWRAP>"+
                    (rootURL.equals("/State")?"Select State to modify.":
                     (rootURL.equals("/ModifyState")?"Modify an existing State.":"Create new State."))+
                    "</TD></TR></TABLE></DIV>"+
                    "<TABLE WIDTH='100%'>");

        if( rootURL.equals("/State") ) {
            Hashtable stateList = (Hashtable)store.get("stateList");
            out.println("<TR><TD WIDTH='100%'>&nbsp;<BR><SELECT ID='Selector_States' SIZE=20 NAME='statelist' "+
                        "ondblclick=\"document.Form.Button_Modify.click()\" "+
                        "onKeyDown=\"if( event.keyCode == 13 ) document.Form.Button_Modify.click();\" MULTIPLE>");

            java.util.List list = Arrays.asList(stateList.entrySet().toArray());
            Collections.sort(list, state_cmp);
            Iterator it = list.iterator();
            while( it.hasNext() ) {
                String state = Utils.nStr(((Hashtable)((Map.Entry)it.next()).getValue()).get("name"));
                out.println("<OPTION VALUE='"+encode(state)+"'>"+Utils.htmlize(state, false));
            }

            out.println("</SELECT><BR><BR><INPUT ID='Button_Add' TYPE='SUBMIT' NAME='action' VALUE='Add'> "+
                        "<INPUT ID='Button_Modify' TYPE='SUBMIT' NAME='action' VALUE='Modify'> "+
                        "<INPUT ID='Button_Delete' TYPE='SUBMIT' NAME='action' VALUE='Delete'></TD></TR>");

            out.println( "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");

        } else { // adding or modifying state
            Hashtable newState = (Hashtable)store.get("newState");

            // Records
            Hashtable user = (Hashtable)session.getAttribute("user");
            Hashtable allRecords = nHash(user.get("records"));

            if( ((Boolean)user.get("all_objects?")).booleanValue() )
                allRecords = (Hashtable)getData(session, "allRecords");

            // Adding a null option (meaning 'associated with no access group')
            allRecords.put("-None-","No Records");

            Object rank      = newState.get("rank");
            boolean denyDNS  = ((Boolean)newState.get("denyDNS")).booleanValue();
            boolean denyDHCP = ((Boolean)newState.get("denyDHCP")).booleanValue();
            String rank_str  = Utils.nStr(newState.get("stateRank"));
            if( rank != null ) rank_str = rank.toString();

            out.println("<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> State</TD>"+
                        "<TD><INPUT ID='TextField_Name' TYPE='TEXT' SIZE='21' NAME='stateName' value='"+
                        Utils.htmlize((String)Utils.n2s(newState.get("name")), false)+
                        "'></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;<font class='star'>*</font> Rank</TD>"+
                        "<TD><INPUT ID='TextField_Rank' TYPE='TEXT' SIZE='4' NAME='stateRank' value='"+
                        Utils.htmlize(rank_str, false)+
                        "'></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Deny DNS</TD>"+
                        "<TD><SELECT ID='Slector_denyDNS' NAME='denyDNS'><OPTION VALUE='true'"+(denyDNS?" SELECTED":"")+">Yes"+
                        "<OPTION VALUE='false'"+(!denyDNS?" SELECTED":"")+">No</SELECT></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Deny DHCP</TD>"+
                        "<TD><SELECT ID='Slector_denyDHCP' NAME='denyDHCP'><OPTION VALUE='true'"+(denyDHCP?" SELECTED":"")+">Yes"+
                        "<OPTION VALUE='false'"+(!denyDHCP?" SELECTED":"")+">No</SELECT></TD></TR>\n"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Comment</TD><TD>"+
                        "<TEXTAREA ID='TextArea_Comment' NAME='stateComment' ROWS='2' COLS='40'>"+
                        Utils.htmlize((String)Utils.n2s(newState.get("description")), false)+
                        "</TEXTAREA></TD></TR>"+
                        "<TR><TD VALIGN='TOP' WIDTH='20%'>&nbsp;Access Group</TD>");

            out.println("<TD><SELECT ID='Selector_Access' NAME='stateAccess'>");
            java.util.List list = Arrays.asList(allRecords.keySet().toArray());
            Collections.sort(list, records_cmp);
            Iterator it = list.iterator();
            String strAccess=(String) newState.get("access");
            if(strAccess == null) strAccess="";
            strAccess=(strAccess.equals("")?"-None-":strAccess);
            while( it.hasNext() ) {
                String record = (String)it.next();
                String rec = record.trim();
                out.println("<OPTION VALUE='"+encode(record.equals("-None-")?"":record)+"' "+
                           (rec.equals(strAccess)?"SELECTED":"")+">"+(rec.equals("Owner")?"Group":rec)+"</OPTION>");
            }
            out.println("</SELECT></TD></TR>\n<TR><TD COLSPAN='2'>&nbsp;</TD></TR>\n");


            out.println("<TR COLSPAN='2'><TD><INPUT ID='Button_Create_Or_Modify' TYPE='SUBMIT' NAME='action' VALUE='"+
                        (rootURL.equals("/AddState")?"Create State":"Modify State")+
                        "'> <INPUT ID='Button_Cancel' TYPE='SUBMIT' NAME='action' VALUE='Cancel'></TD></TR>"+
                        "</TABLE>\n"+
                        "<HR NOSHADE SIZE=2>\n"+
                        "</FORM>\n<BR>\n"+
                        getCommonBottomLabel()+
                        "</DIV>\n"+
                        "</BODY>\n"+
                        "</HTML>\n");
        }
    }
}
