/************************************************************************
 *
 * Formatter.java -
 *
 * $Id: Formatter.java,v 1.2 2006-09-27 19:26:25 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.whois;

import java.util.*;
import java.lang.*;
import java.text.*;
import stanford.netdb.utils.*;

public class Formatter {

    // alignment properties
    public static int LEFT           = 0;
    public static int RIGHT          = 1;

    // color of label in html output
    public static String ncolor      = "#990000";

    // stanford(who|what) url
    public static String st_what_url = "";
    public static String st_who_url  = "";

    protected Hashtable search_result;
    protected int search_for;

    protected Comparator ip_cmp      = null;
    protected Comparator name_cmp    = null;
    protected Comparator in_name_cmp = null;
    protected Comparator intfc_cmp   = null;
    protected SimpleDateFormat sdf   = null;

    public Formatter(Hashtable search_result, int search_for) {

        this.search_result = search_result;
        this.search_for    = search_for;

        sdf = new SimpleDateFormat("MMM dd HH:mm z yyyy");

        in_name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable record1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable record2 = (Hashtable)((Map.Entry)o2).getValue();

                    String name1 = (String)record1.get("name");
                    name1 = name1==null?(String)record1.get("net_name"):name1;
                    name1 = name1==null?"":name1;

                    String name2 = (String)record2.get("name");
                    name2 = name2==null?(String)record2.get("net_name"):name2;
                    name2 = name2==null?"":name2;

                    if( name1.toLowerCase().equals(name2.toLowerCase()) ) {
                        if(  n2s(record1.get("name_type")).equals("MX") && !n2s(record2.get("name_type")).equals("MX") )
                            return 1;
                        if( !n2s(record1.get("name_type")).equals("MX") &&  n2s(record2.get("name_type")).equals("MX") )
                            return -1;
                    }

                    return name1.compareToIgnoreCase(name2);
                }
            };

        name_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    return ((String)o1).compareToIgnoreCase((String)o2);
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        ip_cmp = new Comparator() {
                public int compare(Object ip1, Object ip2) {
                    return Utils.ip2norm((String)ip1).compareToIgnoreCase(Utils.ip2norm((String)ip2));
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };

        intfc_cmp = new Comparator() {
                public int compare(Object o1, Object o2) {
                    Hashtable intfc1 = (Hashtable)((Map.Entry)o1).getValue();
                    Hashtable intfc2 = (Hashtable)((Map.Entry)o2).getValue();

                    boolean hasIPs1 = false;
                    boolean hasIPs2 = false;
                    long ip1 = 0;
                    long ip2 = 0;

                    Hashtable ips = Utils.nHash(intfc1.get("ips"));
                    for( String IP : (Set<String>)ips.keySet() ) {
                        try {
                            long ip = Utils.ipstr2long(IP);
                            if( hasIPs1 )
                                ip1 = ip<ip1?ip:ip1;
                            else {
                                hasIPs1 = true;
                                ip1 = ip;
                            }
                        } catch (Exception ex) { }
                    }

                    ips = Utils.nHash(intfc2.get("ips"));
                    for( String IP : (Set<String>)ips.keySet() ) {
                        try {
                            long ip = Utils.ipstr2long(IP);
                            if( hasIPs2 )
                                ip2 = ip<ip2?ip:ip2;
                            else {
                                hasIPs2 = true;
                                ip2 = ip;
                            }
                        } catch (Exception ex) { }
                    }

                    if( !hasIPs1 && hasIPs2 )
                        return -1;

                    if( hasIPs1 && !hasIPs2 )
                        return 1;

                    if( !hasIPs1 && !hasIPs2 ) {
                        String i1 = (String)((Map.Entry)o1).getKey();
                        String i2 = (String)((Map.Entry)o2).getKey();

                        long v1 = 0;
                        boolean number1 = true;
                        try {
                            v1 = (new Long((String)i1)).longValue();
                        } catch (Exception ex) {
                            number1 = false;
                            try {
                                v1 = (new Long( ((String)i1).substring(((String)i1).indexOf("_")+2) )).longValue();
                            } catch(Exception exx) {
                                v1 = 0;
                            }
                        }

                        long v2 = 0;
                        boolean number2 = true;
                        try {
                            v2 = (new Long((String)i2)).longValue();
                        } catch (Exception ex) {
                            number2 = false;
                            try {
                                v2 = (new Long( ((String)i2).substring(((String)i2).indexOf("_")+2) )).longValue();
                            } catch(Exception exx) {
                                v2 = 0;
                            }
                        }

                        if( number1 && !number2 )
                            return 1;

                        if( !number1 && number2 )
                            return -1;

                        try {
                            long cmp_val =  v2 - v1;
                            return cmp_val>0?1:(cmp_val<0?-1:0);
                        } catch (Exception ex) {}

                        return 0;
                    }

                    try {
                        long cmp_val =  ip1 - ip2;
                        return cmp_val>0?1:(cmp_val<0?-1:0);
                    } catch (Exception ex) {}

                    return 0;
                }
                public boolean equals(Object obj) {
                    return true;
                }
            };
    } // end of Formatter(...)

    public static StringBuffer formatHtmlStr(Object _src,
                                             int min_length,
                                             int max_length,
                                             String link,
                                             int alignment) {
        StringBuffer src = new StringBuffer(_src.toString());
        if( src == null )
            return new StringBuffer("");

        int strlen = src.length();
        if( ( min_length < 1 && max_length < 1 ) ||
            ( min_length > 0 && max_length > 0 && strlen <= max_length && strlen >= min_length) ||
            ( min_length < 1 && max_length > 0 && strlen <= max_length ) ||
            ( min_length > 0 && max_length < 1 && strlen >= min_length ) )
            return src.insert(0, "<a href=\""+link+"\">").append("</a>");

        // at this point we know that 'src' needs some adjustments before returning it.
        String result = "";
        if( min_length > 0 && strlen < min_length ){
            int added_length = min_length - strlen;
            char[] addition = new char[added_length];
            for( int i=0; i<added_length; i++ )
                addition[i] = ' ';
            if( alignment == LEFT )
                return src.insert(0, "<a href=\""+link+"\">").append("</a>").append(addition);
            else // RIGHT
                return (new StringBuffer(new String(addition))).append("<a href=\""+link+"\">").
                    append(src).append("</a>");
        } else { // max_length > 0 && strlen > max_length
            src.setLength(max_length-3);
            src.insert(0, "<a href=\""+link+"\">");
            return src.append("</a>").append("...");
        }
    } // end of formatHtmlStr(...)


    /**
     * This function does formatting which is easier to explain on example:
     * consider that prepend = '    name: ' and
     * src = 'rolo skor olive chevre blue' and max_length = 30 then
     * result will be block like:
     * '    name: rolo skor olive    '
     * '          chevre blue        '
     * Note that \n converted to space prior to building block.
     */
    static public StringBuffer format2Block(String prepend, String src, int max_length)
        throws Exception {
        if( src == null ) src = "";
        StringBuffer result = new StringBuffer();
        int plength = prepend.length();
        int slength = src.length();
        if( max_length <= plength )
            throw new Exception("max_length is too small, cannot format block.");
        char[] prepend1 = new char[plength];
        for( int i=0; i<plength; i++ )
            prepend1[i] = ' ';

        // split src
        int space_index = -1;
        int new_line_index = -1;
        int tab_index = -1;
        int min_index = -1;
        int last_index = 0;
        String element = "";
        Vector tokens = new Vector();
        for(;;) {
            min_index = -1;
            space_index = src.indexOf(' ', last_index);
            new_line_index = src.indexOf('\n', last_index);
            tab_index = src.indexOf('\t', last_index);
            min_index = space_index>0?space_index:slength;
            if( new_line_index < min_index && new_line_index > -1 )
                min_index = new_line_index;
            if( tab_index < min_index && tab_index > 0)
                min_index = tab_index;

            if( min_index == new_line_index )
                element = src.substring(last_index, min_index-1)+" ";
            else
                element = src.substring(last_index, min_index)+" ";
            tokens.addElement(element);
            last_index = min_index+1;
            if( last_index >= slength ) break;
        }
        // build block
        int line_counter = 0;
        StringBuffer line = new StringBuffer(prepend);
        int appended_tokens = 0;
        boolean line_completed = false;
        for(int i=0; i<tokens.size(); i++) {
            element = (String)tokens.elementAt(i);
            if( ( line.length() + element.length() ) > max_length ) {
                if( appended_tokens == 0 ) {
                    // append token anyway
                    line.append(element+"\n");
                    line_completed = true;
                } else {
                    // do not append token now, open new line and try again
                    i--; // has to roll back in order to see the same token again
                    appended_tokens = 0;
                    line_completed = false;
                }
                result.append(rtrim(line)+"\n");
                line = new StringBuffer();
                line.append(prepend1);
                line_counter++;

            } else {
                appended_tokens++;
                line_completed = false;
                line.append(element);
            }
        }
        if( !line_completed )
            result.append(rtrim(line));
        return result;
    } // end of format2Block(...)

     // There should be a better way to do it.
    static public StringBuffer format2HtmlBlock(String prepend, int plength,
                                                String src, int max_length)
        throws Exception {
        if( src == null ) src = "";
        StringBuffer result = new StringBuffer();
        int slength = src.length();
        if( max_length <= plength )
            throw new Exception("max_length is too small, cannot format block.");
        char[] prepend1 = new char[plength];
        for( int i=0; i<plength; i++ )
            prepend1[i] = ' ';

        // split src
        int space_index = -1;
        int new_line_index = -1;
        int tab_index = -1;
        int min_index = -1;
        int last_index = 0;
        String element = "";
        Vector tokens = new Vector();
        for(;;) {
            min_index = -1;
            space_index = src.indexOf(' ', last_index);
            new_line_index = src.indexOf('\n', last_index);
            tab_index = src.indexOf('\t', last_index);
            min_index = space_index>0?space_index:slength;
            if( new_line_index < min_index && new_line_index > -1 )
                min_index = new_line_index;
            if( tab_index < min_index && tab_index > 0)
                min_index = tab_index;

            if( min_index == new_line_index )
                element = src.substring(last_index, min_index-1)+" ";
            else
                element = src.substring(last_index, min_index)+" ";
            tokens.addElement(element);
            last_index = min_index+1;
            if( last_index >= slength ) break;
        }
        // build block
        int line_counter = 0;
        StringBuffer line = new StringBuffer(prepend);
        int length = plength;
        int appended_tokens = 0;
        boolean line_completed = false;
        for(int i=0; i<tokens.size(); i++) {
            element = (String)tokens.elementAt(i);
            if( ( length + html_length(element) ) > max_length ) {
                if( appended_tokens == 0 ) {
                    // append token anyway
                    line.append(element);
                    line_completed = true;
                    length += html_length(element);
                } else {
                    // do not append token now, open new line and try again
                    i--; // has to roll back in order to see the same token again
                    appended_tokens = 0;
                    line_completed = false;
                }
                result.append(rtrim(line)+"\n");
                line = new StringBuffer();
                line.append(prepend1);
                line_counter++;
                length = plength;

            } else {
                appended_tokens++;
                line_completed = false;
                if( i+1 == tokens.size() ) element = element;
                line.append(element);
                length += html_length(element);
            }
        }
        if( !line_completed ) // append last line
            result.append(rtrim(line));

        return result;
    } // end of format2HtmlBlock(...)

    public static String get_display_name(Hashtable name) {
        String domain = (String)Utils.n2s(name.get("domain"));
        if( domain.toLowerCase().equals(WhoIsServer.default_domain) )
            return (String)Utils.n2s(name.get("name"));

        return (String)Utils.n2s(name.get("name"))+"."+domain;
    } // end of get_display_name(...)

    // returns empty string of requested length
    public static String blank(int length) {
        String retval = "";
        for( int i = 0; i < length; i++ )
            retval += " ";
        return retval;
    }

    static public Object n2s(Object src) {
        if( src == null ) return "";
        return src;
    }

    // calculate length of string as it is displayed in a web browser by skipping over html tags
    public static int html_length(String str) {
        int length = 0;
        boolean skip_tag = false;
        boolean skip_amp = false;
        for( int i = 0; i < str.length(); i++ ) {
            if( str.charAt(i) == '&' )
                skip_amp = true;
            else if( str.charAt(i) == '<' )
                skip_tag = true;


            if( skip_amp && str.charAt(i) == ';' )
                skip_amp = false; // not skipping this one as it is counted as one visual char.

            if( skip_tag && str.charAt(i) == '>' ) {
                skip_tag = false;
                continue;
            }

            if( skip_tag || skip_amp ) continue;

            length++;
        }
        return length;
    } // end of html_length(...)

    public static String label(String src) {
        return "<font color=\""+ncolor+"\">"+src+"</font>";
    } // end of label(...)

    // remove trailing spaces
    public static String rtrim(Object src) {
        return src.toString().replaceAll("\\s+$", "");
    }
}
