/************************************************************************
 *
 * HttpFormatter.java -
 *
 * $Id: HttpFormatter.java,v 1.6 2007-03-28 16:48:31 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.whois;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.*;
import java.net.*;
import su.netdb.parser.*;
import stanford.netdb.utils.*;

/**
 * This class does html formatting of data retrieved from db as a result of search
 * and as a result generates html text
 */
public class HttpFormatter extends Formatter {

    public HttpFormatter(Hashtable sr, int search_for) {
        super(sr, search_for);
    }

    /**
     * Does actual formatting of 'search_result' in html.
     *
     * @param result       generated html written into result StringBuffer
     */
    public void format(StringBuffer result, String line) {
        try {
            Hashtable distinct_records = new Hashtable();
            Hashtable distinct_mxes = new Hashtable();
            for( Hashtable record : (Collection<Hashtable>)search_result.values() ) {
                if( record.get("NodeID") != null ) {
                    if( n2s(record.get("name_type")).equals("MX") )
                        distinct_mxes.put(record.get("NodeID"), record);
                    else
                        distinct_records.put(record.get("NodeID"), record);

                } else if( record.get("handle") != null ) // this one will account for networks
                    distinct_records.put(record.get("handle").toString().toLowerCase(), record);

            }

            if( distinct_records.size() == 0 && distinct_mxes.size() == 0 ) {
                result.append("No records matched your search for "+label(line));
                return;
            }

            //int printed = 0; // total printed, i.e. sent to the client
            boolean do_not_print_mx = false;

            boolean do_info   = false;
            String node_id    = null;
            String net_handle = null;
            if( distinct_records.size()==1 || (distinct_records.size()==0 && distinct_mxes.size()==1) ) {
                do_info = true;

                Hashtable record = distinct_records.size()==1?(Hashtable)distinct_records.elements().nextElement():null;
                if( record == null )
                    record = distinct_mxes.size()==1?(Hashtable)distinct_mxes.elements().nextElement():null;

                if( record == null ) { // no way this should happen but let me check for it anyway
                    result.append("INTERNAL ERROR");
                    return;
                }

                if( record.get("NodeID")      != null ) node_id    = record.get("NodeID").toString();
                else if( record.get("handle") != null ) net_handle = record.get("handle").toString();
                else {
                    result.append("INTERNAL ERROR");
                    return;
                }
            }

            if( do_info )
                result.append("Your search for "+label(line)+" found the following match:\n");
            else
                result.append(search_result.size()+" records matched your search for "+
                              label(line)+"<br>\n<font size=\"-1\">Click on a name for "+
                              "more details about the machine, or try "+
                              "<a href=\"#form\">another search</a></font>.\n\n<pre>");

            result.append("<pre>\n");

            Hashtable outv = new Hashtable();
            boolean mxes_on_top = false;
            if( search_for == WhoIsServer.SEARCH_FOR_NAME ) {
                java.util.List alist = Arrays.asList(search_result.entrySet().toArray());
                Collections.sort(alist, in_name_cmp);
                Object[] records = alist.toArray();
                String last_mx_referent_id = "";
                for (int i = 0; i < records.length; i++) {
                    Hashtable record = (Hashtable)((Map.Entry)records[i]).getValue();

                    if( record.get("name") != null ) {
                        if( record.get("name_type").equals("MX") ) { // mx name
                            if( do_info ) {
                                result.append((last_mx_referent_id.length()>0&&!last_mx_referent_id.equals((String)record.get("name"))?"\n":"")+
                                              "     "+label("mailhost:")+" "+format(record.get("name"), 23, 23, '\0', LEFT)+
                                          " MX    "+
                                          format(record.get("preference"),   2,  4, '\0', LEFT)+" "+
                                          formatHtmlStr(record.get("received_by_fullname"), -1, -1,
                                                        st_what_url+URLEncoder.encode(record.get("received_by_fullname").toString(), "UTF-8"), LEFT)+"\n");
                                mxes_on_top = true;
                            } else
                                result.append((last_mx_referent_id.length()>0&&!last_mx_referent_id.equals((String)record.get("name"))?"\n":"")+
                                              format(record.get("name"), 23, 23, '\0', RIGHT)+
                                              "  MX   "+
                                              format(record.get("preference"),   2,  4, '\0', LEFT)+" "+
                                              formatHtmlStr(record.get("received_by_fullname"), -1, -1,
                                                            st_what_url+URLEncoder.encode(record.get("received_by_fullname").toString(), "UTF-8"), LEFT)+"\n");
                            last_mx_referent_id = (String)record.get("name");

                        } else if( !do_info ) { // regular name (i.e. not mx)
                            if( record.get("name_type").equals("ipc_addr_name") ) // ipc name
                                result.append((last_mx_referent_id.length() > 0?"\n":"")+
                                              formatHtmlStr(record.get("name"), 23, 23,
                                                            st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                              format(n2s(record.get("ip")), 10, 19, '\0', LEFT)+"\n"+
                                              blank(25)+format(n2s(record.get("department")), 40, -1, '\0', LEFT)+
                                              "\n                         *IPC Address Name*\n\n");
                            else
                                result.append((last_mx_referent_id.length() > 0?"\n":"")+
                                              formatHtmlStr(record.get("name"), 23, 23,
                                                            st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                              format(n2s(record.get("ip")), 10, 19, '\0', LEFT)+"\n"+
                                              blank(25)+format(n2s(record.get("department")), -1, -1, '\0', LEFT)+"\n"+
                                              blank(25)+format(n2s(record.get("make_and_model")), -1, -1, '\0', LEFT)+" running "+
                                              format(n2s(record.get("os")), -1, -1, '\0', LEFT)+"\n\n");
                            last_mx_referent_id = "";
                        }

                    } else if( record.get("dhcp_name") != null && !do_info ) {
                        result.append((last_mx_referent_id.length() > 0?"\n":"")+
                                      formatHtmlStr(record.get("dhcp_name"), 23, 23,
                                                    st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                      format(record.get("ip"), 17, 17, '\0', LEFT)+
                                      "*Dynamic DHCP Name*\n\n");

                        last_mx_referent_id = "";

                    } else if( record.get("net_name") != null && !do_info ) {
                        String net_comment = n2s(record.get("comment")).toString().trim();
                        result.append((last_mx_referent_id.length() > 0?"\n":"")+
                                      formatHtmlStr(record.get("net_name"), 23, 23,
                                                    st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                      format(n2s(record.get("ip")), -1, -1, '\0', LEFT)+"\n"+
                                      (net_comment.length()>0?("                         "+
                                                               format(new StringBuffer(net_comment), 0, 57, '\0', LEFT)+"\n"):"")+
                                      format(new StringBuffer("*Network*"), 34, 34, '\0', RIGHT)+"\n\n");
                        last_mx_referent_id = "";
                    }
                } // end of loop over all rows

            } else if( search_for == WhoIsServer.SEARCH_FOR_IP_ADDR && !do_info ) {
                java.util.List alist = Arrays.asList(search_result.keySet().toArray());
                Collections.sort(alist, ip_cmp);
                Object[] keys = alist.toArray();
                for (int i = 0; i < keys.length; i++) {
                    Hashtable record = (Hashtable)search_result.get(keys[i]);

                    String ip_type = n2s(record.get("ip_type")).toString();
                    if( ip_type.equals("intfc_ip") )
                        result.append(formatHtmlStr(record.get("ip"), 23, 23,
                                                    st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                      format(record.get("name"), -1, -1, '\0', LEFT)+"\n"+
                                      "                         "+
                                      format(n2s(record.get("department")), -1, -1, '\0', LEFT)+"\n"+
                                      "                         "+
                                      format(n2s(record.get("make_and_model")), -1, -1, '\0', LEFT)+
                                      " running "+
                                      format(n2s(record.get("os")), -1, -1, '\0', LEFT)+"\n\n");

                    else if( ip_type.equals("ipc_ip") )
                        result.append(formatHtmlStr(record.get("ip"), 23, 23,
                                                    st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                      format(record.get("name"), -1, -1, '\0', LEFT)+"\n"+
                                      "                         "+
                                      format(n2s(record.get("department")), -1, -1, '\0', LEFT)+"\n"+
                                      "                         *IPC Address*\n\n");

                    else if( record.get("net_name") != null ) {
                        // it can be either dhcp ip or address space
                        if( record.get("dhcp_name") == null ) {// address space
                            String net_comment = n2s(record.get("comment")).toString().trim();
                            result.append(formatHtmlStr(record.get("ip"), 23, 23,
                                                        st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                          format(record.get("net_name"), -1, -1, '\0', LEFT)+"\n"+
                                          (net_comment.length()>0?("                         "+
                                                                   format(new StringBuffer(net_comment), -1, -1, '\0', LEFT)):"")+"\n\n");

                        } else // dhcp ip
                            result.append(formatHtmlStr(record.get("ip"), 23, 23,
                                                        st_what_url+URLEncoder.encode(record.get("handle").toString(), "UTF-8"), RIGHT)+"  "+
                                          format(n2s(record.get("dhcp_name")), -1, -1, '\0', LEFT)+"\n"+
                                          "                         *Dynamic DHCP Address*\n\n");
                    }
                }
            }

            if( do_info ) {
                if( mxes_on_top ) result.append("\n");

                if( net_handle != null )
                    formatNetworkInfo(WhoIsServer.netdb.getInfo(Constants.NETWORK, net_handle), result);
                if( node_id != null )
                    formatNodeInfo(WhoIsServer.netdb.getInfo(Constants.NODE, Integer.parseInt(node_id)), result);
            }

            result.append("</pre>\n");

        } catch (Exception ex) {
            ex.printStackTrace();
            WhoIsServer.error(ex);
            result.append("INTERNAL ERROR");
            return;
        }

    } // end of format(...)


    // Format and print node info for single match
    public void formatNodeInfo(Hashtable node, StringBuffer result) throws Exception {
        boolean show_state = System.getProperty(WhoIsServer.SHOW_STATE_KEY, "true" ).trim().equals("true");
        boolean show_room  = System.getProperty(WhoIsServer.SHOW_ROOM_KEY,  "false").trim().equals("true");
        boolean show_tags  = System.getProperty(WhoIsServer.SHOW_TAGS_KEY,  "true" ).trim().equals("true");

        // print names
        Hashtable names = (Hashtable)node.get("names");
        List list1 = null;
        Iterator it1 = null;
        List list2 = null;
        Iterator it2 = null;
        List list3 = null;
        Iterator it3 = null;
        Hashtable name = null;
        Hashtable aliases = null;
        Hashtable mxs = null;
        List list = Arrays.asList(names.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();

        String key = "";
        String key1 = "";
        long start_time = (new java.util.Date()).getTime();
        if( WhoIsServer.debug ) {
            System.out.print("Formatting node info... ");
            System.out.flush();
        }

        while( it.hasNext() ) {
            key = (String)it.next(); // it is a name
            name = (Hashtable)names.get(key);
            result.append("         "+label("name:")+" "+get_display_name(name)+"\n");
            aliases = (Hashtable)name.get("aliases");
            list1 = Arrays.asList(aliases.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();

            // print aliases
            Vector aliases_v = new Vector();
            String alias = "            "+label("alias:")+" ";
            int index = 0;
            while( it1.hasNext() ) {
                key1 = (String)it1.next(); // it is an alias
                Hashtable alias_h = (Hashtable)aliases.get(key1);
                if( alias.length()+key1.length() > 77 ) {
                    alias += ",";
                    aliases_v.addElement(alias);
                    alias = "                   "+get_display_name(alias_h);
                    index = 1;
                }else{
                    index++;
                    if( index == 1 )
                        alias += get_display_name(alias_h);
                    else
                        alias += ", "+get_display_name(alias_h);
                }
            }
            if( index > 0 )
                aliases_v.addElement(alias);

            if( aliases_v.size() > 0 ) {
                for( int i=0; i<aliases_v.size(); i++ )
                    result.append((String)aliases_v.elementAt(i)+"\n");
            }

            // print mx's
            Vector mxs_v = new Vector();
            String mx = "    "+label("recv mail for:")+" ";
            index = 0;
            mxs = (Hashtable)name.get("mxs");
            list1 = Arrays.asList(mxs.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();

            while( it1.hasNext() ) {
                key1 = (String)it1.next(); // it is an mx
                Hashtable mx_h = (Hashtable)mxs.get(key1);
                String fqdn = mx_h.get("name")+"."+mx_h.get("domain");
                String dsp_name = "<a href="+st_what_url+URLEncoder.encode(fqdn, "UTF-8")+">"+
                    get_display_name(mx_h)+"</a>"+" ("+mx_h.get("pref")+")";
                if( html_length(mx)+html_length(dsp_name) > 77 ) {
                    mx += ",";
                    mxs_v.addElement(mx);
                    mx = "                   "+dsp_name;
                    index = 1;
                }else{
                    index++;
                    mx += (index==1?"":", ")+dsp_name;
                }
            }
            if( index > 0 )
                mxs_v.addElement(mx);

            if( mxs_v.size() > 0 ) {
                for( int i=0; i<mxs_v.size(); i++ )
                    result.append((String)mxs_v.elementAt(i)+"\n");
            }
        }

        // the rest
        Hashtable types = Utils.nHash(node.get("types"));
        String type_list = types.size()==0?"Host":"";
        for( Map.Entry ent : (java.util.List<Map.Entry>)Utils.sort(types.entrySet(), Utils.node_type_cmp) )
            type_list += (type_list.length()>0?", ":"")+((Hashtable)ent.getValue()).get("type");
        result.append("         "+label("type:")+" Node ("+Utils.htmlize(type_list, false)+")\n");

        if( show_state ) result.append("        "+label("state:")+" "+node.get("state")+"\n");

        String make_and_model = n2s(node.get("make"))+" "+n2s(node.get("model"));
        result.append("          "+label("cpu:")+" "+
                      format(make_and_model, 0, 64, '\0', LEFT)+"\n");

        result.append("       "+label("op-sys:")+" ");
        Hashtable oses = Utils.nHash(node.get("oses"));
        Object[] os = Utils.sorted_keys(oses, name_cmp);
        for( int i = 0; i < os.length; i++ ) {
            String version = ((String)oses.get(os[i])).trim();
            result.append(os[i]+(version.length()>0?(" ("+version+")"):"")+(i+1<os.length?", ":""));
        }
        result.append("\n");

        result.append("   "+label("department:")+" "+
                      format(n2s(node.get("dept")), 0, 64, '\0', LEFT)+"\n");
        result.append("     "+label("location:")+" "+
                      format(node.get("location")+" ("+node.get("site")+"-"+
                             node.get("code")+")", 0, 64, '\0', LEFT)+"\n");

        if( show_room )
            result.append("         "+label("room:")+" "+
                          format(n2s(node.get("room")), 0, 64, '\0', LEFT)+"\n");

        // print address space, it should appear only if it is a template
        String addr_space = (String)node.get("addr_space");
        if( addr_space != null )
            result.append(label("address-space:")+" "+addr_space+"\n");

        // print interfaces
        Hashtable interfaces = (Hashtable)node.get("interfaces");
        if( interfaces != null ) {
            if( interfaces.size() > 0 )
                result.append("   "+label("interfaces:")+"\n");
            String prepend = "";

            java.util.List alist = Arrays.asList(interfaces.entrySet().toArray());
            Collections.sort(alist, intfc_cmp);
            Object[] interfaces_array = alist.toArray();
            for (int i = 0; i < interfaces_array.length; ++i) {
                prepend = "  "+format((new StringBuffer()).append(i+1), 4, 4, ')', RIGHT)+" ";
                Hashtable intfc = (Hashtable)((Map.Entry)interfaces_array[i]).getValue();
                names = (Hashtable)intfc.get("names");
                // print interface names
                if( names != null ) {
                    list1 = Arrays.asList(names.keySet().toArray());
                    Collections.sort(list1, name_cmp);
                    it1 = list1.iterator();
                    int names_counter = 0;
                    while( it1.hasNext() ) {
                        names_counter++;
                        name = (Hashtable)names.get(it1.next());
                        result.append(prepend + "   "+label("name:")+" "+get_display_name(name)+"\n");
                        prepend = "        ";
                        if( name != null ) {
                            Hashtable aliases_h = Utils.nHash(name.get("aliases"));
                            list2 = Arrays.asList(aliases_h.keySet().toArray());
                            Collections.sort(list2, name_cmp);
                            it2 = list2.iterator();
                            while( it2.hasNext() )
                                result.append("                "+label("alias:")+" "+
                                              get_display_name((Hashtable)aliases_h.get(it2.next()))+"\n");
                        }
                    }
                }
                if( intfc.get("hw") != null && intfc.get("hw").toString().trim().length() > 0 ) {
                    String hw = (String)intfc.get("hw");
                    result.append(prepend+label("hw-addr:")+" "+
                               (hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+
                                hw.substring(8)).toLowerCase());
                    if( intfc.get("use-dhcp?") != null && ((Boolean)intfc.get("use-dhcp?")).booleanValue() )
                        result.append("  dhcp");
                    if( intfc.get("roaming?") != null && ((Boolean)intfc.get("roaming?")).booleanValue() )
                        result.append("  roam");
                    result.append("\n");
                    prepend = "        ";
                }
                display_dhcp_options(Utils.nHash(intfc.get("dhcp-options")), "       "+label("dhcp-opt:")+" ", result);

                Hashtable ip_addresses = (Hashtable)intfc.get("ips");
                if( ip_addresses != null ) {
                    list1 = Arrays.asList(ip_addresses.keySet().toArray());
                    Collections.sort(list1, ip_cmp);
                    it1 = list1.iterator();
                    int ip_counter = 0;
                    while( it1.hasNext() ) {
                        ip_counter++;
                        String ip = (String)it1.next();
                        Hashtable ip_addr = (Hashtable)ip_addresses.get(ip);
                        String active = ip_addr.get("active?")!=null&&((Boolean)ip_addr.get("active?")).booleanValue()?"   ":"off";
                        String dsp_ip = format(ip, 16, 16, '\0', LEFT).toString();
                        dsp_ip += (types.get("Advanced")!=null?Utils.nStr(ip_addr.get("PTRprefAbbr")):" ")+" "+active;

                        if( Utils.nHash(ip_addr.get("names")).size()==0 )
                            dsp_ip = dsp_ip.trim();
                        if( ip_counter == 1 ) {
                            result.append(prepend+label("ip-addr:")+" "+dsp_ip);
                            prepend = "                 ";
                        } else
                            result.append(prepend+dsp_ip);

                        names = (Hashtable)ip_addr.get("names");
                        if( names != null ) {
                            list2 = Arrays.asList(names.keySet().toArray());
                            Collections.sort(list2, name_cmp);
                            it2 = list2.iterator();
                            while( it2.hasNext() ) {
                                name = (Hashtable)names.get(it2.next());
                                result.append("  "+format(get_display_name(name), -1, 41, '\0', LEFT));
                            }
                        }
                        result.append("\n");
                    }
                }
            }
        }

        // print IPC addresses
        Hashtable ipc_addresses = (Hashtable)node.get("ipc_addresses");
        if( ipc_addresses != null ) {
            it1 = Utils.sortedKeys(ipc_addresses, ip_cmp);
            int ipc_counter = 0;
            while( it1.hasNext() ) {
                ipc_counter++;
                String ip = (String)it1.next();
                Hashtable ipc_address = (Hashtable)ipc_addresses.get(ip);
                String active = ipc_address.get("active?")!=null&&((Boolean)ipc_address.get("active?")).booleanValue()?"   ":"off";
                String dsp_ip = format(ip, 16, 16, '\0', LEFT).append(active).toString();
                if( Utils.nHash(ipc_address.get("names")).size()==0 )
                    dsp_ip = dsp_ip.trim();
                if( ipc_counter == 1 )
                    result.append(label("IPC addresses:")+" "+dsp_ip);
                else
                    result.append("               "+dsp_ip);
                names = Utils.nHash(ipc_address.get("names"));
                it2 = Utils.sortedKeys(names, name_cmp);
                while( it2.hasNext() ) {
                    name = (Hashtable)names.get(it2.next());
                    result.append(" "+format(get_display_name(name), -1, 42, '\0', LEFT));

                }
                result.append("\n");
            }
        }

        // print users
        Hashtable users = (Hashtable)node.get("users");
        if( users != null ) {
            int user_counter = 0;
            for( Hashtable user : (Collection<Hashtable>)users.values() ) {
                user_counter++;
                if( user_counter == 1 )
                    result.append("         "+label("user:")+" "+person(user)+"\n");
                else
                    result.append("               "+person(user)+"\n");
            }
        }

        // print admins
        Hashtable _admins = (Hashtable)node.get("admins");
        if( _admins != null ) {
            int admin_counter = 0;

            Hashtable admins = null;
            boolean do_break = false;
            for(;;) {
                if( admins == null )
                    admins = Utils.nHash(_admins.get("ateams"));
                else {
                    admins = Utils.nHash(_admins.get("admins"));
                    do_break = true;
                }

                for( String akey : (Set<String>)admins.keySet() ) {
                    admin_counter++;
                    Hashtable admin = (Hashtable)admins.get(akey);

                    if( admin_counter == 1 )
                        result.append(label("administrator:")+" ");
                    else
                        result.append("               ");

                    if( admin.get("name") != null )
                        result.append(person(admin)+"\n");
                    else
                        result.append(akey+"\n");

                    if( !do_break ) { // admin team
                        if( admin.get("email") != null )
                            result.append("                  "+label("e-mail:")+" "+admin.get("email")+"\n");

                        if( admin.get("phone") != null )
                            result.append("                   "+label("phone:")+" "+admin.get("phone")+"\n");
                    }
                }
                if( do_break ) break;
            }
        }

        if( node.get("exp.date") != null )
            result.append("     "+label("exp date:")+" "+(new SimpleDateFormat("MMM dd yyyy")).format(node.get("exp.date"))+"\n");

        // print custom fields, a.k.a tags
        if( show_tags ) {
            Hashtable tags = Utils.nHash(node.get("tags"));
            Object[] label = Utils.sorted_keys(tags, name_cmp);

            if( label.length > 0 ) result.append(label("custom fields:")+"\n");

            // first find max length of the keys
            int max_label_length = 15; // 15 is a minimum length
            for( int i = 0; i < label.length; i++ ) {
                int current_length = ((String)label[i]).length();
                max_label_length = current_length>max_label_length?current_length:max_label_length;
            }

            // now print it
            for( int i = 0; i < label.length; i++ ) {
                try {
                    String full_label = label(format(Utils.htmlize((String)label[i], false),
                                                     max_label_length, max_label_length, '\0', RIGHT)+":")+" ";
                    result.append(format2HtmlBlock(full_label, html_length(full_label),
                                                   Utils.htmlize((String)tags.get(label[i]), false), 78)+"\n");
                } catch (Exception ex) { }
            }
        }

        display_groups(node, "        "+label("group:")+" ", result);
        display_comment(node, "      "+label("comment:")+" ", result);
        display_mod_by(node, result);
        result.append(" "+label("date-updated:")+" "+sdf.format(node.get("modified"))+"\n"+"\n");

        if( WhoIsServer.debug )
            System.out.println("Done in "+((new java.util.Date()).getTime()-start_time)+"ms.");
    } // end of formatNodeInfo(...)

    // Format and print network info for single match
    public void formatNetworkInfo(Hashtable network, StringBuffer result) {
        int index = 0;
        String key = null;
        List list1 = null;
        Iterator it1 = null;
        List list2 = null;
        Iterator it2 = null;
        List list3 = null;
        Iterator it3 = null;
        Hashtable address_spaces = null;
        if( network == null ) return;

        long start_time = (new java.util.Date()).getTime();
        if( WhoIsServer.debug ) {
            System.out.print("Formatting network info... ");
            System.out.flush();
        }
        // names, etc.
        Hashtable names = (Hashtable)network.get("names");
        if( names != null ) {
            list1 = Arrays.asList(names.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();
            String name = "         "+label("name:")+" ";
            Vector names_v = new Vector();
            index = 0;
            while( it1.hasNext() ) {
                key = (String)it1.next();
                String name_str = get_display_name((Hashtable)names.get(key));
                if( name.length()+name_str.length() > 77 ) {
                    name += ",";
                    names_v.addElement(name);
                    name = "                   "+name_str;
                    index = 1;
                }else{
                    index++;
                    if( index == 1 )
                        name += name_str;
                    else
                        name += ", "+name_str;
                }
            }
            if( index > 0 )
                names_v.addElement(name);

            for( int i=0; i<names_v.size(); i++ )
                result.append(names_v.elementAt(i)+"\n");
        }

        result.append("         "+label("type:")+" Network\n");

        // groups
        display_groups(network, "        "+label("group:")+" ", result);

        String dhcp_service = (String)network.get("dhcp_service");
        if( dhcp_service != null && dhcp_service.trim().length() > 0 )
            result.append(" "+label("dhcp-service:")+" "+dhcp_service+"\n");

        display_dhcp_options(Utils.nHash(network.get("dhcp_options")), "     "+
                             label("dhcp-opt:")+" ", result);
        display_comment(network, "      "+label("comment:")+" ", result);

        // address spaces (subnets)
        result.append("   "+label("ip-subnets:")+"\n");
        index = 1;
        address_spaces = (Hashtable)network.get("address_spaces");
        Object[] as = Utils.sorted_keys(address_spaces, ip_cmp);
        for( int i = 0; i < as.length; i++ ) {
            Hashtable address_space = (Hashtable)address_spaces.get(as[i]);
            result.append(label(format((new StringBuffer()).append(index++), 7, 7, ')', RIGHT)+
                                " addr space:")+" "+
                          format(address_space.get("addr_space_prefix"), 21, 21, '\0', LEFT)+
                          " "+label("lo:")+" "+
                          format(address_space.get("low_res"),  5, -1, '\0', LEFT)+
                          " "+label("hi:")+" "+
                          format(address_space.get("high_res"), 0, -1, '\0', LEFT)+"\n");

            display_groups(address_space, "              "+label("group:")+" ", result);
            display_comment(address_space, "            "+label("comment:")+" ", result);
            display_dhcp_options(Utils.nHash(address_space.get("dhcp_options")),
                                 "           "+label("dhcp-opt:")+" ", result);

            Hashtable dhcp_addresses = Utils.nHash(address_space.get("dhcp_addresses"));
            it2 = Utils.sortedKeys(dhcp_addresses, ip_cmp);
            int dhcp_addr_counter = -1;
            while( it2.hasNext() ) {
                String ip = (String)it2.next();
                Hashtable ip_address = (Hashtable)dhcp_addresses.get(ip);
                String active = ip_address.get("active?")!=null&&((Boolean)ip_address.get("active?")).booleanValue()?"   ":"off";
                String dsp_ip = format(ip, 18, 18, '\0', LEFT).append(active).toString();
                if( Utils.nHash(ip_address.get("names")).size()==0 )
                    dsp_ip = dsp_ip.trim();
                dhcp_addr_counter++;
                if( dhcp_addr_counter == 0 )
                    result.append("          "+label("dhcp-addr:")+" "+dsp_ip);
                else
                    result.append("                     "+dsp_ip);
                names = Utils.nHash(ip_address.get("names"));
                it3 = Utils.sortedKeys(names, name_cmp);
                while( it3.hasNext() )
                    result.append(" "+format(get_display_name((Hashtable)names.get(it3.next())), -1, 36, '\0', LEFT));
                result.append("\n");
            }
        }

        display_mod_by(network, result);
        result.append(" "+label("date-updated:")+" "+sdf.format(network.get("modified"))+"\n\n");

        if( WhoIsServer.debug )
            System.out.println("Done in "+((new java.util.Date()).getTime()-start_time)+"ms.");
    } // end of formatNetworkInfo(...)

    public void display_groups(Hashtable record, String prepend, StringBuffer result) {
        // print netdb groups
        Hashtable groups = (Hashtable)record.get("groups");
        if( groups != null ) {
            List list1 = Arrays.asList(groups.keySet().toArray());
            Collections.sort(list1, name_cmp);
            Iterator it1 = list1.iterator();

            Vector groups_v = new Vector();
            String groupsStr = prepend;
            int index  = 0;
            while( it1.hasNext() ) {
                String key = (String)it1.next(); // it is a group name
                if( html_length(groupsStr)+key.length() > 77 ) { // wrapping here
                    groups_v.addElement(groupsStr+",");
                    groupsStr = blank(html_length(prepend))+Utils.htmlize(key, false);
                    index = 1;
                } else {
                    index++;
                    groupsStr += (index==1?"":", ")+Utils.htmlize(key, false);
                }
            }
            if( index > 0 ) groups_v.addElement(groupsStr);

            if( groups_v.size() > 0 ) {
                for( int i=0; i<groups_v.size(); i++ )
                    result.append((String)groups_v.elementAt(i)+"\n");
            }
        }
    } // end of display_groups(...)

    public void display_comment(Hashtable record, String prepend, StringBuffer result) {
        String comment = (String)record.get("comment");
        if( comment != null && comment.trim().length() > 0 ) {
            try {
                result.append(format2HtmlBlock(prepend, html_length(prepend),
                                               Utils.htmlize(comment, false), 78)+"\n");
            } catch(Exception ex) {
                result.append(ex+"\n");
            }
        }
    } // end of display_comment(...)

    public void display_dhcp_options(Hashtable dhcp_options, String prepend, StringBuffer result) {
        Iterator options = Utils.sortedKeys(dhcp_options, name_cmp);
        int j = 0;
        while( options.hasNext() )
            result.append((j++==0?prepend:blank(html_length(prepend)))+
                          format(options.next(), -1, -1, '\0', LEFT)+"\n");
    } // end of display_dhcp_options(...)

    public void display_mod_by(Hashtable record, StringBuffer result) {
        Hashtable mod_by = Utils.nHash(record.get("mod.by"));
        String dir_id = (String)mod_by.get("DirID");
        Boolean active = (Boolean)mod_by.get("active?");
        String mod_by_name = Utils.htmlize((String)n2s(mod_by.get("name")), false);
        if( dir_id != null && dir_id.length() > 0 && active != null && active.booleanValue() )
            mod_by_name = "<a href=\""+st_who_url+dir_id+"\">"+mod_by_name+"</a>";
        result.append("   "+label("updated-by:")+" "+mod_by_name+"\n");
    } // end of display_mod_by(...)

    public String person(Hashtable person) {
        String dir_id = (String)person.get("DirID");
        Boolean active = (Boolean)person.get("active?");
        String person_name = Utils.htmlize((String)n2s(person.get("name")), false);
        if( dir_id != null && dir_id.length() > 0 && active != null && active.booleanValue() )
            person_name = "<a href=\""+st_who_url+dir_id+"\">"+person_name+"</a>";
        return person_name;
    }

    /**
     * same as the one in Formater but src is htmlized.
     */
    public static StringBuffer format(Object src,
                                      int min_length,
                                      int max_length,
                                      char last,
                                      int alignment) {
        return
            new StringBuffer(Utils.htmlize(WhoIsFormatter.format(src,  min_length,
                                                                 max_length, last, alignment).toString(),
                                           false));
    } // end of format(...)
}
