/************************************************************************
 *
 * WhoIsFormatter.java -
 *
 * $Id: WhoIsFormatter.java,v 1.6 2007-03-28 16:55:08 priimak Exp $
 *
 *-----------------------------------------------------------------------
 * Copyright (c) 2006, Board of Trustees, Leland Stanford Jr. University
 ************************************************************************/
package stanford.netdb.whois;

import java.util.*;
import java.lang.*;
import java.io.*;
import java.text.*;
import java.net.*;
import stanford.netdb.lib.*;
import stanford.netdb.utils.*;
import su.netdb.parser.*;

/**
 * This class does formatting of data retrieved from db as a result of search
 * and at the same time printing it to user supplied print string.
 */
public class WhoIsFormatter extends Formatter {

    /**
     * Constructor, it is not intented to be constructed explicitly, but through
     * NetDButils.getWhoIsFormatter(...), but of course you can do it yourself.
     */
    public WhoIsFormatter(Hashtable sr, int search_for) {
        super(sr, search_for);
    }

    public void format(PrintStream cout, boolean no_mx_count) {
        try {
            // first lets go over all results and see how many distinct NetDB records we caught.
            Hashtable distinct_records = new Hashtable();
            Hashtable distinct_mxes = new Hashtable();
            for( Hashtable record : (Collection<Hashtable>)search_result.values() ) {
                if( record.get("NodeID") != null ) {
                    if( n2s(record.get("name_type")).equals("MX") ) {
                        if( no_mx_count )
                            continue;
                        else
                            distinct_mxes.put(record.get("NodeID"), record);
                    } else
                        distinct_records.put(record.get("NodeID"), record);

                } else if( record.get("handle") != null ) // this one will account for networks
                    distinct_records.put(record.get("handle").toString().toLowerCase(), record);

            }

            if( ( no_mx_count && distinct_records.size() == 0 ) ||
                ( distinct_records.size() == 0 && distinct_mxes.size() == 0 )  ) {
                cout.println("NO MATCH");
                return;
            }

            boolean do_info   = false;
            String node_id    = null;
            String net_handle = null;
            if( distinct_records.size()==1 || (distinct_records.size()==0 && distinct_mxes.size()==1) ) {
                do_info = true;

                Hashtable record = distinct_records.size()==1?(Hashtable)distinct_records.elements().nextElement():null;
                if( record == null )
                    record = distinct_mxes.size()==1?(Hashtable)distinct_mxes.elements().nextElement():null;

                if( record == null ) { // no way this should happen but let me check for it anyway
                    cout.println("INTERNAL ERROR");
                    return;
                }

                if( record.get("NodeID")      != null ) node_id    = record.get("NodeID").toString();
                else if( record.get("handle") != null ) net_handle = record.get("handle").toString();
                else {
                    cout.println("INTERNAL ERROR");
                    return;
                }
            }

            Vector outv = new Vector();
            if( search_for == WhoIsServer.SEARCH_FOR_NAME ) {
                java.util.List alist = Arrays.asList(search_result.entrySet().toArray());
                Collections.sort(alist, in_name_cmp);
                Object[] records = alist.toArray();
                String last_mx_referent_id = "";
                for (int i = 0; i < records.length; i++) {
                    Hashtable record = (Hashtable)((Map.Entry)records[i]).getValue();

                    if( record.get("name") != null ) {
                        if( record.get("name_type").equals("MX") ) { // mx name
                            if( !no_mx_count ) {
                                outv.addElement((last_mx_referent_id.length()>0&&!last_mx_referent_id.equals((String)record.get("name"))?"\n":"")+
                                                format(record.get("name"),        23, 23, '\0', LEFT)+" "+
                                                "MX    "+
                                                format(record.get("preference"),   2,  4, '\0', LEFT)+" "+
                                                format(record.get("received_by_fullname"), -1, -1, '\0', LEFT));
                                last_mx_referent_id = (String)record.get("name");
                            }

                        } else if( !do_info ) { // regular name (i.e. not mx)
                            if( record.get("name_type").equals("ipc_addr_name") ) // ipc name
                                outv.addElement((last_mx_referent_id.length() > 0?"\n":"")+
                                                format(record.get("name"),            23, 23, '\0', LEFT)+" <"+
                                                format(record.get("handle"),          35, -1,  '>', LEFT)+" "+
                                                format(n2s(record.get("ip")),         10, 19, '\0', LEFT)+"\n    "+
                                                format(n2s(record.get("department")), 58, 58, '\0', LEFT)+
                                                "*IPC Address Name*\n");
                            else
                                outv.addElement((last_mx_referent_id.length() > 0?"\n":"")+
                                                format(record.get("name"),                23, 23, '\0', LEFT)+" <"+
                                                format(record.get("handle"),              35, -1,  '>', LEFT)+" "+
                                                format(n2s(record.get("ip")),             10, 19, '\0', LEFT)+"\n    "+
                                                format(n2s(record.get("department")),     40, 40, '\0', LEFT)+" "+
                                                format(n2s(record.get("make_and_model")), 16, 16, '\0', LEFT)+" "+
                                                format(n2s(record.get("os")),              0, 18, '\0', LEFT)+"\n");
                            last_mx_referent_id = "";
                        }

                    } else if( record.get("dhcp_name") != null ) {
                        if( !do_info ) {
                            outv.addElement((last_mx_referent_id.length() > 0?"\n":"")+
                                            format(record.get("dhcp_name"), 23, 23, '\0', LEFT)+" <"+
                                            format(record.get("handle"),    35, -1,  '>', LEFT)+" "+
                                            format(record.get("ip"),        10, 19, '\0', LEFT)+
                                            "\n    *Dynamic DHCP Name*\n");
                            last_mx_referent_id = "";
                        }

                    } else if( record.get("net_name") != null && !do_info ) {
                        outv.addElement((last_mx_referent_id.length() > 0?"\n":"")+
                                        format(record.get("net_name"),     23, 23, '\0', LEFT)+" <"+
                                        format(record.get("handle"),       35, -1,  '>', LEFT)+" "+
                                        format(n2s(record.get("ip")),      10, 19, '\0', LEFT)+"\n    "+
                                        format(n2s(record.get("comment")), 57, 57, '\0', LEFT)+" "+
                                        "*Network*\n");
                        last_mx_referent_id = "";
                    }
                }

            } else if( search_for == WhoIsServer.SEARCH_FOR_IP_ADDR && !do_info ) {
                for( Object key : Utils.sort(search_result.keySet(), ip_cmp) ) {
                    Hashtable record = (Hashtable)search_result.get(key);
                    String ip_type = (String)n2s(record.get("ip_type"));
                    if( ip_type.equals("intfc_ip") )
                        outv.addElement(format(record.get("ip"),                  20, 20, '\0', LEFT)+" <"+
                                        format(record.get("handle"),              32, -1,  '>', LEFT)+" "+
                                        format(record.get("name"),                0,  25, '\0', LEFT)+"\n    "+
                                        format(n2s(record.get("department")),     40, 40, '\0', LEFT)+" "+
                                        format(n2s(record.get("make_and_model")), 16, 16, '\0', LEFT)+" "+
                                        format(n2s(record.get("os")),             0,  18, '\0', LEFT)+"\n");

                    else if( ip_type.equals("ipc_ip") )
                        outv.addElement(format(record.get("ip"),         20, 20, '\0', LEFT)+" <"+
                                        format(record.get("handle"),     32, -1, '>',  LEFT)+" "+
                                        format(record.get("name"),       0,  25, '\0', LEFT)+"\n    "+
                                        format(record.get("department"), 40, 40, '\0', LEFT)+" "+
                                        "*IPC Address*\n");

                    else if( record.get("net_name") != null ) {
                        // it can be either dhcp ip or address space
                        if( record.get("dhcp_name") == null ) {// address space
                            String comment = (String)n2s(record.get("comment"));
                            outv.addElement(format(record.get("ip"),       20, 20, '\0', LEFT)+" <"+
                                            format(record.get("handle"),   32, -1,  '>', LEFT)+" "+
                                            format(record.get("net_name"), 0,  25, '\0', LEFT)+"\n"+
                                            (comment.length()>0?("    "+
                                                                 format(comment,   0, 74, '\0', LEFT)+"\n"):""));

                        } else // dhcp ip
                            outv.addElement(format(record.get("ip"),             20, 20, '\0', LEFT)+" <"+
                                            format(record.get("handle"),         32, -1,  '>', LEFT)+" "+
                                            format(n2s(record.get("dhcp_name")), 0,  25, '\0', LEFT)+
                                            "\n    *Dynamic DHCP Address*\n");
                    }
                }
            }

            // print what have here as it may contain mxes even if we do info page
            for( int i = 0; i < outv.size(); i++ )
                cout.println(outv.elementAt(i));

            if( do_info ) {
                // if something from outv was printed it means that we printed mxes above and need to add empty line
                if( outv.size() > 0 ) cout.println("");

                if( net_handle != null )
                    formatNetworkInfo(WhoIsServer.netdb.getInfo(Constants.NETWORK, net_handle), cout);
                if( node_id != null )
                    formatNodeInfo(WhoIsServer.netdb.getInfo(Constants.NODE, Integer.parseInt(node_id)), cout);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            WhoIsServer.error(ex);
            cout.println("INTERNAL ERROR");
            return;
        }
    } // end of format(...)

    /**
     * Format and print node info for single match, make sure that result set is an output of infoNodeNew
     *
     * @param node  Hashtable representing NetDB node object.
     * @param cout  Specifies output stream.
     */
    public void formatNodeInfo(Hashtable node, PrintStream cout) {
        boolean show_state = System.getProperty(WhoIsServer.SHOW_STATE_KEY, "true" ).trim().equals("true");
        boolean show_room  = System.getProperty(WhoIsServer.SHOW_ROOM_KEY,  "false").trim().equals("true");
        boolean show_tags  = System.getProperty(WhoIsServer.SHOW_TAGS_KEY,  "true" ).trim().equals("true");

        // print names
        Hashtable names = (Hashtable)node.get("names");
        List list1 = null;
        Iterator it1 = null;
        List list2 = null;
        Iterator it2 = null;
        List list3 = null;
        Iterator it3 = null;
        Hashtable name = null;
        Hashtable aliases = null;
        Hashtable mxs = null;
        List list = Arrays.asList(names.keySet().toArray());
        Collections.sort(list, name_cmp);
        Iterator it = list.iterator();

        String key = "";
        String key1 = "";
        long start_time = (new java.util.Date()).getTime();
        if( WhoIsServer.debug ) {
            System.out.print("Formatting node info... ");
            System.out.flush();
        }

        while( it.hasNext() ) {
            key = (String)it.next(); // it is a name
            name = (Hashtable)names.get(key);
            cout.println("         name: "+get_display_name(name));
            aliases = (Hashtable)name.get("aliases");
            list1 = Arrays.asList(aliases.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();

            // print aliases
            Vector aliases_v = new Vector();
            String alias = "            alias: ";
            int index = 0;
            while( it1.hasNext() ) {
                key1 = (String)it1.next(); // it is an alias
                Hashtable alias_h = (Hashtable)aliases.get(key1);
                if( alias.length()+key1.length() > 77 ) {
                    alias += ",";
                    aliases_v.addElement(alias);
                    alias = "                   "+get_display_name(alias_h);
                    index = 1;
                }else{
                    index++;
                    if( index == 1 )
                        alias += get_display_name(alias_h);
                    else
                        alias += ", "+get_display_name(alias_h);
                }
            }
            if( index > 0 )
                aliases_v.addElement(alias);

            if( aliases_v.size() > 0 ) {
                for( int i=0; i<aliases_v.size(); i++ )
                    cout.println((String)aliases_v.elementAt(i));
            }

            // print mx's
            Vector mxs_v = new Vector();
            String mx = "    recv mail for: ";
            index = 0;
            mxs = (Hashtable)name.get("mxs");
            list1 = Arrays.asList(mxs.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();

            while( it1.hasNext() ) {
                key1 = (String)it1.next(); // it is an mx
                Hashtable mx_h = (Hashtable)mxs.get(key1);
                String dsp_name = get_display_name(mx_h)+" ("+mx_h.get("pref")+")";
                if( mx.length()+dsp_name.length() > 77 ) {
                    mx += ",";
                    mxs_v.addElement(mx);
                    mx = "                   "+dsp_name;
                    index = 1;
                }else{
                    index++;
                    mx += (index==1?"":", ")+dsp_name;
                }
            }
            if( index > 0 )
                mxs_v.addElement(mx);

            if( mxs_v.size() > 0 ) {
                for( int i=0; i<mxs_v.size(); i++ )
                    cout.println((String)mxs_v.elementAt(i));
            }
        }

        // the rest
        Hashtable types = Utils.nHash(node.get("types"));
        String type_list = types.size()==0?"Host":"";
        for( Map.Entry ent : (java.util.List<Map.Entry>)Utils.sort(types.entrySet(), Utils.node_type_cmp) )
            type_list += (type_list.length()>0?", ":"")+((Hashtable)ent.getValue()).get("type");
        cout.println("         type: Node ("+type_list+")");

        if( show_state ) cout.println("        state: "+node.get("state"));

        String make_and_model = n2s(node.get("make"))+" "+n2s(node.get("model"));
        cout.println("          cpu: "+
                     format(make_and_model, 0, 64, '\0', LEFT));

        cout.print("       op-sys: ");
        Hashtable oses = Utils.nHash(node.get("oses"));
        Object[] os = Utils.sorted_keys(oses, name_cmp);
        for( int i = 0; i < os.length; i++ ) {
            String version = ((String)oses.get(os[i])).trim();
            cout.print(os[i]+(version.length()>0?(" ("+version+")"):"")+(i+1<os.length?", ":""));
        }
        cout.println("");

        cout.println("   department: "+
                     format(n2s(node.get("dept")), 0, 64, '\0', LEFT));
        cout.println("     location: "+
                     format(node.get("location")+" ("+node.get("site")+"-"+
                            node.get("code")+")", 0, 64, '\0', LEFT));

        if( show_room )
            cout.println("         room: "+
                         format(n2s(node.get("room")),
                                64, 64, '\0', LEFT));

        // print address space, it should appear only if it is a template
        String addr_space = (String)node.get("addr_space");
        if( addr_space != null )
            cout.println("address-space: "+addr_space);

        // print interfaces
        Hashtable interfaces = (Hashtable)node.get("interfaces");
        if( interfaces != null ) {
            if( interfaces.size() > 0 )
                cout.println("   interfaces:");
            String prepend = "";

            java.util.List alist = Arrays.asList(interfaces.entrySet().toArray());
            Collections.sort(alist, intfc_cmp);
            Object[] interfaces_array = alist.toArray();
            for (int i = 0; i < interfaces_array.length; ++i) {
                prepend = "  "+format((new StringBuffer()).append(i+1), 4, 4, ')', RIGHT)+" ";
                Hashtable intfc = (Hashtable)((Map.Entry)interfaces_array[i]).getValue();
                names = (Hashtable)intfc.get("names");
                // print interface names
                if( names != null ) {
                    list1 = Arrays.asList(names.keySet().toArray());
                    Collections.sort(list1, name_cmp);
                    it1 = list1.iterator();
                    int names_counter = 0;
                    while( it1.hasNext() ) {
                        names_counter++;
                        name = (Hashtable)names.get(it1.next());
                        cout.println(prepend + "   name: "+get_display_name(name));
                        prepend = "        ";
                        if( name != null ) {
                            Hashtable aliases_h = Utils.nHash(name.get("aliases"));
                            list2 = Arrays.asList(aliases_h.keySet().toArray());
                            Collections.sort(list2, name_cmp);
                            it2 = list2.iterator();
                            while( it2.hasNext() )
                                cout.println("                alias: "+
                                             get_display_name((Hashtable)aliases_h.get(it2.next())));
                        }
                    }
                }
                if( intfc.get("hw") != null && intfc.get("hw").toString().trim().length() > 0 ) {
                    String hw = (String)intfc.get("hw");
                    cout.print(prepend + "hw-addr: "+
                               (hw.substring(0, 4)+"."+hw.substring(4, 8)+"."+
                                hw.substring(8)).toLowerCase());
                    if( intfc.get("use-dhcp?") != null && ((Boolean)intfc.get("use-dhcp?")).booleanValue() )
                        cout.print("  dhcp");
                    if( intfc.get("roaming?") != null && ((Boolean)intfc.get("roaming?")).booleanValue() )
                        cout.print("  roam");
                    cout.println("");
                    prepend = "        ";
                }
                display_dhcp_options(Utils.nHash(intfc.get("dhcp-options")), "       dhcp-opt: ", cout);

                Hashtable ip_addresses = (Hashtable)intfc.get("ips");
                if( ip_addresses != null ) {
                    list1 = Arrays.asList(ip_addresses.keySet().toArray());
                    Collections.sort(list1, ip_cmp);
                    it1 = list1.iterator();
                    int ip_counter = 0;
                    while( it1.hasNext() ) {
                        ip_counter++;
                        String ip = (String)it1.next();
                        Hashtable ip_addr = (Hashtable)ip_addresses.get(ip);
                        String active = ip_addr.get("active?")!=null&&((Boolean)ip_addr.get("active?")).booleanValue()?"   ":"off";
                        String dsp_ip = format(ip, 16, 16, '\0', LEFT).toString();
                        dsp_ip += (types.get("Advanced")!=null?Utils.nStr(ip_addr.get("PTRprefAbbr")):" ")+" "+active;

                        if( Utils.nHash(ip_addr.get("names")).size()==0 )
                            dsp_ip = dsp_ip.trim();
                        if( ip_counter == 1 ) {
                            cout.print(prepend + "ip-addr: " + dsp_ip);
                            prepend = "                 ";
                        } else
                            cout.print(prepend + dsp_ip);

                        names = (Hashtable)ip_addr.get("names");
                        if( names != null ) {
                            list2 = Arrays.asList(names.keySet().toArray());
                            Collections.sort(list2, name_cmp);
                            it2 = list2.iterator();
                            while( it2.hasNext() ) {
                                name = (Hashtable)names.get(it2.next());
                                cout.print("  "+format(get_display_name(name), -1, 41, '\0', LEFT));
                            }
                        }
                        cout.println("");
                    }
                }
            }
        }

        // print IPC addresses
        Hashtable ipc_addresses = (Hashtable)node.get("ipc_addresses");
        if( ipc_addresses != null ) {
            it1 = Utils.sortedKeys(ipc_addresses, ip_cmp);
            int ipc_counter = 0;
            while( it1.hasNext() ) {
                ipc_counter++;
                String ip = (String)it1.next();
                Hashtable ipc_address = (Hashtable)ipc_addresses.get(ip);
                String active = ipc_address.get("active?")!=null&&((Boolean)ipc_address.get("active?")).booleanValue()?"   ":"off";
                String dsp_ip = format(ip, 16, 16, '\0', LEFT).append(active).toString();
                if( Utils.nHash(ipc_address.get("names")).size()==0 )
                    dsp_ip = dsp_ip.trim();
                if( ipc_counter == 1 )
                    cout.print("IPC addresses: "+dsp_ip);
                else
                    cout.print("               "+dsp_ip);
                names = Utils.nHash(ipc_address.get("names"));
                it2 = Utils.sortedKeys(names, name_cmp);
                while( it2.hasNext() ) {
                    name = (Hashtable)names.get(it2.next());
                    cout.print(" "+format(get_display_name(name), -1, 42, '\0', LEFT));

                }
                cout.println("");
            }
        }

        // print users
        Hashtable users = (Hashtable)node.get("users");
        if( users != null ) {
            int user_counter = 0;
            for( Hashtable user : (Collection<Hashtable>)users.values() ) {
                user_counter++;
                if( user_counter == 1 )
                    cout.print("         user: "+user.get("name"));
                else
                    cout.print("               "+user.get("name"));
                String dirid = (String)user.get("DirID");
                if( dirid != null && dirid.length() > 0 )
                    cout.print(" ("+dirid.toLowerCase()+")");
                cout.println("");
            }
        }

        // print admins
        Hashtable _admins = (Hashtable)node.get("admins");
        if( _admins != null ) {
            int admin_counter = 0;

            Hashtable admins = null;
            boolean do_break = false;
            for(;;) {
                if( admins == null )
                    admins = Utils.nHash(_admins.get("ateams"));
                else {
                    admins = Utils.nHash(_admins.get("admins"));
                    do_break = true;
                }

                for( String akey : (Set<String>)admins.keySet() ) {
                    admin_counter++;
                    Hashtable admin = (Hashtable)admins.get(akey);

                    if( admin_counter == 1 )
                        cout.print("administrator: ");
                    else
                        cout.print("               ");

                    if( admin.get("name") != null ) {
                        Object dirid = admin.get("DirID");
                        cout.println(admin.get("name")+
                                     (dirid!=null?(" ("+dirid.toString().toLowerCase()+")"):""));
                    } else
                        cout.println(akey);

                    if( !do_break ) { // admin team
                        if( admin.get("email") != null )
                            cout.println("                  e-mail: "+admin.get("email"));

                        if( admin.get("phone") != null )
                            cout.println("                   phone: "+admin.get("phone"));
                    }
                }
                if( do_break ) break;
            }
        }

        if( node.get("exp.date") != null )
            cout.println("     exp date: "+(new SimpleDateFormat("MMM dd yyyy")).format(node.get("exp.date")));

        // print custom fields, a.k.a tags
        if( show_tags ) {
            Hashtable tags = Utils.nHash(node.get("tags"));
            Object[] label = Utils.sorted_keys(tags, name_cmp);

            if( label.length > 0 ) cout.println("custom fields:");

            // first find max length of the keys
            int max_label_length = 15; // 15 is a minimum length
            for( int i = 0; i < label.length; i++ ) {
                int current_length = ((String)label[i]).length();
                max_label_length = current_length>max_label_length?current_length:max_label_length;
            }

            // now print it
            for( int i = 0; i < label.length; i++ ) {
                try {
                    cout.println(format2Block(format(label[i], max_label_length,
                                                     max_label_length, '\0', RIGHT)+": ",
                                              (String)tags.get(label[i]), 78));
                } catch (Exception ex) { }
            }
        }

        display_groups(node, "        group: ", cout);
        display_comment(node, "      comment: ", cout);
        display_mod_by(node, cout);
        cout.println(" date-updated: "+sdf.format(node.get("modified"))+"\n");

        if( WhoIsServer.debug )
            System.out.println("Done in "+((new java.util.Date()).getTime()-start_time)+"ms.");
    } // end of formatNodeInfo(...)

    /**
     * Format and print network info for single match, make sure that result set is an output of infoNodeNew
     *
     * @param network  Hashtable representing NetDB network object.
     * @param cout  Specifies output stream.
     */
    public void formatNetworkInfo(Hashtable network, PrintStream cout) {
        int index = 0;
        String key = null;
        List list1 = null;
        Iterator it1 = null;
        List list2 = null;
        Iterator it2 = null;
        List list3 = null;
        Iterator it3 = null;
        Hashtable address_spaces = null;
        if( network == null ) return;

        long start_time = (new java.util.Date()).getTime();
        if( WhoIsServer.debug ) {
            System.out.print("Formatting network info... ");
            System.out.flush();
        }
        // names, etc.
        Hashtable names = (Hashtable)network.get("names");
        if( names != null ) {
            list1 = Arrays.asList(names.keySet().toArray());
            Collections.sort(list1, name_cmp);
            it1 = list1.iterator();
            String name = "         name: ";
            Vector names_v = new Vector();
            index = 0;
            while( it1.hasNext() ) {
                key = (String)it1.next();
                String name_str = get_display_name((Hashtable)names.get(key));
                if( name.length()+name_str.length() > 77 ) {
                    name += ",";
                    names_v.addElement(name);
                    name = "                   "+name_str;
                    index = 1;
                }else{
                    index++;
                    if( index == 1 )
                        name += name_str;
                    else
                        name += ", "+name_str;
                }
            }
            if( index > 0 )
                names_v.addElement(name);

            for( int i=0; i<names_v.size(); i++ )
                cout.println(names_v.elementAt(i));
        }

        cout.println("         type: Network");

        // groups
        display_groups(network, "        group: ", cout);

        String dhcp_service = (String)network.get("dhcp_service");
        if( dhcp_service != null && dhcp_service.trim().length() > 0 )
            cout.println(" dhcp-service: "+dhcp_service);

        display_dhcp_options(Utils.nHash(network.get("dhcp_options")), "     dhcp-opt: ", cout);
        display_comment(network, "      comment: ", cout);

        // address spaces (subnets)
        cout.println("   ip-subnets:");
        index = 1;
        address_spaces = (Hashtable)network.get("address_spaces");
        Object[] as = Utils.sorted_keys(address_spaces, ip_cmp);
        for( int i = 0; i < as.length; i++ ) {
            Hashtable address_space = (Hashtable)address_spaces.get(as[i]);
            cout.println(format((new StringBuffer()).append(index++), 7, 7, ')', RIGHT)+
                         " addr space: "+format(address_space.get("addr_space_prefix"), 21, 21, '\0', LEFT)+
                         " lo: "+
                         format(address_space.get("low_res"),  5, -1, '\0', LEFT)+
                         " hi: "+
                         format(address_space.get("high_res"), 0, -1, '\0', LEFT));

            display_groups(address_space, "              group: ", cout);
            display_comment(address_space, "            comment: ", cout);
            display_dhcp_options(Utils.nHash(address_space.get("dhcp_options")), "           dhcp-opt: ", cout);

            Hashtable dhcp_addresses = Utils.nHash(address_space.get("dhcp_addresses"));
            it2 = Utils.sortedKeys(dhcp_addresses, ip_cmp);
            int dhcp_addr_counter = -1;
            while( it2.hasNext() ) {
                String ip = (String)it2.next();
                Hashtable ip_address = (Hashtable)dhcp_addresses.get(ip);
                String active = ip_address.get("active?")!=null&&((Boolean)ip_address.get("active?")).booleanValue()?"   ":"off";
                String dsp_ip = format(ip, 18, 18, '\0', LEFT).append(active).toString();
                if( Utils.nHash(ip_address.get("names")).size()==0 )
                    dsp_ip = dsp_ip.trim();
                dhcp_addr_counter++;
                if( dhcp_addr_counter == 0 )
                    cout.print("          dhcp-addr: "+dsp_ip);
                else
                    cout.print("                     "+dsp_ip);
                names = Utils.nHash(ip_address.get("names"));
                it3 = Utils.sortedKeys(names, name_cmp);
                while( it3.hasNext() )
                    cout.print(" "+format(get_display_name((Hashtable)names.get(it3.next())), -1, 36, '\0', LEFT));
                cout.println("");
            }
        }

        display_mod_by(network, cout);
        cout.println(" date-updated: "+sdf.format(network.get("modified"))+"\n");

        if( WhoIsServer.debug )
            System.out.println("Done in "+((new java.util.Date()).getTime()-start_time)+"ms.");
    } // end of formatNetworkInfo(...)

    public void display_groups(Hashtable record, String prepend, PrintStream cout) {
        // print netdb groups
        Hashtable groups = (Hashtable)record.get("groups");
        if( groups != null ) {
            List list1 = Arrays.asList(groups.keySet().toArray());
            Collections.sort(list1, name_cmp);
            Iterator it1 = list1.iterator();

            Vector groups_v = new Vector();
            String groupsStr = prepend;
            int index  = 0;
            while( it1.hasNext() ) {
                String key = (String)it1.next(); // it is a group name
                if( groupsStr.length()+key.length() > 77 ) { // wrapping here
                    groups_v.addElement(groupsStr+",");
                    groupsStr = blank(prepend.length())+key;
                    index = 1;
                } else {
                    index++;
                    groupsStr += (index==1?"":", ")+key;
                }
            }
            if( index > 0 ) groups_v.addElement(groupsStr);

            if( groups_v.size() > 0 ) {
                for( int i=0; i<groups_v.size(); i++ )
                    cout.println((String)groups_v.elementAt(i));
            }
        }
    } // end of display_groups(...)

    public void display_comment(Hashtable record, String prepend, PrintStream cout) {
        String comment = (String)record.get("comment");
        if( comment != null && comment.trim().length() > 0 ) {
            try {
                cout.println(format2Block(prepend, comment, 78));
            } catch(Exception ex) {
                cout.println(ex);
            }
        }
    } // end of display_comment(...)

    public void display_mod_by(Hashtable record, PrintStream cout) {
        Hashtable mod_by = Utils.nHash(record.get("mod.by"));
        cout.print("   updated-by: "+mod_by.get("name"));
        String dirid = (String)mod_by.get("DirID");
        if( dirid != null && dirid.length() > 0 )
            cout.print(" ("+dirid.toLowerCase()+")");
        cout.println("");
    } // end of display_mod_by(...)

    public void display_dhcp_options(Hashtable dhcp_options, String prepend, PrintStream cout) {
        Iterator options = Utils.sortedKeys(dhcp_options, name_cmp);
        int j = 0;
        while( options.hasNext() )
            cout.println(format((j++==0?prepend:blank(prepend.length()))+options.next(), -1, 79, '\0', LEFT));
    } // end of display_dhcp_options(...)

    /**
     * Simple string formatter for whois output, it manipulates string directly on a
     * passed StringBuffer without making a copy. Note that is up to programmer to ensure
     * that min_length < max_length.
     *
     * @param src         Source string buffer which has to be formatted, if src is null then empty
     *                    string is returned.
     *
     * @param min_length  Minimum length of resulted string, if length of string is less then
     *                    min_length then white space(s) whill be added to the right of the string.
     *                    If min_length is negative or zero then then min_length constrains won't
     *                    be applied.
     *
     * @param max_length  Maximum length of resulted string, if length of string is more then
     *                    max_length then string will be truncated and '...' will added at the end.
     *                    Becouse of adding '...' at the end max_length has to be <b>grater then 2!!!</b>
     *                    if max_length is negative or zero then then maz_length constrains won't
     *                    be applied.
     *
     * @param last        if not 0 then this char will be appented at the end.
     *
     * @param alignment   Alignment can be NetDButils.LEFT or RIGHT
     *
     * @return            Returns string buffer formatted and alligned to the left.
     */
    public static StringBuffer format(StringBuffer src,
                                      int min_length,
                                      int max_length,
                                      char last,
                                      int alignment) {
        if( src == null )
            return new StringBuffer("");

        int strlen = src.length();
        if( ( min_length < 1 && max_length < 1 ) ||
            ( min_length > 0 && max_length > 0 && strlen <= max_length && strlen >= min_length) ||
            ( min_length < 1 && max_length > 0 && strlen <= max_length ) ||
            ( min_length > 0 && max_length < 1 && strlen >= min_length ) ){
            if( last != 0 )
                if( max_length == strlen ){
                    src.setLength(max_length-4);
                    return src.append("...").append(last);
                }else
                    return src.append(last);
            return src;
        }

        // at this point we know that 'src' needs some adjustments before returning it.
        String result = "";
        if( min_length > 0 && strlen < min_length ){
            int added_length = min_length - strlen;
            char[] addition = new char[added_length];
            for( int i=0; i<added_length; i++ )
                addition[i] = ' ';
            if( last != 0 && alignment == LEFT )
                addition[0] = last;
            if( alignment == LEFT )
                return src.append(addition);
            else { // RIGHT
                StringBuffer sb = (new StringBuffer(new String(addition))).append(src);
                if( last != 0 ) sb.append(last);
                return sb;
            }
        }else // max_length > 0 && strlen > max_length
            if( last != 0 ){
                src.setLength(max_length-4);
                return src.append("...").append(last);
            }else{
                src.setLength(max_length-3);
                return src.append("...");
            }
    } // end of format(...)

    /** Same as above only difference is that it takes String not StringBuffer as an argument */
    public static StringBuffer format(String src, int min_length, int max_length,
                                      char last,  int alignment) {
        return format(new StringBuffer(src), min_length, max_length, last, alignment);
    }

    /** Same as above only difference is that it takes String not StringBuffer as an argument */
    public static StringBuffer format(Object src, int min_length, int max_length,
                                      char last,  int alignment) {
        return format(new StringBuffer(src.toString()), min_length, max_length, last, alignment);
    }
}
